/**@name LIBRARY: unCPC_declarations.ctl

@author: Enrique Blanco (EN/ICE)

Creation Date: 14/11/2011

version 1.0

External Functions: None

Internal Functions: None

Purpose: This library contains unicos CPC constants

Usage: Public

Modification History:
    13/09/2012: Alexey
        [UCPC-931] add constants for the alarms menu item

    16/05/2012: Alexey
        [UCPC-787] introduce AlSt bit and letter constants

    14/05/2012: Alexey
        [UCPC-803] unify ConfigW bit

    27/03/2012: Alexey
        [UCPC-733] added FuStopISt bit position for PCO

    13/03/2012: Alexey
        [UCPC-688] added set parameter constants

    20/01/2012: Alexey
        [UCPC-599] added MFC constants

    12/12/2011: Alexey
        [UCPC-621] modified alarm constants

    23/11/2011: Alexey
        removed custom AA/DA mask event constants
        cloned right menu constants

    25/10/2011: Alexey
        [UCPC-587] all letter constants has been ported

    20/10/2011: Alexey
        [UCPC-585] added CPC_STSREG01_CONTROLLER_ARMRCPST

    21/09/2011: Alexey
        [UCPC-555] added CPC_CONFIG_ALARM_LEVEL

    20/09/2011: Alexey
        [CPC_PARAMS_HLDRIVE] added CPC_PARAMS_HLDRIVE

    13/09/2011: Alexey
        [UCPC-531] added CPC_STSREG02_ANALOGONST/CPC_STSREG02_ANALOGOFFST
        [UCPC-511] fixed AA inhibited thresholds constants
        [UCPC-532] added CPC_STSREG02_OUTOFFOVST and CPC_PARAMS_OUTOFF

    07/09/2011: Alexey
        [UCPC-430] added CPC_STSREG02_OUTONOVST

    25/08/2011: Alexey
        [UCPC-511] removed UN_CPC_MANREG01_MNEWTHRESR/UN_CPC_MANREG01_MEDHH/UN_CPC_MANREG01_MEDH/UN_CPC_MANREG01_MEDL/UN_CPC_MANREG01_MEDLL

    24/08/2011: Alexey
        [UCPC-466] added CPC_PARAMS_ALDT constant for alarm's delay time parameter

	11/08/2011: Alexey
        [UCPC-476] added CPC_WIDGET_TEXT_CONFIGW/CPC_WIDGET_TEXT_CONFIGW/CPC_WIDGET_TEXT_POSHHW/CPC_WIDGET_TEXT_POSLLW/CPC_WIDGET_TEXT_POSW

	10/08/2011: Alexey
		[UCPC-460] added CPC_POPUPMENU_MASK_IST_TEXT

	04/08/2011: Alexey
		[UCPC-456] added CPC_PARAMS_OUTMAIN and CPC_PARAMS_PWMMODE

	28/07/2011: Alexey
		[UCPC-436] added sms message storage's key CPC_CONFIG_SMS_MESSAGE

	15/07/2011: Alexey
		[UCPC-411] removed UN_STSREG02_PLDCMDST

	13/07/2011: Alexey
		[UCPC-392] changed CPC_MANREG02_MNEWPOSR and added CPC_MANREG01_MPSAVE/CPC_MANREG01_MPREST

	08/07/2011: Alexey
		[UCPC-384] removed "parameters" field from the meta-info block

	01/07/2011: Alexey
		added param constants
		added AA enable/disable manreg constants
		[UCPC-374] added PRstartFS constant

	2011/06/30 Alexey
		[UCPC-362] added manreg constants to set AA limits

	2011/06/28 Alexey
		[UCPC-311] added CPC_STSREG02_SOFTLDST
		added UN_WIDGET_TEXT_CONTROL to follow cpcGenericObject_WidgetControlStateAnimation convention
		[UCPC-345] added CPC_STSREG01_SOFTLDST

	2011/06/23 Alexey
		[UCPC-323] added CPC_STSREG02_NEEDRESTART and CPC_WIDGET_TEXT_MANUAL_RESTART

	2011/06/22 Alexey
		[UCPC-323] added CPC_MANREG01_MRESTART

	2011/06/20 Alexey
		[UCPC-316] added CPC_MANREG01_MRSETTSASFS and CPC_MANREG01_MDALACT
		[UCPC-320] removed UN_MANREG02_ARMRCP

	2011/05/31 Alexey
		[UCPC-298] added CPC_CONFIG_CONTROLLER_SCALING_METHOD_KEY constant

	2011/05/30 Alexey
		[UCPC-292] removed UN_CPC_CONFIG length constants

	2011/05/25 Alexey
		added CPC_WIDGET_TEXT_WARNING_ALARM_BLOCKED

	2011/05/19 amerezhi
		removed UN_STSREG01_AUINHSAVRES as it exists in unicoObjects_declarations

	2011/05/17 amerezhi
		added unCPC6 StsReg and ManReg bit constants
		added interlock status constants for widget
		added IOErrorBlocking popup constant and SetAALimit and OutputPositioning faceplate constants

	2011/05/10 amerezhi
		[UCPC-268] added second ALIAS to metainfo
		added key constant for metainfo

	2011/05/09 amerezhi
		[UCPC-235] added CPC length constants and meta info position constants

	2011/05/09 amerezhi
		[UCPC-195][UCPC-196] added StsReg constants for Alarm objects

	Revision 1.22  2011/01/13 14:00:10  amerezhi
	Merged with gcsObjects_declarations.ctl
	
	Revision 1.22  2009/03/20 14:00:10  gthomas
	-Updated variable names <_GCSDELAYEDALARM_CLASS> with <_ALERT_CLASS_>
	
	Revision 1.21  2008/09/04 14:54:31  gthomas
	-moved back to version 1.15  and added "MFCSetParameters" constant

	Revision 1.20  2008/06/23 08:57:31  gthomas
	added variable const string CPC_CONFIG_EMPTY_DESCRIPTION_STRING = "-";

	Revision 1.19  2008/06/23 08:29:10  gthomas
	Kept only variables for xStatus and xParameters

	Revision 1.18  2008/06/19 12:24:25  gthomas
	uncommented 2 variables used in xParameters lib and pnl

	Revision 1.17  2008/06/13 15:16:38  aabadz
	CPC_MENU_ALARM_MASK_REPLY commented
	because is already declared in another file.

	Revision 1.16  2008/06/09 14:54:27  aabadz
	Result after move some constants, only use use inside this file, from gcsObjects_declarations.

	Revision 1.15  2007/10/30 16:22:09  gthomas
	Introduced the CVS keyword $Log to track changes made in the library

*/

const string CPC_UNDEFINED_CONSTANT_VALUE = "undefined";

const unsigned CPC_METAINFO_LENGTH = 5;
const unsigned CPC_CONFIG_MASTER_NAME	= 1;
const unsigned CPC_CONFIG_PARENTS 		= 2;
const unsigned CPC_CONFIG_CHILDREN 		= 3;
const unsigned CPC_CONFIG_TYPE			= 4;
const unsigned CPC_CONFIG_SECOND_ALIAS	= 5;
const string CPC_CONFIG_MASTER_NAME_KEY	= "MASTER_NAME";
const string CPC_CONFIG_PARENTS_KEY 		= "PARENTS";
const string CPC_CONFIG_CHILDREN_KEY 	= "CHILDREN";
const string CPC_CONFIG_TYPE_KEY			= "TYPE";
const string CPC_CONFIG_SECOND_ALIAS_KEY	= "SECOND_ALIAS";
const string CPC_CONFIG_SMS_MESSAGE		= "SMS_MESSAGE";
const string CPC_CONFIG_ALARM_LEVEL      = "ALARM_LEVEL";

const string UN_CONFIG_CPC_CONTROLLER_SCALING_METHOD_KEY = "SCALING_METHOD";


//---------
// WIDGETS
//---------
const string CPC_WIDGET_TEXT_CONTROL_AUIHFOMOST = "h";
const string CPC_WIDGET_TEXT_CONTROL_AUINHFMO = "h";
const string CPC_WIDGET_TEXT_CONTROL_AUINHMMO = "h";
const string CPC_WIDGET_TEXT_MANUAL_RESTART = "R";
const string CPC_WIDGET_TEXT_ALARM_STARTIST = "I";
const string CPC_WIDGET_TEXT_ALARM_TSTOPIST = "S";
const string CPC_WIDGET_TEXT_ALARM_FUSTOPIST = "F";
const string CPC_WIDGET_TEXT_ALARM_ALST = "A";
const string CPC_WIDGET_TEXT_BLOCK_TFUSTOP = "T";
const string CPC_WIDGET_TEXT_WARNING_ALARM_BLOCKED = "B";
const string CPC_WIDGET_TEXT_CONTROL_MMOST = "M";
const string CPC_WIDGET_TEXT_CONTROL_LDST = "HL";
const string CPC_WIDGET_TEXT_CONTROL_FORCED = "F";
const string CPC_WIDGET_TEXT_CONTROL_MANUAL = "M";
const string CPC_WIDGET_TEXT_CONTROL_SOFTLDST = "L";
const string CPC_WIDGET_TEXT_CONTROL_FOMOST = "F";
const string CPC_WIDGET_TEXT_CONTROL_REMOST = "R";
const string CPC_WIDGET_TEXT_CONTROL_TST = "T";
const string CPC_WIDGET_TEXT_CONFIGW = "C";
const string CPC_WIDGET_TEXT_POSHHW = "W";
const string CPC_WIDGET_TEXT_POSLLW = "W";
const string CPC_WIDGET_TEXT_POSW = "W";
const string CPC_WIDGET_TEXT_WARNING_DEFAULT = "W";
const string CPC_WIDGET_TEXT_WARNING_SIMU = "S";
const string CPC_WIDGET_TEXT_WARNING_ERROR = "E";
const string CPC_WIDGET_TEXT_OLD_DATA = "O";
const string CPC_WIDGET_TEXT_INVALID = "N";
const string CPC_WIDGET_TEXT_ALARM_MASKED = "M";
const string CPC_WIDGET_TEXT_WARNING_BLOCKED = "B";
const string CPC_WIDGET_TEXT_ALARM_BLOCKED = "B";
const string CPC_WIDGET_TEXT_ALARM_DOES_NOT_EXIST = "";

// StsReg01 constants
const unsigned CPC_STSREG01_AST =			0;					/// Alarm Status AA (for PosAl DPE; un_POSAL = 9)
const unsigned CPC_STSREG01_SOFTLDST =		0;					/// Controller software local mode
const unsigned CPC_STSREG01_WST =			1;					/// Warning Status AnalogAlarm
const unsigned CPC_STSREG01_AuMoSt =			2;					/// Auto Mode
const unsigned CPC_STSREG01_MaMoSt =			3;					/// Manual Mode
const unsigned CPC_STSREG01_FoMoSt =			4;					/// Forced mode
const unsigned CPC_STSREG01_MONRST = 		4;					//
const unsigned CPC_STSREG01_MOFFRST = 		5;					//
const unsigned CPC_STSREG01_AUEAIST = 		8;
const unsigned CPC_STSREG01_PREMASKALARM =	8;					/// Premask alarm for DA
const unsigned CPC_STSREG01_REGST = 			9;					// 
const unsigned CPC_STSREG01_POSW = 			9;					//
const unsigned CPC_STSREG01_MIOERBRST = 		9;					// I/O Error blocked
const unsigned CPC_STSREG01_FUSTOPIST = 9;						// Full stop interlock of PCO
const unsigned CPC_STSREG01_TRST = 			10;					// 
const unsigned CPC_STSREG01_POSALST = 		10;					//
const unsigned CPC_STSREG01_TSTOPIST = 		11;					// Temporary Stop Interlock status
const unsigned CPC_STSREG01_OUTPST = 		11;					// 
const unsigned CPC_STSREG01_AUIHFOMOST = 	13;					// Forced mode inhibited
const unsigned CPC_STSREG01_AUIHMBST = 		15;
const unsigned CPC_STSREG01_LLST =			8;					/// ANALOG ALARM -LL
const unsigned CPC_STSREG01_LST=			 	9;					/// ANALOG ALARM -L
const unsigned CPC_STSREG01_HST =			10;					/// ANALOG ALARM -H
const unsigned CPC_STSREG01_HHST =			11;					/// ANALOG ALARM -HH

const unsigned CPC_STSREG01_MAMSKST =		15;					/// STSREG info alarm setMask
const unsigned CPC_STSREG01_PFSPOSONST_MFC = 5;					/// Auto inhibit manual mode (MFC)
const unsigned CPC_STSREG01_POSOV =			11;					/// DC Status Position		????

const unsigned CPC_MANREG01_AMSKST =			11;					/// AlarmMaskOn (Off is when is not bit 11) AA, DA. Output
const unsigned CPC_MANREG01_AMSKRST =		12;					/// AalarmMaskOff (Reset alarm AA, DA) Output.

// AnalogAlarm constants
const unsigned CPC_STSREG01_IST		= 0;
const unsigned CPC_STSREG01_CONFIGW = 5;
const unsigned CPC_STSREG01_ARMRCPST= 3;
const unsigned CPC_STSREG01_CONTROLLER_ARMRCPST = 5;
const unsigned CPC_STSREG01_ACTHHST	= 4;
const unsigned CPC_STSREG01_ACTLLST	= 5;
const unsigned CPC_STSREG01_POSHHW	= 8;
const unsigned CPC_STSREG01_POSHW	= 9;
const unsigned CPC_STSREG01_POSLW	= 10;
const unsigned CPC_STSREG01_POSLLW	= 11;
const unsigned CPC_STSREG01_MALBRST	= 14;
const unsigned CPC_STSREG01_ALST	= 14;
const unsigned CPC_STSREG01_AUIHMB	= 15;

const unsigned CPC_STSREG02_EHHST	= 0;						// 
const unsigned CPC_STSREG02_EHST		= 1;						// 
const unsigned CPC_STSREG02_ELST		= 2;						// 
const unsigned CPC_STSREG02_ELLST	= 3;						// 
const unsigned CPC_STSREG02_HHALST	= 4;						// 
const unsigned CPC_STSREG02_HWST		= 5;						// 
const unsigned CPC_STSREG02_LWST		= 6;						// 
const unsigned CPC_STSREG02_LLALST	= 7;						// 
const unsigned CPC_STSREG02_FUSTOPIST = 10;						// Full  Stop Interlock status
const unsigned CPC_STSREG02_NEEDRESTART = 11;
const unsigned CPC_STSREG02_SOFTLDST = 12;
const unsigned CPC_STSREG02_IHMHHST	=  8;						//
const unsigned CPC_STSREG02_IHMHST	=  9;						//
const unsigned CPC_STSREG02_IHMLST	= 10;						//
const unsigned CPC_STSREG02_IHMLLST	= 11;						//
const unsigned CPC_STSREG02_POUTMAIN = 15;
const unsigned CPC_STSREG02_OUTONOVST    =  0;
const unsigned CPC_STSREG02_ANALOGONST   =  8;
const unsigned CPC_STSREG02_ANALOGOFFST  =  9;
const unsigned CPC_STSREG02_OUTOFFOVST   = 14;



// ManReg01 constants --- seems we not use any more
const unsigned CPC_MANREG02_MNEWPOSR		=  3;						/// Manual Position request trigger bit in ManReg01
const unsigned CPC_MANREG01_MRESTART			=  9;
const unsigned CPC_MANREG01_MIOERBSETRST 	= 10;						// 
const unsigned CPC_MANREG01_MALBSETRST 		= 10;						//
const unsigned CPC_MANREG01_MRSETTSASFS	=  8;                       // Manual reset temporary as full stop bit position in Manreg01^M
const unsigned CPC_MANREG01_MDALACT		= 10;                       // Manual deblock alarm action bit position in ManReg01^M
const unsigned CPC_MANREG01_MNEWHHR		=  8;
const unsigned CPC_MANREG01_MNEWHR		=  9;
const unsigned CPC_MANREG01_MNEWLR		= 12;
const unsigned CPC_MANREG01_MNEWLLR		= 13;

const unsigned CPC_MANREG02_MREGR 			= 0;						//
const unsigned CPC_MANREG02_MOUTPR			= 1;						//
const unsigned CPC_MANREG01_MPSAVE =			 5;					// Request restoring PID parameters
const unsigned CPC_MANREG01_MPREST =			 6;					// Request SAVE PID parameters (version PID without addressing)

// Button constants
const string CPC_FACEPLATE_BUTTON_OUTPUTPOSITIONING_MODE = "OutputPositioning";// Output positioning button

const string CPC_FACEPLATE_BUTTON_PARAMETER_SET_VALUE	=		"Parameter_SetValue"; 			/// Set Value for AnalogParameter and WordParameter
const string CPC_FACEPLATE_BUTTON_DIGITALPARAMETER_SET_VALUE	=	"DigitalParameter_SetValue"; 	/// Set Value DigitalParameter
const string CPC_FACEPLATE_BUTTON_PUMP_SET_FREQUENCY   = 		"SetFrecuency";					/// Set Frecuency Pump

const string CPC_FACEPLATE_BUTTON_AA_SUBPANEL_SET_ALARM   = "SetAlarm";			/// Test subpanel Analog Alarm
const string CPC_FACEPLATE_BUTTON_AA_SUBPANEL_RESET_ALARM   = "ResetAlarm";		/// Test subpanel Analog Alarm
const string CPC_FACEPLATE_BUTTON_AA_SUBPANEL_SET_WARNING   = "SetWarning";		/// Test subpanel Analog Alarm
const string CPC_FACEPLATE_BUTTON_AA_SUBPANEL_RESET_WARNING   = "ResetWarning";	/// Test subpanel Analog Alarm
const string CPC_FACEPLATE_BUTTON_SET_AA_LIMITS = "SetAALimits";		// Setting limits for AA
const string CPC_FACEPLATE_BUTTON_SET_DA_MAIL_SMS = "SetDAMailSMS";		// Setting limits for DA

const string CPC_FACEPLATE_BUTTON_MFC_PARAMS ="MFCSetParameters";
const string CPC_FACEPLATE_BUTTON_ALLOW_RESTART = "AllowRestart";
const string CPC_FACEPLATE_BUTTON_SAVE_VALUE = "SaveValue";


//	AA and DA RBMenu
const unsigned CPC_MENU_ALARM_MASK_REPLY		 = 55;			///Menu Choice result for Digital Alarm and Analog Alarm
const unsigned CPC_MENU_RECIPES_REPLY		 = 56;			///Menu Choice result for Recipes from PCO
const unsigned CPC_MENU_CURRNTPARAMS_REPLY	 = 57;			///Menu Choice result for Current Parameters from PCO
const string CPC_POPUPMENU_RECIPES  			 = "Recipes";	///Menu Item for calling Recipes from PCO
const string CPC_POPUPMENU_CURRENTPARAMS 	 = "Current Parameters";	///Menu Item for calling Current Parameters from PCO
const string CPC_POPUPMENU_MANUAL_IO_ERROR_BLOCK = "IOErrorBlocking";

const string CPC_POPUPMENU_DIAG_INFO_TEXT = "Info_Diagnostic";	///	Unicos "info-diagnostic" menu-text
const string CPC_POPUPMENU_MASK_IST_TEXT = "MaskISt";
const string CPC_POPUPMENU_MASK_ALST_TEXT = "MaskAlSt";
const string CPC_POPUPMENU_ALARMS_TEXT = "Alarms";

const string CPC_POPUPMENU_HIERARCHY = "HierarchyPopup";


const int CPC_POPUPMENU_ACK_NUMBER               = 2;
const int CPC_POPUPMENU_MASK_POSST_NUMBER        = 3;
const int CPC_POPUPMENU_BLOCK_PLC_NUMBER         = 5; // cpc
const int CPC_POPUPMENU_FACEPLATE_NUMBER         = 6;
const int CPC_POPUPMENU_DIAGNOSTIC_NUMBER        = 7;
const int CPC_POPUPMENU_INFO_NUMBER              = 8;
const int CPC_POPUPMENU_DEVICE_CONFIG_NUMBER     = 10;
const int CPC_POPUPMENU_UNMASKEVENT_NUMBER       = 11; // cpc
const int CPC_POPUPMENU_MASKEVENT_NUMBER         = 12; // cpc
const int CPC_POPUPMENU_OPENSYNOPTIC_NUMBER      = 15;
const int CPC_POPUPMENU_MASK_ALST_NUMBER         = 16; // cpc
const int CPC_POPUPMENU_MANUAL_IO_ERROR_NUMBER   = 17; // cpc
const int CPC_POPUPMENU_MASK_IST_NUMBER          = 18; // cpc
const int CPC_POPUPMENU_HIERARCHY_NUMBER         = 19; // cpc
const int CPC_POPUPMENU_DEVICELINK_NUMBER        = 20;
const int CPC_POPUPMENU_ALARMS_NUMBER            = 301; // UN_POPUPMENU_MIN + 1


// MFC constants
const unsigned CPC_MANREG01_MSPR        = 6;					///Manual Set Point Request trigger
const unsigned CPC_MANREG01_MDRMODE     = 12;					///Manual Drive Mode Request Trigger 
const unsigned CPC_MANREG01_MTOTMRT     = 13;					///Manual Totalizer Mode Request Trigger 
const unsigned CPC_MANREG01_MCCRT       = 14;					///Manual Calibration Curve Mode Request Trigger 

// Config constants
const string CPC_CONFIG_EMPTY_DESCRIPTION_STRING = "-";							/// A dummy string to be put in description when no actual string is valid
const string CPC_CONFIG_DEFAULT_CALIBRATION_CURVE = "1=default";	/// default string when there is not string for the calibration curve

// Alarm
const string CPC_WIDGET_ALARM_HIGH		= "H";						///High alarm AnalogAlarm
const string CPC_WIDGET_ALARM_HIGHHIGH	= "HH";						///High High alarm AnalogAlarm
const string CPC_WIDGET_ALARM_LOW		= "L";						///Low alarm AnalogAlarm
const string CPC_WIDGET_ALARM_LOWLOW	= "LL";						///Low Low alarm - for AnalogAlarm
const string CPC_WIDGET_ALARM			= "A";						///Alarm for Digital Alarm

// Mail-SMS alert class constants
const string CPC_CONFIG_ALARM_BASE_CLASS   = "_cpcAnalog";
const string CPC_CONFIG_ALARM_ACK_POSTFIX  = "_Ack";
const string CPC_CONFIG_ALARM_MAIL_POSTFIX = "_Mail";

const int CPC_INT16 	= 1;
const int CPC_INT32 	= 2;
const int CPC_UINT16	= 3;
const int CPC_BOOL 	= 4;
const int CPC_FLOAT 	= 5;

const string CPC_PARAMS_FSPOSON = "PFSPosOn";
const string CPC_PARAMS_HFONST  = "PHFOnSt";
const string CPC_PARAMS_HFOFFST = "PHFOffSt";
const string CPC_PARAMS_PULSEST = "PPulseSt";
const string CPC_PARAMS_HFON    = "PHFOn";
const string CPC_PARAMS_HFOFF   = "PHFOff";
const string CPC_PARAMS_MRESTART= "PEnRstart";
const string CPC_PARAMS_RSTARTFS= "PRstartFS";
const string CPC_PARAMS_OUTOFF  = "POutOff";
const string CPC_PARAMS_OUTMAIN = "POutMain";
const string CPC_PARAMS_PWMMODE = "PPWMMode";
const string CPC_PARAMS_ALDT    = "PAlDt";
const string CPC_PARAMS_HLDRIVE = "PHLDrive";
const string CPC_PARAMS_NOISE_FILTER = "PNoiseFilter";
const string CPC_PARAMS_FEEDBACK_SIM = "PFeedbackSim";
const string CPC_PARAMS_INHIBITMANUALTOTALIZER = "PIhMTot";
const string CPC_MFC_CALIBRATION_UNIT = "CalibrationFlowUnit";
const string CPC_MFC_CALIBRATION_GAS_NAME = "CalibrationGasName";
