/**@name LIBRARY: cpcExportGenericFunctions.ctl

@author: Nikolay KULMAN (IT/CO)

Creation Date: 06 04 2006

Modification History:
	$Log: cpcExportGenericFunctions.ctl,v $
    11/04/2012: Alexey
        [UCPC-764] return N for ack if undef; return 0 for deadband if undef

    23/03/2012: Alexey
        [UCPC-251][UCPC-253] add cpcExportGenericFunctions_getAutonomous

    12/12/2011: Alexey
        [UCPC-621] updated cpcExportGenericFunctions_get5AlertAcknowledgeAlarmValue and cpcExportGenericFunctions_getAcknowledgeAlarmValue according new alarm classes

    26/10/2011: Alexey
        [UCPC-557] removed cpcExportGenericFunctions_getScalingMethod and cpcExportGenericFunctions_getSmsMessage

    03/10/2011: Alexey
        [UCPC-562] removed cpcExportGenericFunctions_getAnalogNormalPosition and cpcExportGenericFunctions_CheckAnalogAlarmType
        instead of cpcExportGenericFunctions_get5RangesAlert and cpcExportGenericFunctions_get5AlertClasses now cpcExportGenericFunctions_get5AlertLimits and cpcExportGenericFunctions_get5AlertClasses
        fixed cpcExportGenericFunctions_get5AlertAcknowledgeAlarmValue
        added cpcExportGenericFunctions_getAlarmActive

    21/09/2011: Alexey
        [UCPC-555] added cpcExportGenericFunctions_getKeyDeviceConfiguration which retrive configuration value by key

	02/08/2011: Alexey
		[UCPC-437] fixed cpcExportGenericFunctions_get5AlertAcknowledgeAlarmValue - don't take H/L into account

	28/07/2011: Alexey
		[UCPC-436] added cpcExportGenericFunctions_getSmsMessage which retrieve sms message

	08/07/2011: Alexey
		[UCPC-384] removed "parameters" field from the meta-info block

	07/07/2011: Alexey
		ported unGenericObject_CheckAnalogAlarmType and used it in cpcExportGenericFunctions_get5AlertClasses (to avoid having g_iAlertRange)

	06/07/2011: Alexey
		[UCPC-336] in cpcExportGenericFunctions_getDeadband use straight deadband value (removed range reference)

	01/07/2011: Alexey
		made auto export of params

	2011/06/15 amerezhi
		modified cpcExportGenericFunctions_getAcknowledgeAlarmValue signature

	2011/05/31 amerezhi
		[UCPC-298] added cpcExportGenericFunctions_getScalingMethod

	2011/05/27 amerezhi
		created separated functions for obtaining is alert acknowledge

	2011/05/17 amerezhi
		moved device specific functions into corresponding unConfigDPT files
		removed unused functions

	2011/05/10 amerezhi
		[IS-527] process metainfo with API; cpcExportGenericFunctions_getMetainfo params extended with exceptionInfo
		[UCPC-268] added second ALIAS
		added dynJoin method which turns dyn_string into string merging elements with separator

	2011/05/09 amerezhi
		[UCPC-235] added metainfo (types relationships configuraiton)
		fixed archive names exporting for Local

	2011/05/09 amerezhi
		[UCPC-195][UCPC-196] added AnalogAlarm and DigitalAlarm export functions

	2011/04/27 amerezhi
		[UCPC-246] fixed Analog dpe position constants according specification

	2011/04/26 amerezhi
		[UCPC-191] updated WordParameter export functions according unCPC6 specs

	21/04/2011: amerezhi
		added alarm acknowledge and 5 range alarm support (DI/DO/AI/AO/Local)

	2011/04/19 amerezhi
		[UCPC-236] updated Word2AnalogStatus export functions according unCPC6 specs

	2011/04/19 amerezhi
		[UCPC-192] updated WordStatus export functions according unCPC6 specs

	2011/04/18 amerezhi
		[UCPC-193] updated WordParameter export functions according unCPC6 specs

	2011/04/15 amerezhi
		[UCPC-208] updated OnOff export functions according unCPC6 specs

	2011/04/13 amerezhi
		updated cpcExportGenericFunctions_getCategories according new API [IS-518]

	2011/04/11 amerezhi
		added cpcExportGenericFunctions_getCategories
		updated export of SMS_CATEGORIES for AI, AO, DI, DO and Local

	2011/04/07 amerezhi
		[UCPC-209] updated Local export functions according unCPC6 specs

	2011/04/06 amerezhi
		[UCPC-206] simplified export of Analog

	2011/04/05 amerezhi
		[UCPC-201] simplified export of Analog Input
		[UCPC-203] simplified export of Analog Output
		[UCPC-151] updated Digital Output for unCPC6 format
		[UCPC-199] simplified export of Digital Input

	2011/03/30 amerezhi
		[UCPC-209] simplified export of Local
		[UCPC-207] simplified export of AnaDig

	2011/03/29 amerezhi
		[UCPC-201] simplified export of AI

	2011/03/28 amerezhi
	simplified export of PCO
	simplified export of DO

	Revision 1.33  2010/09/01 11:10:32 amerezhi
	Exporting default value for WordParameter is fixed
	Exporting achive time filter is fixed
	Added *_MAIL alert classes to the cpcExportGenericFunctions_getAlarmIsAck 
	Added PV_RANGE to the WordComputed export results.
	Changed ARCHIVE_TIME_FILTER's exported value from UN_CONFIG_EXPORT_UNDEF to ""
	Fixed exporting archive time filter for PUMP and MFC

	Revision 1.32  2010/03/25 15:30:58  gthomas
	Fixed #53414 (bad dpe name reference, renamed ASt-->PosAl)
	
	Revision 1.31  2009/03/20 13:59:43  gthomas
	-updated variable names <_GCSDELAYEDALARM_CLASS> with <_ALERT_CLASS_>
	-update variable <GCS_CONFIG_EXPORT_UNDEF> with <UN_CONFIG_EXPORT_UNDEF>

	Revision 1.30  2009/03/17 10:11:45  gthomas
	-added MFC and Pump Export functions from 1.21 version

	Revision 1.29  2009/03/10 15:28:24  gthomas
	added UNICOS Changes made on 17/02/2009
	-Function _UnPlc_DigitalParameter_ExportConfig exports the default value in text mode (True instead of TRUE)

	Revision 1.28  2009/01/15 15:13:17  gthomas
	Added functions to export new type: Word2AnalogStatus

	Revision 1.27  2008/07/23 08:41:25  gthomas
	changed CVS log comment in previous version (1.26)

	Revision 1.26  2008/07/18 07:16:55  gthomas
	added functions for other gcs types

	Revision 1.25  2008/06/20 07:48:46  aabadz
	Some auxiliary functions were deleted because the were not going to be used in the new package to UNICOS.

	Revision 1.24  2008/06/20 06:36:46  gthomas
	introduced CVS key Log to track changes

	20/08/2007: enrique Blanco
	_UnPlc_AnalogStatus_ExportConfig has been modified. In case of no archive defined it shoud give "" instead of UN_CONFIG_EXPORT_UNDEF

version 1.0

External Function : 
	cpcExportGenericFunctions_processAddress(string sDp, string sObject, string object, bool isInput, dyn_string &smoothDpParameters)
	cpcExportGenericFunctions_getMetainfo(string sDp, dyn_string &dsDpParameters, dyn_string &exceptionInfo)
	cpcExportGenericFunctions_getUnit(string sDp, dyn_string &dsDpParameters, string dpe = ".ProcessInput.PosSt")
	cpcExportGenericFunctions_getFormat(string sDp, dyn_string &dsDpParameters, string dpe = ".ProcessInput.PosSt")
	cpcExportGenericFunctions_getRange(string sDp, dyn_string &dsDpParameters, dyn_string &exceptionInfo, string dpe = ".ProcessInput.PosSt")
	cpcExportGenericFunctions_getOutRange(string sDp, dyn_string &dsDpParameters, dyn_string &exceptionInfo, string dpe = ".ProcessInput.PosSt")
	cpcExportGenericFunctions_getDeadband(string sDp, dyn_string &dsDpParameters, dyn_string &exceptionInfo, string dpe = ".ProcessInput.PosSt")
	cpcExportGenericFunctions_getArchive(string sDp, dyn_string &dsDpParameters, string dpe = ".ProcessInput.PosSt")
	cpcExportGenericFunctions_getMaskEvent(string sDp, dyn_string &dsDpParameters)
	cpcExportGenericFunctions_getDefaultValue(string sDp, dyn_string &dsDpParameters, string dfltElementName = "DfltVal")
	cpcExportGenericFunctions_getBoolDefaultValue(string sDp, dyn_string &dsDpParameters, string dfltElementName = "DfltVal")
	cpcExportGenericFunctions_getPIDName(string sDp, dyn_string &dsDpParameters)
	cpcExportGenericFunctions_getPCOName(string sDp, dyn_string &dsDpParameters)
	cpcExportGenericFunctions_getDigitalNormalPosition(string sDP, string sObject, dyn_string dsDpElements, dyn_string &dsDpParameters)
	cpcExportGenericFunctions_getPCOParameters(string sDP, dyn_string &dsDpParameters)
	cpcExportGenericFunctions_get5AlertLimits(string sDP, dyn_string &dsDpParameters)
	cpcExportGenericFunctions_getArchiveNameForDpes(string sDp, string defaultValue, dyn_string dsDpes, dyn_string &dsDpParameters)
	cpcExportGenericFunctions_getCategories(string sDP, dyn_string &dsDpParameters, dyn_string &exceptionInfo)
	cpcExportGenericFunctions_getAcknowledgeAlarm(string sDP, string alarmDP, dyn_string &dsDpParameters, dyn_string &exceptionInfo)
	cpcExportGenericFunctions_get5AlertClasses(string sDP, dyn_string &dsClasses, int &iNormalPosition, dyn_string &exceptionInfo, string dpe = ".ProcessInput.PosSt")
	cpcExportGenericFunctions_get5AlertAcknowledgeAlarm(string sDP, string alarmDP, dyn_string &dsDpParameters, dyn_string &exceptionInfo)
	string dynJoin(dyn_string array, string separator)
*/

void cpcExportGenericFunctions_processAddress(string sDp, string sObject, string object, bool isInput, dyn_string &smoothDpParameters) {
	string temp;
	string prefix = isInput ? ".ProcessInput." : ".ProcessOutput.";

	dpGet(sDp + prefix + object + ":_address.._drv_ident" , temp);
	string sFunction = "DRV_"+temp+"_convertToUnicosAddress";
	if(isFunctionDefined(sFunction)) {
		evalScript(temp, "string main(string sDeviceDpeName, string sDeviceType) {"+
											"	string temp;"+
											"	temp = "+sFunction+"(sDeviceDpeName, sDeviceType);"+
											"	return temp;"+
											"}", makeDynString(), sDp + prefix + object, sObject);
	} else {
		temp = UN_CONFIG_EXPORT_UNDEF;
	}
	dynAppend(smoothDpParameters, temp);
}

void cpcExportGenericFunctions_getParameters(string sDp, dyn_string &dsDpParameters, dyn_string &exceptionInfo) {
	dyn_string params;
	dyn_string paramKeys = cpcConfigGenericFunctions_getDeviceParamNames(dpTypeName(sDp));
	for (int i = 1; i <= dynlen(paramKeys); i++) {
		string value = cpcGenericDpFunctions_getDeviceProperty(sDp, paramKeys[i], exceptionInfo);
		dynAppend(params, paramKeys[i] + "=" + value);
	}

	dynAppend(dsDpParameters, dynJoin(params, ","));
}

void cpcExportGenericFunctions_getMetainfo(string sDp, dyn_string &dsDpParameters, dyn_string &exceptionInfo) {
	dyn_string master, parents, children, type, second_alias;
	unGenericDpFunctions_getKeyDeviceConfiguration(sDp, CPC_CONFIG_MASTER_NAME_KEY, master, exceptionInfo);
	unGenericDpFunctions_getKeyDeviceConfiguration(sDp, CPC_CONFIG_PARENTS_KEY, parents, exceptionInfo);
	unGenericDpFunctions_getKeyDeviceConfiguration(sDp, CPC_CONFIG_CHILDREN_KEY, children, exceptionInfo);
	unGenericDpFunctions_getKeyDeviceConfiguration(sDp, CPC_CONFIG_TYPE_KEY, type, exceptionInfo);
	unGenericDpFunctions_getKeyDeviceConfiguration(sDp, CPC_CONFIG_SECOND_ALIAS_KEY, second_alias, exceptionInfo);

	dynAppend(dsDpParameters, dynJoin(master, ","));
	dynAppend(dsDpParameters, dynJoin(parents, ","));
	dynAppend(dsDpParameters, dynJoin(children, ","));
	dynAppend(dsDpParameters, dynJoin(type, ","));
	dynAppend(dsDpParameters, dynJoin(second_alias, ","));
}

dyn_string cpcExportGenericFunctions_getParams(string sDp, dyn_string paramKeys, dyn_string &exceptionInfo) {
	dyn_string values;
	return values;
}

void cpcExportGenericFunctions_getUnit(string sDp, dyn_string &dsDpParameters, string dpe = ".ProcessInput.PosSt") {
	string unit;
	unit = dpGetUnit(sDp + dpe);
	dynAppend(dsDpParameters, unit);
}

void cpcExportGenericFunctions_getFormat(string sDp, dyn_string &dsDpParameters, string dpe = ".ProcessInput.PosSt") {
	string format;
	format = unExportDevice_convertFormat(dpGetFormat(sDp + dpe), "#");
	dynAppend(dsDpParameters, format);
}

void cpcExportGenericFunctions_getRange(string sDp, dyn_string &dsDpParameters, dyn_string &exceptionInfo, string dpe = ".ProcessInput.PosSt") {
	bool bDoesExist, bNegateRange, bIgnoreOutside, bInclusiveMin, bInclusiveMax;
	float fMinValue, fMaxValue;

	fwPvRange_get(sDp + dpe, bDoesExist, fMinValue, fMaxValue, bNegateRange, bIgnoreOutside, bInclusiveMin, bInclusiveMax, exceptionInfo);
	dynAppend(dsDpParameters, unExportDevice_floatFormat(fMaxValue));
	dynAppend(dsDpParameters, unExportDevice_floatFormat(fMinValue));
}

void cpcExportGenericFunctions_getOutRange(string sDp, dyn_string &dsDpParameters, dyn_string &exceptionInfo, string dpe = ".ProcessInput.PosSt") {
	float rangeMin, rangeMax, outRangeMax, outRangeMin;
	bool bDoesExist, bNegateRange, bIgnoreOutside, bInclusiveMin, bInclusiveMax, configExists;
	int conversionType, order;
	dyn_float arguments;

	fwPvRange_get(sDp + dpe, bDoesExist, rangeMin, rangeMax, bNegateRange, bIgnoreOutside, bInclusiveMin, bInclusiveMax, exceptionInfo);
	fwConfigConversion_get(sDp + dpe, configExists, DPCONFIG_CONVERSION_RAW_TO_ENG_MAIN, conversionType, order, arguments, exceptionInfo);
	outRangeMax = arguments[2]*rangeMax + arguments[1];
	outRangeMin = arguments[2]*rangeMin + arguments[1];

	dynAppend(dsDpParameters, unExportDevice_floatFormat(outRangeMax));
	dynAppend(dsDpParameters, unExportDevice_floatFormat(outRangeMin));
}

void cpcExportGenericFunctions_getDeadband(string sDp, dyn_string &dsDpParameters, dyn_string &exceptionInfo, string dpe = ".ProcessInput.PosSt") {
	float deadband, timeInterval;
	int smoothProcedure, deadbandType;
	bool isSmoothDefined;

	unExportDevice_fwSmoothing_getTemp(sDp + dpe, isSmoothDefined, smoothProcedure, deadband, timeInterval);
	if (isSmoothDefined) {
		switch (smoothProcedure) {
			case DPATTR_VALUE_REL_SMOOTH:
				deadbandType = UN_CONFIG_DEADBAND_RELATIF;
				break;
			case DPATTR_VALUE_SMOOTH:
				deadbandType = UN_CONFIG_DEADBAND_VALUE;
				break;
			case DPATTR_COMPARE_OLD_NEW:
				deadbandType = UN_CONFIG_DEADBAND_OLD_NEW;
				deadband = 0;
				break;
			default:
				deadbandType = DPCONFIG_NONE;
				deadband = 0;
				break;
		}
	} else {
		deadbandType = DPCONFIG_NONE;
		deadband = 0;
	}

    dynAppend(dsDpParameters, deadband);
    dynAppend(dsDpParameters, deadbandType);
}

void cpcExportGenericFunctions_getArchive(string sDp, dyn_string &dsDpParameters, string dpe = ".ProcessInput.PosSt") {
	string sArchiveActive;
	float fArchiveTimeFilter;
	unExportDevice_getArchiveParametersDpe(sDp + dpe, sArchiveActive, fArchiveTimeFilter);

	dynAppend(dsDpParameters, sArchiveActive);
	if (fArchiveTimeFilter < 0) {
		dynAppend(dsDpParameters, "");
	} else {
		dynAppend(dsDpParameters, fArchiveTimeFilter);
	}
}

void cpcExportGenericFunctions_getMaskEvent(string sDp, dyn_string &dsDpParameters) {
	int iMask;
	string sMask;
	unGenericDpFunctions_getMaskEvent(sDp, iMask);
	sprintf(sMask, "%d", iMask);
	dynAppend(dsDpParameters, sMask);
}

void cpcExportGenericFunctions_getDefaultValue(string sDp, dyn_string &dsDpParameters, string dfltElementName = "DfltVal") {
	float defaultValue;
	dpGet(sDp + ".ProcessOutput." + dfltElementName + ":_online.._value", defaultValue);
	dynAppend(dsDpParameters, defaultValue);
}

void cpcExportGenericFunctions_getBoolDefaultValue(string sDp, dyn_string &dsDpParameters, string dfltElementName = "DfltVal") {
	bool defaultValue;
	dpGet(sDp + ".ProcessOutput." + dfltElementName + ":_online.._value", defaultValue);
	dynAppend(dsDpParameters, defaultValue);
}

void cpcExportGenericFunctions_getPIDName(string sDp, dyn_string &dsDpParameters) {
	string sPIDName;
	unExportDevice_getPIDName(sDp, sPIDName);
	dynAppend(dsDpParameters, sPIDName);
}

void cpcExportGenericFunctions_getPCOName(string sDp, dyn_string &dsDpParameters) {
	string sPCOName;
	unExportDevice_getPCOName(sDp, sPCOName);
	dynAppend(dsDpParameters, sPCOName);
}

void cpcExportGenericFunctions_getKeyDeviceConfiguration(string sDp, dyn_string &dsDpParameters, string key, dyn_string &exceptionInfo) {
	string value;
	unGenericDpFunctions_getKeyDeviceConfiguration(sDp, key, value, exceptionInfo);
	dynAppend(dsDpParameters, value);
}
//------------------------------------------------------------------------------------------------------------------------
// cpcExportGenericFunctions_getDigitalNormalPosition
void cpcExportGenericFunctions_getDigitalNormalPosition(string sDP, string sObject, dyn_string dsDpElements, dyn_string &dsDpParameters) {
	//	we don't need sObject at the moment, as unicos functions need it, but not used. we can pass "".
	bool bAlarmMask, bOkRange, bBoolAlert, bSMS = false;
	dyn_int diAlertClass;
	int iAlertType, iRes;
	string sDataPoint, sAlertClass1, sAlertClass2;
	int iNormalPosition = -1;

	for(int i = 1; i <= dynlen(dsDpElements); i++) {
		iRes = dpGet(sDP + dsDpElements[i] + ":_alert_hdl.._type", iAlertType);
		dynAppend(diAlertClass, iAlertType);
	}

	if (dynUnique(diAlertClass) != 1) {
		iAlertType = DPCONFIG_NONE;
	} else {
		iAlertType = diAlertClass[1];
	}

	if(iAlertType == DPCONFIG_NONE)	{
		iNormalPosition = 2;
	} else {
		sDataPoint = sDP + dsDpElements[1];
		bBoolAlert = ((iRes >= 0) && (iAlertType == DPCONFIG_ALERT_BINARYSIGNAL)); 
		if (bBoolAlert) { 
			dpGet(sDataPoint + ":_alert_hdl.._ok_range", bOkRange);
		}

		dpGet(sDataPoint + ":_alert_hdl.._active",  bAlarmMask);
		dpGet(sDataPoint + ":_alert_hdl.1._class",  sAlertClass1);
		dpGet(sDataPoint + ":_alert_hdl.2._class",  sAlertClass2);

		bSMS = (patternMatch(UN_PROCESSALARM_PATTERN,sAlertClass1)) || (patternMatch(UN_PROCESSALARM_PATTERN,sAlertClass2));

		if(bAlarmMask) { //	Alarm unmasked
			iNormalPosition = bOkRange ? 1 : 0;
		} else { //	Alarm masked
			iNormalPosition = bOkRange ? 4 : 3;
		}
	}

	if (bSMS) {
		iNormalPosition += 100;
	}
	dynAppend(dsDpParameters, iNormalPosition);
}

void cpcExportGenericFunctions_getPCOParameters(string sDP, dyn_string &dsDpParameters) {
	dyn_string dsPCOParameters;
	
	unExportDevice_getPCOParameters(sDP, dsPCOParameters);
	for(int i = 1; i <= dynlen(dsPCOParameters); i++) {
		dynAppend(dsDpParameters, dsPCOParameters[i]);
	}
}

void cpcExportGenericFunctions_getAutonomous(string sDP, dyn_string &dsDpParameters, string dpe = ".ProcessInput.PosSt") {
    int iType, iResult;
    iResult=dpGet(sDP + dpe + ":_dp_fct.._type", iType);
    dynAppend(dsDpParameters, iType == DPCONFIG_DP_FUNCTION ? "TRUE" : "FALSE");
}

//------------------------------------------------------------------------------------------------------------------------
// unExportDevice_getArchiveNamesDpe
/**
Purpose: Get Archive name (Bool, Analog or Event values) for given dpes

Parameters:

Usage: Internal function

PVSS manager usage: NG, NV
*/
void cpcExportGenericFunctions_getArchiveNameForDpes(string sDp, string defaultValue, dyn_string dsDpes, dyn_string &dsDpParameters) {
    dyn_string dsCheck;
    string archiveName;

    if (dynlen(dsDpes) == 0) {
        archiveName = defaultValue;
    } else {
        for (int i = 1; i <= dynlen(dsDpes); i++) {
            string sArchiveClass, sName;
            
            dpGet(sDp + "." + dsDpes[i] + ":_archive.1._class", sArchiveClass);

            if(dpExists(sArchiveClass + ".general.arName")) {
                dpGet(sArchiveClass + ".general.arName:_online.._value", sName);
            } else {
                sName = defaultValue;
            }
            dynAppend(dsCheck, sName);
        }
        dynUnique(dsCheck);
        if (dynContains(dsCheck, "") > 0) {
            dynRemove(dsCheck, dynContains(dsCheck, ""));
        }
        if (dynlen(dsCheck)==1) {       //check if same archive for same type
            archiveName = dsCheck[1];
        } else {
            archiveName = "";
        }
    }

    dynAppend(dsDpParameters, archiveName);
}

void cpcExportGenericFunctions_getCategories(string sDP, dyn_string &dsDpParameters, dyn_string &exceptionInfo) {
	dyn_dyn_string ddsCategory;
	unProcessAlarm_getCategory(makeDynString(sDP), ddsCategory, exceptionInfo);

	if (dynlen(ddsCategory) > 0) {
		string categories = "";
		dyn_string dpCategories = ddsCategory[1];
		for (int i = 1; i <= dynlen(dpCategories); i++) {
			if (categories != "") {
				categories += ",";
			}
			categories += dpCategories[i];
		}
		dynAppend(dsDpParameters, categories);
	} else {
		dynAppend(dsDpParameters, "");
	}
}

string cpcExportGenericFunctions_getAcknowledgeAlarmValue(string sDP, string dpe = ".ProcessInput.PosSt") {
	int iRes, iAlertType;
	string class;

	iRes = dpGet(sDP + dpe + ":_alert_hdl.._type", iAlertType);
	if((iRes < 0) || (iAlertType == DPCONFIG_NONE)) {
		return "N";
	} else {
		dpGet(sDP + dpe + ":_alert_hdl.._class", class);
		if(strpos(class, CPC_CONFIG_ALARM_ACK_POSTFIX) >= 0) {
			return "Y";
		} else {
			return "N";
		}
	}
}

void cpcExportGenericFunctions_getAcknowledgeAlarm(string sDP, string alarmDP, dyn_string &dsDpParameters, dyn_string &exceptionInfo) {
	string isAck = cpcExportGenericFunctions_getAcknowledgeAlarmValue(sDP, alarmDP);
	dynAppend(dsDpParameters, isAck);
}

void cpcExportGenericFunctions_get5AlertLimits(string sDP, dyn_string &dsDpParameters) {
    int rangesNumber;
    dyn_string dsLimits = makeDynString("", "", "", "");

    dpGet(sDP + ".ProcessInput.PosSt:_alert_hdl.._num_ranges", rangesNumber);
    for(int i = 1; i <= rangesNumber; i++) {
        float value;
        string rangeClass = "#";
        dpGet(sDP + ".ProcessInput.PosSt:_alert_hdl." + i + "._class", rangeClass);
        if (strpos(rangeClass, "_cpcAnalogHH") > 0) {
            dpGet(sDP + ".ProcessInput.PosSt:_alert_hdl."+i+"._l_limit", value);
            dsLimits[1] = value;
        } else if (strpos(rangeClass, "_cpcAnalogH") > 0) {
            dpGet(sDP + ".ProcessInput.PosSt:_alert_hdl."+i+"._l_limit", value);
            dsLimits[2] = value;
        } else if (strpos(rangeClass, "_cpcAnalogLL") > 0) {
            dpGet(sDP + ".ProcessInput.PosSt:_alert_hdl."+i+"._u_limit", value);
            dsLimits[4] = value;
        } else if (strpos(rangeClass, "_cpcAnalogL") > 0) {
            dpGet(sDP + ".ProcessInput.PosSt:_alert_hdl."+i+"._u_limit", value);
            dsLimits[3] = value;
        }
    }

    dynAppend(dsDpParameters, dsLimits[1]);
    dynAppend(dsDpParameters, dsLimits[2]);
    dynAppend(dsDpParameters, dsLimits[3]);
    dynAppend(dsDpParameters, dsLimits[4]);
}

void cpcExportGenericFunctions_get5AlertClasses(string sDP, dyn_string &dsClasses, dyn_string &exceptionInfo, string dpe = ".ProcessInput.PosSt") {
    int rangesNumber;

    dynClear(dsClasses);

    dpGet(sDP + dpe + ":_alert_hdl.._num_ranges", rangesNumber);
    for(int i = 1; i <= rangesNumber; i++) {
        string rangeClass;
        dpGet(sDP + dpe + ":_alert_hdl." + i + "._class", rangeClass);
        if (rangeClass != "") {
            dynAppend(dsClasses, rangeClass);
        }
    }
}

string cpcExportGenericFunctions_get5AlertAcknowledgeAlarmValue(string sDP, dyn_string &exceptionInfo) {
	dyn_string dsLimits;
	dyn_string dsClasses;

	cpcExportGenericFunctions_get5AlertClasses(sDP, dsClasses, exceptionInfo);
	if (dynlen(dsClasses) == 0) {
		return "FALSE";
	} else {
		int ackAmount = 0;
		for (int i = 1; i <= dynlen(dsClasses); i++) {
			if (strpos(dsClasses[i], CPC_CONFIG_ALARM_ACK_POSTFIX) > 0) {
				ackAmount += 1;
			}
		}
		return ackAmount > 0;
	}
}

void cpcExportGenericFunctions_get5AlertAcknowledgeAlarm(string sDP, string alarmDP, dyn_string &dsDpParameters, dyn_string &exceptionInfo) {
	string isAck = cpcExportGenericFunctions_get5AlertAcknowledgeAlarmValue(sDP, alarmDP, exceptionInfo);
	dynAppend(dsDpParameters, isAck);
}

void cpcExportGenericFunctions_getAlarmActive(string sDP, dyn_string &dsDpParameters, string dpe = ".ProcessInput.PosSt") {
    bool isAlarmActive;
    dpGet(sDP + dpe + ":_alert_hdl.._active", isAlarmActive);
    dynAppend(dsDpParameters, isAlarmActive);
}

/**
Purpose: Convertion dyn_string into string; elements is glued via separator

Parameters:
	array: dyn_string, input, data to process
	separator: string, input

	return:	empty string if input array is empty
			string with all elements of array merged with separator
*/
string dynJoin(dyn_string array, string separator) {
	if (dynlen(array) == 0) {
		return "";
	} else {
		string result = array[1];
		for (int i = 2; i <= dynlen(array); i++) {
			result = result + "," + array[i];
		}
		return result;
	}
}
