/** @name LIBRARY: unCPCFaceplate.ctl

@author: Alexey Merezhin (EN-ICE-SIC)
@date 03.10.2011
@brief This library contains generic functions for faceplate animation.

Modifications History:
    08/02/2012: Alexey
        [UCPC-656] fixed collision in cpcFaceplate_fetchAnimationCBOnlineValue (use dot in the beginning of dpe pattern)

    20/01/2012: Alexey
        cpcFaceplate_animateTextParameter, cpcFaceplate_animateTranslatedValue and cpcFaceplate_animateInterlock
        extracted cpcFaceplate_fetchStsRegBitAndValidness from cpcFaceplate_animateStsRegBit

*/

cpcFaceplate_animateTextParameter(dyn_string dpes, dyn_string values, string paramName, string value)
{
    string color = (!values[2]) || (values[1] > c_unSystemIntegrity_no_alarm_value) ? "unDataNotValid" : "unDisplayValue_Parameter";

    if(g_bSystemConnected) {
        setMultiValue(paramName + ".display", "foreCol", color, paramName + ".display", "text", value);
    }
}

cpcFaceplate_animateTranslatedValue(dyn_string dpes, dyn_string values, string dpeName, mapping dict, string color) {
    int value;
    bool invalid;

    cpcFaceplate_fetchAnimationCBOnlineValue(dpes, values, dpeName, value, invalid);
    invalid = invalid || (!values[2]) || (values[1] > c_unSystemIntegrity_no_alarm_value);
    
    cpcGenericObject_animateTranslatedValue(dpeName, dict, value, color, invalid);
}

cpcFaceplate_animateInterlock(dyn_string dpes, dyn_string values, string bitName, string interlockName, string unAckBitName = "UN_STSREG01_ALUNACK") {
    bool value, invalid, unAck, unAckInvalid;

    int interlockState = cpcFaceplate_fetchAnimationCBValue(dpes, values, interlockName);
    cpcFaceplate_fetchStsRegBitAndValidness(dpes, values, bitName, value, invalid);
    cpcFaceplate_fetchStsRegBitAndValidness(dpes, values, unAckBitName, unAck, unAckInvalid);

    if(g_bSystemConnected)
        cpcGenericObject_ColorBoxAnimateInterlock(interlockName, interlockState, value, unAck,
                    invalid || unAckInvalid || (!values[2]) || (values[1] > c_unSystemIntegrity_no_alarm_value));

}

cpcFaceplate_animateOnlineValue(dyn_string dpes, dyn_string values, string dpeName, string dpeUnit, string dpeFormat, string color) {
    anytype value;
    bool invalid;
    cpcFaceplate_fetchAnimationCBOnlineValue(dpes, values, dpeName, value, invalid);
    if(g_bSystemConnected) {
        unGenericObject_DisplayValue(dpeFormat, dpeUnit, value, dpeName, color, 
            invalid || (!values[2]) || (values[1] > c_unSystemIntegrity_no_alarm_value));
    }
}

cpcFaceplate_animateStsRegBit(dyn_string dpes, dyn_string values, string bitName, bool enabled, string color) {
    bool invalid, value;
    dyn_string parts = strsplit(bitName, "_");
    cpcFaceplate_fetchStsRegBitAndValidness(dpes, values, bitName, value, invalid);

    if(g_bSystemConnected)
        unGenericObject_ColorBoxAnimate(parts[3], value, enabled, color, 
                    invalid || (!values[2]) || (values[1] > c_unSystemIntegrity_no_alarm_value));
}

cpcFaceplate_animateRange(dyn_string dpes, dyn_string values, string dpeName, int rangeType, string dpeUnit, string dpeFormat) {
    float rangeMin, rangeMax;
    if (rangeType == DPCONFIG_MINMAX_PVSS_RANGECHECK) {
        rangeMin = cpcFaceplate_fetchAnimationCBValue(dpes, values, dpeName + ":_pv_range.._min");
        rangeMax = cpcFaceplate_fetchAnimationCBValue(dpes, values, dpeName + ":_pv_range.._max");
    } else {
        rangeMin = 0;
        rangeMax = 0;
    }
    if(g_bSystemConnected)
        unGenericObject_DisplayRange(rangeType, dpeFormat, dpeUnit, rangeMin, rangeMax, dpeName + "RangeMin", dpeName + "RangeMax", "unDisplayValue_Parameter");
}

cpcFaceplate_animateMaskEvent(dyn_string dpes, dyn_string values) {
    bool maskEvent, invalid;
    cpcFaceplate_fetchAnimationCBOnlineValue(dpes, values, "eventMask", maskEvent, invalid);
    if(g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("eventState", !maskEvent, 1, "unFaceplate_WarningActive", 
                    invalid || (!values[2]) || (values[1] > c_unSystemIntegrity_no_alarm_value));

}


cpcFaceplate_fetchStsRegBitAndValidness(dyn_string dpes, dyn_string values, string bitName, bool &value, bool &invalid) {
    int stsReg;
    dyn_string exceptionInfo;
    int bitPosition = cpcConfigGenericFunctions_getConstantValueByName(bitName);
    if (bitPosition < 0) {
        fwException_raise(exceptionInfo,"ERROR", "cpcFaceplate_fetchStsRegBitAndValidness: no position for " + bitName, "EMPTY");
        return;
    }
    dyn_string parts = strsplit(bitName, "_");
    if (dynlen(parts) != 3) {
        fwException_raise(exceptionInfo,"ERROR", "cpcFaceplate_fetchStsRegBitAndValidness: wrong bit name " + bitName, "EMPTY");
        return;
    }
    if (parts[2] == "STSREG01") {
        cpcFaceplate_fetchAnimationCBOnlineValue(dpes, values, "StsReg01", stsReg, invalid);
    } else if (parts[2] == "STSREG02") {
        cpcFaceplate_fetchAnimationCBOnlineValue(dpes, values, "StsReg02", stsReg, invalid);
    } else {
        fwException_raise(exceptionInfo,"ERROR", "cpcFaceplate_fetchStsRegBitAndValidness: wrong stsreg " + bitName, "EMPTY");
        return;
    }
    value = cpcGenericObject_getBit(stsReg, bitPosition);
}

cpcFaceplate_fetchAnimationCBOnlineValue(dyn_string dpes, dyn_string values, string dpeName, anytype &value, bool &invalid) {
    string dpeValue = "." + dpeName + ":_online.._value";
    string dpeValid = "." + dpeName + ":_online.._invalid";
    for (int i = 1; i <= dynlen(dpes); i++) {
        if (strpos(dpes[i], dpeValue) >= 0) {
            value = values[i];
        }
        if (strpos(dpes[i], dpeValid) >= 0) {
            invalid = values[i];
        }
    }
}

anytype cpcFaceplate_fetchAnimationCBValue(dyn_string dpes, dyn_string values, string dpe) {
    for (int i = 1; i <= dynlen(dpes); i++) {
        if (strpos(dpes[i], dpe) >= 0) {
            return values[i];
        }
    }
    return "missed";
}
