/**@name LIBRARY: cpcGenericDpFunctions.ctl

@author: Alexey Merezhin

Creation Date: 20/06/2011

Modification History:

External Functions: 
	cpcGenericDpFunctions_setDeviceProperty
	cpcGenericDpFunctions_getDeviceProperty

Purpose:
This library contains CPC generic functions.

*/

const string UN_CPC_DEVICE_PROPERTY_PREFIX = "PROP_";

//------------------------------------------------------------------------------------------------------------------------
// cpcGenericDpFunctions_setDeviceProperty
/** Save the device property for a given property name
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDeviceDpName  input, device name DP name
@param propertyName input, property name
@param propertyValue output, property value
@param exceptionInfo output, error returned here
*/
cpcGenericDpFunctions_setDeviceProperty(string sDeviceDpName, string propertyName, string propertyValue, dyn_string &exceptionInfo)
{
	unGenericDpFunctions_setKeyDeviceConfiguration(sDeviceDpName, UN_CPC_DEVICE_PROPERTY_PREFIX + propertyName, makeDynString(propertyValue), exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// cpcGenericDpFunctions_getDeviceProperty
/** Get the device property for a given key
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDeviceDpName  input, device name DP name
@param propertyName input, property name
@param exceptionInfo output, error returned here
@return propertyValue output, property value
*/
string cpcGenericDpFunctions_getDeviceProperty(string sDeviceDpName, string propertyName, dyn_string &exceptionInfo) {
	dyn_string dsConfig;
	unGenericDpFunctions_getKeyDeviceConfiguration(sDeviceDpName, UN_CPC_DEVICE_PROPERTY_PREFIX + propertyName, dsConfig, exceptionInfo);
	if (dynlen(dsConfig) == 0) {
		return "";
	} else {
		return dsConfig[1];
	}
}
