/** @name LIBRARY: cpcGenericObject.ctl

@author: Nikolay Kulman (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 08.07.2003
@brief This library contains generic functions for Unicos objects.
Modifications History:
    13/09/2012: Alexey
        [UCPC-931] allows to open alarm list from the right click menu

    16/05/2012: Alexey
        [UCPC-787] cpcGenericObject_WidgetAnimationDoubleStsReg: widget doesn't blink on AlSt

    29/02/2012: Alexey
        ported unGenericObject_WidgetDisplayValueAnimation

    14/02/2012: Alexey
        [UCPC-665] don't animate letters in widget when stsreg is invalid

    09/02/2012: Alexey
        [UCPC-646] disable diagnostic popup item when device has it empty

    20/01/2012: Alexey
        removed cpcGenericObject_DisplayValueOrLabel as unused
        added cpcGenericObject_animateTranslatedValue

    13/12/2011: Alexey
        [UCPC-630] grayed info popup item when needed

    12/12/2011: Alexey
        added AA 'mask PVSS alarm' handler

    24/11/2011: Alexey
        fixed 'Manual IO Error Block' menu item text

    23/11/2011: Alexey
        removed cpcGenericObject_HandleAlarmMaskMenu, cpcGenericObject_addMaskToMenu
        moved cpcGenericObject_HandleEventMaskMenu to the cpcGenericObject_HandleUnicosMenu
        [UCPC-603] cloned cpcGenericObject_addDefaultUnicosActionToMenu and cpcGenericObject_addTrendActionToMenu; clean up menu builders and handlers

    31/10/2011: Alexey
        [UCPC-593] used cpcGenericObject_GetAnimationFunction and cpcGenericObject_WidgetDisconnection instead of core

    28/10/2011: Alexey
        [UCPC-434] added damper animation method

    20/10/2011: Alexey
        [UCPC-572] Analog/OnOff/Anadig/AnaDO: blink when unack without specific alert
        cpcGenericObject_translateAlarmAcronym works with types in a format "type:alias"

    04/10/2011: Alexey
        added isAlertExists to cpcGenericObject_ColorBoxAnimateWarning and cpcGenericObject_ColorBoxAnimateAlarm

    03/10/2011: Alexey
        added Mail/SMS handle in the cpcGenericObject_HandleUnicosMenu
        removed cpcGenericObject_WidgetAIAOAnimation
        extended cpcGenericObject_CheckboxAnimate with enabled option
        added cpcGenericObject_ColorBoxAnimate
        [UCPC-562] added cpcGenericObject_GetAnalogAlarmType, cpcGenericObject_isAlarmWithMail (instead of unProcessAlarm_isDpConfigured) and cpcGenericObject_getBit to process StsReg as float not bit32

    30/09/2011: Alexey
        [UCPC-567] added interlock status to the cpcGenericObject_WidgetAnimationDoubleStsReg; updated cpcGenericObject_WidgetAnimation

    21/09/2011: Alexey
        [UCPC-546] added cpcGenericObject_ColorBoxAnimateAlarm and refactored cpcGenericObject_ColorBoxAnimateWarning

    20/09/2011: Alexey
        [UCPC-548] added cpcGenericObject_CheckboxAnimate

    16/09/2011: Alexey
        [UCPC-525] use cpcConfigGenericFunctions_setMaskEvent in cpcGenericObject_HandleEventMaskMenu

    14/09/2011: Alexey
        [UCPC-534] cpcGenericObject_translateAlarmAcronym process types as a dyn_string

    13/09/2011: Alexey
        [UCPC-531] added cpcGenericObject_StsRegConstantNameToValue
        moved cpcGenericObject_WidgetAnimateManualRestart into cpcGenericObject_WidgetAnimationDoubleStsReg
        changed on/off constants to string in the cpcGenericObject_WidgetAnimationDoubleStsReg

    25/08/2011: Alexey
        [UCPC-387] added multiple type to the cpcGenericObject_translateAlarmAcronym

	10/08/2011: Alexey
		[UCPC-460] added MaskISt popup item
        removed Mail-SMS from generic as related only to the DA
        fixed cpcGenericObject_addUnicosActionToMenu rights granting

	04/08/2011: Alexey
		[UCPC-454] fixed Deblock PLC Alarm popup's condition

	02/08/2011: Alexey
		[UCPC-437] display H/L thresholds as warnings

	27/07/2011: Alexey
		[UCPC-387] added cpcGenericObject_translateAlarmAcronym

	13/07/2011: Alexey
		use widget-specific animation function in cpcGenericObject_WidgetAnimateManualRestart 

	05/07/2011: Alexey
		[UCPC-323] moved ManRestart animation from cpcGenericObject_WidgetAnimationDoubleStsReg to cpcGenericObject_WidgetAnimateManualRestart

	28/06/2011: Alexey
		added cpcGenericObject_WidgetControlStateAnimation; use it in cpcGenericObject_WidgetAnimation

	27/06/2011: Alexey
		[UCPC-323] color Field objects widget with orange when NeedRestart bit is on

	23/06/2011: Alexey
		[UCPC-206] moved cpcGenericObject_getCtrlLink from unAnaDig; now it returns links to stsreg01 dpes
		added cpcGenericObject_WidgetCtrlAnimationCB which animated control object letters (used in Analog/AnaDig)
		[UCPC-328] added cpcGenericObject_ColorBoxAnimateInterlock

	2011/05/25 Alexey
		display 'Manual IO Error Block' in widget in Input/Output objects

	2011/05/20 Alexey
		made cpcGenericObject_notifyUsers to simplify use notification (refactor cpcGenericObject_SendBinaryOrder to use it)
		notify about mask/unmask events and ioErrorBlocking

	2011/05/19 Alexey
		made getIOErrorBlocking function for getting number of IOErrorBlocking menu item

	2011/05/17 amerezhi
		cloned unGenericObject_WidgetDIDOAnimation and unGenericObject_WidgetAIAOAnimation functions
		cloned cpcGenericObject_WidgetAnimation, cpcGenericObject_WidgetAnimationDoubleStsReg and cpcGenericObject_WidgetAlarmTextAnimationDoubleStsReg functions
		created wrapping cpcGenericObject_HandleUnicosMenu function
		ported cpcGenericObject_addUnicosActionToMenu

  - 23/11/10 quique
      [UCPC-33] WordStatus bit description not correctly shown; Function: cpcGenericObject_GetLabelsFromString() 
      
	- 22/10/10 amerezhi: merged with ABCO version:
		28/01/2010: Herve
		- modify the function: cpcGenericObject_GetLabelsFromString
		add the missing bits in the description if all the bits are not defined.

	Revision 1.18  2008/11/12 09:17:39  gthomas
	-removed _isRoot function
	
	Revision 1.17  2008/09/12 13:45:08  gthomas
	Fixed variable typo in <cpcGenericObject_DisplayValueOrLabel> fct.

	Revision 1.16  2008/09/04 14:48:11  gthomas
	-moved back to version 1.12

	Revision 1.15  2008/06/13 12:29:47  aabadz
	Modifications asked by UNICOS for their version 4.X

	Revision 1.14  2008/06/09 14:54:26  aabadz
	Result after move some constants, only use use inside this file, from gcsObjects_declarations.

	Revision 1.13  2008/05/26 12:18:19  gthomas
	Merged files with modifications made by ABCO on xS and xP (GCS-ABCO-v4-1)
	to be used with PVSS 3.6 QT

	Revision 1.12  2008/01/22 14:46:36  aabadz
	Fixed:
	- Naming
	- Call to other functions to reuse code
	- Some error check

	Revision 1.11  2008/01/15 09:50:13  aabadz
	Changes after code review and before tests

	Revision 1.10  2007/10/30 16:21:58  gthomas
	Introduced the CVS keyword $Log to track changes made in the library

*/


/*
Internal Functions :
*/

//---------------------------------------------------------------------------------------------------------------------------------------
// cpcGenericObject_WidgetXC_Animation
/**
Animate AC / WC  widget
*/
cpcGenericObject_WidgetXC_Animation(bool bLocked, string sSelectedManager, bit32 bit32StsReg01, bool bStsReg01Invalid,
		string sDpPosOV, int iPosOV, bool bInvalidBody, bool bInvalidLetter,
		int iSystemIntegrityAlarmValue, bool bSystemIntegrityAlarmEnabled, string sEvtMask, int iMask, bool lookupForLabel)
{
	string sFormattedValue;
	string sWarningLetter, sWarningColor, sControlLetter, sControlColor, sBody1Text, sBody1Color, sSelectColor;
	bool bLockVisible;

	// Control animation
	sBody1Color = "unWidget_ControlStateAuto";					// Auto mode (default)
	sControlLetter = "";
	if ((!bStsReg01Invalid) && (getBit(bit32StsReg01, UN_STSREG01_FOMOST) == 1))
	{
		sBody1Color = "unWidget_ControlStateForced";				// Forced mode
		sControlLetter = CPC_WIDGET_TEXT_CONTROL_FORCED;
		sControlColor = "unWidget_ControlStateForced";
	}
	if ((!bStsReg01Invalid) && (getBit(bit32StsReg01, UN_STSREG01_MMOST) == 1))
	{
		sBody1Color = "unWidget_ControlStateManualLocal";			// Manual mode
		sControlLetter = CPC_WIDGET_TEXT_CONTROL_MANUAL;
		sControlColor = "unWidget_ControlStateManualLocal";
	}
	if (!bStsReg01Invalid)
	{
		unGenericObject_WidgetWarningAnimation(bit32StsReg01,
		   makeDynInt(UN_STSREG01_AUMRW, UN_STSREG01_IOSIMUW, UN_STSREG01_IOERRORW),
		   makeDynString(CPC_WIDGET_TEXT_WARNING_DEFAULT, CPC_WIDGET_TEXT_WARNING_SIMU, CPC_WIDGET_TEXT_WARNING_ERROR),
		   sWarningLetter, sWarningColor);
		if (bInvalidLetter)
		{
			sWarningLetter = CPC_WIDGET_TEXT_OLD_DATA;
			sWarningColor = "unDataNotValid";
		}
	}// if

	if((!bSystemIntegrityAlarmEnabled)||(iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value))
	{
		sWarningLetter = CPC_WIDGET_TEXT_OLD_DATA;
		sWarningColor = "unDataNotValid";
	}

	if (lookupForLabel && mappingHasKey(g_mLabels, (string)iPosOV)) {
		sBody1Text = g_mLabels[(string) iPosOV];
	} else {
		sFormattedValue = unGenericObject_FormatValue(g_sPosOvFormat, iPosOV);
		sBody1Text = sFormattedValue + " " + g_sPosOvUnit;
	}

	if (bInvalidBody)
		sBody1Color = "unDataNotValid";

	if (bInvalidLetter)
	{
		sWarningLetter = CPC_WIDGET_TEXT_INVALID;
		sWarningColor = "unDataNotValid";
	}

	unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, sSelectColor, bLockVisible);
	if(g_bSystemConnected)
	{
		setMultiValue("Body1", "text", sBody1Text, /*"Body1", "fill", "[solid]", */
			"Body1", "foreCol", sBody1Color, /*"Body1", "backCol", "unWidget_Background",*/
			"WarningText", "text", sWarningLetter, "WarningText", "foreCol", sWarningColor,
			"ControlStateText", "text", sControlLetter, "ControlStateText", "foreCol", sControlColor,
			"SelectArea", "foreCol", sSelectColor, "LockBmp", "visible", bLockVisible,
			"WidgetArea", "visible", true);
	}

// display event state
	if(g_bSystemConnected)
	{
		if(iMask == 0)
			eventState.visible	= true;
		else
			eventState.visible	= false;
	}
}

// cpcGenericObject_StringToMapping
/**
Split a string like "key1=value1,key2=value2" into the mapping

	@return 	If function finishedf successfully
	@param[in]  data     		Input string with labels information
	@param[out] pairs			Result mappint
*/
cpcGenericObject_StringToMapping(string data, mapping& pairs) {
	dyn_string pairList, pair;

	mappingClear(pairs);

	pairList = strsplit(data, ","); // list of "key=value" strings
	for (int i = 1; i <= dynlen(pairList); i++) {
		if(strpos(pairList[i], "=") < 0) continue; //	we have incorrect element
		pair = strsplit(pairList[i], "=");
		if(dynlen(pair) != 2) continue; // wrong number of elements - not a "key=value"
		pairs[pair[1]] = pair[2];
	}
}

//------------------------------------------------------------------------------------------------------------------------
// cpcConfigGenericFunctions_getPlcDpName
/**
Returns the full name of the Plc data point. Checks that Plc name is not empty and that Plc dp exists inside DB.

  @param[in]  sSystemName The name of the distributed system (can be empty if not set).
  @param[in]  sPlcName    The name of the plc.
  @param[out] sPlcDpName  The full name of the plc dp.
  @param[out] exceptionInfo The list of exceptions.
*/
cpcConfigGenericFunctions_getPlcDpName(string sSystemName, string sPlcName, string &sPlcDpName, dyn_string &exceptionInfo)
{
  sPlcDpName = "";

  if ("" == sPlcName){
    dynAppend(exceptionInfo, "Plc name is not specified");
    return;
  }

  string sTemp = sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName;
  if (!dpExists(sTemp)) {
    dynAppend(exceptionInfo, "Dp for specified Plc does not exist. Plc name: " + sPlcName + ", system name: " + sSystemName);
    return;
  }
  sPlcDpName = sTemp;
}

// cpcGenericObject_SendBinaryOrder
/**
Purpose: send a binary order to the PLC. Example : auto mode request etc.

Parameters:	
	- sDpName, string, input, device name
	- iManRegBit, int, input, bit position in ManReg01
	- sKey, string, input, catalogue key
	- exceptionInfo, dyn_string, output, for errors

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. In catalogue "unOperator.cat", keys <sKey + "FAILED"> and <sKey + "OK"> must exist
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cpcGenericObject_SendBinaryOrder(string sDpName, string regName, int iManRegBit, string sKey, dyn_string &exceptionInfo)
{
	dyn_string exceptionInfoTemp;
	string alias, deviceName;
	bool alreadySent, bRes;
	
	deviceName = unGenericDpFunctions_getDpName(sDpName);
// 1. Select device again
	unGenericObject_NeedSelect(dpTypeName(deviceName), bRes, exceptionInfo);
	if (bRes)
		{
		unSelectDeselectHMI_select(deviceName, true, exceptionInfo);
		dynClear(exceptionInfo);
		}
// 2. Send pulse
	cpcGenericObject_SendManRegPulse(deviceName, regName, iManRegBit, exceptionInfo, alreadySent);
// 3. Display result
	if (!alreadySent) {
		cpcGenericObject_notifyUsers(deviceName, sKey, exceptionInfo);
	}
}

// cpcGenericObject_SendManRegPulse
/**
Purpose: send a pulse on a specified bit of ManReg01  

Parameters:
	sDpName: string, input, object name
	bitPosition: string, input, the pulse position in ManReg (see Constants)
	exceptionInfo: dyn_string, output, details of any exceptions are returned here
	alreadySent: bool, output, set to true if ManReg bit is already equal to 1 at the beginning of the function

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT, W2000 and Linux, but tested only under W2000.
	. distributed system: yes but not tested.
*/

cpcGenericObject_SendManRegPulse(string sDpName, string regName, unsigned bitPosition, dyn_string &exceptionInfo, bool &alreadySent)
{
	unGenericObject_SendPulse(sDpName, ".ProcessOutput." + regName,  bitPosition, exceptionInfo, alreadySent);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cpcGenericObject_WidgetDIDOAnimation
/**
Purpose: animate DI DO widget body

Parameters:
	- bLocked, bool, input, for select animation
	- sSelectedManager, string, input, for select animation
	- bit32StsReg01, bit32, input, status register value
	- bStsReg01Invalid, bool, input, data status
	- bBoolAlert, bool, input, alert hdl exists ?
	- bIsAlarmActive, bool, input, alert hdl active ?
	- iActState, int, input, alert hdl act state
	- bAckPossible, bool, input, has alert to be acknowleged ?
	- iSystemIntegrityAlarmValue, int, input, the value of the systemIntegrity alarm for the PLC: 
							c_unSystemIntegrity_no_alarm_value (0) no alarm PLC is running, 
							c_unSystemIntegrity_alarm_value_level1 (10): alarm PLC is not correctly running
	- bSystemIntegrityAlarmEnabled, bool, input, the systemIntegrity alarm is enabled
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cpcGenericObject_WidgetDIDOAnimation(bool bLocked, string sSelectedManager, bit32 bit32StsReg01, bool bStsReg01Invalid, 
									bool bBoolAlert, bool bIsAlarmActive, int iActState, bool bAckPossible, int iSystemIntegrityAlarmValue,
									bool bSystemIntegrityAlarmEnabled)
{
	string warningLetter, warningColor, alarmLetter, alarmColor, controlLetter, controlColor;
	string body1Color, body1BackColor, selectColor;
	bool lockVisible;
	
	if (bStsReg01Invalid)
		{
		warningLetter = CPC_WIDGET_TEXT_INVALID;
		warningColor = "unDataNotValid";
		body1Color = "unDataNotValid";
		body1BackColor = "unWidget_Background";
		}
	else
		{
		body1Color = "unWidget_ControlStateAuto";				// Auto mode
		if (getBit(bit32StsReg01, UN_STSREG01_FOMOST) == 1) {	// Forced mode
			body1Color = "unWidget_ControlStateForced";				
			controlLetter = CPC_WIDGET_TEXT_CONTROL_FORCED;
			controlColor = "unWidget_ControlStateForced";
		} else if (getBit(bit32StsReg01, CPC_STSREG01_AUIHFOMOST) == 1) {	// Forced mode inhibited
			controlLetter = CPC_WIDGET_TEXT_CONTROL_AUIHFOMOST;
		}
		if (bBoolAlert) 										// Alarm config exists
			{
			if (bIsAlarmActive)									// Alarm active
				{
				if ((iActState == DPATTR_ALERTSTATE_APP_NOT_ACK) || (iActState == DPATTR_ALERTSTATE_APP_ACK)) // Alarm ?
					{
					body1Color = "unFaceplate_AlarmActive";
					}
				if (bAckPossible)								// Alarm not ack ?
					{
					body1Color = "unWidget_AlarmNotAck";
					}
				}
			else												// Alarm masked
				{
				alarmLetter = CPC_WIDGET_TEXT_ALARM_MASKED;
				alarmColor = "unAlarmMasked";
				}
			}
		body1BackColor = "unWidget_Background";
		if (getBit(bit32StsReg01, UN_STSREG01_POSST) == 1)
			{
			body1BackColor = body1Color;
			}
		unGenericObject_WidgetWarningAnimation(bit32StsReg01, 
											   makeDynInt(UN_STSREG01_FODIPRO, UN_STSREG01_IOSIMUW, UN_STSREG01_IOERRORW, CPC_STSREG01_MIOERBRST),
											   makeDynString(CPC_WIDGET_TEXT_WARNING_DEFAULT, CPC_WIDGET_TEXT_WARNING_SIMU, CPC_WIDGET_TEXT_WARNING_ERROR, CPC_WIDGET_TEXT_WARNING_ALARM_BLOCKED),
											   warningLetter, warningColor);

// 6.1 if the device is not correclty running (the iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value) 
// or the checking is not enabled then set the letter to O and the color to not valid data

		if ((!bSystemIntegrityAlarmEnabled) || (iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value))
			{
			warningLetter = CPC_WIDGET_TEXT_OLD_DATA;
			warningColor = "unDataNotValid";
			}

		}
	unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, selectColor, lockVisible);
	if(g_bSystemConnected)
		setMultiValue("Body1", "foreCol", body1Color, "Body1", "fill", "[solid]", "Body1", "backCol", body1BackColor,
					  "WarningText", "text", warningLetter, "WarningText", "foreCol", warningColor,
					  "AlarmText", "text", alarmLetter, "AlarmText", "foreCol", alarmColor,
					  "ControlStateText", "text", controlLetter, "ControlStateText", "foreCol", controlColor,
					  "SelectArea", "foreCol", selectColor, "LockBmp", "visible", lockVisible,
					  "WidgetArea", "visible", true);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cpcGenericObject_WidgetAlarmTextAnimationDoubleStsReg
/**
Purpose: animate the alarm part of the widget

Parameters :
	- bit32StsReg01, bit32, input, StsReg01 value
	- bit32StsReg02, bit32, input, StsReg02 value
	- diBits, dyn_int, input, bit positions
	- dsLetters, dyn_string, input, associated letters to be displayed
diBits and dsLetters must have the same length. dyn arrays must be defined in priority order: lowest -> highest priority.
	- diBits2, dyn_int, input, bit positions
	- dsLetters2, dyn_string, input, associated letters to be displayed
diBits2 and dsLetters2 must have the same length. dyn arrays must be defined in priority order: lowest -> highest priority.
	- letter, string, output, the letter to be displayed
	- color, string, output, the color to be used

Usage: Internal function

PVSS manager usage:  NG, NV

Constraints:
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cpcGenericObject_WidgetAlarmTextAnimationDoubleStsReg(bit32 bit32StsReg01, bit32 bit32StsReg02, dyn_int diBits, dyn_string dsLetters, dyn_int diBits2, dyn_string dsLetters2, string &letter, string &color)
{
	letter = "";
	color = "unFaceplate_AlarmActive";
	if (dynlen(diBits) == dynlen(dsLetters)) {
		for(int i = 1; i <= dynlen(diBits); i++) {
			if (getBit(bit32StsReg01, diBits[i]) == 1) {
				letter = dsLetters[i];
			}
		}
	}
	if (dynlen(diBits2) == dynlen(dsLetters2)) {
		for(int i = 1; i <= dynlen(diBits2); i++) {
			if (getBit(bit32StsReg02, diBits2[i]) == 1) {
				letter = dsLetters2[i];
			}
		}
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cpcGenericObject_WidgetAnimationDoubleStsReg
/**
Purpose: animate widget

Parameters:
	- sWidgetType, string, input, widget type = <dpType> + <widget type>. ex: AnaDigHeater
	- sDpType, string, input, object
	- bLocked, bool, input, for select animation
	- sSelectedManager, string, input, for select animation
	- bit32StsReg01, bit32, input, status register
        - bit32StsReg01, bit32, input, status register (empty in case of not using the stsReg02)
	- diWarningBits, dyn_int, input, warning bit positions in status register.
        - diWarningBits2, dyn_int, input, warning bit positions in status register. (empty in case of not using the stsReg02)
	- dsWarningLetters, dyn_string, input, associated warning letters to be displayed.
        - dsWarningLetters2, dyn_string, input, associated warning letters to be displayed. (empty in case of not using the stsReg02)
	- diAlarmBits, dyn_int, input, alarm bit positions in status register.
	- dsAlarmLetters, dyn_string, input, associated alarm letters to be displayed.
	- diControlBits, dyn_int, input, control bit positions in status register.
	- dsControlLetters, dyn_string, input, associated control letters to be displayed.
	- iOnbitPosition, int, input, on status bit position in status register
	- iOffBitPosition, int, input, off status bit position in status register
	- iAlUnackBitPosition, int, input, alarm unack status bit position in status register
	- bInvalidLetter, bool, input, true if invalid letter as to be shown
	- bInvalidBody, bool, input, invalid animation on body ?
	- iSystemIntegrityAlarmValue, int, input, the value of the systemIntegrity alarm for the PLC: 
							c_unSystemIntegrity_no_alarm_value (0) no alarm PLC is running, 
							c_unSystemIntegrity_alarm_value_level1 (10): alarm PLC is not correctly running
	- bSystemIntegrityAlarmEnabled, bool, input, the systemIntegrity alarm is enabled

di<...>Bits and ds<...>Letters must have the same length. dyn arrays must be defined in priority order: lowest -> highest priority .

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cpcGenericObject_WidgetAnimationDoubleStsReg(string sWidgetType, string sDpType,
				bool bLocked, string sSelectedManager, bit32 bit32StsReg01, bit32 bit32StsReg02,
				dyn_int diWarningBits, dyn_int diWarningBits2,
				dyn_string dsWarningLetters, dyn_string dsWarningLetters2,
				dyn_int diAlarmBits, dyn_string dsAlarmLetters,
				dyn_int diAlarmBits2, dyn_string dsAlarmLetters2,
                int iStartIStState, int iTStopIStState, int iFuStopIStState, int iAlSt,
				dyn_string diControlBitNames,
				string onBitConstant, string offBitConstant, int iAlUnackBitPosition,
				bool bInvalidLetter, bool bInvalidBody,
				int iSystemIntegrityAlarmValue,
				bool bSystemIntegrityAlarmEnabled)
{
	string sColorSelect, sWarningLetter, sWarningColor, sAlarmLetter, sAlarmColor, sControlLetter, sControlColor, sBodyColor, sFunction;
	bool bLockVisible, isRestartNeeded = false;
	dyn_string dsFunctions, exceptionInfo;

  if (onBitConstant != "" && offBitConstant != "") {
    isRestartNeeded = g_bMRestart && getBit(bit32StsReg02, CPC_STSREG02_NEEDRESTART) == 0;
  }
    // 1. Select
	unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, sColorSelect, bLockVisible);
    // 2. Warning (if the stsreg02 exists then the treatment is different)
	unGenericObject_WidgetWarningAnimationDoubleStsReg(bit32StsReg01, bit32StsReg02, diWarningBits, diWarningBits2,dsWarningLetters,dsWarningLetters2, sWarningLetter, sWarningColor);

    // 2.1 if the device is not correclty running (the iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value) 
    // or the checking is not enabled then set the letter to O and the color to not valid data
	if ((!bSystemIntegrityAlarmEnabled) || (iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value)) {
		sWarningLetter = CPC_WIDGET_TEXT_OLD_DATA;
		sWarningColor = "unDataNotValid";
	}

	if (bInvalidLetter) {
		sWarningLetter = CPC_WIDGET_TEXT_INVALID;
		sWarningColor = "unDataNotValid";
	}
    // 3. Alarm
    if (!bInvalidBody) {
        cpcGenericObject_WidgetAlarmTextAnimationDoubleStsReg(bit32StsReg01, bit32StsReg02, diAlarmBits, dsAlarmLetters, diAlarmBits2, dsAlarmLetters2, sAlarmLetter, sAlarmColor);
        if (onBitConstant != "" && offBitConstant != "") {
            if (sAlarmLetter == "" && isRestartNeeded) {
                sAlarmLetter = CPC_WIDGET_TEXT_MANUAL_RESTART;
                sAlarmColor = "unFaceplate_WarningActive";
            }
        }
    }

    // 4. Control state
    if (!bInvalidBody) {
        cpcGenericObject_WidgetControlStateAnimation(bit32StsReg01, bit32StsReg02, diControlBitNames, sControlLetter, sControlColor);
    }
    // 5. Animate all except body
	if(g_bSystemConnected)
	setMultiValue("WarningText", "text", sWarningLetter, "WarningText", "foreCol", sWarningColor,
				  "AlarmText", "text", sAlarmLetter, "AlarmText", "foreCol", sAlarmColor,
				  "ControlStateText", "text", sControlLetter, "ControlStateText", "foreCol", sControlColor,
				  "SelectArea", "foreCol", sColorSelect, "LockBmp", "visible", bLockVisible,
				  "WidgetArea", "visible", true);
    // 6. Animate body
	sBodyColor = "cpcColor_Widget_Status";					// Auto mode (default)
	if (bInvalidBody) {
		sBodyColor = "unDataNotValid";							// Invalid data
	} else {
		if (sControlLetter == CPC_WIDGET_TEXT_CONTROL_FORCED) {
			sBodyColor = "unWidget_ControlStateForced";			// Forced mode
		}
    if (isRestartNeeded) {
      sBodyColor = "unFaceplate_WarningActive";
    }
    if (sAlarmLetter == CPC_WIDGET_TEXT_ALARM_TSTOPIST || sAlarmLetter == CPC_WIDGET_TEXT_ALARM_FUSTOPIST) {
      sBodyColor = "unFaceplate_AlarmActive";
    }
    if (iAlUnackBitPosition >= 0 && getBit(bit32StsReg01, iAlUnackBitPosition) == 1 && (iTStopIStState == 1 || iTStopIStState == 3 || iFuStopIStState == 1 || iFuStopIStState == 3)) {
      sBodyColor = "unWidget_AlarmNotAck";
    }
  }

    sFunction = cpcGenericObject_GetAnimationFunction(sWidgetType);
    
    int iOnBit = cpcGenericObject_StsRegConstantNameToValue(onBitConstant, bit32StsReg01, bit32StsReg02);
    int iOffBit = cpcGenericObject_StsRegConstantNameToValue(offBitConstant, bit32StsReg01, bit32StsReg02);
    if (isFunctionDefined(sFunction) && sFunction != "") {
        if(g_bSystemConnected) {
            execScript("main(int iOn, int iOff, string sBodyColor) {" +
                   sFunction + "(iOn, iOff, sBodyColor);}",
                   makeDynString(), iOnBit, iOffBit, sBodyColor);
        }
    }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cpcGenericObject_WidgetAnimation
/**
Purpose: animate widget

Parameters:
	- sWidgetType, string, input, widget type = <dpType> + <widget type>. ex: AnaDigHeater
	- sDpType, string, input, object
	- bLocked, bool, input, for select animation
	- sSelectedManager, string, input, for select animation
	- bit32StsReg01, bit32, input, status register
	- diWarningBits, dyn_int, input, warning bit positions in status register.
	- dsWarningLetters, dyn_string, input, associated warning letters to be displayed.
	- diAlarmBits, dyn_int, input, alarm bit positions in status register.
	- dsAlarmLetters, dyn_string, input, associated alarm letters to be displayed.
	- diControlBits, dyn_int, input, control bit positions in status register.
	- dsControlLetters, dyn_string, input, associated control letters to be displayed.
	- iOnbitPosition, int, input, on status bit position in status register
	- iOffBitPosition, int, input, off status bit position in status register
	- iAlUnackBitPosition, int, input, alarm unack status bit position in status register
	- bInvalidLetter, bool, input, true if invalid letter as to be shown
	- bInvalidBody, bool, input, invalid animation on body ?
	- iSystemIntegrityAlarmValue, int, input, the value of the systemIntegrity alarm for the PLC:
							c_unSystemIntegrity_no_alarm_value (0) no alarm PLC is running,
							c_unSystemIntegrity_alarm_value_level1 (10): alarm PLC is not correctly running
	- bSystemIntegrityAlarmEnabled, bool, input, the systemIntegrity alarm is enabled

di<...>Bits and ds<...>Letters must have the same length. dyn arrays must be defined in priority order: lowest -> highest priority .

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cpcGenericObject_WidgetAnimation(string sWidgetType, string sDpType,
				bool bLocked, string sSelectedManager, bit32 bit32StsReg01,
				dyn_int diWarningBits, dyn_string dsWarningLetters,
				dyn_int diAlarmBits, dyn_string dsAlarmLetters,
				dyn_string diControlBitNames,
				string onBitConstant, string offBitConstant, int iAlUnackBitPosition,
				bool bInvalidLetter, bool bInvalidBody,
				int iSystemIntegrityAlarmValue,
				bool bSystemIntegrityAlarmEnabled)
{
	dyn_int diWarningBits_EMPTY, diAlarmBits_EMPTY;
	dyn_string dsWarningLetters_EMPTY, dsAlarmLetters_EMPTY;
	bit32 bit32StsReg02_EMPTY;

	diWarningBits_EMPTY = makeDynInt();
	diAlarmBits_EMPTY = makeDynInt();
	dsWarningLetters_EMPTY = makeDynString();
	dsAlarmLetters_EMPTY = makeDynString();

	cpcGenericObject_WidgetAnimationDoubleStsReg(sWidgetType, sDpType, bLocked, sSelectedManager,
				bit32StsReg01, bit32StsReg02_EMPTY,
				diWarningBits, diWarningBits_EMPTY, dsWarningLetters, dsWarningLetters_EMPTY,
				diAlarmBits, dsAlarmLetters, diAlarmBits_EMPTY, dsAlarmLetters_EMPTY,
                0, 0, 0, 0,
				diControlBitNames, onBitConstant, offBitConstant, iAlUnackBitPosition,
				bInvalidLetter, bInvalidBody, iSystemIntegrityAlarmValue, bSystemIntegrityAlarmEnabled);
}

// cpcGenericObject_WidgetDisplayValueAnimation
/**
Purpose: animation of widget display values

Parameters:
	- dsShapes, dyn_string, input, display values shapes

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cpcGenericObject_WidgetDisplayValueAnimation(string sShape, string sUnit, string sFormat, float fValue, string sColor, bool bInvalid)
{
  string displayColor, formattedValue;

  displayColor = sColor;
  formattedValue = unGenericObject_FormatValue(sFormat, fValue);
  if (bInvalid) {
    displayColor = "unDataNotValid";
  }
  if (sUnit != "") {
    formattedValue = formattedValue + " " + sUnit;
  }
  if (g_bSystemConnected) {
    setMultiValue(sShape, "text", formattedValue, sShape, "foreCol", displayColor);
  }
}


//---------------------------------------------------------------------------------------------------------------------------------------

// cpcGenericObject_HandleUnicosMenu
/**
Purpose: handle the result of the pop-up menu for the UNICOS device

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList
	- exceptionInfo, dyn_string, output, the error is returned here
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cpcGenericObject_HandleUnicosMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer, dyn_string &exceptionInfo)
{
	bool bRes;
	string sCatKey;
    dyn_string exceptionInfoTemp1, exceptionInfoTemp2;

	switch (menuAnswer)
	{
		case CPC_POPUPMENU_BLOCK_PLC_NUMBER:
			bRes = dynContains(menuList, "PUSH_BUTTON, Block PLC Alarm, " + CPC_POPUPMENU_BLOCK_PLC_NUMBER + ", 1");
			sCatKey = (bRes > 0) ? "MASKPLC" : "UNMASKPLC";
			unGenericObject_SendBinaryOrder(deviceName, CPC_MANREG01_MALBSETRST, sCatKey, exceptionInfo);
			break;
		case CPC_POPUPMENU_MANUAL_IO_ERROR_NUMBER:
			cpcGenericObject_SendBinaryOrder(deviceName, "ManReg01", CPC_MANREG01_MIOERBSETRST, "IOERRORBLOCKING", exceptionInfo);
			break;
        case CPC_POPUPMENU_HIERARCHY_NUMBER:
            cpcGenericObject_OpenHierarchy(deviceName);
            break;
        case CPC_POPUPMENU_ALARMS_NUMBER:
            cpcGenericObject_OpenAlarmsPanel(deviceName);
            break;
        case CPC_POPUPMENU_MASK_ALST_NUMBER:
            dpGet(deviceName + ".ProcessInput.HHAlSt:_alert_hdl.._active", bRes);
            unAlarmConfig_mask(deviceName + ".ProcessInput.HHAlSt", bRes, exceptionInfoTemp1);
            unAlarmConfig_mask(deviceName + ".ProcessInput.HWSt", bRes, exceptionInfoTemp1);
            unAlarmConfig_mask(deviceName + ".ProcessInput.LWSt", bRes, exceptionInfoTemp1);
            unAlarmConfig_mask(deviceName + ".ProcessInput.LLAlSt", bRes, exceptionInfoTemp2);
            sCatKey = bRes ? "MASKPVSS" : "UNMASKPVSS";
            cpcGenericObject_notifyUsers(deviceName, sCatKey, exceptionInfoTemp1);
            if (dynlen(exceptionInfoTemp1) > 0) {
                dynAppend(exceptionInfo, exceptionInfoTemp1);
            }
            if (dynlen(exceptionInfoTemp2) > 0) {
                dynAppend(exceptionInfo, exceptionInfoTemp2);
            }
			break;
        case CPC_POPUPMENU_MASK_IST_NUMBER:
			dpGet(deviceName + ".ProcessInput.ISt:_alert_hdl.._active", bRes);
			unAlarmConfig_mask(deviceName + ".ProcessInput.ISt", bRes, exceptionInfoTemp1);
			sCatKey = bRes ? "MASKPVSS" : "UNMASKPVSS";
            cpcGenericObject_notifyUsers(deviceName, sCatKey, exceptionInfoTemp1);
            if (dynlen(exceptionInfoTemp1) > 0) {
                dynAppend(exceptionInfo, exceptionInfoTemp1);
            }
			break;
		case CPC_POPUPMENU_UNMASKEVENT_NUMBER:				// unmask event
            cpcConfigGenericFunctions_setMaskEvent(sDpType, deviceName, true, exceptionInfo);
			cpcGenericObject_notifyUsers(deviceName, "UNMASKEVENT", exceptionInfo);
			break;
		case CPC_POPUPMENU_MASKEVENT_NUMBER:				// mask event
            cpcConfigGenericFunctions_setMaskEvent(sDpType, deviceName, false, exceptionInfo);
			cpcGenericObject_notifyUsers(deviceName, "MASKEVENT", exceptionInfo);
			break;
		default:
			unGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);
			break;
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cpcGenericObject_addUnicosActionToMenu
/**
Purpose: add the UNICOS action to the UNICOS device

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- dsMenuConfig, dyn_string, input, list of requested action
	- dsAccessOk, dyn_string, input, list of authorized action
	- menuList, dyn_string, output, output menu
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cpcGenericObject_addUnicosActionToMenu(string deviceName, string sDpType, dyn_string dsMenuConfig, dyn_string dsAccessOk, dyn_string &menuList)
{
    bool isAlertActive;
    bit32 bit32StsReg01;
    string menuText, currentAlertClass;
    int iRes, alertType = DPCONFIG_NONE, iManReg01;

    unGenericObject_addSelectToMenu(deviceName, sDpType, dsMenuConfig, dsAccessOk, menuList);
    if (unGenericDpFunctions_isMaskEvent(deviceName)) {
        int iMask;
        unGenericDpFunctions_getMaskEvent(deviceName, iMask);

        if (iMask == 0) {
            cpcGenericObject_addItemToMenu(menuList, "Unmask Event", CPC_POPUPMENU_UNMASKEVENT_NUMBER, dynContains(dsAccessOk, UN_FACEPLATE_MASKEVENT) > 0);
        } else {
            cpcGenericObject_addItemToMenu(menuList, "Mask Event",   CPC_POPUPMENU_MASKEVENT_NUMBER, dynContains(dsAccessOk, UN_FACEPLATE_MASKEVENT) > 0);
        }
    }
    if(dynlen(menuList) > 0) {
        if (menuList[dynlen(menuList)] != "SEPARATOR") {
            dynAppend(menuList, "SEPARATOR");
        }
    }
    // 2. Ack Alarm
    if (dynContains(dsMenuConfig, UN_POPUPMENU_ACK_TEXT) > 0) {
        if (dynContains(dsAccessOk, UN_FACEPLATE_BUTTON_ACK_ALARM) > 0) {
            // reset manreg01: to be able to ack alarm afterwards. Workaround due to a PVSS bug: right click event cancelled if another right click is executed before the end of the previous one.
            iRes = dpGet(deviceName + ".ProcessOutput.ManReg01", iManReg01);
            if ((iRes < 0) || (iManReg01 != 0)) {
                dpSet(deviceName + ".ProcessOutput.ManReg01", 0);
            }
        }
        cpcGenericObject_addItemToMenu(menuList, "Ack. Alarm", CPC_POPUPMENU_ACK_NUMBER, dynContains(dsAccessOk, UN_FACEPLATE_BUTTON_ACK_ALARM) > 0);
    }
    // 3. Mask PosSt
    if (dynContains(dsMenuConfig, UN_POPUPMENU_MASK_POSST_TEXT) > 0) {
        dpGet(deviceName + ".ProcessInput.PosSt:_alert_hdl.._type", alertType);
        if (alertType != DPCONFIG_NONE) {
            iRes = dpGet(deviceName + ".ProcessInput.PosSt:_alert_hdl.._active", isAlertActive);
            if (iRes >= 0) {
                cpcGenericObject_addItemToMenu(menuList, isAlertActive ? "Mask PVSS Alarm" : "Unmask PVSS Alarm", CPC_POPUPMENU_MASK_POSST_NUMBER, dynContains(dsAccessOk, UN_FACEPLATE_BUTTON_MASK_ALARM) > 0);
            }
        }
    }
    // 4. Mask ISt
    if (dynContains(dsMenuConfig, CPC_POPUPMENU_MASK_IST_TEXT) > 0) {
        dpGet(deviceName + ".ProcessInput.ISt:_alert_hdl.._type", alertType);
        if (alertType != DPCONFIG_NONE) {
            iRes = dpGet(deviceName + ".ProcessInput.ISt:_alert_hdl.._active", isAlertActive);
            if (iRes >= 0) {
                cpcGenericObject_addItemToMenu(menuList, isAlertActive ? "Mask PVSS Alarm" : "Unmask PVSS Alarm", CPC_POPUPMENU_MASK_IST_NUMBER, dynContains(dsAccessOk, UN_FACEPLATE_BUTTON_MASK_ALARM) > 0);
            }
        }
    }
    if (dynContains(dsMenuConfig, CPC_POPUPMENU_MASK_ALST_TEXT) > 0) {
        dpGet(deviceName + ".ProcessInput.HHAlSt:_alert_hdl.._type", alertType);
        if (alertType != DPCONFIG_NONE) {
            iRes = dpGet(deviceName + ".ProcessInput.HHAlSt:_alert_hdl.._active", isAlertActive);
            if (iRes >= 0) {
                cpcGenericObject_addItemToMenu(menuList, isAlertActive ? "Mask PVSS Alarm" : "Unmask PVSS Alarm", CPC_POPUPMENU_MASK_ALST_NUMBER, dynContains(dsAccessOk, UN_FACEPLATE_BUTTON_MASK_ALARM) > 0);
            }
        }
    }
    // 5. Block Plc
    if (dynContains(dsMenuConfig, UN_POPUPMENU_BLOCK_PLC_TEXT) > 0) {
        if (dynContains(dsAccessOk, UN_FACEPLATE_BUTTON_BLOCK_ALARM_ACTION) > 0) {
            iRes = dpGet(deviceName + ".ProcessOutput.ManReg01", iManReg01);
            if ((iRes < 0) || (iManReg01 != 0)) {
                dpSet(deviceName + ".ProcessOutput.ManReg01", 0);
            }
        }
        dpGet(deviceName + ".ProcessInput.StsReg01", bit32StsReg01);
        bool allowed = (!getBit(bit32StsReg01, CPC_STSREG01_AUIHMBST)) && (dynContains(dsAccessOk, UN_FACEPLATE_BUTTON_BLOCK_ALARM_ACTION) > 0);
        cpcGenericObject_addItemToMenu(menuList, getBit(bit32StsReg01, CPC_STSREG01_MALBRST) ? "Deblock PLC Alarm" : "Block PLC Alarm", CPC_POPUPMENU_BLOCK_PLC_NUMBER, allowed);
    }
    // 7. Manual IO Error Block
    if (dynContains(dsMenuConfig, CPC_POPUPMENU_MANUAL_IO_ERROR_BLOCK) > 0) {
        iRes = dpGet(deviceName + ".ProcessInput.StsReg01", bit32StsReg01);
        cpcGenericObject_addItemToMenu(menuList, getBit(bit32StsReg01, CPC_STSREG01_MIOERBRST) ? "Manual IO Error Unblock" : "Manual IO Error Block", CPC_POPUPMENU_MANUAL_IO_ERROR_NUMBER, dynContains(dsAccessOk, UN_FACEPLATE_BUTTON_MASK_ALARM) > 0);
    }
}

/**
Purpose: add the UNICOS default action to the UNICOS device: faceplate, diagnostic and Info

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- dsMenuConfig, dyn_string, input, list of requested action
	- dsAccessOk, dyn_string, input, list of authorized action
	- menuList, dyn_string, output, output menu
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cpcGenericObject_addDefaultUnicosActionToMenu(string deviceName, string sDpType, dyn_string dsMenuConfig, dyn_string dsAccessOk, dyn_string &menuList)
{
    dyn_string exceptionInfo;
    string sLink, sEna, sSystemName = unGenericDpFunctions_getSystemName(deviceName), sDeviceLink;
    int i, len, iCste, pos;
    dyn_string dsSplit, dsParameters;
    bool bConnected;
    string sDeviceDpLink, sSystemLink, strMenu;
    bit32 b32Sts;
        
    string sTriggerDp = _unGraphicalFrame_getInternalTriggerName(myModuleName());
    unGenericDpFunctions_getParameters(deviceName, dsParameters);

    if(dynlen(menuList) > 0) {
        if (menuList[dynlen(menuList)] != "SEPARATOR") {
            dynAppend(menuList, "SEPARATOR");
        }
    }
    // 1. faceplate
    if (dynContains(dsMenuConfig, UN_POPUPMENU_FACEPLATE_TEXT) > 0) {
        cpcGenericObject_addItemToMenu(menuList, "Faceplate", CPC_POPUPMENU_FACEPLATE_NUMBER, true);
    }
    cpcGenericObject_addItemToMenu(menuList, "Hierarchy", CPC_POPUPMENU_HIERARCHY_NUMBER, true);
    if (dynContains(dsMenuConfig, CPC_POPUPMENU_ALARMS_TEXT) > 0) {
      cpcGenericObject_addItemToMenu(menuList, "Alarms", CPC_POPUPMENU_ALARMS_NUMBER, dynlen(cpcGenericObject_getLinkedAlarms(deviceName, exceptionInfo)) > 0);
    }
    cpcGenericObject_addItemToMenu(menuList, "Diagnostic", CPC_POPUPMENU_DIAGNOSTIC_NUMBER, dpExists(sTriggerDp) && dsParameters[UN_PARAMETER_DIAGNOSTIC] != "");
    cpcGenericObject_addItemToMenu(menuList, "Info", CPC_POPUPMENU_INFO_NUMBER, dsParameters[UN_PARAMETER_HTML] != "");
    cpcGenericObject_addItemToMenu(menuList, "Synoptic", CPC_POPUPMENU_OPENSYNOPTIC_NUMBER, dpExists(sTriggerDp) && dsParameters[UN_PARAMETER_PANEL] != "");

    cpcGenericObject_addItemToMenu(menuList, "Device configuration", CPC_POPUPMENU_DEVICE_CONFIG_NUMBER, true);

    unGenericDpFunctions_getDeviceLink(deviceName, sLink);
    if(sLink != "") {
        dynAppend(menuList,"CASCADE_BUTTON, Device link(s), 1");
        dynAppend(menuList,"Device link(s)");
        dsSplit = strsplit(sLink, UN_CONFIG_GROUP_SEPARATOR);
        len = dynlen(dsSplit);
        iCste = CPC_POPUPMENU_DEVICELINK_NUMBER;
        for(i=1;i<=len;i++) {
            iCste++;
            bool enabled = true;
            pos = strpos(dsSplit[i], ":");
            if(pos <= 0) // no system name add device system name
                sDeviceLink = sSystemName + dsSplit[i];
            else
                sDeviceLink = dsSplit[i];
                
            sSystemLink = unGenericDpFunctions_getSystemName(sDeviceLink);
            unDistributedControl_isConnected(bConnected, sSystemLink);
            if (!bConnected) {
                enabled = false;
            } else {
                sDeviceDpLink = unGenericDpFunctions_getWidgetDpName(sDeviceLink);
                if(!dpExists(sDeviceDpLink))
                    enabled = false;
            }
            string itemName = dsSplit[i];
            if (sDpType == UN_CONFIG_CPC_ANADIG_DPT_NAME || sDpType == UN_CONFIG_CPC_ANALOG_DPT_NAME || sDpType == UN_CONFIG_CPC_ANADO_DPT_NAME) {
                if(enabled) { // dev existing
                    if (sSystemLink == unGenericDpFunctions_getSystemName(deviceName)) {
                        if (dpTypeName(sDeviceDpLink) == UN_CONFIG_CPC_CONTROLLER_DPT_NAME) { // controller
                            dpGet(sDeviceDpLink+".ProcessInput.StsReg01", b32Sts);
                            if(getBit(b32Sts, UN_STSREG01_ACTIVE)) {
                                itemName = "* " + itemName;
                            }
                        }
                    }
                }
            }
            cpcGenericObject_addItemToMenu(menuList, itemName, iCste, enabled);
        }
    }
}

/**
Purpose: add the trend action: all the possible DPE

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- dsMenuConfig, dyn_string, input, list of requested action
	- dsAccessOk, dyn_string, input, list of authorized action
	- menuList, dyn_string, output, output menu
        - bUseAlias, bool, input, true=use alias instead of DPE/false=use DPE (default)
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cpcGenericObject_addTrendActionToMenu(string deviceName, string sDpType, dyn_string dsMenuConfig, dyn_string dsAccessOk, dyn_string &menuList, bool bUseAlias=false)
{
	dyn_string dsAnalog, dsRange, exceptionInfo;
	int i, length, iCste = UN_POPUPMENU_TREND_CSTE, iMenuValue, pos;
	string sDeleteAllowed;
        dyn_string dsDPEAnalog, dsDPEAlias, dsProxy, dsDPE, dsDPERange, dsCurveDPE;
        bool bProxy;
        dyn_bool dbCurveVisible;
        dyn_string dsCurveColor;
        dyn_float dfCurveMin, dfCurveMax;
        dyn_string dsFaceplateDPE;
        
	if (dynContains(dsMenuConfig, UN_POPUPMENU_TREND_TEXT) > 0)
	{
		unGenericObject_GetTrendDPEList(sDpType, dsAnalog, dsRange, exceptionInfo);
                unGenericDpFunctions_getProxyDeviceList(deviceName, bProxy, dsProxy);
                unGenericObject_deviceGetAllTrendDPE(deviceName, bProxy, bUseAlias, dsProxy, 
                                                     dsAnalog, dsRange, makeDynBool(), makeDynString(),
                                                     makeDynFloat(), makeDynFloat(),
                                                     dsFaceplateDPE, dsDPEAnalog, dsDPE, dsDPEAlias, dsDPERange, dsCurveDPE,
                                                     dbCurveVisible, dsCurveColor, 
                                                     dfCurveMin, dfCurveMax);
		length = dynlen(dsDPEAnalog);
		if(length > 0) {
			pos = dynContains(menuList, "Device link(s)");
                        if(_unGenericObject_checkAccessRigth(fwAccessControl_CONFIG_AccessRight_DP, exceptionInfo))
                          sDeleteAllowed = "1";
                        else
                          sDeleteAllowed = "0";
			if(pos > 0){
				if((pos-1) > 0) {
					if (menuList[pos-1] != "SEPARATOR")
					{
						dynInsertAt(menuList,"SEPARATOR", pos++);
					}
				}
				dynInsertAt(menuList,"PUSH_BUTTON, Delete dynamic trend, "+UN_POPUPMENU_TREND_CSTE+", "+sDeleteAllowed, pos++);
				dynInsertAt(menuList,"CASCADE_BUTTON, Add to dynamic trend, 1", pos++);
				//dynInsertAt(menuList,"SEPARATOR", pos++); // don't need a separator between trend related items
				dynInsertAt(menuList,"CASCADE_BUTTON, Single DPE trend, 1", pos++);
			}
			else {
				if(dynlen(menuList) > 0) {
					if (menuList[dynlen(menuList)] != "SEPARATOR")
					{
						dynAppend(menuList, "SEPARATOR");
					}
				}
				dynAppend(menuList,"PUSH_BUTTON, Delete dynamic trend, "+UN_POPUPMENU_TREND_CSTE+", "+sDeleteAllowed);
				dynAppend(menuList,"CASCADE_BUTTON, Add to dynamic trend, 1");
				//dynAppend(menuList, "SEPARATOR"); // don't need a separator between trend related items
				dynAppend(menuList,"CASCADE_BUTTON, Single DPE trend, 1");
			}
			dynAppend(menuList,"Add to dynamic trend");
//  DebugN("addTrend",dsAnalog, dsRange, "ANA", dsDPEAnalog, "DPE", dsDPE, "ALIAS", dsDPEAlias, "RAN", dsDPERange, "CUR", dsCurveDPE, 
//         "VICI", dbCurveVisible, "COLOR", dsCurveColor, "MIN", dfCurveMin, "MAX", dfCurveMax, "FAC", dsFaceplateDPE);
                        length = dynlen(dsDPEAnalog);
			for(i=1;i<=length;i++) {
				iMenuValue = iCste+i;
                                // if proxy DPE & exits add
                                if(bUseAlias)
                                {
                                  if(dsDPEAlias[i] != "")
                                    dynAppend(menuList,"PUSH_BUTTON, "+dsDPEAlias[i]+", "+iMenuValue+", 1");
                                }
                                else
				  dynAppend(menuList,"PUSH_BUTTON, "+dsDPEAnalog[i]+", "+iMenuValue+", 1");
			}
			dynAppend(menuList,"Single DPE trend");
			for(i=1;i<=length;i++) {
				iMenuValue = 10000+iCste+i;
                                // if proxy DPE & exits add
                                if(bUseAlias)
                                {
                                  if(dsDPEAlias[i] != "")
                                    dynAppend(menuList,"PUSH_BUTTON, "+dsDPEAlias[i]+", "+iMenuValue+", 1");
                                }
                                else
				  dynAppend(menuList,"PUSH_BUTTON, "+dsDPEAnalog[i]+", "+iMenuValue+", 1");
			}
		}
	}
//DebugN(menuList);
}

cpcGenericObject_addItemToMenu(dyn_string &menuList, string menuText, int menuNumber, bool granted)
{
    string access = granted ? "1" : "0";
    dynAppend(menuList,"PUSH_BUTTON, "+menuText+", "+menuNumber+", "+access);
}

void cpcGenericObject_notifyUsers(string deviceName, string sKey, dyn_string exceptionInfo) {
	dyn_string exceptionInfoTemp;
	string alias = unGenericDpFunctions_getAlias(deviceName);
	if(dynlen(exceptionInfo) > 0) {
		unSendMessage_toAllUsers(alias + getCatStr("unCPCOperator", sKey + "FAILED"), exceptionInfoTemp);
		if (dynlen(exceptionInfoTemp) > 0) {
			fwExceptionHandling_display(exceptionInfoTemp);
		}
		unSendMessage_toExpertException(this.name(), exceptionInfo);
	} else {
		unSendMessage_toAllUsers(alias + getCatStr("unCPCOperator", sKey + "OK"), exceptionInfoTemp);
		if (dynlen(exceptionInfoTemp) > 0) {
			fwExceptionHandling_display(exceptionInfoTemp);
		}
	}
}



// cpcGenericObject_getCtrlLink
/**
Purpose: get the list of Controller

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnaDig/Analog widgets
*/
dyn_string cpcGenericObject_getCtrlLink(string sSystemName, dyn_string dsDeviceName)
{
	dyn_string dsCtrl;
	string sDp;
	int i, len, iPos;
	
	len = dynlen(dsDeviceName);
	for(i=1;i<=len;i++) {
		iPos = strpos(dsDeviceName[i], ":");
		if(iPos <=0) 
			dsDeviceName[i] = sSystemName+dsDeviceName[i];
		sDp = unGenericDpFunctions_getWidgetDpName(dsDeviceName[i]);
		if(sSystemName == unGenericDpFunctions_getSystemName(sDp)) {
			if(dpExists(sDp)) {
				if(dpTypeName(sDp) == UN_CONFIG_CPC_CONTROLLER_DPT_NAME) {
					dynAppend(dsCtrl, sDp + ".ProcessInput.StsReg01");
				}
			}
		}
	}
	
	return dsCtrl;
}

cpcGenericObject_WidgetCtrlAnimationCB(dyn_string dsCtrlNames, dyn_anytype daCtrlStsReg) {
	int activeCtrlIdx = 0;
	for (int i = 1; i <= dynlen(daCtrlStsReg); i++) {
		if (getBit(daCtrlStsReg[i], UN_STSREG01_ACTIVE)) {
			activeCtrlIdx = i;
			break;
		}
	}

	if (activeCtrlIdx == 0) {
		setMultiValue("RegulationText1", "visible", false,
						"RegulationText2", "visible", false);
	} else {
		bit32 activeStsReg = daCtrlStsReg[activeCtrlIdx];
		string operationModeLetter, operationModeColor, workingStateLetter, workingStateColor;
		unGenericObject_WidgetControlStateAnimation(activeStsReg, makeDynInt(UN_STSREG01_AUMOST, UN_STSREG01_MMOST, UN_STSREG01_FOMOST), makeDynString("A", CPC_WIDGET_TEXT_CONTROL_MANUAL, CPC_WIDGET_TEXT_CONTROL_FORCED), operationModeLetter, operationModeColor);
		unGenericObject_WidgetControlStateAnimation(activeStsReg, makeDynInt(CPC_STSREG01_REGST, CPC_STSREG01_TRST, CPC_STSREG01_OUTPST), makeDynString("R", "T", "P"), workingStateLetter, workingStateColor);

		if(g_bSystemConnected) {
			setMultiValue("RegulationText1", "text", operationModeLetter, "RegulationText1", "visible", true,
							"RegulationText2", "text", workingStateLetter, "RegulationText2", "visible", true);
		}
	}
}

cpcGenericObject_animateTranslatedValue(string sShape, mapping dict, anytype value, string color, bool invalid) {
    if (mappingHasKey(dict, (string)value)) { // display label
        if (g_bSystemConnected) {
            setMultiValue(sShape + ".display", "foreCol", invalid ? "unDataNotValid" : color, sShape + ".display", "text", dict[(string)value]);
        }
    } else { // display original value
        if (g_bSystemConnected) {
            unGenericObject_DisplayValue("", "", (float) value, sShape, color, invalid);
        }
    }
}

cpcGenericObject_ColorBoxAnimateInterlock(string sShape, int pvssAlarmState, bool alarmState, bool alarmAck, bool bInvalid) {
	if (alarmState) {
		if(g_bSystemConnected) {
			unGenericObject_ColoredSquareAnimate(sShape, alarmState, alarmAck && pvssAlarmState == 1 || pvssAlarmState == 3, "unFaceplate_AlarmNotAck", "unFaceplate_AlarmActive", bInvalid);
		}
	} else {
		if(g_bSystemConnected) {
			unGenericObject_ColoredSquareAnimate(sShape, !(alarmAck && (pvssAlarmState == 1 || pvssAlarmState == 3)), true, "unFaceplate_Unactive", "unFaceplate_AlarmNotAck", bInvalid);
		}
	}
}

cpcGenericObject_ColorBoxAnimate(string sShape, int iCommandbit, int iEnableBit, string sOneBackCol, bool bInvalid, string sZeroBackCol="unFaceplate_Unactive")
{
    if(g_bSystemConnected) {
        unGenericObject_ColorBoxAnimate(sShape, iCommandbit, iEnableBit, sOneBackCol, bInvalid, sZeroBackCol);
    }
}

cpcGenericObject_ColorBoxDisable(string sShape) {
    if(g_bSystemConnected) {
        setMultiValue(sShape + ".colorbox", "enabled", false, 
                      sShape + ".colorbox", "backCol", "unFaceplate_Disabled");
    }
}


cpcGenericObject_CheckboxAnimate(string sShape, bool state, bool enabled = true) {
    if(g_bSystemConnected) {
        setValue(sShape + ".checkbox", "state", 0, state);
    }
    if (shapeExists(sShape + "Text")) {
        if(g_bSystemConnected)
            setValue(sShape + "Text", "enabled", enabled);
    }
}

cpcGenericObject_WidgetControlStateAnimation(bit32 bit32StsReg01, bit32 bit32StsReg02, dyn_string d_sBits, string &letter, string &color)
{
	letter = "";
	color = "unWidget_ControlStateManualLocal";
	dyn_string exceptionInfo;

	for (int i = 1; i <= dynlen(d_sBits); i++) {
		string bit = d_sBits[i];
		dyn_string names = strsplit(bit, "_");
		if (dynlen(names) != 3) {
			fwException_raise(exceptionInfo, "ERROR", "cpcGenericObject_WidgetControlStateAnimation: wrong bit name - "+bit, "");
			break;
		}
		int pos = cpcConfigGenericFunctions_getConstantValueByName(bit);
		bool enabled = false;
		if (names[2] == "STSREG01") {
			enabled = getBit(bit32StsReg01, pos) == 1;
		} else if (names[2] == "STSREG02") {
			enabled = getBit(bit32StsReg02, pos) == 1;
		} else {
			fwException_raise(exceptionInfo, "ERROR", "cpcGenericObject_WidgetControlStateAnimation: wrong register in "+bit, "");
		}
		if (enabled) {
			letter = cpcConfigGenericFunctions_getStringConstantValueByName("CPC_WIDGET_TEXT_CONTROL_" + names[3]);
			break;
		}
	}
	if (letter == CPC_WIDGET_TEXT_CONTROL_FORCED) {
		color = "unWidget_ControlStateForced";
	}
}

int cpcGenericObject_StsRegConstantNameToValue(string name, bit32 bit32StsReg01, bit32 bit32StsReg02) {
    dyn_string exceptionInfo;

    if (name == "") {
        return -1;
    }

    int pos = cpcConfigGenericFunctions_getConstantValueByName(name);
    if (pos < 0) {
        fwException_raise(exceptionInfo, "ERROR", "cpcGenericObject_StsRegConstantNameToValue: negative constant value for "+name, "");
        return -1;
    }

    dyn_string names = strsplit(name, "_");
    if (dynlen(names) != 3) {
        fwException_raise(exceptionInfo, "ERROR", "cpcGenericObject_StsRegConstantNameToValue: wrong constant name - " + name, "");
        return -1;
    }

    bool value = false;
    if (names[2] == "STSREG01") {
        return getBit(bit32StsReg01, pos);
    } else if (names[2] == "STSREG02") {
        return getBit(bit32StsReg02, pos);
    } else {
        fwException_raise(exceptionInfo, "ERROR", "cpcGenericObject_StsRegConstantNameToValue: wrong register in "+name, "");
        return -1;
    }
}

// cpcGenericObject_SendBigPulse
/**
Purpose: send a pulse with a specified number

Parameters:
	sDpName: string, input, object name
	sDpe: string, input, Dpe name
	value: string, input, the pulse value of ManReg
	exceptionInfo: dyn_string, output, details of any exceptions are returned here
	alreadySent: bool, output, set to true if ManReg bit is already equal to 1 at the beginning of the function

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: XP, W2000 and Linux, but tested only under Wds.
	. distributed system: yes but not tested.
*/

cpcGenericObject_SendBigPulse(string sDpName, string sDpe, bit32 value, dyn_string &exceptionInfo)
{
	string regName, dpType, alias;
	bit32 regValue;
	dyn_string exceptionInfoTemp;

	regName = unGenericDpFunctions_getDpName(sDpName) + sDpe;
	if (dpGet(regName, regValue) == -1) {
		fwException_raise(exceptionInfo,"ERROR","unGenericObject_sendPulse : " + regName + getCatStr("unPVSS","DPGETFAILED"),"");
	} else {
		if (dpSet(regName, value) == -1) {
				fwException_raise(exceptionInfo,"ERROR","unGenericObject_sendPulse : " + regName + getCatStr("unPVSS","DPSETFAILED"),"");
		}
	}

	unGenericObject_WaitPulse();

	if (dpSet(regName, regValue) == -1) {
		fwException_raise(exceptionInfo,"ERROR","unFaceplateButtons_sendPulse : " + regName + getCatStr("unPVSS","DPSETFAILED"),"");
	}
}

string cpcGenericObject_translateAlarmAcronym(dyn_string acronyms) {
    if (dynlen(acronyms) == 1) {
        switch(substr(acronyms[1], 0, 2)) {
            case "AL": return "No Interlock (AL)";
            case "FS": return "Full Stop Interlock (FS)";
            case "TS": return "Temp Stop Interlock (TS)";
            case "SI": return "Start Interlock (SI)";
            default: return "Unknown (" + acronyms + ")";
        }
    } else if (dynlen(acronyms) > 1) {
        dyn_string clearTypes;
        for (int i = 1; i <= dynlen(acronyms); i++) {
            dynAppend(clearTypes, substr(acronyms[i], 0, 2));
        }
        return "Multiple (" + dynJoin(clearTypes, ",") + ")";
    } else {
    }
    return "No type";
}

// cpcGenericObject_ColorBoxAnimateWarning
/**
Purpose: animate warning colorBoxes

Parameters:
    - sShape, string, input, colorbox shape name
    - iAlarmState, int, input, act state alarm
    - bBoolAlert, bool, input, is alert hdl defined ?
    - bInvalid, bool, input, invalid data ?
    
Color : - if alert hdl does not exist, backcolor = "unAlarmDoesNotExist"
        - else if bInvalid = true, backcolor = "unDataNotValid"
        -      else if alarm unack., backcolor = "unFaceplate_AlarmNotAck"
        -           else if alarm active and ack., backcolor = "unFaceplate_AlarmActive"
        -                else backcolor = "unFaceplate_Unactive"
Usage: External

PVSS manager usage: NG, NV

Constraints:
    . PVSS version: 2.12.1 
    . operating system: NT and W2000, but tested only under W2000.
    . distributed system: yes.
*/
cpcGenericObject_ColorBoxAnimateWarning(string sShape, int iAlarmState, bool isAlertExists, bool isAlertActive, bool isAlertInvalid)
{
    cpcGenericObject_ColorBoxAnimateAlarm(sShape, iAlarmState, isAlertExists, isAlertActive, isAlertInvalid, "unFaceplate_WarningActive", "unFaceplate_WarningActive");
}

// cpcGenericObject_ColorBoxAnimateAlarm
/**
Purpose: animate alarm colorBoxes

Parameters:
	- sShape, string, input, colorbox shape name
	- iAlarmState, int, input, act state alarm
	- bBoolAlert, bool, input, is alert hdl defined ?
	- bInvalid, bool, input, invalid data ?
	
Color : - if alert hdl does not exist, backcolor = "unAlarmDoesNotExist"
		- else if bInvalid = true, backcolor = "unDataNotValid"
		-	   else if alarm unack., backcolor = "unFaceplate_AlarmNotAck"
		-			else if alarm active and ack., backcolor = "unFaceplate_AlarmActive"
		-				 else backcolor = "unFaceplate_Unactive"
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cpcGenericObject_ColorBoxAnimateAlarm(string sShape, int iAlarmState, bool isAlertExists, bool isAlertActive, bool isAlertInvalid, string activeColor = "unFaceplate_AlarmActive", string notAckColor = "unFaceplate_AlarmNotAck")
{
    string sColor;
    bool textEnabled = true;
    
    if (!isAlertExists) {
        sColor = "_3DFace";
        textEnabled = false;
    } else if (isAlertInvalid) {
        sColor = "unDataNotValid";
    } else if (!isAlertActive) {
        sColor = "unFaceplate_Disabled";
        textEnabled = false;
    } else {
        switch (iAlarmState) {
            case DPATTR_ALERTSTATE_NONE:
                sColor = "unFaceplate_Unactive";
                break;
            case DPATTR_ALERTSTATE_APP_ACK:
                sColor = activeColor;
                break;
            case DPATTR_ALERTSTATE_APP_NOT_ACK:
            case DPATTR_ALERTSTATE_DISAPP_NOT_ACK:
            case DPATTR_ALERTSTATE_APP_DISAPP_NOT_ACK:
                sColor = notAckColor;
                break;
            default:
                sColor = "unFaceplate_Unactive";
                break;
        }
    }

    if(g_bSystemConnected)
        setValue(sShape + ".colorbox", "backCol", sColor);
    if (shapeExists(sShape + "Text")) {
        if(g_bSystemConnected)
            setValue(sShape + "Text", "enabled", textEnabled);
    }
}

// cpcGenericObject_GetAnalogAlarmType
/**
Purpose: retrieve 5 ranges alarm type

Parameters:
	- sDpe, string, input, dpe
	- iAlertType, int, output, alert type
	- exceptionInfo, dyn_string, output, for errors
	
Return in iAlertType : 1. DPCONFIG_ALERT_NONBINARYSIGNAL if Unicos 5 ranges alert_hdl is correct (and if sDpe is analogic)
					   2. DPCONFIG_ALERT_BINARYSIGNAL for boolean sDpe
					   3. DPCONFIG_NONE if no alert_hdl is defined or if alert_hdl is a bad analogic alarm

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. Unicos constants for alert_hdl
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

dyn_bool cpcGenericObject_GetAnalogAlarmType(string sDpe, int &iAlertType, dyn_string &exceptionInfo)
{
    int alertDPType = DPCONFIG_NONE, iRes, rangesNumber = -1, rangeOk;
    iAlertType = DPCONFIG_NONE;
    dyn_bool ranges = makeDynBool(false, false, false, false);
    bit32 alertRange = 0;

    if (!dpExists(sDpe)) {
        fwException_raise(exceptionInfo, "ERROR", "cpcGenericObject_GetAnalogAlarmType: " + sDpe + getCatStr("unGeneration", "DPDOESNOTEXIST"), "");
    } else {
        dpGet(sDpe + ":_alert_hdl.._type", alertDPType);
        switch (alertDPType) {
            case DPCONFIG_ALERT_BINARYSIGNAL:
                iAlertType = DPCONFIG_ALERT_BINARYSIGNAL;
                break;
            case DPCONFIG_ALERT_NONBINARYSIGNAL:
                iAlertType = DPCONFIG_ALERT_NONBINARYSIGNAL; //@TODO false when invalid alarms
                dpGet(sDpe + ":_alert_hdl.._num_ranges", rangesNumber);
                //@TODO: optimize it!
                //@TODO : check if rangeOK is valid! - rangesNumber <= 5 && rangeOK in the middle...
                for(int i = 1; i <= rangesNumber; i++) {
                    string rangeClass = "#";
                    dpGet(sDpe + ":_alert_hdl." + i + "._class", rangeClass);
                    if (rangeClass == "") {
                        rangeOk = i; // Range Ok
                    }
                }
                for(int i = 1; i < rangeOk; i++) {
                    string rangeClass = "#";
                    dpGet(sDpe + ":_alert_hdl." + i + "._class", rangeClass);
                    if (strpos(rangeClass, "cpcAnalogLL") >= 0) {
                        setBit(alertRange, 0, true);
                        ranges[1] = true;
                    } else if (strpos(rangeClass, "cpcAnalogL") >= 0) {
                        setBit(alertRange, 1, true);
                        ranges[2] = true;
                    } else {
                    }
                }
                for(int i = rangeOk+1; i <= rangesNumber; i++) {
                    string rangeClass = "#";
                    dpGet(sDpe + ":_alert_hdl." + i + "._class", rangeClass);
                    if (strpos(rangeClass, "cpcAnalogHH") >= 0) {
                        setBit(alertRange, 3, true);
                        ranges[4] = true;
                    } else if (strpos(rangeClass, "cpcAnalogH") >= 0) {
                        setBit(alertRange, 2, true);
                        ranges[3] = true;
                    } else {
                    }
                }
                //DebugN("rangesNumber", rangesNumber, rangeOk, alertRange, ranges);
                break;
            case DPCONFIG_NONE:
            default:
                iAlertType = DPCONFIG_NONE;
                break;
        }
    }
    return ranges;
}

bool cpcGenericObject_isAlarmWithMail(string dpName, int alertType) {
    string alertClass;
    int rangesNumber;
    dyn_bool isConfigured;
    if(alertType == DPCONFIG_ALERT_NONBINARYSIGNAL) {
        dpGet(dpName + ":_alert_hdl.._num_ranges", rangesNumber);
        for(int i = 1; i <= rangesNumber; i++) {
            dpGet(dpName + ":_alert_hdl." + i + "._class", alertClass);
            if (alertClass != "") { // skip OK class
                dynAppend(isConfigured, patternMatch(UN_PROCESSALARM_PATTERN, alertClass));
            }
        }
        dynUnique(isConfigured);
        if (dynlen(isConfigured) == 1) {
            return isConfigured[1];
        } else {
            return false;
        }
   } else {
        dpGet(dpName + ":_alert_hdl.1._class", alertClass);
        return patternMatch(UN_PROCESSALARM_PATTERN, alertClass);
    }
}

bool cpcGenericObject_getBit(int value, int bitPosition)
{
    int pattern = pow(2, bitPosition);
    return value & pattern;
}


// cpcGenericObject_WidgetDisconnection
/**
Purpose: animate widget disconnection

Parameters:
	- sWidgetType, string, input, widget type
	- sDpType, string, input, object

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cpcGenericObject_WidgetDisconnection(string sWidgetType, string sDpType)
{
    dyn_string dsFunctions, exceptionInfo;
    string sFunction;

    // 1. Disconnection of widget body
    sFunction = "cpcWidget_Widget" + sWidgetType + "Disconnection";
    if (isFunctionDefined(sFunction)) {
        execScript("main() {" + sFunction + "();}", makeDynString());
    }
    // 2. Disconnection of alarm, warning, mode letter & select animation & right click
    setMultiValue("WarningText", "text", "", "AlarmText", "text", "", "ControlStateText", "text", "", 
                  "SelectArea", "foreCol", "", "LockBmp", "visible", false, "WidgetArea", "visible", false);
}

string cpcGenericObject_GetAnimationFunction(string sWidgetType) {
    return "cpcWidget_Widget" + sWidgetType + "Animation";
}

cpcGenericObject_OpenHierarchy(string deviceName) {
	dyn_string exceptionInfo;
	string aliasName;
	int xPos, yPos;
	dyn_int diPanelSize;
	bool bRes;
	float width, heigth;

	aliasName = unGenericDpFunctions_getAlias(deviceName);
	getValue("", "position", xPos, yPos);
	getValue("", "size", width, heigth);
	xPos = xPos + 2 * width;
	diPanelSize = getPanelSize("vision/cpcObjects/General/cpcHierarchy.pnl");
	if (dynlen(diPanelSize) == 2) {
		xPos = xPos - diPanelSize[1];
		yPos = yPos - diPanelSize[2] - 15;
    }
	unGenericObject_NeedSelect(dpTypeName(deviceName), bRes, exceptionInfo);
	if (bRes) {
		unSelectDeselectHMI_select(deviceName, true, exceptionInfo);
    }
	unGraphicalFrame_ChildPanelOn("vision/cpcObjects/General/cpcHierarchy.pnl",
					  aliasName + " hierarchy",
					  makeDynString("$sDpName:" + deviceName),
					  xPos, yPos);
}

cpcGenericObject_OpenAlarmsPanel(string deviceName) {
	dyn_string exceptionInfo;
	string aliasName, descripName;
	int xPos, yPos;
	dyn_int diPanelSize;
	bool bRes;
	float width, heigth;
	
	aliasName = unGenericDpFunctions_getAlias(deviceName);
	descripName = unGenericDpFunctions_getDescription(deviceName);
	getValue("", "position", xPos, yPos);
	getValue("", "size", width, heigth);
	xPos = xPos + 2 * width;
	diPanelSize = getPanelSize("vision/cpcObjects/General/cpcLinkedAlarmList.pnl");
	if (dynlen(diPanelSize) == 2) {
		xPos = xPos - diPanelSize[1];
		yPos = yPos - diPanelSize[2] - 15;
    }
	unGenericObject_NeedSelect(dpTypeName(deviceName), bRes, exceptionInfo);
	if (bRes) {
		unSelectDeselectHMI_select(deviceName, true, exceptionInfo);
    }
	unGraphicalFrame_ChildPanelOn("vision/cpcObjects/General/cpcLinkedAlarmList.pnl",
					  aliasName + " " + descripName + " Configured alarms",
					  makeDynString("$sDpName:" + deviceName),
					  xPos, yPos);
}

dyn_int cpcGenericObject_getMaxWidgetSize(dyn_string objects)
{
    dyn_int maxDim = makeDynInt(1, 1);
    for (int i = 1; i <= dynlen(objects); i++) {
        string widget = cpcGenericObject_getDeviceWidget(objects[i]);
        if (widget != "") {
            dyn_int widgetDim = getPanelSize(widget);
            if (widgetDim[1] > maxDim[1]) {
                maxDim[1] = widgetDim[1];
            }
            if (widgetDim[2] > maxDim[2]) {
                maxDim[2] = widgetDim[2];
            }
        }
    }
    return maxDim;
}

string cpcGenericObject_getDeviceWidget(string device)
{
    string sWidget;
    dyn_string dsWidgets;

    string objectDP = unGenericDpFunctions_getWidgetDpName(device);
    string objectType = dpTypeName(objectDP);

    unGenericDpFunctions_getDeviceWidgetList(objectType, dsWidgets);
    unGenericDpFunctions_getDeviceWidget(objectDP, sWidget, dsWidgets);

    return sWidget;
}

dyn_string cpcGenericObject_getLinkedAlarms(string device, dyn_string& exceptionInfo)
{
  dyn_string children, alarmList;
  string deviceSystemName = unGenericDpFunctions_getSystemName(device);
  unGenericDpFunctions_getKeyDeviceConfiguration(device, CPC_CONFIG_CHILDREN_KEY, children, exceptionInfo);
  dynUnique(children);
  for (int i = 1; i <= dynlen(children); i++) {
    string child = unGenericDpFunctions_getWidgetDpName(deviceSystemName + children[i]);
    if (child != "" && strpos(dpTypeName(child), "Alarm") > 0) {
      dynAppend(alarmList, child);
    }
  }
  return alarmList;
}