/**@name LIBRARY: cpcAnaDigConfig.ctl

@author: Vincent Forest (LHC-IAS)

Creation Date: 05 12 2002

Modification History: 
    16/05/2012: Alexey
        [UCPC-787] introduce AlSt bit and letter constants

    20/09/2011: Alexey
        [UCPC-552] added HWLocal parameter

	04/08/2011: Alexey
		[UCPC-456] added POutMain and PPWMMode to the params list

	08/07/2011: Alexey
		[UCPC-389] moved "parameters" field

	01/07/2011: Alexey
		[UCPC-364] added getParamNames method which returns all possible params
		[UCPC-374] added PRstartFS

	2011/05/30 Alexey
		[UCPC-292] moved meta-info to the end of config line; removed UN_CPC_CONFIG length constants

	2011/05/16 amerezhi
		moved export functions from cpcExportGenericFunctions

	2011/05/09 amerezhi
		[UCPC-235] added metainfo (types relationships configuraiton)

	2011/03/30 amerezhi
		[UCPC-207] changes code structure according new model

	18/04/2006: Herve
		- implementation of mask event

	15/05/2005: Enrique
		- S7 adaptation, new functions:
			. S7_PLC_CPC_AnaDig_setConfig
			. S7_PLC_CPC_AnaDig_checkConfig
			
	22/04/2005: Herve
		- allow archive in import file per device

	15/04/2005: Herve
		- Modbus set and check front-end functions: _UnPlc_CPC_AnaDig_checkConfig, _UnPlc_CPC_AnaDig_setConfig

version 1.0

External Function : 
	. CPC_AnaDigConfig_setConfig :  set configuration of given AnaDig datapoint
	. CPC_AnaDigConfig_checkConfig : check data before configuration
	
Internal Functions :
	
Purpose: 
This library contains the function used to set Unicos AnaDig configuration.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

// Constants
const string UN_CONFIG_CPC_ANADIG_DPT_NAME = "CPC_AnaDig";

const unsigned UN_CONFIG_CPC_ANADIG_LENGTH = 23;

const unsigned UN_CONFIG_CPC_ANADIG_UNIT = 1;
const unsigned UN_CONFIG_CPC_ANADIG_FORMAT = 2;
const unsigned UN_CONFIG_CPC_ANADIG_RANGEMAX = 3;
const unsigned UN_CONFIG_CPC_ANADIG_RANGEMIN = 4;
const unsigned UN_CONFIG_CPC_ANADIG_LIMITON = 5;
const unsigned UN_CONFIG_CPC_ANADIG_LIMITOFF = 6;
const unsigned UN_CONFIG_CPC_ANADIG_DEADBAND = 7;
const unsigned UN_CONFIG_CPC_ANADIG_DEADBAND_TYPE = 8;
const unsigned UN_CONFIG_CPC_ANADIG_ARCHIVE_ACTIVE = 9;
const unsigned UN_CONFIG_CPC_ANADIG_ARCHIVE_TIME_FILTER = 10;
const unsigned UN_CONFIG_CPC_ANADIG_NORMAL_POSITION = 11;
const unsigned UN_CONFIG_CPC_ANADIG_ADDRESS_STSREG01 = 12;
const unsigned UN_CONFIG_CPC_ANADIG_ADDRESS_STSREG02 = 13;
const unsigned UN_CONFIG_CPC_ANADIG_ADDRESS_EVSTSREG01 = 14;
const unsigned UN_CONFIG_CPC_ANADIG_ADDRESS_EVSTSREG02 = 15;
const unsigned UN_CONFIG_CPC_ANADIG_ADDRESS_POSST = 16;
const unsigned UN_CONFIG_CPC_ANADIG_ADDRESS_AUPOSRST = 17;
const unsigned UN_CONFIG_CPC_ANADIG_ADDRESS_MPOSRST = 18;
const unsigned UN_CONFIG_CPC_ANADIG_ADDRESS_POSRST = 19;
const unsigned UN_CONFIG_CPC_ANADIG_ADDRESS_MANREG01 = 20;
const unsigned UN_CONFIG_CPC_ANADIG_ADDRESS_MPOSR = 21;
const unsigned UN_CONFIG_CPC_ANADIG_ADDRESS_PLION = 22;
const unsigned UN_CONFIG_CPC_ANADIG_ADDRESS_PLIOFF = 23;

const unsigned UN_CONFIG_CPC_ANADIG_ADDITIONAL_LENGTH          = 7;
const unsigned UN_CONFIG_CPC_ANADIG_ADDITIONAL_MASKEVENT       = 1;
const unsigned UN_CONFIG_CPC_ANADIG_ADDITIONAL_PARAMETERS      = 2;
const unsigned UN_CONFIG_CPC_ANADIG_ADDITIONAL_MASTER_NAME     = 3;
const unsigned UN_CONFIG_CPC_ANADIG_ADDITIONAL_PARENTS         = 4;
const unsigned UN_CONFIG_CPC_ANADIG_ADDITIONAL_CHILDREN        = 5;
const unsigned UN_CONFIG_CPC_ANADIG_ADDITIONAL_TYPE            = 6;
const unsigned UN_CONFIG_CPC_ANADIG_ADDITIONAL_SECOND_ALIAS    = 7;

mapping CPC_AnaDigConfig_getConfig() {
	mapping config;
	mapping props;

	mappingClear(props);
	props["address"] 		= ".ProcessInput.StsReg01";
	props["dataType"] 		= CPC_UINT16;
	config["StsReg01"] 		= props;

	mappingClear(props);
	props["address"] 		= ".ProcessInput.StsReg02";
	props["dataType"] 		= CPC_UINT16;
	config["StsReg02"] 		= props;

	mappingClear(props);
	props["hasUnit"] 		= true;
	props["hasFormat"]		= true;
	props["hasPvRange"] 	= true;
	props["hasArchive"] 	= true;
	props["hasSmooth"] 		= true;
	props["address"] 		= ".ProcessInput.PosSt";
	props["dataType"] 		= CPC_FLOAT;
	config["PosSt"] 		= props;

	mappingClear(props);
	props["hasUnit"] 		= true;
	props["hasFormat"]		= true;
	props["hasArchive"] 	= true;
	props["hasSmooth"] 		= true;
	props["address"] 		= ".ProcessInput.PosRSt";
	props["dataType"] 		= CPC_FLOAT;
	config["PosRSt"] 		= props;

	mappingClear(props);
	props["hasUnit"] 		= true;
	props["hasFormat"]		= true;
	props["hasSmooth"] 		= true;
	props["address"] 		= ".ProcessInput.AuPosRSt";
	props["dataType"] 		= CPC_FLOAT;
	config["AuPosRSt"] 		= props;

	mappingClear(props);
	props["hasUnit"] 		= true;
	props["hasFormat"]		= true;
	props["hasSmooth"] 		= true;
	props["address"] 		= ".ProcessInput.MPosRSt";
	props["dataType"] 		= CPC_FLOAT;
	config["MPosRSt"] 		= props;

	mappingClear(props);
	props["address"] 		= ".ProcessInput.StartISt";
	props["dpe"]			= "StsReg01";
	props["bitPosition"]	= UN_STSREG01_STARTIST;
	props["isAlarm"]		= true;
	config["StartISt"] 		= props;

	mappingClear(props);
	props["address"] 		= ".ProcessInput.TStopISt";
	props["dpe"]			= "StsReg01";
	props["bitPosition"]	= CPC_STSREG01_TSTOPIST;
	props["isAlarm"]		= true;
	config["TStopISt"] 		= props;

	mappingClear(props);
	props["address"] 		= ".ProcessInput.FuStopISt";
	props["dpe"]			= "StsReg02";
	props["bitPosition"]	= CPC_STSREG02_FUSTOPIST;
	props["isAlarm"]		= true;
	config["FuStopISt"] 		= props;

  mappingClear(props);
  props["address"]      = ".ProcessInput.AlSt";
  props["dpe"]          = "StsReg01";
  props["bitPosition"]  = CPC_STSREG01_ALST;
  props["isAlarm"]      = true;
  config["AlSt"]        = props;

	mappingClear(props);
	props["hasArchive"] 	= true;
	props["address"] 		= ".ProcessInput.evStsReg01";
	props["dataType"] 		= CPC_INT32;
	config["evStsReg01"]	= props;

	mappingClear(props);
	props["hasArchive"] 	= true;
	props["address"] 		= ".ProcessInput.evStsReg02";
	props["dataType"] 		= CPC_INT32;
	config["evStsReg02"]	= props;

	mappingClear(props);
	props["address"] 		= ".ProcessOutput.ManReg01";
	props["dataType"] 		= CPC_UINT16;
	config["ManReg01"] 		= props;

	mappingClear(props);
	props["address"] 		= ".ProcessOutput.MPosR";
	props["dataType"] 		= CPC_FLOAT;
	config["MPosR"] 		= props;

	mappingClear(props);
	props["hasUnit"] 		= true;
	props["hasFormat"]		= true;
	props["address"] 		= ".ProcessOutput.PLiOn";
	props["dataType"] 		= CPC_FLOAT;
	config["PLiOn"] 		= props;

	mappingClear(props);
	props["hasUnit"] 		= true;
	props["hasFormat"]		= true;
	props["address"] 		= ".ProcessOutput.PLiOff";
	props["dataType"] 		= CPC_FLOAT;
	config["PLiOff"] 		= props;

	return config;
}

dyn_string CPC_AnaDigConfig_getParamNames() {
	return makeDynString(CPC_PARAMS_OUTMAIN, CPC_PARAMS_PWMMODE, CPC_PARAMS_FSPOSON, CPC_PARAMS_MRESTART, CPC_PARAMS_RSTARTFS, CPC_PARAMS_HLDRIVE);
}

//------------------------------------------------------------------------------------------------------------------------
// CPC_AnaDigConfig_checkConfig
/**
Check data before configuration

  @param[in]  configLine   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_AnaDigConfig_checkConfig(dyn_string configLine, dyn_string &exceptionInfo) {
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
		// 1. Common
		unConfigGenericFunctions_checkParameters(configLine, UN_CONFIG_CPC_ANADIG_DPT_NAME, exceptionInfo);
		// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, UN_CONFIG_CPC_ANADIG_DPT_NAME, exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","CPC_AnaDigConfig_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

_UnPlc_CPC_AnaDig_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_ANADIG_DPT_NAME, configLine, exceptionInfo);
}

S7_PLC_CPC_AnaDig_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_ANADIG_DPT_NAME, configLine, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// CPC_AnaDigConfig_checkCustomConfig
/**
Check custom data before configuration

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_AnaDigConfig_checkCustomConfig(dyn_string configLine, bool hasArchive, dyn_string &exceptionInfo) {
	unConfigGenericFunctions_checkRange(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_ANADIG_LIMITOFF], configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_ANADIG_LIMITON], exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// CPC_AnaDigConfig_setConfig
/**
Creates dp (or checks if it is already presented)and sets configuration for AnaDig object.

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_AnaDigConfig_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo) {
	string sDpName;

	// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, UN_CONFIG_CPC_ANADIG_DPT_NAME, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, UN_CONFIG_CPC_ANADIG_DPT_NAME, exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
			// 2. Generate device
			// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
			// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, UN_CONFIG_CPC_ANADIG_DPT_NAME, exceptionInfo);
		}
	}
}

_UnPlc_CPC_AnaDig_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

S7_PLC_CPC_AnaDig_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}
//------------------------------------------------------------------------------------------------------------------------
// CPC_AnaDigConfig_setConfig
/**
Set custom configuration of given AnaDig datapoint

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_AnaDigConfig_setCustomConfig(dyn_string dsConfigs, bool hasArchive, dyn_string &exceptionInfo) {
	dpSet(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.PLiOn", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_ANADIG_LIMITON]);
	dpSet(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessOutput.PLiOff", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_ANADIG_LIMITOFF]);
}

//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_CPC_AnaDig_ExportConfig
/**
Purpose: Export Anadig Devices for _UnPlc_CPC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
_UnPlc_CPC_AnaDig_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	string sObject = UN_CONFIG_CPC_ANADIG_DPT_NAME;
	int amount = dynlen(dsDpList);
	for(int i = 1; i <= amount; i++) {
		dyn_string dsDpParameters;
		string currentDP = dsDpList[i];
		string sLimitOn, sLimitOff;

		if(!dpExists(currentDP)) continue;

		unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
		cpcExportGenericFunctions_getUnit(currentDP, dsDpParameters);
		cpcExportGenericFunctions_getFormat(currentDP, dsDpParameters);
		cpcExportGenericFunctions_getRange(currentDP, dsDpParameters, exceptionInfo);
		dpGet(currentDP + ".ProcessOutput.PLiOn", sLimitOn);
		dynAppend(dsDpParameters, unExportDevice_floatFormat(sLimitOn));
		dpGet(currentDP + ".ProcessOutput.PLiOff", sLimitOff);
		dynAppend(dsDpParameters, unExportDevice_floatFormat(sLimitOff));
		cpcExportGenericFunctions_getDeadband(currentDP, dsDpParameters, exceptionInfo);
		cpcExportGenericFunctions_getArchive(currentDP, dsDpParameters);
		cpcExportGenericFunctions_getDigitalNormalPosition(currentDP, sObject, makeDynString(".ProcessInput.StartISt", ".ProcessInput.TStopISt", ".ProcessInput.FuStopISt"), dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg01", 	true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg02", 	true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "evStsReg01", 	true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "evStsReg02", 	true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "PosSt", 		true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "AuPosRSt", 	true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "MPosRSt", 	true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "PosRSt", 		true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "ManReg01", 	false, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "MPosR", 		false, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "PLiOn", 		false, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "PLiOff", 		false, dsDpParameters);
		cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString(), dsDpParameters);
		cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_2, makeDynString("ProcessInput.PosSt", "ProcessInput.PosRSt"), dsDpParameters);
		cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString("ProcessInput.evStsReg01", "ProcessInput.evStsReg02"), dsDpParameters);
		cpcExportGenericFunctions_getMaskEvent(currentDP, dsDpParameters);
		cpcExportGenericFunctions_getParameters(currentDP, dsDpParameters, exceptionInfo);
		cpcExportGenericFunctions_getMetainfo(currentDP, dsDpParameters, exceptionInfo);

		unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
	}
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_CPC_AnaDig_ExportConfig
/**
Purpose: Export Anadig Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
S7_PLC_CPC_AnaDig_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_CPC_AnaDig_ExportConfig(dsDpList, exceptionInfo);
}
