/**@name LIBRARY: unAnalog.ctl

@author: Vincent Forest (AB-CO)

Creation Date: 21 05 2003

Modification History:
    13/09/2012: Alexey
        [UCPC-931] add alarms item to the right click menu

    16/05/2012: Alexey
        [UCPC-787] introduce AlSt bit and letter constants

    29/02/2012: Alexey
        [UCPC-684] "allow restart" available for "operator"

    23/11/2011: Alexey
        [UCPC-603] refactor menu creation and handling

    30/09/2011: Alexey
        [UCPC-567] added interlock status to the widget animation cb

    20/09/2011: Alexey
        [UCPC-548][UCPC-547] upd faceplate
        [UCPC-552] animate HWLocal parameter

    13/09/2011: Alexey
        [UCPC-531] took off cpcGenericObject_WidgetAnimateManualRestart, updated cpcGenericObject_WidgetAnimationDoubleStsReg

	08/08/2011: Alexey
		fixed faceplate Disconnect animation

	27/07/2011: Alexey
		[UCPC-412] disabled "Manual restart needed" faceplate box when it's necessary

	05/07/2011: Alexey
		[UCPC-323] fixed ManRestart animation

	01/07/2011: Alexey
		use constant for params

	28/06/2011: Alexey
		[UCPC-311] animate Soft/Hard local mode in the faceplate and widget

	27/06/2011: Alexey
		[UCPC-328] animate interlock status in the new way
		[UCPC-323] animate NeedRestart box

	23/06/2011: Alexey
		[UCPC-206] animate active control object in the widget
		renamed cpcFaceplateButtonState_updateAccess -> cpcFaceplateButtonState_ButtonSetState

	22/06/2011: Alexey
		[UCPC-323] added AllowRestart button

	18/05/2011: Alexey
		use cpcGenericObject_HandleEventMaskMenu and CPC functions to manage menu
		use cpcGenericObject_WidgetAnimationDoubleStsReg instead of unGenericObject_WidgetAnimationDoubleStsReg

	2011/04/06 amerezhi
		[UCPC-206] renamed OutOVSt -> PosRSt
					splited StopISt to TStopISt and FuStopISt

  15/11/2010: Quique
  - [UCPC-154] Inhibit manual/forced mode must be shown in faceplate and widget (letter "h")
  
        28/04/2009: Quique
                - Include animation for Warning_IOSimulated. it was cutted from the last modification (bug)
				
  27/03/2009: Frederic
	  - ..._ButtonRegisterCB(): remove fwException_raise(...) before g_dsUserAccess
        
  12/10/2008: Quique
    - New animation for the stsReg02 bit 13 : Dependent Alarm blocked warning
    - Faceplate animation: Include in the callbacks the stsReg02
    - Widget animation : "B" warning letter low priority
                  
  28/12/2007: Herve
    - device acccess control
    
	29/06/2007: Herve
		- animate widget with Ctrl if any
		
	14/12/2006: Herve
		- new param in unGenericDpFunctions_checkDeviceType
		
	08/05/2006: Herve
		- Implementation of the R
	
	02/05/2006: Herve
		- implementation of maskEvent in widget, faceplate and rigthClick
		
	11/04/2006: Herve
		- remove dynAppend(menuList, "SEPARATOR"); in CPC_Analog_MenuConfiguration function

	14/12/2005: Celine
		- CPC_Analog_WidgetRegisterCB, CPC_Analog_FaceplateStatusRegisterCB, CPC_Analog_FaceplateStatusAnimationCB, CPC_Analog_WidgetAnimationCB: 
		add g_bSystemConnected
		
	09/08/2005: Frederic
		- CPC_Analog_ButtonRegisterCB(), add evalScript to run dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS] function
 
	15/04/2005: Herve
		- unGenericObject_GetPLCNameFromDpName: return the PLCtype+"_"+plcName, in case of _UnPlc it returns _unPlc_+plcName
		
	05/11/2004: Herve
		- in the WidgetRegisterCB, faceplateRegiterCB: use the systemName of the $sIdentifier to dpConnect to the PLC

	26/10/2004: Herve
		- in the CPC_Analog_ButtonRegisterCB function: call the CPC_Analog_ButtonUserAccess to get the list of allowed buttons. This list is 
		reseted if the user logs in/out while the remote system is off.

	22/10/2004: Herve
		- modify the unGenericObject_FormatValue, unGenericObject_WidgetDisplayValueAnimation, unGenericObject_DisplayValue call

	18/10/2004: Herve
		- add to CPC_Analog_MenuConfiguration the function call unGenericObject_addTrendActionToMenu

	03/05/2004: Herve
		- during WidgetRegister and FaceplateRegister call unGenericObject_GetPLCNameFromDpName to get the PLC DPName
		- during WidgetRegister check if the dpType is correct

	28/04/2004: Herve
		- add function CPC_Analog_ObjectListGetValueTime

	14/04/2004: Herve
		- add the function CPC_Analog_HandleMenu
		- add the function CPC_Analog_UserLoginGetButtonState

	13/04/2004: Herve
		- in function CPC_Analog_ButtonRegisterCB add the set Manreg01 to 0 in case of dpConnect.
		- modification of the signature CPC_Analog_MenuConfiguration and add the creation of the menu.

	08/04/2004: Herve 
		modification of the signature of the function CPC_Analog_ButtonSetState

	16/01/2004: Herve
		remove the dpConnect, dpDisconnect and callback function in the case of no PLC systemAlarm. Ease the development 
		and test.
		
	01/01/2004: Herve
		- add in all dpConnect and dpDisconnect the _unSystemAlarm_PLC_DS dp if the PLC is configured for systemIntegrity. 
		The g_bUnSystemAlarmPlc is set to true if the PLC is configured for systemIntegrity
		change the dpConnect and dpDisconnect of _aut_inv to _invalid

version 1.0

External Function : 
	. CPC_Analog_AcknowledgeAlarm : acknowledge Analog alarms
	. CPC_Analog_FaceplateStatusDisconnection : animate faceplate disconnection
	. CPC_Analog_ButtonUserAccess : User access for Analog buttons
	. CPC_Analog_ButtonSetState : set button state
	. CPC_Analog_WidgetDisconnection : animate widget disconnection
	. CPC_Analog_MenuConfiguration : configuration of popup menu

Internal Functions :
	. CPC_Analog_FaceplateStatusRegisterCB : register callback function
	. CPC_Analog_FaceplateStatusAnimationCB_NoRangeNoAlarm : animate faceplate
	. CPC_Analog_FaceplateStatusAnimationCB_NoAlarm : animate faceplate
	. CPC_Analog_FaceplateStatusAnimationCB_NoRange : animate faceplate
	. CPC_Analog_FaceplateStatusAnimationCB : animate faceplate
	. CPC_Analog_ButtonRegisterCB : register callback function
	. CPC_Analog_ButtonAnimationCB : animate buttons
	. CPC_Analog_ButtonDisconnection : animate buttons disconnection
	. CPC_Analog_WidgetRegisterCB : register callback function
	. CPC_Analog_WidgetAnimationCB : animate widget
		
Purpose: This library contains Analog object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Analog object (DPType etc.)
	. Global variables and $parameters defined in Analog faceplate and widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/


//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Analog_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Analog_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	float value;
	string sTime;
	
	dpGet(sDeviceName+".ProcessInput.PosSt", value, sDeviceName+".ProcessInput.PosSt:_online.._stime", sTime);
	dsReturnData[1] = sTime;
	dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.PosSt"), value) + " " 
							+ dpGetUnit(sDeviceName+".ProcessInput.PosSt");
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Analog_AcknowledgeAlarm
/**
Purpose: acknowledge Analog alarms

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsNeedAck, dyn_string, output, DPEs names that need acknowledgement. 
									 The PLC acknowledgement is specified using UN_ACKNOWLEDGE_PLC string
	
Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Analog faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Analog_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
	dsNeedAck = makeDynString("ProcessInput.StartISt", "ProcessInput.TStopISt", "ProcessInput.FuStopISt", "ProcessInput.AlSt", UN_ACKNOWLEDGE_PLC);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Analog_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsAccessOk, dyn_string, input, authorized action
	- menuList, dyn_string, output, menu to pop-up
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Analog_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
	dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_ACK_TEXT, UN_POPUPMENU_FACEPLATE_TEXT, CPC_POPUPMENU_ALARMS_TEXT, UN_POPUPMENU_TREND_TEXT);

	cpcGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	cpcGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	cpcGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Analog_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_Analog_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
	dyn_string exceptionInfo;

	cpcGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);

	if (dynlen(exceptionInfo) > 0) {
		unSendMessage_toExpertException("Right Click",exceptionInfo);
	}

}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Analog_FaceplateStatusRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Analog faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Analog_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName,sSystemName;
	int iAction;
	dyn_string exceptionInfo;
	bool bRemote;
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			CPC_Analog_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
			CPC_Analog_FaceplateConnect(deviceName);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			CPC_Analog_FaceplateDisconnect(deviceName);
			CPC_Analog_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Analog_FaceplateConnect
/**
Purpose: connect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Analog faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_Analog_FaceplateConnect(string deviceName)
{
	int iAction, iRes, iAlertType1, iAlertType2, iAlertType3, iAlertType4, iRangeType;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;

// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;

	iRes = dpGet(deviceName + ".ProcessInput.StartISt:_alert_hdl.._type", iAlertType1,
						 deviceName + ".ProcessInput.TStopISt:_alert_hdl.._type", iAlertType2,
						 deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._type", iAlertType3,
						 deviceName + ".ProcessInput.AlSt:_alert_hdl.._type", iAlertType4);
	g_bBoolAlert = ((iRes >= 0) && (iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType2 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType3 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType4 == DPCONFIG_ALERT_BINARYSIGNAL));
	iRes = dpGet(deviceName + ".ProcessInput.PosSt:_pv_range.._type", iRangeType);
	if ((iRes >= 0) && (iRangeType == DPCONFIG_MINMAX_PVSS_RANGECHECK))
		{
		g_iRangeType = DPCONFIG_MINMAX_PVSS_RANGECHECK;
		}
	else
		{
		g_iRangeType = DPCONFIG_NONE;
		}

	g_sPosStFormat=dpGetFormat(deviceName + ".ProcessInput.PosSt");
	g_sPosStUnit=dpGetUnit(deviceName + ".ProcessInput.PosSt");
	g_sAuPosRStFormat=dpGetFormat(deviceName + ".ProcessInput.AuPosRSt");
	g_sAuPosRStUnit=dpGetUnit(deviceName + ".ProcessInput.AuPosRSt");
	g_sPosRStFormat=dpGetFormat(deviceName + ".ProcessInput.PosRSt");
	g_sPosRStUnit=dpGetUnit(deviceName + ".ProcessInput.PosRSt");
	g_sMPosRStFormat=dpGetFormat(deviceName + ".ProcessInput.MPosRSt");
	g_sMPosRStUnit=dpGetUnit(deviceName + ".ProcessInput.MPosRSt");
	g_sPLiOnFormat=dpGetFormat(deviceName + ".ProcessOutput.PLiOn");
	g_sPLiOnUnit=dpGetUnit(deviceName + ".ProcessOutput.PLiOn");
	g_sPLiOffFormat=dpGetFormat(deviceName + ".ProcessOutput.PLiOff");
	g_sPLiOffUnit=dpGetUnit(deviceName + ".ProcessOutput.PLiOff");
	g_bPFSPosOn = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_FSPOSON, exceptionInfo);
	g_bMRestart = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_MRESTART, exceptionInfo);
    g_bHLDrive = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_HLDRIVE, exceptionInfo);

	if(g_bUnSystemAlarmPlc) {
		if ((g_bBoolAlert) && (g_iRangeType == DPCONFIG_MINMAX_PVSS_RANGECHECK))
			{
			iRes = dpConnect("CPC_Analog_FaceplateStatusAnimationCB",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							 deviceName + ".ProcessInput.StsReg02", 
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt",
							 deviceName + ".ProcessInput.PosSt:_online.._invalid",
							 deviceName + ".ProcessInput.AuPosRSt",
							 deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.PosRSt",
							 deviceName + ".ProcessInput.PosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.MPosRSt",
							 deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOn",
							 deviceName + ".ProcessOutput.PLiOn:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOff",
							 deviceName + ".ProcessOutput.PLiOff:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt:_pv_range.._min",
							 deviceName + ".ProcessInput.PosSt:_pv_range.._max",
							 deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");
			}
		if ((!g_bBoolAlert) && (g_iRangeType == DPCONFIG_MINMAX_PVSS_RANGECHECK))
			{
			iRes = dpConnect("CPC_Analog_FaceplateStatusAnimationCB_NoAlarm",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							 deviceName + ".ProcessInput.StsReg02", 
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt",
							 deviceName + ".ProcessInput.PosSt:_online.._invalid",
							 deviceName + ".ProcessInput.AuPosRSt",
							 deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.PosRSt",
							 deviceName + ".ProcessInput.PosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.MPosRSt",
							 deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOn",
							 deviceName + ".ProcessOutput.PLiOn:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOff",
							 deviceName + ".ProcessOutput.PLiOff:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt:_pv_range.._min",
							 deviceName + ".ProcessInput.PosSt:_pv_range.._max",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");
			}	
		if ((g_bBoolAlert) && (g_iRangeType != DPCONFIG_MINMAX_PVSS_RANGECHECK))
			{
			iRes = dpConnect("CPC_Analog_FaceplateStatusAnimationCB_NoRange",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							 deviceName + ".ProcessInput.StsReg02", 
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt",
							 deviceName + ".ProcessInput.PosSt:_online.._invalid",
							 deviceName + ".ProcessInput.AuPosRSt",
							 deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.PosRSt",
							 deviceName + ".ProcessInput.PosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.MPosRSt",
							 deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOn",
							 deviceName + ".ProcessOutput.PLiOn:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOff",
							 deviceName + ".ProcessOutput.PLiOff:_online.._invalid",
							 deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");				
			}
		if ((!g_bBoolAlert) && (g_iRangeType != DPCONFIG_MINMAX_PVSS_RANGECHECK))
			{
			iRes = dpConnect("CPC_Analog_FaceplateStatusAnimationCB_NoRangeNoAlarm",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							 deviceName + ".ProcessInput.StsReg02", 
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt",
							 deviceName + ".ProcessInput.PosSt:_online.._invalid",
							 deviceName + ".ProcessInput.AuPosRSt",
							 deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.PosRSt",
							 deviceName + ".ProcessInput.PosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.MPosRSt",
							 deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOn",
							 deviceName + ".ProcessOutput.PLiOn:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOff",
							 deviceName + ".ProcessOutput.PLiOff:_online.._invalid",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");				
			}
	}
	else
		CPC_Analog_FaceplateStatusDisconnection();
	g_bCallbackConnected = (iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Analog_FaceplateDisconnect
/**
Purpose: disconnect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Analog faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_Analog_FaceplateDisconnect(string deviceName)
{
	int iRes;
	string sPlcName, sSystemName;

	if(g_bUnSystemAlarmPlc) {
// get the systemName
		sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
		sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);

		if ((g_bBoolAlert) && (g_iRangeType == DPCONFIG_MINMAX_PVSS_RANGECHECK))
			{
			iRes = dpDisconnect("CPC_Analog_FaceplateStatusAnimationCB",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                         deviceName + ".ProcessInput.StsReg02", 
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt",
							 deviceName + ".ProcessInput.PosSt:_online.._invalid",
							 deviceName + ".ProcessInput.AuPosRSt",
							 deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.PosRSt",
							 deviceName + ".ProcessInput.PosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.MPosRSt",
							 deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOn",
							 deviceName + ".ProcessOutput.PLiOn:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOff",
							 deviceName + ".ProcessOutput.PLiOff:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt:_pv_range.._min",
							 deviceName + ".ProcessInput.PosSt:_pv_range.._max",
							 deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");
			}
		if ((!g_bBoolAlert) && (g_iRangeType == DPCONFIG_MINMAX_PVSS_RANGECHECK))
			{
			iRes = dpDisconnect("CPC_Analog_FaceplateStatusAnimationCB_NoAlarm",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                         deviceName + ".ProcessInput.StsReg02", 
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt",
							 deviceName + ".ProcessInput.PosSt:_online.._invalid",
							 deviceName + ".ProcessInput.AuPosRSt",
							 deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.PosRSt",
							 deviceName + ".ProcessInput.PosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.MPosRSt",
							 deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOn",
							 deviceName + ".ProcessOutput.PLiOn:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOff",
							 deviceName + ".ProcessOutput.PLiOff:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt:_pv_range.._min",
							 deviceName + ".ProcessInput.PosSt:_pv_range.._max",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");
			}	
		if ((g_bBoolAlert) && (g_iRangeType != DPCONFIG_MINMAX_PVSS_RANGECHECK))
			{
			iRes = dpDisconnect("CPC_Analog_FaceplateStatusAnimationCB_NoRange",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                         deviceName + ".ProcessInput.StsReg02", 
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt",
							 deviceName + ".ProcessInput.PosSt:_online.._invalid",
							 deviceName + ".ProcessInput.AuPosRSt",
							 deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.PosRSt",
							 deviceName + ".ProcessInput.PosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.MPosRSt",
							 deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOn",
							 deviceName + ".ProcessOutput.PLiOn:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOff",
							 deviceName + ".ProcessOutput.PLiOff:_online.._invalid",
							 deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");				
			}
		if ((!g_bBoolAlert) && (g_iRangeType != DPCONFIG_MINMAX_PVSS_RANGECHECK))
			{
			iRes = dpDisconnect("CPC_Analog_FaceplateStatusAnimationCB_NoRangeNoAlarm",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                         deviceName + ".ProcessInput.StsReg02", 
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt",
							 deviceName + ".ProcessInput.PosSt:_online.._invalid",
							 deviceName + ".ProcessInput.AuPosRSt",
							 deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.PosRSt",
							 deviceName + ".ProcessInput.PosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.MPosRSt",
							 deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOn",
							 deviceName + ".ProcessOutput.PLiOn:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOff",
							 deviceName + ".ProcessOutput.PLiOff:_online.._invalid",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");				
			}
	}
	g_bCallbackConnected = !(iRes >= 0);
}
//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Analog_FaceplateStatusAnimationCB_NoRangeNoAlarm
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Analog_FaceplateStatusAnimationCB_NoRangeNoAlarm(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
                                                   string sStsReg02, bit32 bit32StsReg02, string sDp3, bool bStsReg02Invalid,
						   string sDpPosSt, float fPosSt, string sDp4, bool bPosStInvalid, 
				 		   string sDpAuto, float fAuPosRSt, string sDp6, bool bAuPosRStInvalid,
				 		   string sDpOutput, float fPosRSt, string sDp8, bool bPosRStInvalid,
				 		   string sDpMPosRSt, float fMPosRSt, string sDp10, bool bMPosRStInvalid,
				 		   string sDpLimitOn, float fPLiOn, string sDp12, bool bPLiOnInvalid,
				 		   string sDpLimitOff, float fPLiOff, string sDp14, bool bPLiOffInvalid,
				 		   string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled,
				 		   string sDpEvt, int iMask)
{
	CPC_Analog_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid,
                                            sStsReg02, bit32StsReg02, sDp3, bStsReg02Invalid,  
									    sDpPosSt, fPosSt, sDp4, bPosStInvalid,
					 			        sDpAuto, fAuPosRSt, sDp6, bAuPosRStInvalid,
					 			        sDpOutput, fPosRSt, sDp8, bPosRStInvalid,
					 			        sDpMPosRSt, fMPosRSt, sDp10, bMPosRStInvalid,
					 			        sDpLimitOn, fPLiOn, sDp12, bPLiOnInvalid,
					 			        sDpLimitOff, fPLiOff, sDp14, bPLiOffInvalid,
					 			        "", 0, "", 0, "", 0, "", 0,
					 			        sDpAl, iValAlarm, sDpEnabled, bPlcEnabled, sDpEvt, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Analog_FaceplateStatusAnimationCB_NoRange
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Analog_FaceplateStatusAnimationCB_NoRange(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
                                            string sStsReg02, bit32 bit32StsReg02, string sDp3, bool bStsReg02Invalid,
					    string sDpPosSt, float fPosSt, string sDp4, bool bPosStInvalid, 
				 	    string sDpAuto, float fAuPosRSt, string sDp6, bool bAuPosRStInvalid,
				 	    string sDpOutput, float fPosRSt, string sDp8, bool bPosRStInvalid,
				 	    string sDpMPosRSt, float fMPosRSt, string sDp10, bool bMPosRStInvalid,
				 	    string sDpLimitOn, float fPLiOn, string sDp12, bool bPLiOnInvalid,
				 	    string sDpLimitOff, float fPLiOff, string sDp14, bool bPLiOffInvalid,
					    string sDp17, int iStartIStState, string sDp18, int iTStopIStState, string sDp19, int iFuStopIStState, string sDp20, int iAlSt,
				 	    string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled,
				 	    string sDpEvt, int iMask)
{
	CPC_Analog_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, 
                                            sStsReg02, bit32StsReg02, sDp3, bStsReg02Invalid, 
									    sDpPosSt, fPosSt, sDp4, bPosStInvalid,
					 			        sDpAuto, fAuPosRSt, sDp6, bAuPosRStInvalid,
					 			        sDpOutput, fPosRSt, sDp8, bPosRStInvalid,
					 			        sDpMPosRSt, fMPosRSt, sDp10, bMPosRStInvalid,
					 			        sDpLimitOn, fPLiOn, sDp12, bPLiOnInvalid,
					 			        sDpLimitOff, fPLiOff, sDp14, bPLiOffInvalid,
					 			        "", 0, "", 0, sDp17, iStartIStState, sDp18, iTStopIStState, sDp19, iFuStopIStState, sDp20, iAlSt,
					 			        sDpAl, iValAlarm, sDpEnabled, bPlcEnabled, sDpEvt, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Analog_FaceplateStatusAnimationCB_NoAlarm
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Analog_FaceplateStatusAnimationCB_NoAlarm(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
                                            string sStsReg02, bit32 bit32StsReg02, string sDp3, bool bStsReg02Invalid,
									string sDpPosSt, float fPosSt, string sDp4, bool bPosStInvalid, 
				 			        string sDpAuto, float fAuPosRSt, string sDp6, bool bAuPosRStInvalid,
				 			        string sDpOutput, float fPosRSt, string sDp8, bool bPosRStInvalid,
				 			        string sDpMPosRSt, float fMPosRSt, string sDp10, bool bMPosRStInvalid,
				 			        string sDpLimitOn, float fPLiOn, string sDp12, bool bPLiOnInvalid,
				 			        string sDpLimitOff, float fPLiOff, string sDp14, bool bPLiOffInvalid,
									string sDp15, float minPosSt, string sDp16, float maxPosSt,
				 			        string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled,
				 			        string sDpEvt, int iMask)
{
	CPC_Analog_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid,
                                            sStsReg02, bit32StsReg02, sDp3, bStsReg02Invalid, 
									    sDpPosSt, fPosSt, sDp4, bPosStInvalid,
					 			        sDpAuto, fAuPosRSt, sDp6, bAuPosRStInvalid,
					 			        sDpOutput, fPosRSt, sDp8, bPosRStInvalid,
					 			        sDpMPosRSt, fMPosRSt, sDp10, bMPosRStInvalid,
					 			        sDpLimitOn, fPLiOn, sDp12, bPLiOnInvalid,
					 			        sDpLimitOff, fPLiOff, sDp14, bPLiOffInvalid,
					 			        sDp15, minPosSt, sDp16, maxPosSt, "", 0, "", 0, "", 0,
					 			        sDpAl, iValAlarm, sDpEnabled, bPlcEnabled, sDpEvt, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Analog_FaceplateStatusAnimationCB
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Analog_FaceplateStatusAnimationCB(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
									string sStsReg02, bit32 bit32StsReg02, string sDp3, bool bStsReg02Invalid,
									string sDpPosSt, float fPosSt, string sDp4, bool bPosStInvalid, 
									string sDpAuto, float fAuPosRSt, string sDp6, bool bAuPosRStInvalid,
									string sDpOutput, float fPosRSt, string sDp8, bool bPosRStInvalid,
									string sDpMPosRSt, float fMPosRSt, string sDp10, bool bMPosRStInvalid,
									string sDpLimitOn, float fPLiOn, string sDp12, bool bPLiOnInvalid,
									string sDpLimitOff, float fPLiOff, string sDp14, bool bPLiOffInvalid,
									string sDp15, float minPosSt, string sDp16, float maxPosSt,
									string sDp17, int iStartIStState, string sDp18, int iTStopIStState, string sDp19, int iFuStopIStState, string sDp20, int iAlSt,
									string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
									string sDpEvt, int iMask)
{
if(g_bSystemConnected)
	unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fPosSt, "StatusPosition", "cpcColor_Faceplate_Status", 
				bPosStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_DisplayValue(g_sAuPosRStFormat, g_sAuPosRStUnit, fAuPosRSt, "RequestAuto", "cpcColor_Faceplate_Request", 
				bAuPosRStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_DisplayValue(g_sPosRStFormat, g_sPosRStUnit, fPosRSt, "RequestOutput", "cpcColor_Faceplate_Request", 
				bPosRStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_DisplayValue(g_sMPosRStFormat, g_sMPosRStUnit, fMPosRSt, "RequestManual", "cpcColor_Faceplate_Request", 
				bMPosRStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_DisplayValue(g_sPLiOnFormat, g_sPLiOnUnit, fPLiOn, "StatusLimitOn", "unDisplayValue_Parameter", 
				bPLiOnInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_DisplayValue(g_sPLiOffFormat, g_sPLiOffUnit, fPLiOff, "StatusLimitOff", "unDisplayValue_Parameter", 
				bPLiOffInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_DisplayRange (g_iRangeType, g_sPosStFormat, g_sPosStUnit, minPosSt, maxPosSt, "RangeMin", "RangeMax", "unDisplayValue_Parameter");
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Auto", getBit(bit32StsReg01, UN_STSREG01_AUMOST), 1, "cpcColor_Faceplate_Status", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Manual", getBit(bit32StsReg01, UN_STSREG01_MMOST), 1, "cpcColor_Faceplate_Status", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Status_AuInhMan", getBit(bit32StsReg01, UN_STSREG01_AUINHMMO), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Forced", getBit(bit32StsReg01, UN_STSREG01_FOMOST), 1, "cpcColor_Faceplate_Status", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Status_AuInhFor", getBit(bit32StsReg01, UN_STSREG01_AUINHFMO), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Local", getBit(bit32StsReg01, UN_STSREG01_LDST), g_bHLDrive, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_SoftLocal", getBit(bit32StsReg02, CPC_STSREG02_SOFTLDST), 1, "cpcColor_Faceplate_Status", 
			bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_IOError", getBit(bit32StsReg01, UN_STSREG01_IOERRORW), 1, "unFaceplate_WarningActive", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
     unGenericObject_ColorBoxAnimate("Warning_IOSimulated", getBit(bit32StsReg01,UN_STSREG01_IOSIMUW), 1, "unFaceplate_WarningActive",
                            bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_AlBW", getBit(bit32StsReg02, UN_STSREG02_ALBW), 1, "unFaceplate_WarningActive", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_ManualAuto", getBit(bit32StsReg01, UN_STSREG01_AUMRW), 1, "unFaceplate_WarningActive", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_PositionWarning", getBit(bit32StsReg01, UN_STSREG01_POSALW), 1, "unFaceplate_WarningActive", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
    cpcGenericObject_CheckboxAnimate("Status_Failsafe", g_bPFSPosOn);
	cpcGenericObject_ColorBoxAnimateInterlock("Alarm_StartISt", iStartIStState, getBit(bit32StsReg01, UN_STSREG01_STARTIST), getBit(bit32StsReg01, UN_STSREG01_ALUNACK),
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	cpcGenericObject_ColorBoxAnimateInterlock("Alarm_TStopISt", iTStopIStState, getBit(bit32StsReg01, CPC_STSREG01_TSTOPIST), getBit(bit32StsReg01, UN_STSREG01_ALUNACK),
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	cpcGenericObject_ColorBoxAnimateInterlock("Alarm_FuStopISt", iFuStopIStState, getBit(bit32StsReg02, CPC_STSREG02_FUSTOPIST), getBit(bit32StsReg01, UN_STSREG01_ALUNACK),
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	cpcGenericObject_ColorBoxAnimateInterlock("Alarm_AlSt", iAlSt, getBit(bit32StsReg01, CPC_STSREG01_ALST), getBit(bit32StsReg01, UN_STSREG01_ALUNACK),
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Alarm_UnAck", getBit(bit32StsReg01, UN_STSREG01_ALUNACK), 1, "cpcColor_Alarm_Bad", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));

	if (g_bMRestart) {
		if(g_bSystemConnected)
			unGenericObject_ColorBoxAnimate("NeedRestart", getBit(bit32StsReg02, CPC_STSREG02_NEEDRESTART) == 0, 1, "unFaceplate_WarningActive",
				bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	} else {
        cpcGenericObject_ColorBoxDisable("NeedRestart");
	}
    if(g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("eventState", iMask == 0, 1, "unFaceplate_WarningActive", 
                bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Analog_FaceplateStatusDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Analog_FaceplateStatusDisconnection()
{
    unGenericObject_ColoredSquareDisconnect(makeDynString("Alarm_StartISt", "Alarm_TStopISt", "Alarm_FuStopISt", "Alarm_AlSt"));
    unGenericObject_ColorBoxDisconnect(makeDynString("Alarm_UnAck", "OperationMode_Auto",
                                                     "OperationMode_Forced", "OperationMode_Local", "OperationMode_SoftLocal", "OperationMode_Manual",
                                                     "Status_AuInhFor", "Status_AuInhMan",
                                                     "Warning_IOError", "Warning_IOSimulated", "Warning_ManualAuto", "Warning_AlBW", "Warning_PositionWarning", "NeedRestart", 
                                                     "eventState"));
    unGenericObject_DisplayValueDisconnect(makeDynString("RangeMax", "RangeMin", "RequestAuto", "RequestManual", "RequestOutput",
                                                         "StatusLimitOff", "StatusLimitOn", "StatusPosition"));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Analog_ButtonRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Analog faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Analog_ButtonRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sDpType, sFunction;
	int iAction, iRes, iManReg01;
	dyn_string exceptionInfo, dsUserAccess, dsFunctions;
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			CPC_Analog_ButtonDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
// Send 0 to ManReg01 if necessary
			iRes = dpGet(deviceName + ".ProcessOutput.ManReg01", iManReg01);
			if ((iRes < 0) || (iManReg01 != 0))
			{
				dpSet(deviceName + ".ProcessOutput.ManReg01", 0);
			}
			
// call the CPC_Analog_ButtonUserAccess to get the list of allowed buttons.
//			CPC_Analog_ButtonUserAccess(deviceName, UN_CONFIG_CPC_ANALOG_DPT_NAME, g_dsUserAccess);

			sDpType=UN_CONFIG_CPC_ANALOG_DPT_NAME;
			unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
			sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
						 
			if (sFunction != "")
				{
				evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" + 
							 "dyn_string dsUserAccessReturn;" +
							 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
							 "    {" +
					 			sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
							 "    }" +
							 "return dsUserAccessReturn; }", makeDynString(), deviceName, sDpType);
						
				g_dsUserAccess=dsUserAccess;	
				}
			else
				fwException_raise(exceptionInfo,"ERROR", "CPC_Analog_ButtonRegisterCB: " + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");
					
			
			iRes = dpConnect("CPC_Analog_ButtonAnimationCB",
							 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 deviceName + ".statusInformation.selectedManager",
							 deviceName + ".ProcessInput.StsReg01",
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                         deviceName + ".ProcessInput.StsReg02",
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid");
			g_bCallbackConnected = (iRes >= 0);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			iRes = dpDisconnect("CPC_Analog_ButtonAnimationCB",
							 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 deviceName + ".statusInformation.selectedManager",
							 deviceName + ".ProcessInput.StsReg01",
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                         deviceName + ".ProcessInput.StsReg02",
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid");
			g_bCallbackConnected = !(iRes >= 0);
			CPC_Analog_ButtonDisconnection();
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Analog_ButtonAnimationCB
/**
Purpose: animate buttons

Parameters: 
	- dbPermissions, dyn_bool, input, list of permissions, length = 4, order : 1=monitor, 2=operator, 3=expert, 4=admin
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Analog_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager, 
							string sDpStsReg01, bit32 stsReg01Value, string sDpStsReg01Bad, bool bStsReg01Bad,
							string sDpStsReg02, bit32 stsReg02Value, string sDpStsReg02Bad, bool bStsReg02Bad)
{
	CPC_Analog_ButtonSetState(unGenericDpFunctions_getDpName(sDpStsReg01), UN_CONFIG_CPC_ANALOG_DPT_NAME, g_dsUserAccess, 
						 	makeDynString(bLocked, sSelectedManager, stsReg01Value, bStsReg01Bad, stsReg02Value, bStsReg02Bad));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Analog_ButtonUserAccess
/**
Purpose: User access for Analog buttons

Parameters: 
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Analog_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	bool operator, expert, admin;
	int i, length;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
        string sOperAction, sExpAction, sAdminAction;
        bool bActionDefined = false;

        unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
        if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
           (sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
          bActionDefined=true;
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	if (operator)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
  	    dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE, UN_FACEPLATE_BUTTON_SET_VALUE, 
							UN_FACEPLATE_BUTTON_ONOPEN_REQUEST, UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST, UN_FACEPLATE_BUTTON_ACK_ALARM,
							UN_FACEPLATE_BUTTON_INC_VALUE, UN_FACEPLATE_BUTTON_DEC_VALUE, UN_FACEPLATE_BUTTON_SET_STATUS_LIMITS,
							CPC_FACEPLATE_BUTTON_ALLOW_RESTART, UN_FACEPLATE_BUTTON_SELECT));
	}
	if (expert)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
	    dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_MASKEVENT));
	}
        if(admin){
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
        }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Analog_UserLoginGetButtonState
/**
Purpose: function called before the button set state when a user logs in. 

Parameters:
	deviceName: string, input, the device name
	dsReturnData: dyn_string, output, the data are returned here

Usage: Internal function

PVSS manager usage:  Ctrl, NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
		-> $sDpName, string, the device DP name
		-> g_sFaceplateButtonType, string, faceplate type
	. PVSS version: 2.12.1 
	. operating system: NT, W2000 and Linux, but tested only under W2000.
	. distributed system: yes but not tested.
*/

CPC_Analog_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
	unGenericObject_UserCBGetButtonState(deviceName, sType, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Analog_ButtonSetState
/**
Purpose: set button state

Parameters: 
	- deviceName, string, input, device
	- sDpType, string, input, dptype
	- dsUserAccess, dyn_string, input, user access
	- dsData, dyn_string, parameter
			
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Analog_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected, buttonEnabled, bit1, bit2, bit3;
	dyn_string dsButtons;
	int i, length;
	string localManager;
	bool bLocked, bStsReg01Bad, bStsReg02Bad;
	string sSelectedManager;
	bit32 stsReg01Value, stsReg02Value; 

	bLocked = dsData[1];
	sSelectedManager = dsData[2];
	stsReg01Value = dsData[3];
	bStsReg01Bad = dsData[4];
	stsReg02Value = dsData[5];
	bStsReg02Bad = dsData[6];

//DebugN("Analog_SetState", bLocked, sSelectedManager, stsReg01Value, bStsReg01Bad, stsReg02Value, bStsReg02Bad);

	dsButtons = makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE, UN_FACEPLATE_BUTTON_SET_VALUE, 
							  UN_FACEPLATE_BUTTON_ONOPEN_REQUEST, UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST, UN_FACEPLATE_BUTTON_ACK_ALARM,
							  UN_FACEPLATE_BUTTON_INC_VALUE, UN_FACEPLATE_BUTTON_DEC_VALUE, UN_FACEPLATE_BUTTON_SET_STATUS_LIMITS,
							  UN_FACEPLATE_BUTTON_FORCED_MODE); // Each button except "Select" button
	length = dynlen(dsButtons);
// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");
// 2. Button state
	for(i=1;i<=length;i++)
		{
		buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);	// User access
		switch (dsButtons[i])
			{
			case UN_FACEPLATE_BUTTON_FORCED_MODE:
				buttonEnabled = buttonEnabled && bSelected && 
					!getBit(stsReg01Value, UN_STSREG01_FOMOST) &&
					!getBit(stsReg01Value, UN_STSREG01_AUINHFMO) &&
					!getBit(stsReg01Value, UN_STSREG01_LDST) &&
					!bStsReg01Bad && !bStsReg02Bad;
				break;
			case UN_FACEPLATE_BUTTON_MANUAL_MODE:
				buttonEnabled = buttonEnabled && bSelected &&
					!getBit(stsReg01Value, UN_STSREG01_MMOST) &&
					!getBit(stsReg01Value, UN_STSREG01_AUINHMMO) &&
					!getBit(stsReg01Value, UN_STSREG01_LDST) &&
					!getBit(stsReg02Value, CPC_STSREG02_SOFTLDST) &&
					!bStsReg01Bad && !bStsReg02Bad;
				break;
			case UN_FACEPLATE_BUTTON_INC_VALUE:
			case UN_FACEPLATE_BUTTON_DEC_VALUE:
			case UN_FACEPLATE_BUTTON_ONOPEN_REQUEST:
			case UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST:
			case UN_FACEPLATE_BUTTON_AUTO_MODE:
			case UN_FACEPLATE_BUTTON_SET_VALUE:
				buttonEnabled = buttonEnabled && bSelected &&
					!getBit(stsReg01Value, UN_STSREG01_AUMOST) &&
					!getBit(stsReg01Value, UN_STSREG01_LDST) &&
					!getBit(stsReg02Value, CPC_STSREG02_SOFTLDST) &&
					!bStsReg01Bad && !bStsReg02Bad;
				break;
			case UN_FACEPLATE_BUTTON_SET_STATUS_LIMITS:
				buttonEnabled = buttonEnabled && bSelected &&
					!getBit(stsReg01Value, UN_STSREG01_LDST) &&
					!getBit(stsReg02Value, CPC_STSREG02_SOFTLDST) &&
					!bStsReg01Bad && !bStsReg02Bad;
				break;
			case UN_FACEPLATE_BUTTON_ACK_ALARM:
				buttonEnabled = buttonEnabled &&
					!getBit(stsReg01Value, UN_STSREG01_LDST) &&
					!getBit(stsReg02Value, CPC_STSREG02_SOFTLDST) &&
					!bStsReg01Bad && !bStsReg02Bad;
			default:
				break;
			}
		setValue(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", buttonEnabled);
		}
	dyn_string exceptionInfo;
	cpcFaceplateButtonState_ButtonSetState(makeDynString(CPC_FACEPLATE_BUTTON_ALLOW_RESTART), dsUserAccess, deviceName, sDpType, dsData, exceptionInfo);
// 3. Button Select state
	unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Analog_ButtonDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Analog_ButtonDisconnection()
{
	unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE, 
												   UN_FACEPLATE_BUTTON_SET_VALUE, UN_FACEPLATE_BUTTON_ONOPEN_REQUEST, 
												   UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST, UN_FACEPLATE_BUTTON_ACK_ALARM,
												   UN_FACEPLATE_BUTTON_INC_VALUE, UN_FACEPLATE_BUTTON_DEC_VALUE, 
												   UN_FACEPLATE_BUTTON_SET_STATUS_LIMITS, UN_FACEPLATE_BUTTON_FORCED_MODE,
												   UN_FACEPLATE_BUTTON_SELECT));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Analog_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Analog widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Analog_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName;
	int iAction;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;
	bool bRemote;
	
	deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
	if (deviceName == "") {// In case of disconnection
		deviceName = g_sDpName;
	}

	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	if(sPlcName != "") {
		g_bUnSystemAlarmPlc = dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName);
	} else {
		g_bUnSystemAlarmPlc = false;
	}

	unDistributedControl_isRemote(bRemote, sSystemName);
	g_bSystemConnected = bRemote ? bSystemConnected : true;

	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			CPC_Analog_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = deviceName;
			CPC_Analog_WidgetConnect(deviceName, sPlcName, sSystemName);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			CPC_Analog_WidgetDisconnect(deviceName, sPlcName, sSystemName);
			CPC_Analog_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Analog_WidgetConnect
/**
Purpose: Connect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_Analog_WidgetConnect(string deviceName, string sPlcName, string sSystemName)
{
	int iRes, iCtrlRes, iAlertType1, iAlertType2, iAlertType3, iAlertType4;
	string sDeviceLink;
	dyn_string exceptionInfo;

//  check if the device name is of type UN_CONFIG_CPC_ANALOG_DPT_NAME if not like of no PLC. -> not available.
	if(dpTypeName(deviceName) != UN_CONFIG_CPC_ANALOG_DPT_NAME) {
		g_bUnSystemAlarmPlc = false;
	}

	RegulationText1.visible = false;
	RegulationText2.visible = false;
	g_bMRestart = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_MRESTART, exceptionInfo);
    iRes = dpGet(deviceName + ".ProcessInput.StartISt:_alert_hdl.._type", iAlertType1,
                 deviceName + ".ProcessInput.TStopISt:_alert_hdl.._type", iAlertType2,
                 deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._type", iAlertType3,
                 deviceName + ".ProcessInput.AlSt:_alert_hdl.._type", iAlertType4);
    g_bBoolAlert = ((iRes >= 0) && (iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType2 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType3 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType4 == DPCONFIG_ALERT_BINARYSIGNAL));

	if(g_bUnSystemAlarmPlc) {
		g_sPosStFormat = dpGetFormat(deviceName + ".ProcessInput.PosSt");
		g_sPosRStFormat = dpGetFormat(deviceName + ".ProcessInput.PosRSt");
		unGenericDpFunctions_getDeviceLink(deviceName, sDeviceLink);
		g_dsCtrl = cpcGenericObject_getCtrlLink(sSystemName, strsplit(sDeviceLink, UN_CONFIG_GROUP_SEPARATOR));

        if (g_bBoolAlert) {
            iRes = dpConnect("CPC_Analog_WidgetAnimationCB", 
                            deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                            deviceName + ".statusInformation.selectedManager",
                            deviceName + ".ProcessInput.StsReg01",
                            deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                            deviceName + ".ProcessInput.StsReg02",
                            deviceName + ".ProcessInput.StsReg02:_online.._invalid",
                            deviceName + ".ProcessInput.PosSt",
                            deviceName + ".ProcessInput.PosSt:_online.._invalid",
                            deviceName + ".ProcessInput.PosRSt",
                            deviceName + ".ProcessInput.PosRSt:_online.._invalid",
                            deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
                            deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
                            deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
                            deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state",
                            deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state",
                            deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
                            deviceName + ".eventMask");
        } else {
            iRes = dpConnect("CPC_Analog_WidgetAnimationCB_NoAlarm", 
                            deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                            deviceName + ".statusInformation.selectedManager",
                            deviceName + ".ProcessInput.StsReg01",
                            deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                            deviceName + ".ProcessInput.StsReg02",
                            deviceName + ".ProcessInput.StsReg02:_online.._invalid",
                            deviceName + ".ProcessInput.PosSt",
                            deviceName + ".ProcessInput.PosSt:_online.._invalid",
                            deviceName + ".ProcessInput.PosRSt",
                            deviceName + ".ProcessInput.PosRSt:_online.._invalid",
                            deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
                            deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
                            deviceName + ".eventMask");
        }

		if (dynlen(g_dsCtrl) > 0) {
			iCtrlRes = dpConnect("cpcGenericObject_WidgetCtrlAnimationCB", g_dsCtrl);
		}
		g_bCallbackConnected = (iRes >= 0) && (iCtrlRes >= 0);
	} else {
		CPC_Analog_WidgetDisconnection(g_sWidgetType);
	}
}
//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Analog_WidgetDisconnect
/**
Purpose: Disconnect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_Analog_WidgetDisconnect(string deviceName, string sPlcName, string sSystemName)
{
	int iRes, iCtrlRes;

	if(g_bUnSystemAlarmPlc) {
        if (g_bBoolAlert) {
            iRes = dpDisconnect("CPC_Analog_WidgetAnimationCB", 
                            deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                            deviceName + ".statusInformation.selectedManager",
                            deviceName + ".ProcessInput.StsReg01",
                            deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                            deviceName + ".ProcessInput.StsReg02",
                            deviceName + ".ProcessInput.StsReg02:_online.._invalid",
                            deviceName + ".ProcessInput.PosSt",
                            deviceName + ".ProcessInput.PosSt:_online.._invalid",
                            deviceName + ".ProcessInput.PosRSt",
                            deviceName + ".ProcessInput.PosRSt:_online.._invalid",
                            deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
                            deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
                            deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
                            deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state",
                            deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state",
                            deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
                            deviceName + ".eventMask");
        } else {
            iRes = dpDisconnect("CPC_Analog_WidgetAnimationCB_NoAlarm", 
                            deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                            deviceName + ".statusInformation.selectedManager",
                            deviceName + ".ProcessInput.StsReg01",
                            deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                            deviceName + ".ProcessInput.StsReg02",
                            deviceName + ".ProcessInput.StsReg02:_online.._invalid",
                            deviceName + ".ProcessInput.PosSt",
                            deviceName + ".ProcessInput.PosSt:_online.._invalid",
                            deviceName + ".ProcessInput.PosRSt",
                            deviceName + ".ProcessInput.PosRSt:_online.._invalid",
                            deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
                            deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
                            deviceName + ".eventMask");
        }
		if (dynlen(g_dsCtrl) > 0) {
			iCtrlRes = dpDisconnect("cpcGenericObject_WidgetCtrlAnimationCB", g_dsCtrl);
		}
		g_bCallbackConnected = !(iRes >= 0) && !(iCtrlRes >= 0);
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------
CPC_Analog_WidgetAnimationCB_NoAlarm(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
							string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
							string sDpStsReg02, bit32 bit32StsReg02, string sDp5, bool bStsReg02Invalid,
							string sDpPosSt, float fPosSt, string sDp6, bool bPosStInvalid,
							string sDpOutput, float fPosRSt, string sDp7, bool bPosRStInvalid,
							string sDp8, bool bAuPosRStInvalid, string sDp9, bool bMPosRStInvalid,
							string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled,
							string sEvtMask, int iMask)
{
    CPC_Analog_WidgetAnimationCB(sDpLock, bLocked, sDpManager, sSelectedManager,
                                sDpStsReg01, bit32StsReg01, sDp4, bStsReg01Invalid,
                                sDpStsReg02, bit32StsReg02, sDp5, bStsReg02Invalid,
                                sDpPosSt, fPosSt, sDp6, bPosStInvalid,
                                sDpOutput, fPosRSt, sDp7, bPosRStInvalid,
                                sDp8, bAuPosRStInvalid, sDp9, bMPosRStInvalid,
                                "", 0, "", 0, "", 0,
                                sDpAl, iAlValue, sDpEnabled, bPlcEnabled,
                                sEvtMask, iMask);
}
// CPC_Analog_WidgetAnimationCB
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_Analog_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
							string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
							string sDpStsReg02, bit32 bit32StsReg02, string sDp5, bool bStsReg02Invalid,
							string sDpPosSt, float fPosSt, string sDp6, bool bPosStInvalid,
							string sDpOutput, float fPosRSt, string sDp7, bool bPosRStInvalid,
							string sDp8, bool bAuPosRStInvalid, string sDp9, bool bMPosRStInvalid,
                            string sDp10, int iStartIStState, string sDp11, int iTStopIStState, string sDp12, int iFuStopIStState, string sDp13, int iAlSt,
							string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled,
							string sEvtMask, int iMask)
{
// 1. Generic animation (Common to Analog/OnOff/AnaDig and PCO)
  	cpcGenericObject_WidgetAnimationDoubleStsReg(g_sWidgetType, UN_CONFIG_CPC_ANALOG_DPT_NAME, 
		bLocked, sSelectedManager, bit32StsReg01, bit32StsReg02,
		makeDynInt(UN_STSREG01_AUMRW, UN_STSREG01_POSALW, UN_STSREG01_IOSIMUW, UN_STSREG01_IOERRORW),
		makeDynInt(UN_STSREG02_ALBW),
		makeDynString(CPC_WIDGET_TEXT_WARNING_DEFAULT, CPC_WIDGET_TEXT_WARNING_DEFAULT, CPC_WIDGET_TEXT_WARNING_SIMU, CPC_WIDGET_TEXT_WARNING_ERROR),
		makeDynString(CPC_WIDGET_TEXT_WARNING_BLOCKED),
		makeDynInt(CPC_STSREG01_ALST, UN_STSREG01_STARTIST, CPC_STSREG01_TSTOPIST), 
		makeDynString(CPC_WIDGET_TEXT_ALARM_ALST, CPC_WIDGET_TEXT_ALARM_STARTIST, CPC_WIDGET_TEXT_ALARM_TSTOPIST),
		makeDynInt(CPC_STSREG02_FUSTOPIST), 
		makeDynString(CPC_WIDGET_TEXT_ALARM_FUSTOPIST),
        iStartIStState, iTStopIStState, iFuStopIStState, iAlSt,
		makeDynString("UN_STSREG01_LDST", "CPC_STSREG02_SOFTLDST", "CPC_STSREG01_AUIHFOMOST", "UN_STSREG01_AUINHMMO", "UN_STSREG01_MMOST", "UN_STSREG01_FOMOST"),
		"UN_STSREG01_ONST", "UN_STSREG01_OFFST", UN_STSREG01_ALUNACK,
		bStsReg01Invalid || bStsReg02Invalid || bPosStInvalid || bPosRStInvalid || bMPosRStInvalid || bAuPosRStInvalid,
		bStsReg01Invalid || bStsReg02Invalid, iAlValue, bPlcEnabled);

// 2. Display values
	if (g_bSystemConnected)
		unGenericObject_WidgetDisplayValueAnimation("Display1", g_sPosStFormat, fPosSt, "cpcColor_Widget_Status", bPosStInvalid);
	if (g_bSystemConnected)	
		unGenericObject_WidgetDisplayValueAnimation("Display2", g_sPosRStFormat, fPosRSt, "cpcColor_Widget_Request", bPosRStInvalid);

// display event state
	if(g_bSystemConnected) {
		eventState.visible = (iMask == 0);
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Analog_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType, string, input, widget type. ex: "heater"
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Analog_WidgetDisconnection(string sWidgetType)
{
	cpcGenericObject_WidgetDisconnection(sWidgetType, UN_CONFIG_CPC_ANALOG_DPT_NAME);
	unGenericObject_WidgetDisplayValueDisconnect(makeDynString("Display1", "Display2"));
	// set event state
	eventState.visible	= false;
	// set Regulation visibility.
	RegulationText1.visible = false;
	RegulationText2.visible = false;
}
