/**@name LIBRARY: unAnalogAlarm.ctl

@author: Francisco Mico (IT-CO)

Creation Date: 17 06 2003

Modification History:
	$Log: unAnalogAlarm.ctl,v $
    14/05/2012: Alexey
        [UCPC-783] AA_Analog widget has added

    29/02/2012: Alexey
        [UCPC-684] "Set Limits" available for "operator"

    12/12/2011: Alexey
        [UCPC-621] refactored AA - added separated dpe for 4 limits alerts, removed general and positional alerts

    23/11/2011: Alexey
        [UCPC-603] refactor menu creation and handling
        [UCPC-609] fixed right for Block PLC alarm
        [UCPC-611] added faceplate item to the right click menu
        [UCPC-610] removed set limit item from the right click menu

    20/10/2011: Alexey
        [UCPC-584] added ArmedRecipe box to the AA faceplate

    20/09/2011: Alexey
        [UCPC-548][UCPC-547] upd faceplate

    15/09/2011: Alexey
        [UCPC-540] apply format and unit of APar PosSt to the delay time

    14/09/2011: Alexey
        [UCPC-534] type is a list now

    30/08/2011: Alexey
        [UCPC-514] animate widget when no alert handler

    26/08/2011: Alexey
        added CPC_AnalogAlarm_animatePlcDisconnection

    24/08/2011: Alexey
        [UCPC-483] added CPC_AnalogAlarm_UserLoginGetButtonState function
        [UCPC-466] animate delay time parameter

	11/08/2011: Alexey
        [UCPC-476] display PosHHW/PosLLW/ConfigW in the widget

	10/08/2011: Alexey
		[UCPC-460] fixed popup rights; added Mask/Unmask PVSS alarm
		animate Mask alarm in the AA widget
		[UCPC-473] added "Select" popup item

	04/08/2011: Alexey
		[UCPC-454] added Block Alarm to the popup-menu; made smart label for it

	02/08/2011: Alexey
		[UCPC-451] 'Alarm Not Ack' is never blinking
		[UCPC-450] added AuIhMB information to the faceplate

	01/08/2011: Alexey
		[UCPC-448] HH/LL box blinks in the faceplate

	27/07/2011: Alexey
		[UCPC-387] more human alarm's type

	14/07/2011: Alexey
		use ISt to detect if ack alarm

	07/07/2011: Alexey
		[UCPC-265] update alarm acknowledge

	06/07/2011: Alexey
		[UCPC-358] fixed handling event mask

	04/07/2011: Alexey
		[UCPC-358] updated dpe's to mask

	28/06/2011: Alexey
		[UCPC-360] removed set mask popup item

	18/05/2011: Alexey
		use cpcGenericObject_HandleEventMaskMenu and CPC functions to manage menu

	12/05/2011: amerezhi
		[UCPC-196] display alarm type in the faceplate

	Revision 1.17  2009/03/30 12:14:48  gamx
	fixed bug in animation: priority of "N" is higher than "O"
	
	Revision 1.16  2009/01/28 14:05:58  gamx
	Fixes in _WidgetDisconnection function:
	A standard "unGenericObject_WidgetDisconnections" function cannot be used for ADA or DDA objects (due to additional graphical objects on the widget panel).
	A custom disconnection function has been introduced.

	Revision 1.15  2008/11/12 08:22:17  gthomas
	-removed function calls to _ButtonUserAccessGcs and _ButtonUserAccessUn

	Revision 1.14  2008/07/18 07:16:55  gthomas
	added new Access control functionality for unicos-4-1
	modified  xx_ButtonUserAccess function.

	Revision 1.13  2008/06/18 14:54:51  aabadz
	Changes done following the recomendations from AB-CO (from Herve)

	Revision 1.12  2008/03/17 10:52:36  aabadz
	Fixed a problem found in animation when we have not database.

	Revision 1.11  2008/01/22 14:46:36  aabadz
	Fixed:
	- Naming
	- Call to other functions to reuse code
	- Some error check

	Revision 1.10  2008/01/14 09:58:25  aabadz
	Changes after code review and before tests

	Revision 1.9  2008/01/14 09:48:47  aabadz
	After tests

	Revision 1.8  2007/10/30 16:12:14  gthomas
	Introduced the CVS keyword $Log to track changes made in the library

	25.09.2006 Kulman Nikolay
		- Porting to Unicos 3.6
	25.11.2004 Kulman Nikolay
		- function CPC_AnalogAlarm_HandleMenu modified to deselect widget automaticly
		- function CPC_AnalogAlarm_WidgetAnimationCB modified
	03.11.2004 Kulman Nikolay
		- function CPC_AnalogAlarm_HandleMenu modified to handle Mask/UnMask menu commands
	05/05/2004: Herve AB/CO
		- function CPC_AnalogAlarm_MenuConfiguration modified: signature modified and add the creation of the menu
		- function CPC_AnalogAlarm_HandleMenu added: this function handles the rigth click menu
		- function CPC_AnalogAlarm_ButtonSetState modified: modification of the signature of the function
		- function CPC_AnalogAlarm_UserLoginGetButtonState added: called when user logs in, returns the state of the buttons
		- function CPC_AnalogAlarm_ButtonRegister modified : add the set Manreg01 to 0 in case of dpConnect
		- function CPC_AnalogAlarm_ObjectListGetValueTime added: called in object list.
		- function CPC_AnalogAlarm_ButtonAnimationCB modified: signature of the function CPC_AnalogAlarm_ButtonSetState was modified
		- function CPC_AnalogAlarm_FaceplateConnect, CPC_AnalogAlarm_FaceplateDisconnect and CPC_AnalogAlarm_WidgetRegisterCB
		bug in case the PLC is not defined and if the PVSS system name contains '-',
		call unGenericObject_GetPLCNameFromDpName to get the PLC DPName instead (this function corrects the bugs).

version 1.0

External Function :
	. CPC_AnalogAlarm_AcknowledgeAlarm : animate AnalogAlarm alarms
	. CPC_AnalogAlarm_FaceplateStatusDisconnection : animate faceplate disconnection
	. CPC_AnalogAlarm_ButtonUserAccess : User access for AnalogAlarm buttons
	. CPC_AnalogAlarm_ButtonSetState : set button state
	. CPC_AnalogAlarm_WidgetDisconnection : animate widget disconnection
	. CPC_AnalogAlarm_MenuConfiguration : configuration of popup menu


Internal Functions :
	. CPC_AnalogAlarm_FaceplateStatusRegisterCB : register callback function
	. CPC_AnalogAlarm_FaceplateConnect : dpConnect to the DataPoint
	. CPC_AnalogAlarm_FaceplateDisconnect : dpDisconnect to the DataPoint
	. CPC_AnalogAlarm_FaceplateStatusAnimationCB : animate faceplate
	. CPC_AnalogAlarm_ButtonRegisterCB : register callback function
	. CPC_AnalogAlarm_ButtonAnimationCB : animate buttons
	. CPC_AnalogAlarm_ButtonDisconnection : animate buttons disconnection
	. CPC_AnalogAlarm_WidgetRegisterCB : register callback function
	. CPC_AnalogAlarm_WidgetConnect : dpConnect to the DataPoint
	. CPC_AnalogAlarm_WidgetDisconnect : dpDisconnect to the DataPoint
	. CPC_AnalogAlarm_WidgetAnimationCB : animate widget

Purpose: This library contains AnalogAlarm object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. AnalogAlarm object (DPType etc.)
	. Global variables and $parameters defined in AnalogAlarm faceplate and widget
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogAlarm_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogAlarm_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
    float value;
    string sTime;

    dpGet(sDeviceName+".ProcessInput.PosSt", value,
          sDeviceName+".ProcessInput.PosSt:_online.._stime", sTime);

    dsReturnData[1] = sTime;
    dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.PosSt"), value) + " "
                + dpGetUnit(sDeviceName+".ProcessInput.PosSt");
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogAlarm_AcknowledgeAlarm
/**
Purpose: acknowledge AnalogAlarm alarms

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsNeedAck, dyn_string, output, DPEs names that need acknowledgement.
									 The PLC acknowledgement is specified using UN_ACKNOWLEDGE_PLC string

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in OnOff faceplate
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogAlarm_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
	dsNeedAck = makeDynString("ProcessInput.HHAlSt", "ProcessInput.LLAlSt", UN_ACKNOWLEDGE_PLC);
}


//------------------------------------------------------------------------------------------------------------
// CPC_AnalogAlarm_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsMenuConfig, dyn_string, output, configuration of menu. Use defined constants UN_POPUPMENU_***

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogAlarm_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
    dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_FACEPLATE_TEXT, UN_POPUPMENU_ACK_TEXT, CPC_POPUPMENU_DIAG_INFO_TEXT, CPC_POPUPMENU_MASK_ALST_TEXT,
        UN_POPUPMENU_TREND_TEXT, UN_POPUPMENU_BLOCK_PLC_TEXT);

    cpcGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
    cpcGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
    cpcGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogAlarm_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogAlarm_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
    dyn_string exceptionInfo;

    cpcGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);

    if (dynlen(exceptionInfo) > 0) {
        unSendMessage_toExpertException("Right Click",exceptionInfo);
    }
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogAlarm_ButtonUserAccess
/**
Purpose: User access for AnalogAlarm buttons

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogAlarm_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	
	bool operator, expert, admin;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
	string sOperAction, sExpAction, sAdminAction;
	bool bActionDefined = false;
	
	unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
	if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
	(sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
	bActionDefined=true;
	
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	
	if (operator){
		if(bActionDefined) {
			dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
		} else {
			dynAppend(dsAccess, makeDynString(CPC_FACEPLATE_BUTTON_SET_AA_LIMITS, UN_FACEPLATE_BUTTON_MASK_ALARM, UN_FACEPLATE_BUTTON_SELECT));
			if (cpcExportGenericFunctions_getAcknowledgeAlarmValue(sDpName, ".ProcessInput.HHAlSt") != "N") {
				dynAppend(dsAccess, UN_FACEPLATE_BUTTON_ACK_ALARM);
			}
		}
	}
	// to check why Block/unblock action is missing !
	if (expert){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
		else
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_MASKEVENT, UN_FACEPLATE_BUTTON_BLOCK_ALARM_ACTION));
	}
	if(admin){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
		else
        		dynAppend(dsAccess, makeDynString());
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogAlarm_UserLoginGetButtonState
/**
Function is called before the button set state when a user logs in.

	@param[in]	sDeviceName		The device name
	@param[in]	sType			Device type <b>(Currently not used)</b>
	@param[out]	dsReturnData	The data are returned here
	@note	This library works with an unicos button panel and need global variables declared in this panel
		- g_dsUserAccess, dyn_string, button identifier is added to this variable if user access
		- $sDpName, string, the device DP name
		- g_sFaceplateButtonType, string, faceplate type
*/
CPC_AnalogAlarm_UserLoginGetButtonState(string sDeviceName, string sType, dyn_string &dsReturnData)
{
	unGenericObject_UserCBGetButtonState(sDeviceName, sType, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogAlarm_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogAlarm widget
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogAlarm_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sPlcName, sSystemName;
	int iAction;
	string sPlcDpName;
	dyn_string exceptionInfo;
	bool bRemote;

	deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
	if (deviceName == "")	// In case of disconnection
	{
		deviceName = g_sDpName;
	}

	sSystemName = unGenericDpFunctions_getSystemName(deviceName);

	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	cpcConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)
		g_bUnSystemAlarmPlc = true;
	else
		g_bUnSystemAlarmPlc = false;


	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);

	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			CPC_AnalogAlarm_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = deviceName;

			if(dpTypeName(deviceName) == UN_CONFIG_CPC_ANALOGALARM_DPT_NAME)
				CPC_AnalogAlarm_WidgetConnect(deviceName, sPlcName, sSystemName);
			else
			{
				g_bUnSystemAlarmPlc = false;
				CPC_AnalogAlarm_WidgetDisconnection(g_sWidgetType);
			}
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			CPC_AnalogAlarm_WidgetDisconnect(deviceName, sPlcName, sSystemName);
			CPC_AnalogAlarm_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING: break;
		default:
			break;
		}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogAlarm_WidgetConnect
/**
23.04.2004
Teodora Miteva

Purpose: do dpConnect

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnalogAlarm faceplate
	. PVSS version: 2.12.1
	. operating system: XP and W2000, but tested only under XP.
	. distributed system: yes.
*/
CPC_AnalogAlarm_WidgetConnect(string deviceName, string sPlcName, string sSystemName)
{
    int iRes, iAlertType;
    if(g_bUnSystemAlarmPlc) {
        iRes = dpGet(deviceName + ".ProcessInput.HHAlSt:_alert_hdl.._type", iAlertType);
        g_bBoolAlert = ((iRes >= 0) && (iAlertType == DPCONFIG_ALERT_BINARYSIGNAL));
        if (g_sWidgetType == "AA_Analog") {
          g_sPosStFormat = dpGetFormat(deviceName + ".ProcessInput.PosSt");
          g_sPosStUnit = dpGetUnit(deviceName + ".ProcessInput.PosSt");
        }
        if (g_bBoolAlert) {
            iRes = dpConnect("CPC_AnalogAlarm_WidgetAnimationCB",
                deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                deviceName + ".statusInformation.selectedManager",
                deviceName + ".ProcessInput.StsReg01",
                deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                deviceName + ".ProcessInput.StsReg02",
                deviceName + ".ProcessInput.StsReg02:_online.._invalid",
                deviceName + ".ProcessInput.PosSt",
                deviceName + ".ProcessInput.PosSt:_online.._invalid",
                deviceName + ".ProcessInput.HHAlSt:_alert_hdl.._active",
                sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
                sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
                deviceName + ".eventMask");
        } else {
            iRes = dpConnect("CPC_AnalogAlarm_WidgetAnimationCB_NoAlarm",
                deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                deviceName + ".statusInformation.selectedManager",
                deviceName + ".ProcessInput.StsReg01",
                deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                deviceName + ".ProcessInput.StsReg02",
                deviceName + ".ProcessInput.StsReg02:_online.._invalid",
                deviceName + ".ProcessInput.PosSt",
                deviceName + ".ProcessInput.PosSt:_online.._invalid",
                sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
                sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
                deviceName + ".eventMask");
        }
        g_bCallbackConnected = (iRes >= 0);
    } else {
        CPC_AnalogAlarm_WidgetDisconnection(g_sWidgetType);
    }
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogAlarm_WidgetDisconnect
/**
23.04.2004
Teodora Miteva

Purpose: do dpDisconnect

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnalogAlarm faceplate
	. PVSS version: 2.12.1
	. operating system: XP and W2000, but tested only under XP.
	. distributed system: yes.
*/
CPC_AnalogAlarm_WidgetDisconnect(string deviceName, string sPlcName, string sSystemName)
{
    int iRes;
    if (g_bUnSystemAlarmPlc) {
        if (g_bBoolAlert) {
            iRes = dpDisconnect("CPC_AnalogAlarm_WidgetAnimationCB",
                deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                deviceName + ".statusInformation.selectedManager",
                deviceName + ".ProcessInput.StsReg01",
                deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                deviceName + ".ProcessInput.StsReg02",
                deviceName + ".ProcessInput.StsReg02:_online.._invalid",
                deviceName + ".ProcessInput.PosSt",
                deviceName + ".ProcessInput.PosSt:_online.._invalid",
                deviceName + ".ProcessInput.HHAlSt:_alert_hdl.._active",
                sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
                sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
                deviceName + ".eventMask");
        } else {
            iRes = dpDisconnect("CPC_AnalogAlarm_WidgetAnimationCB_NoAlarm",
                deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                deviceName + ".statusInformation.selectedManager",
                deviceName + ".ProcessInput.StsReg01",
                deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                deviceName + ".ProcessInput.StsReg02",
                deviceName + ".ProcessInput.StsReg02:_online.._invalid",
                deviceName + ".ProcessInput.PosSt",
                deviceName + ".ProcessInput.PosSt:_online.._invalid",
                sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
                sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
                deviceName + ".eventMask");
        }
        g_bCallbackConnected = !(iRes >= 0);
    }
}

CPC_AnalogAlarm_WidgetAnimationCB_NoAlarm(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
        string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
        string sDpStsReg02, bit32 bit32StsReg02, string sDp5, bool bStsReg02Invalid,
        string sDpPosSt, float fPosSt, string sDp6, bool bPosStInvalid,
        string sDpAl, int iSystemIntegrityAlarmValue, string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
        string sEvtMask, int iMask)
{
    CPC_AnalogAlarm_WidgetAnimationCB(sDpLock, bLocked, sDpManager, sSelectedManager,
            sDpStsReg01, bit32StsReg01, sDp4, bStsReg01Invalid,
            sDpStsReg02, bit32StsReg02, sDp5, bStsReg02Invalid,
            sDpPosSt, fPosSt, sDp6, bPosStInvalid,
            "", false,
            sDpAl, iSystemIntegrityAlarmValue, sDpEnabled, bSystemIntegrityAlarmEnabled,
            sEvtMask, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogAlarm_WidgetAnimationCB
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnalogAlarm faceplate
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogAlarm_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
   		string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
   		string sDpStsReg02, bit32 bit32StsReg02, string sDp5, bool bStsReg02Invalid,
      string sDpPosSt, float fPosSt, string sDp6, bool bPosStInvalid,
        string sDp2, bool bIsAlarmActive,
		string sDpAl, int iSystemIntegrityAlarmValue, string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
		string sEvtMask, int iMask)
{
	string sBody1Color, sBody1BColor = "", sWarningColor = "",sWarningLetter = "", sColorSelect;
	string sAlarmTypeLetter = "", sAlarmTypeColor = "", sAlarmTextLetter = "", sAlarmTextColor = "";
    bool bLockVisible;

	sBody1Color = "unDataNotValid";

	if (!bIsAlarmActive) {
		sAlarmTextLetter = CPC_WIDGET_TEXT_ALARM_MASKED;
		sAlarmTextColor = "unAlarmMasked";
	}

	if (!bStsReg01Invalid && !bStsReg02Invalid) {
		// all is OK - so alarm is green
		sBody1Color = "unAlarm_Ok";
		sBody1BColor = "";

		if (getBit(bit32StsReg01, CPC_STSREG01_IST) == 1) {		//	Alarm is active
			sBody1Color  = "cpcColor_Alarm_Bad";
			sBody1BColor = "cpcColor_Alarm_Bad";
		} else if (getBit(bit32StsReg01, CPC_STSREG01_WST) == 1)	{	//	Warning is active
			sBody1Color  = "cpcColor_Alarm_Warning";
		}

//	Animate alarm type notification (H, HH, L, LL)
		if(getBit(bit32StsReg02, CPC_STSREG02_HWST) == 1) {		//	H
			sAlarmTypeLetter = CPC_WIDGET_ALARM_HIGH;
			sAlarmTypeColor  = "cpcColor_Alarm_Warning";
		}
		if(getBit(bit32StsReg02, CPC_STSREG02_LWST) == 1) {		//	L
			sAlarmTypeLetter = CPC_WIDGET_ALARM_LOW;
			sAlarmTypeColor  = "cpcColor_Alarm_Warning";
		}
		if(getBit(bit32StsReg02, CPC_STSREG02_HHALST) == 1) {	//	HH
			sAlarmTypeLetter = CPC_WIDGET_ALARM_HIGHHIGH;
			sAlarmTypeColor  = "cpcColor_Alarm_Bad";
		}
		if(getBit(bit32StsReg02, CPC_STSREG02_LLALST) == 1) {	//	LL
			sAlarmTypeLetter = CPC_WIDGET_ALARM_LOWLOW;
			sAlarmTypeColor  = "cpcColor_Alarm_Bad";
		}

		if (getBit(bit32StsReg01, UN_STSREG01_ALUNACK) == 1) {
			sBody1Color = "cpcColor_Alarm_AlarmNotAck";
			if (getBit(bit32StsReg01, CPC_STSREG01_IST) == 1) {
				sBody1BColor = "cpcColor_Alarm_AlarmNotAck";
			} else if (getBit(bit32StsReg01, CPC_STSREG01_WST) == 1) {
				sBody1Color  = "cpcColor_Alarm_WarningNotAck";
			}
		}

		if (getBit(bit32StsReg01, CPC_STSREG01_MALBRST) == 1) {			// Alarm blocked
			sAlarmTextLetter = CPC_WIDGET_TEXT_ALARM_BLOCKED;
			sAlarmTextColor = "unAlarmMasked";
			sBody1Color = "unAlarmMasked";
			sBody1BColor = "";
		}

		unGenericObject_WidgetWarningAnimation(bit32StsReg01,
			makeDynInt(CPC_STSREG01_CONFIGW, CPC_STSREG01_POSHHW, CPC_STSREG01_POSLLW, UN_STSREG01_IOSIMUW, UN_STSREG01_IOERRORW),
			makeDynString(CPC_WIDGET_TEXT_CONFIGW, CPC_WIDGET_TEXT_POSHHW, CPC_WIDGET_TEXT_POSLLW, CPC_WIDGET_TEXT_WARNING_SIMU, CPC_WIDGET_TEXT_WARNING_ERROR),
			sWarningLetter, sWarningColor);
	}

	//data quality animation
	if(!bSystemIntegrityAlarmEnabled || iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value) {
		sWarningLetter = CPC_WIDGET_TEXT_OLD_DATA;
		sWarningColor = "unDataNotValid";
	}
	
	if (bStsReg01Invalid || bStsReg02Invalid) {
		sWarningLetter = CPC_WIDGET_TEXT_INVALID;
		sWarningColor = "unDataNotValid";
	}

	unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, sColorSelect, bLockVisible);
	if(g_bSystemConnected) {
		setMultiValue("WarningText", "text", sWarningLetter, "WarningText", "foreCol", sWarningColor,
			"AlarmType", "text", sAlarmTypeLetter, "AlarmType", "foreCol", sAlarmTypeColor,
			"AlarmText", "text", sAlarmTextLetter, "AlarmText","foreCol", sAlarmTextColor,
			"ControlStateText", "text", "", "SelectArea", "foreCol", sColorSelect,
			"LockBmp", "visible", bLockVisible, "Body1", "fill", "[solid]",
			"Body1", "foreCol", sBody1Color, "Body1", "backCol", sBody1BColor,
			"WidgetArea", "visible", true);
      if (g_sWidgetType == "AA_Analog") {
        cpcGenericObject_WidgetDisplayValueAnimation("Display1", g_sPosStUnit, g_sPosStFormat, fPosSt, sBody1Color, 
            bPosStInvalid || (!bSystemIntegrityAlarmEnabled) || (iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value));
      }
	}
    if (g_bSystemConnected) {
        eventState.visible = iMask == 0;
    }
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogAlarm_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType, string, input, widget type. ex: "heater"

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogAlarm_WidgetDisconnection(string sWidgetType)
{
	setMultiValue("Body1", "foreCol", "unDataNoAccess", "Body1", "backCol", "unWidget_Background", "Body1", "fill", "[solid]",
	   "WarningText", "text", "", "WidgetArea", "visible", false, "AlarmType", "text", "", "AlarmText", "text", "");
  if (sWidgetType == "AA_Analog") {
    unGenericObject_WidgetDisplayValueDisconnect(makeDynString("Display1"));
  }
	eventState.visible = false;
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogAlarm_FaceplateStatusRegisterCB
/**
Register faceplate callback function

	@param[in]	sSystemName 	The device dp
	@param[in] 	bSystemConnected 	Flag
*/
CPC_AnalogAlarm_FaceplateStatusRegisterCB(string sSystemStatus, bool bSystemConnected)
{
	string sDeviceName, sSystemName, sPlcName;
	int iAction;
	dyn_string exceptionInfo;
	string sPlcDpName;
	bool bRemote;

	sDeviceName = unGenericDpFunctions_getDpName($sDpName);
	if (sDeviceName)
	{
		sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
		unDistributedControl_isRemote(bRemote, sSystemName);
		if(bRemote)
			g_bSystemConnected = bSystemConnected;
		else
			g_bSystemConnected = true;

		sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
		cpcConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)
			g_bUnSystemAlarmPlc = true;
		else
			g_bUnSystemAlarmPlc = false;

		unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);

		switch (iAction)
		{
			case UN_ACTION_DISCONNECT:
				CPC_AnalogAlarm_animatePlcDisconnection();
				break;
			case UN_ACTION_DPCONNECT:
				CPC_AnalogAlarm_FaceplateConnect(sDeviceName);
				break;
			case UN_ACTION_DPDISCONNECT_DISCONNECT:
				CPC_AnalogAlarm_FaceplateDisconnect(sDeviceName);
				CPC_AnalogAlarm_animatePlcDisconnection();
				break;
			case UN_ACTION_NOTHING: break;
			default:
				break;
		}
	}

}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogAlarm_FaceplateConnect
/**
Do faceplate dpConnect

	@param[in]	sDeviceName	Device DP name
*/
CPC_AnalogAlarm_FaceplateConnect(string sDeviceName)
{
	int iRes=0;
	dyn_string exceptionInfo;
	dyn_string sPlcDpName;
	string sPlcName, sSystemName;

	sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
	sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
	cpcConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	g_bUnSystemAlarmPlc = (dynlen(exceptionInfo) <= 0);
	g_bAlAck = cpcExportGenericFunctions_getAcknowledgeAlarmValue(sDeviceName, ".ProcessInput.HHAlSt") != "N";
	g_sAlarmDelay = cpcGenericDpFunctions_getDeviceProperty(sDeviceName, CPC_PARAMS_ALDT, exceptionInfo);
    string alarmDelayDPe = dpAliasToName(sSystemName + g_sAlarmDelay) + "ProcessInput.PosSt";
    g_bAlarmDelayInhibited = dpExists(dpAliasToName(sSystemName + g_sAlarmDelay));
    g_sAlarmDelayFormat = g_bAlarmDelayInhibited ? dpGetFormat(alarmDelayDPe) : "%2.1f";
    g_sAlarmDelayUnit = g_bAlarmDelayInhibited ? dpGetUnit(alarmDelayDPe) : "s";
    g_bAlarmDelayIsValue = g_bAlarmDelayInhibited || g_sAlarmDelay == (string)((int) g_sAlarmDelay);

	if (g_bUnSystemAlarmPlc) {
		g_sPosStFormat = dpGetFormat(sDeviceName + ".ProcessInput.PosSt");
		g_sPosStUnit = dpGetUnit(sDeviceName + ".ProcessInput.PosSt");

		g_bMailSMS = cpcGenericObject_isAlarmWithMail(sDeviceName + ".ProcessInput.HHAlSt", DPCONFIG_ALERT_BINARYSIGNAL);;
		unGenericDpFunctions_getKeyDeviceConfiguration(sDeviceName, CPC_CONFIG_TYPE_KEY, g_dsAlarmTypes, exceptionInfo);

        if (g_bAlarmDelayInhibited) {
            iRes = dpConnect("CPC_AnalogAlarm_FaceplateStatusAnimationCB",
                 sDeviceName + ".ProcessInput.StsReg01",
                 sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
                 sDeviceName + ".ProcessInput.StsReg02",
                 sDeviceName + ".ProcessInput.StsReg02:_online.._invalid",
                 sDeviceName + ".ProcessInput.HHSt",
                 sDeviceName + ".ProcessInput.HHSt:_online.._invalid",
                 sDeviceName + ".ProcessInput.HSt",
                 sDeviceName + ".ProcessInput.HSt:_online.._invalid",
                 sDeviceName + ".ProcessInput.LSt",
                 sDeviceName + ".ProcessInput.LSt:_online.._invalid",
                 sDeviceName + ".ProcessInput.LLSt",
                 sDeviceName + ".ProcessInput.LLSt:_online.._invalid",
                 sDeviceName + ".ProcessInput.PosSt",
                 sDeviceName + ".ProcessInput.PosSt:_online.._invalid",
                 alarmDelayDPe,
                 alarmDelayDPe + ":_online.._invalid",
                 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
                 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
                 sDeviceName + ".eventMask");
        } else {
            iRes = dpConnect("CPC_AnalogAlarm_FaceplateStatusAnimationCB_ConstantDelay",
                 sDeviceName + ".ProcessInput.StsReg01",
                 sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
                 sDeviceName + ".ProcessInput.StsReg02",
                 sDeviceName + ".ProcessInput.StsReg02:_online.._invalid",
                 sDeviceName + ".ProcessInput.HHSt",
                 sDeviceName + ".ProcessInput.HHSt:_online.._invalid",
                 sDeviceName + ".ProcessInput.HSt",
                 sDeviceName + ".ProcessInput.HSt:_online.._invalid",
                 sDeviceName + ".ProcessInput.LSt",
                 sDeviceName + ".ProcessInput.LSt:_online.._invalid",
                 sDeviceName + ".ProcessInput.LLSt",
                 sDeviceName + ".ProcessInput.LLSt:_online.._invalid",
                 sDeviceName + ".ProcessInput.PosSt",
                 sDeviceName + ".ProcessInput.PosSt:_online.._invalid",
                 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
                 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
                 sDeviceName + ".eventMask");
        }
	} else {
		CPC_AnalogAlarm_animatePlcDisconnection();
	}

	g_bCallbackConnected = (iRes >= 0);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogAlarm_FaceplateDisconnect
/**
Do faceplate dpDisconnect

	@param[in]	sDeviceName	Device DP name
*/
CPC_AnalogAlarm_FaceplateDisconnect(string sDeviceName)
{
	int iRes = -1;
	string sPlcName, sSystemName;
	string sPlcDpName;
	dyn_string exceptionInfo;

	if (g_bUnSystemAlarmPlc) {
		sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
		sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
        string alarmDelayDPe = dpAliasToName(sSystemName + g_sAlarmDelay) + "ProcessInput.PosSt";

		cpcConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
		if (dynlen(exceptionInfo) <= 0) g_bUnSystemAlarmPlc = true;
		else g_bUnSystemAlarmPlc = false;

        if (g_bAlarmDelayInhibited) {
            iRes = dpDisconnect("CPC_AnalogAlarm_FaceplateStatusAnimationCB",
                 sDeviceName + ".ProcessInput.StsReg01",
                 sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
                 sDeviceName + ".ProcessInput.StsReg02",
                 sDeviceName + ".ProcessInput.StsReg02:_online.._invalid",
                 sDeviceName + ".ProcessInput.HHSt",
                 sDeviceName + ".ProcessInput.HHSt:_online.._invalid",
                 sDeviceName + ".ProcessInput.HSt",
                 sDeviceName + ".ProcessInput.HSt:_online.._invalid",
                 sDeviceName + ".ProcessInput.LSt",
                 sDeviceName + ".ProcessInput.LSt:_online.._invalid",
                 sDeviceName + ".ProcessInput.LLSt",
                 sDeviceName + ".ProcessInput.LLSt:_online.._invalid",
                 sDeviceName + ".ProcessInput.PosSt",
                 sDeviceName + ".ProcessInput.PosSt:_online.._invalid",
                 alarmDelayDPe,
                 alarmDelayDPe + ":_online.._invalid",
                 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
                 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
                 sDeviceName + ".eventMask");
        } else {
            iRes = dpDisconnect("CPC_AnalogAlarm_FaceplateStatusAnimationCB_ConstantDelay",
                 sDeviceName + ".ProcessInput.StsReg01",
                 sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
                 sDeviceName + ".ProcessInput.StsReg02",
                 sDeviceName + ".ProcessInput.StsReg02:_online.._invalid",
                 sDeviceName + ".ProcessInput.HHSt",
                 sDeviceName + ".ProcessInput.HHSt:_online.._invalid",
                 sDeviceName + ".ProcessInput.HSt",
                 sDeviceName + ".ProcessInput.HSt:_online.._invalid",
                 sDeviceName + ".ProcessInput.LSt",
                 sDeviceName + ".ProcessInput.LSt:_online.._invalid",
                 sDeviceName + ".ProcessInput.LLSt",
                 sDeviceName + ".ProcessInput.LLSt:_online.._invalid",
                 sDeviceName + ".ProcessInput.PosSt",
                 sDeviceName + ".ProcessInput.PosSt:_online.._invalid",
                 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
                 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
                 sDeviceName + ".eventMask");
        }

		g_bCallbackConnected = !(iRes >= 0);
	} else {
		CPC_AnalogAlarm_WidgetDisconnection(g_sWidgetType);
		g_bCallbackConnected = false;
	}
}


CPC_AnalogAlarm_FaceplateStatusAnimationCB_ConstantDelay(string sStsReg01, bit32 bit32StsReg01, string sDp1, bool bStsReg01Invalid,
		string sStsReg02, bit32 bit32StsReg02, string sDp2, bool bStsReg02Invalid,
		string sDpHHSt, float fHHSt, string sDp3, bool bHHStInvalid,
		string sDpHSt,  float fHSt,  string sDp4, bool bHStInvalid,
		string sDpLSt,  float fLSt,  string sDp5, bool bLStInvalid,
		string sDpLLSt, float fLLSt, string sDp6, bool bLLStInvalid,
		string sDpPosSt, float fPosSt, string sDp7, bool bPosStInvalid, 
		string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled, string sDpEvt, int iMask)
{
    CPC_AnalogAlarm_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp1, bStsReg01Invalid,
            sStsReg02, bit32StsReg02, sDp2, bStsReg02Invalid,
            sDpHHSt, fHHSt, sDp3, bHHStInvalid,
            sDpHSt,  fHSt,  sDp4, bHStInvalid,
            sDpLSt,  fLSt,  sDp5, bLStInvalid,
            sDpLLSt, fLLSt, sDp6, bLLStInvalid,
            sDpPosSt, fPosSt, sDp7, bPosStInvalid, 
            "", g_sAlarmDelay, "", false,
            sDpAl, iValAlarm, sDpEnabled, bSystemIntegrityAlarmEnabled, sDpEvt, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogAlarm_FaceplateStatusAnimationCB
/**
Animate faceplate
*/
CPC_AnalogAlarm_FaceplateStatusAnimationCB(string sStsReg01, bit32 bit32StsReg01, string sDp1, bool bStsReg01Invalid,
		string sStsReg02, bit32 bit32StsReg02, string sDp2, bool bStsReg02Invalid,
		string sDpHHSt, float fHHSt, string sDp3, bool bHHStInvalid,
		string sDpHSt,  float fHSt,  string sDp4, bool bHStInvalid,
		string sDpLSt,  float fLSt,  string sDp5, bool bLStInvalid,
		string sDpLLSt, float fLLSt, string sDp6, bool bLLStInvalid,
		string sDpPosSt, float fPosSt, string sDp7, bool bPosStInvalid, 
		string sDp8, anytype alarmDelay, string sDp9, bool alarmDelayInvalid,
		string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled, string sDpEvt, int iMask)
{
	if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fPosSt, "StatusPosition", "unDisplayValue_Status", 
					bPosStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fHHSt, "HHSt", "unDisplayValue_Parameter",
			bHHStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fHSt, "HSt", "unDisplayValue_Parameter",
			bHStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fLSt, "LSt", "unDisplayValue_Parameter",
			bLStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fLLSt, "LLSt", "unDisplayValue_Parameter",
			bLLStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("AuIhMBSt", getBit(bit32StsReg01, CPC_STSREG01_AUIHMBST), 1, "cpcColor_Faceplate_Status",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));

	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("EHHSt", getBit(bit32StsReg02, CPC_STSREG02_EHHST), 1, "cpcColor_Faceplate_Status",
			bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("EHSt", getBit(bit32StsReg02, CPC_STSREG02_EHST), 1, "cpcColor_Faceplate_Status",
			bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("ELSt", getBit(bit32StsReg02, CPC_STSREG02_ELST), 1, "cpcColor_Faceplate_Status",
			bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("ELLSt", getBit(bit32StsReg02, CPC_STSREG02_ELLST), 1, "cpcColor_Faceplate_Status",
			bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("IhMHHSt", getBit(bit32StsReg02, CPC_STSREG02_IHMHHST), 1, "cpcColor_Faceplate_Status",
			bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("IhMHSt", getBit(bit32StsReg02, CPC_STSREG02_IHMHST), 1, "cpcColor_Faceplate_Status",
			bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("IhMLSt", getBit(bit32StsReg02, CPC_STSREG02_IHMLST), 1, "cpcColor_Faceplate_Status",
			bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("IhMLLSt", getBit(bit32StsReg02, CPC_STSREG02_IHMLLST), 1, "cpcColor_Faceplate_Status",
			bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("HWSt", getBit(bit32StsReg02, CPC_STSREG02_HWST), 1, "unFaceplate_WarningActive", 
					bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("LWSt", getBit(bit32StsReg02, CPC_STSREG02_LWST), 1, "unFaceplate_WarningActive", 
					bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("ActHHSt", getBit(bit32StsReg02, CPC_STSREG02_HHALST), 1, getBit(bit32StsReg01, UN_STSREG01_ALUNACK) ? "unFaceplate_AlarmNotAck" : "unFaceplate_AlarmActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("ActLLSt", getBit(bit32StsReg02, CPC_STSREG02_LLALST), 1, getBit(bit32StsReg01, UN_STSREG01_ALUNACK) ? "unFaceplate_AlarmNotAck" : "unFaceplate_AlarmActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("AlUnAck", getBit(bit32StsReg01, UN_STSREG01_ALUNACK), g_bAlAck, "cpcColor_Alarm_Bad",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
    if(g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("ArmedRecipe", getBit(bit32StsReg01, CPC_STSREG01_ARMRCPST), 1, "cpcColor_Faceplate_Status",
            bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
    if(g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("eventState", iMask == 0, 1, "unFaceplate_WarningActive", 
                bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("ConfigW", getBit(bit32StsReg01, CPC_STSREG01_CONFIGW), 1, "unFaceplate_WarningActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("MAlBRSt", getBit(bit32StsReg01, CPC_STSREG01_MALBRST), 1, "unFaceplate_WarningActive", 
					bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("PosHHW", getBit(bit32StsReg01, CPC_STSREG01_POSHHW), 1, "unFaceplate_WarningActive", 
					bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("PosHW", getBit(bit32StsReg01, CPC_STSREG01_POSHW), 1, "unFaceplate_WarningActive", 
					bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("PosLW", getBit(bit32StsReg01, CPC_STSREG01_POSLW), 1, "unFaceplate_WarningActive", 
					bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("PosLLW", getBit(bit32StsReg01, CPC_STSREG01_POSLLW), 1, "unFaceplate_WarningActive", 
					bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("IOErrorW", getBit(bit32StsReg01, UN_STSREG01_IOERRORW), 1, "unFaceplate_WarningActive", 
					bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("IOSimuW", getBit(bit32StsReg01, UN_STSREG01_IOSIMUW), 1, "unFaceplate_WarningActive", 
					bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
    cpcGenericObject_CheckboxAnimate("MailSMS", g_bMailSMS);
	string color = (!bSystemIntegrityAlarmEnabled || iValAlarm > c_unSystemIntegrity_no_alarm_value) ? "unDataNotValid" : "unDisplayValue_Parameter";
	if(g_bSystemConnected)
		setMultiValue("AlarmType" + ".display", "foreCol", color, "AlarmType" + ".display", "text", cpcGenericObject_translateAlarmAcronym(g_dsAlarmTypes));
    if (g_bSystemConnected) {
        if (g_bAlarmDelayIsValue) {
            unGenericObject_DisplayValue(g_sAlarmDelayFormat, g_sAlarmDelayUnit, alarmDelay, "AlarmDelayText", "unDisplayValue_Parameter", 
                        alarmDelayInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
        } else {
            setMultiValue("AlarmDelayText" + ".display", "foreCol", color, "AlarmDelayText" + ".display", "text", alarmDelay);
        }
    }
    setMultiValue("AlarmDelayButton", "enabled", g_bAlarmDelayInhibited, "AlarmDelayButton", "text", g_bAlarmDelayInhibited ? g_sAlarmDelay : "none");
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogAlarm_animatePlcDisconnection
/**
Animate faceplate disconnection
*/
CPC_AnalogAlarm_animatePlcDisconnection()
{
    unGenericObject_DisplayValueDisconnect(makeDynString("StatusPosition", "HHSt", "HSt", "LSt", "LLSt", "AlarmDelayText", "AlarmType"));
    unGenericObject_ColorBoxDisconnect(makeDynString("AuIhMBSt", "EHHSt", "EHSt", "ELSt", "ELLSt", "IhMHHSt", "IhMHSt", "IhMLSt", "IhMLLSt", "HWSt",
            "LWSt", "ActHHSt", "ActLLSt", "AlUnAck", "eventState", "ConfigW", "MAlBRSt", "PosHHW", "PosHW", "PosLW", "PosLLW", "IOErrorW", "IOSimuW", "ArmedRecipe"));
    setMultiValue("AlarmDelayButton", "enabled", false);
}

CPC_AnalogAlarm_ButtonRegisterCB(string sSystemStatus, bool bSystemConnected)
{
	string sDeviceName, sDpType, sFunction, sSystemName;
	int iAction, iRes;
	dyn_string exceptionInfo, dsFunctions, dsUserAccess;
	string sPlcName;

	sDeviceName = unGenericDpFunctions_getDpName($sDpName);
	if (sDeviceName) {
		unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);

		switch (iAction) {
			case UN_ACTION_DISCONNECT:
				CPC_AnalogAlarm_ButtonDisconnection();
				break;
			case UN_ACTION_DPCONNECT:
				sDpType=UN_CONFIG_CPC_ANALOGALARM_DPT_NAME;
				unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
				if (dynlen(exceptionInfo) <= 0)	CPC_AnalogAlarm_ProvideAccessToButtons(sDeviceName, sDpType, dsFunctions, exceptionInfo);
				
				iRes = dpConnect("CPC_AnalogAlarm_ButtonAnimationCB",
						sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
						sDeviceName + ".statusInformation.selectedManager",
						sDeviceName + ".ProcessInput.StsReg01",
						sDeviceName + ".ProcessInput.StsReg01:_online.._aut_inv");
				g_bCallbackConnected = (iRes >= 0);
				break;
			case UN_ACTION_DPDISCONNECT_DISCONNECT:
				CPC_AnalogAlarm_ButtonsDisconnect(sDeviceName);
				CPC_AnalogAlarm_ButtonDisconnection();
				break;
			case UN_ACTION_NOTHING: break;
			default:
				break;
		}
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------
//CPC_AnalogAlarm_ButtonsDisconnect
/**
Disconnects callback function to animate buttons states
	@param[in] 	sDeviceName	The name of the device
*/
CPC_AnalogAlarm_ButtonsDisconnect(string deviceName)
{
	int iRes = -1;

	iRes = dpDisconnect("CPC_AnalogAlarm_ButtonAnimationCB",
				deviceName + ".statusInformation.selectedManager:_lock._original._locked",
				deviceName + ".statusInformation.selectedManager",
				deviceName + ".ProcessInput.StsReg01",
				deviceName + ".ProcessInput.StsReg01:_online.._aut_inv");
	g_bCallbackConnected = !(iRes >= 0);
}

CPC_AnalogAlarm_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager,
    string sDpStsReg01, bit32 fStsReg01Value, string sDpStsReg01Bad, bool bStsReg01Bad)
{
	CPC_AnalogAlarm_ButtonSetState(unGenericDpFunctions_getDpName(sDpStsReg01), UN_CONFIG_CPC_ANALOGALARM_DPT_NAME, g_dsUserAccess,
		makeDynString(bLocked, sSelectedManager, fStsReg01Value, bStsReg01Bad, 0, 0));
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogAlarm_ButtonSetState
/**
Set button state

	@param[in]	 sDeviceName		The device name
	@param[in]	 sDpType		Device type
	@param[in]	 dsUserAccess	User access
	@param[in]	 dsData			List of various parameters
*/
CPC_AnalogAlarm_ButtonSetState(string sDeviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected;
	dyn_string dsButtons;
	string localManager;
	bool bLocked, bStsReg01Bad;
	string sSelectedManager;
	bit32 stsReg01Value;

	bLocked = dsData[1];
	sSelectedManager = dsData[2];
	stsReg01Value = dsData[3];
	bStsReg01Bad = dsData[4];

	dsButtons = makeDynString(UN_FACEPLATE_BUTTON_ACK_ALARM, CPC_FACEPLATE_BUTTON_SET_AA_LIMITS, UN_FACEPLATE_BUTTON_BLOCK_ALARM_ACTION);// Each button except "Select" button
	// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = localManager == "S";
	// 2. Button state
	for(int i = 1; i <= dynlen(dsButtons); i++) {
		bool buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);	// User access
		switch (dsButtons[i]) {
			case CPC_FACEPLATE_BUTTON_SET_AA_LIMITS:
				buttonEnabled = buttonEnabled && bSelected;
				break;
            case UN_FACEPLATE_BUTTON_BLOCK_ALARM_ACTION:
                buttonEnabled = buttonEnabled && !getBit(stsReg01Value, CPC_STSREG01_AUIHMBST);
			default:
				break;
		}
		setValue(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", buttonEnabled);
	}
    setValue(UN_FACEPLATE_BUTTON_PREFIX + UN_FACEPLATE_BUTTON_BLOCK_ALARM_ACTION, "text", getBit(stsReg01Value, CPC_STSREG01_MALBRST) ? "Deblock Alarm" : "Block Alarm");
	// 3. Button Select state
	unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogAlarm_ButtonDisconnection
/**
Animate disconnection
*/
CPC_AnalogAlarm_ButtonDisconnection()
{
	unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_SELECT, UN_FACEPLATE_BUTTON_ACK_ALARM, CPC_FACEPLATE_BUTTON_SET_AA_LIMITS, UN_FACEPLATE_BUTTON_BLOCK_ALARM_ACTION));
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogAlarm_ProvideAccessToButtons
/**
Defines the list of buttons available for a user. The result is passed in global variable.
	@param[in]	sDeviceName	Name of the device
	@param[in]	sDpType		Device Type
	@param[in] 	dsFunctions	A list of functions for a device, specified in device internal configuration.
	@param[out]	exceptionInfo	A list of exceptions.
*/
CPC_AnalogAlarm_ProvideAccessToButtons(string sDeviceName, string sDpType, dyn_string dsFunctions, dyn_string &exceptionInfo)
{
	string sFunction;
	dyn_string dsUserAccess;

	if (dynlen(dsFunctions) >= UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS) {
		sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
	} else {
		dynAppend(exceptionInfo, "AnalogAlarm_ButtonRegisterCB: the list of functions is not correct - number of functions is less than expected");
		return;
	}

	if (sFunction != "") {
		evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" +
					 "dyn_string dsUserAccessReturn;" +
					 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
					 "    {" + sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
					 "    }" +
					 "return dsUserAccessReturn; }", makeDynString(), sDeviceName, sDpType);

			g_dsUserAccess=dsUserAccess;
	} else {
		fwException_raise(exceptionInfo,"ERROR", "AnalogAlarm_ButtonRegisterCB:" + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");
	}
}

