/**@name LIBRARY: unAnalogInput.ctl

@author: Vincent Forest (AB-CO)

Creation Date: 21 05 2003

Modification History:
    02/03/2012: Alexey
        removed all dpGet from CPC_AnalogInput_FaceplateConnectionDPEs

    20/01/2012: Alexey
        fixed faceplate disconnection

    23/11/2011: Alexey
        [UCPC-603] refactor menu creation and handling

    03/11/2011: Alexey
        use stepPLC code from cpcStepPLC.ctl; same for transitionPLC

    31/10/2011: Alexey
        [UCPC-571] removed bargraph references

    04/10/2011: Alexey
        fixed alarm box animation in the faceplate

    03/10/2011: Alexey
        [UCPC-562] faceplate animation callback is now based on dyn_string of dpes
        animation of widget is moved from cpcGenericObject_WidgetAIAOAnimation; alarm animation based on _act_text

    29/09/2011: Alexey
        [UCPC-563] upd faceplate's buttons when alarms changed
        ack available when ack == true

    21/09/2011: Alexey
        [UCPC-546] use cpcGenericObject_ColorBoxAnimateAlarm

    20/09/2011: Alexey
        [UCPC-549][UCPC-548][UCPC-547] upd faceplate

	08/08/2011: Alexey
		fixed faceplate Disconnect animation

	02/08/2011: Alexey
		[UCPC-437] display H/L thresholds as warnings
		[UCPC-451] 'Alarm Not Ack' is never blinking

	07/07/2011: Alexey
		[UCPC-265] update alarm acknowledge

	18/05/2011: Alexey
		use cpcGenericObject_HandleEventMaskMenu and CPC functions to manage menu
		renamed unGenericObject_WidgetAIAOAnimation -> cpcGenericObject_WidgetAIAOAnimation

	18/04/2011: amerezhi
		[UCPC-234] added faceplate's box for the forced mode inhibited
	
	05/04/2011: amerezhi
		- [UCPC-201] added 'Manual IO Error Block' popup menu
					 added 'I/O Error blocked' box to the faceplate

  27/03/2009: Frederic
	  - ..._WidgetRegisterCB(): fix problem with deviceType check during disconnection
	  - ..._ButtonRegisterCB(): remove fwException_raise(...) before g_dsUserAccess
 
  03/09/2008: Herve
    - allow rigth click if lock taken.
    
  28/12/2007: Herve
    - device acccess control
    
	20/07/2007: Herve
		in faceplate: 
			- case alarm masked show the alarm limits 
			- when no alarm limits show "" instead of "???"
		
	26/06/2007: Herve
		- move all bargraph function to unGenericWidget
		
	21/11/2006: Herve
		- all bargraph function: new widget bargraphSimple, identical as bargraph without the range, alarm and value
		
	02/05/2006: Herve
		- implementation of maskEvent in widget, faceplate and rigthClick
		
	11/04/2006: Herve
		- remove dynAppend(menuList, "SEPARATOR"); in CPC_AnalogInput_MenuConfiguration function

	13/12/2005: Celine
		CPC_AnalogInput_FaceplateStatusRegisterCB,
		CPC_AnalogInput_FaceplateStatusAnimationCB,
		CPC_AnalogInput_WidgetRegisterCB,
		CPC_AnalogInput_bargraph_rangeAlertCB,
		CPC_AnalogInput_step_PLC_CB,
		CPC_AnalogInput_transition_PLC_CB : add g_bSystemConnected checking.
		
	09/08/2005: Frederic
		- CPC_AnalogInput_ButtonRegisterCB(), add evalScript to run dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS] function

	22/06/2005:
		- CPC_AnalogInput_FaceplateStatusAnimationCB, CPC_AnalogInput_FaceplateStatusDisconnection() :add Mail/SMS colorbox in the faceplate
	15/04/2005: Herve
		- unGenericObject_GetPLCNameFromDpName: return the PLCtype+"_"+plcName, in case of _UnPlc it returns _unPlc_+plcName
		
	10/11/2004: Herve
		- bargraph callback: if fMax-fMin = 0.0 set the scale to 0.

	05/11/2004: Herve
		- in the WidgetRegisterCB, faceplateRegiterCB: use the systemName of the $sIdentifier to dpConnect to the PLC

	26/10/2004: Herve
		- in the CPC_AnalogInput_ButtonRegisterCB function: call the CPC_AnalogInput_ButtonUserAccess to get the list of allowed buttons. This list is 
		reseted if the user logs in/out while the remote system is off.
		
	22/10/2004: Herve
		- modify the unGenericObject_FormatValue call
		- modify the unGenericObject_DisplayValue, unGenericObject_DisplayRange, unGenericObject_WidgetAIAOAnimation call

	18/10/2004: Herve
		- add to CPC_AnalogInput_MenuConfiguration the function call unGenericObject_addTrendActionToMenu

	11/05/2004: Frederic
		- update CPC_AnalogInput_transition_PLC_WidgetConnect to add checking of g_iBit <= Position Max
		- update CPC_AnalogInput_transition_PLC_WidgetDisconnect to call unGenericObject_GetPLCNameFromDpName
		- update CPC_AnalogInput_step_PLC_WidgetDisconnect to call unGenericObject_GetPLCNameFromDpName
	
	03/05/2004: Herve
		- during WidgetRegister and FaceplateRegister call unGenericObject_GetPLCNameFromDpName to get the PLC DPName
		- during WidgetRegister check if the dpType is correct

	28/04/2004: Herve
		- add function CPC_AnalogInput_ObjectListGetValueTime

	14/04/2004: Herve
		- add the function CPC_AnalogInput_HandleMenu
		- add the function CPC_AnalogInput_UserLoginGetButtonState

	13/04/2004: Herve
		- in function CPC_AnalogInput_ButtonRegisterCB add the set Manreg01 to 0 in case of dpConnect.
		- modification of the signature CPC_AnalogInput_MenuConfiguration and add the creation of the menu.

	08/04/2004: Herve 
		modification of the signature of the function CPC_AnalogInput_ButtonSetState

	25/03/2004: Herve
		in the function CPC_AnalogInput_ButtonUserAccess, the set_alarm_limit is accessible only for expert (was before also operator)
		add HH, H, L and LL, L, H case

	27/01/2004: Frederic
		- include Step_PLC and Transition_PLC Widget into the lib
	
	15/01/2004: Herve
		- remove the dpConnect, dpDisconnect and callback function in the case of no PLC systemAlarm. Ease the development 
		and test.
		- include the bargraph function into the lib

	26/11/2003: Herve
		- add in all dpConnect and dpDisconnect the _unSystemAlarm_PLC_DS dp if the PLC is configured for systemIntegrity. 
		The g_bUnSystemAlarmPlc is set to true if the PLC is configured for systemIntegrity
		change the dpConnect and dpDisconnect of h_aut_inv to _invalid
		- widget: remove the dpConnect on pv_range, not anymore needed, old version 2.3
		
	06/10/2003: Herve
		add dpConnect on the _lock._alert_hdl._locked in order to detect when the alert_hdl config modified.
		
	22/10/2003: Herve:
		add the following cases for alarm: H&L, HH&H, LL&L, H, L

version 1.0

External Function : 
	. CPC_AnalogInput_AcknowledgeAlarm : acknowledge AnalogInput alarms
	. CPC_AnalogInput_FaceplateStatusDisconnection : animate faceplate disconnection
	. CPC_AnalogInput_ButtonUserAccess : User access for AnalogInput buttons
	. CPC_AnalogInput_ButtonSetState : set button state
	. CPC_AnalogInput_WidgetDisconnection : animate widget disconnection
	. CPC_AnalogInput_MenuConfiguration : configuration of popup menu

Internal Functions :
	. CPC_AnalogInput_FaceplateStatusRegisterCB : register callback function
	. CPC_AnalogInput_FaceplateLockCallBack : set up the lock callback function for the faceplate
	. CPC_AnalogInput_FaceplateConnect: Connect function of the faceplate data
	. CPC_AnalogInput_FaceplateDisconnect: Disconnect function of the faceplate data
	. CPC_AnalogInput_FaceplateStatusAnimationCB_NoRangeNoAlarm : animate faceplate
	. CPC_AnalogInput_FaceplateStatusAnimationCB_NoAlarm : animate faceplate
	. CPC_AnalogInput_FaceplateStatusAnimationCB_NoRange : animate faceplate
	. CPC_AnalogInput_FaceplateStatusAnimationCB : animate faceplate
	. CPC_AnalogInput_ButtonRegisterCB : register callback function
	. CPC_AnalogInput_ButtonAnimationCB : animate buttons
	. CPC_AnalogInput_ButtonDisconnection : animate buttons disconnection
	. CPC_AnalogInput_WidgetRegisterCB : register callback function
	. CPC_AnalogInput_WidgetLockCallBack : set up the lock callback function for the widget
	. CPC_AnalogInput_WidgetConnect : Connect function of the widget data
	. CPC_AnalogInput_WidgetDisconnect : Disconnect function of the widget data
	. CPC_AnalogInput_WidgetAnimationCB_NoRangeNoAlarm : animate widget
	. CPC_AnalogInput_WidgetAnimationCB_NoRange : animate widget
	. CPC_AnalogInput_WidgetAnimationCB_NoAlarm : animate widget
	. CPC_AnalogInput_WidgetAnimationCB : animate widget
	

Purpose: This library contains AnalogInput object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. AnalogInput object (DPType etc.)
	. Global variables and $parameters defined in AnalogInput faceplate and widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogInput_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogInput_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	float value;
	string sTime;
	
	dpGet(sDeviceName+".ProcessInput.PosSt", value, sDeviceName+".ProcessInput.PosSt:_online.._stime", sTime);
	dsReturnData[1] = sTime;
	dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.PosSt"), value) + " " 
							+ dpGetUnit(sDeviceName+".ProcessInput.PosSt");
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogInput_AcknowledgeAlarm
/**
Purpose: acknowledge AnalogInput alarms

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsNeedAck, dyn_string, output, DPEs names that need acknowledgement. 
									 The PLC acknowledgement is specified using UN_ACKNOWLEDGE_PLC string
	
Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogInput_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
	dsNeedAck = makeDynString("ProcessInput.PosSt");
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogInput_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsAccessOk, dyn_string, input, authorized action
	- menuList, dyn_string, output, menu to pop-up
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogInput_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
	dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_MASK_POSST_TEXT, CPC_POPUPMENU_MANUAL_IO_ERROR_BLOCK,
								 UN_POPUPMENU_ACK_TEXT, UN_POPUPMENU_FACEPLATE_TEXT, UN_POPUPMENU_TREND_TEXT);

    cpcGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
    cpcGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
    cpcGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogInput_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogInput_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
	dyn_string exceptionInfo;

	cpcGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);

	if (dynlen(exceptionInfo) > 0)
	{
		unSendMessage_toExpertException("Right Click",exceptionInfo);
	}

}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogInput_FaceplateStatusRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogInput_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
    string deviceName, sSystemName, sPlcName;
    int iAction, iRes, iRangeType;
    dyn_string exceptionInfo;
    bool bRemote;
    
    deviceName = unGenericDpFunctions_getDpName($sDpName);
    sSystemName = unGenericDpFunctions_getSystemName(deviceName);
    
    unDistributedControl_isRemote(bRemote, sSystemName);
    if (bRemote) g_bSystemConnected = bSystemConnected;
    else g_bSystemConnected = true;

    sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
    g_bUnSystemAlarmPlc = sPlcName != "" && dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName);

	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			CPC_AnalogInput_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
			g_bDisconnectCB = false;
			if(g_bUnSystemAlarmPlc) {
				iRes = dpConnect("CPC_AnalogInput_FaceplateLockCallBack", deviceName + ".ProcessInput.PosSt:_lock._alert_hdl._locked");

				g_bCallbackConnected = (iRes >= 0);
			}
			else
				CPC_AnalogInput_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			if(g_bUnSystemAlarmPlc) 
				iRes = dpDisconnect("CPC_AnalogInput_FaceplateLockCallBack", deviceName + ".ProcessInput.PosSt:_lock._alert_hdl._locked");

			if(g_bDisconnectCB)
				CPC_AnalogInput_FaceplateDisconnect(deviceName, iRes);

			g_bCallbackConnected = !(iRes >= 0);
			g_bDisconnectCB = false;
			CPC_AnalogInput_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogInput_FaceplateLockCallBack
/**
Purpose: set up the lock callback function for the faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogInput widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogInput_FaceplateLockCallBack(string sdpe, bool bLockState)
{
	int iRes;
	string deviceName;
//DebugN("CPC_AnalogInput_FaceplateLockCallBack", bLockState, g_bDisconnectCB);

	deviceName = unGenericDpFunctions_getDpName(sdpe);
	if(bLockState) { // lock set so disconnect if not already disconnected
// no need to check if it was already set before because it is not possible to do 2 dpSet on the lock the Ev reject the message
		if(g_bDisconnectCB) {
			CPC_AnalogInput_FaceplateDisconnect(deviceName, iRes);
//DebugN("CPC_AnalogInput_FaceplateLockCallBack: disconnect");
			g_bDisconnectCB = false;
		}
		CPC_AnalogInput_FaceplateStatusDisconnection();
	}
	else { // lock not set so connect if not already connected
// no need to check if it was already set before because it is not possible to do 2 dpSet on the lock the Ev reject the message
		g_bDisconnectCB = true;
		CPC_AnalogInput_FaceplateConnect(deviceName);
//DebugN("CPC_AnalogInput_FaceplateLockCallBack: connect");
	}
}

dyn_string CPC_AnalogInput_FaceplateConnectionDPEs(string deviceName, string systemName, string plcName) {
    dyn_string dpes = makeDynString(systemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+plcName+".alarm",
                                    systemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+plcName+".enabled",
                                    deviceName + ".eventMask",
                                    deviceName + ".ProcessInput.StsReg01", 
                                    deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                    deviceName + ".ProcessInput.PosSt",
                                    deviceName + ".ProcessInput.PosSt:_online.._invalid",
                                    deviceName + ".ProcessInput.HFStOrAuPosRSt",
                                    deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid");
    if (g_iRangeType == DPCONFIG_MINMAX_PVSS_RANGECHECK) {
        dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_pv_range.._min");
        dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_pv_range.._max");
    }
    g_alertPositions["HH"] = 0;
    g_alertPositions["H"]  = 0;
    g_alertPositions["L"]  = 0;
    g_alertPositions["LL"] = 0;

    if (g_bBoolAlert) {
        dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_alert_hdl.._active");
        dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_range");
        dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible");
        for(int i = 1; i <= dynlen(g_rangeClasses); i++) {
            string rangeClass = g_rangeClasses[i];
            if (strpos(rangeClass, "_cpcAnalogHH") > 0) {
                dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_alert_hdl."+i+"._act_state");
                dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_alert_hdl."+i+"._l_limit");
                g_alertPositions["HH"] = i;
            } else if (strpos(rangeClass, "_cpcAnalogH") > 0) {
                dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_alert_hdl."+i+"._act_state");
                dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_alert_hdl."+i+"._l_limit");
                g_alertPositions["H"] = i;
            } else if (strpos(rangeClass, "_cpcAnalogLL") > 0) {
                dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_alert_hdl."+i+"._act_state");
                dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_alert_hdl."+i+"._u_limit");
                g_alertPositions["LL"] = i;
            } else if (strpos(rangeClass, "_cpcAnalogL") > 0) {
                dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_alert_hdl."+i+"._act_state");
                dynAppend(dpes, deviceName + ".ProcessInput.PosSt:_alert_hdl."+i+"._u_limit");
                g_alertPositions["L"] = i;
            }
       }
    }
    return dpes;
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogInput_FaceplateConnect
/**
Purpose: Connect function of the faceplate data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogInput widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogInput_FaceplateConnect(string deviceName)
{
    int iRes, iAlertType, iRangeType;
    dyn_string exceptionInfo, lastError;
    string plcName, systemName;

    systemName = unGenericDpFunctions_getSystemName(deviceName);
    plcName = unGenericObject_GetPLCNameFromDpName(deviceName);
    g_bUnSystemAlarmPlc = plcName != "" && dpExists(systemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+plcName);

    g_dbAlarmRanges = cpcGenericObject_GetAnalogAlarmType(deviceName + ".ProcessInput.PosSt", iAlertType, exceptionInfo);
    g_bBoolAlert = (iAlertType == DPCONFIG_ALERT_NONBINARYSIGNAL);
    g_rangeClasses = makeDynString();
    if (g_bBoolAlert) {
      int rangesNumber;
      dpGet(deviceName + ".ProcessInput.PosSt:_alert_hdl.._num_ranges", rangesNumber);
      for (int i = 1; i <= rangesNumber; i++) {
        string rangeClass = "#";
        dpGet(deviceName + ".ProcessInput.PosSt:_alert_hdl." + i + "._class", rangeClass);
        dynAppend(g_rangeClasses, rangeClass);
      }
    }
    iRes = dpGet(deviceName + ".ProcessInput.PosSt:_pv_range.._type", iRangeType);
    if ((iRes >= 0) && (iRangeType == DPCONFIG_MINMAX_PVSS_RANGECHECK)) {
        g_iRangeType = DPCONFIG_MINMAX_PVSS_RANGECHECK;
    } else {
        g_iRangeType = DPCONFIG_NONE;
    }

    g_sPosStFormat = dpGetFormat(deviceName + ".ProcessInput.PosSt");
    g_sPosStUnit = dpGetUnit(deviceName + ".ProcessInput.PosSt");
    g_sHFStOrAuPosRStFormat = dpGetFormat(deviceName + ".ProcessInput.HFStOrAuPosRSt");
    g_sHFStOrAuPosRStUnit = dpGetUnit(deviceName + ".ProcessInput.HFStOrAuPosRSt");
    g_bAlAck = cpcExportGenericFunctions_get5AlertAcknowledgeAlarmValue(deviceName, exceptionInfo) == "TRUE";
    g_bMailSMS = cpcGenericObject_isAlarmWithMail(deviceName + ".ProcessInput.PosSt", iAlertType);

    if(g_bUnSystemAlarmPlc) {
        dyn_string dpes = CPC_AnalogInput_FaceplateConnectionDPEs(deviceName, systemName, plcName);
        iRes = dpConnect("CPC_AnalogInput_FaceplateStatusAnimationCB", dpes);
        lastError = getLastError();
    } else {
        CPC_AnalogInput_FaceplateStatusDisconnection();
    }
    g_bCallbackConnected = (iRes >= 0) && dynlen(lastError) == 0;
    if (dynlen(lastError) > 0) {
        DebugN(lastError);
        fwException_raise(exceptionInfo,"ERROR", "CPC_AnalogInput_FaceplateConnect: dpConnect error: " + lastError, "EMPTY");
    }
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogInput_FaceplateDisconnect
/**
Purpose: Disconnect function of the faceplate data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogInput widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogInput_FaceplateDisconnect(string deviceName, int &iRes)
{
    string plcName, systemName;

    if(g_bUnSystemAlarmPlc) {
        systemName = unGenericDpFunctions_getSystemName(deviceName);
        plcName = unGenericObject_GetPLCNameFromDpName(deviceName);
        dyn_string dpes = CPC_AnalogInput_FaceplateConnectionDPEs(deviceName, systemName, plcName);
        iRes = dpDisconnect("CPC_AnalogInput_FaceplateStatusAnimationCB", dpes);
        g_bCallbackConnected = !(iRes >= 0);
    } else {
        g_bCallbackConnected = false;
    }
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogInput_FaceplateStatusAnimationCB
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnalogInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogInput_FaceplateStatusAnimationCB(dyn_string dpes, dyn_anytype values)
{
    // unit and format can be as global mappings
    cpcFaceplate_animateOnlineValue(dpes, values, "PosSt", g_sPosStUnit, g_sPosStFormat, "cpcColor_Faceplate_Status");
    cpcFaceplate_animateOnlineValue(dpes, values, "HFStOrAuPosRSt", g_sHFStOrAuPosRStUnit, g_sHFStOrAuPosRStFormat, "cpcColor_Faceplate_Status");

    cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_AUMOST", true, "cpcColor_Faceplate_Status");
    cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_FOMOST", true, "cpcColor_Faceplate_Status");
    cpcFaceplate_animateStsRegBit(dpes, values, "CPC_STSREG01_AUIHFOMOST", true, "cpcColor_Faceplate_Status");

    cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_IOERRORW", true, "unFaceplate_WarningActive");
    cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_IOSIMUW", true, "unFaceplate_WarningActive");
    cpcFaceplate_animateStsRegBit(dpes, values, "UN_STSREG01_FODIPRO", true, "unFaceplate_WarningActive");
    cpcFaceplate_animateStsRegBit(dpes, values, "CPC_STSREG01_MIOERBRST", true, "unFaceplate_WarningActive");
    cpcFaceplate_animateMaskEvent(dpes, values);
    
    cpcFaceplate_animateRange(dpes, values, "PosSt", g_iRangeType, g_sPosStUnit, g_sPosStFormat);

    if (!g_bBoolAlert) { // if does not exist
        cpcGenericObject_ColorBoxAnimateAlarm("Alarm_HH", 0, false, false, false);
        cpcGenericObject_ColorBoxAnimateWarning("Alarm_H", 0, false, false, false);
        cpcGenericObject_ColorBoxAnimateWarning("Alarm_L", 0, false, false, false);
        cpcGenericObject_ColorBoxAnimateAlarm("Alarm_LL", 0, false, false, false);
        cpcGenericObject_ColorBoxAnimateAlarm("Alarm_UnAck", 0, false, false, false);
        cpcGenericObject_CheckboxAnimate("MailSMS", false, false);
         if(g_bSystemConnected)
            setMultiValue("ValueHH.display", "text", "", "ValueHH.display", "foreCol", "unAlarmDoesNotExist",
                          "ValueH.display", "text", "", "ValueH.display", "foreCol", "unAlarmDoesNotExist",
                          "ValueL.display", "text", "", "ValueL.display", "foreCol", "unAlarmDoesNotExist",
                          "ValueLL.display", "text", "", "ValueLL.display", "foreCol", "unAlarmDoesNotExist");
    } else { // animate alarms
        bool ackPossible = cpcFaceplate_fetchAnimationCBValue(dpes, values, "ProcessInput.PosSt:_alert_hdl.._ack_possible");
        bool alarmActive = cpcFaceplate_fetchAnimationCBValue(dpes, values, "ProcessInput.PosSt:_alert_hdl.._active");

        cpcGenericObject_CheckboxAnimate("MailSMS", g_bMailSMS, alarmActive);
        cpcGenericObject_ColorBoxAnimate("Alarm_UnAck", ackPossible, alarmActive && g_bAlAck, "cpcColor_Alarm_Bad", false);

        float alarmRange;
        int alarmState;
        if (g_alertPositions["HH"] > 0) {
            alarmRange = cpcFaceplate_fetchAnimationCBValue(dpes, values, "ProcessInput.PosSt:_alert_hdl." + g_alertPositions["HH"] + "._l_limit");
            alarmState = cpcFaceplate_fetchAnimationCBValue(dpes, values, "ProcessInput.PosSt:_alert_hdl." + g_alertPositions["HH"] + "._act_state");
            unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, alarmRange, "ValueHH", "unDisplayValue_Parameter", false);
            cpcGenericObject_ColorBoxAnimateAlarm("Alarm_HH", alarmState, g_bBoolAlert, alarmActive, false);
        } else {
            setMultiValue("ValueHH.display", "text", "", "ValueHH.display", "foreCol", "unAlarmDoesNotExist");
            cpcGenericObject_ColorBoxAnimateAlarm("Alarm_HH", 0, false, false, false);
        }
        if (g_alertPositions["LL"] > 0) {
            alarmRange = cpcFaceplate_fetchAnimationCBValue(dpes, values, "ProcessInput.PosSt:_alert_hdl." + g_alertPositions["LL"] + "._u_limit");
            alarmState = cpcFaceplate_fetchAnimationCBValue(dpes, values, "ProcessInput.PosSt:_alert_hdl." + g_alertPositions["LL"] + "._act_state");
            unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, alarmRange, "ValueLL", "unDisplayValue_Parameter", false);
            cpcGenericObject_ColorBoxAnimateAlarm("Alarm_LL", alarmState, g_bBoolAlert, alarmActive, false);
        } else {
            setMultiValue("ValueLL.display", "text", "", "ValueLL.display", "foreCol", "unAlarmDoesNotExist");
            cpcGenericObject_ColorBoxAnimateAlarm("Alarm_LL", 0, false, false, false);
        }
        if (g_alertPositions["H"] > 0) {
            alarmRange = cpcFaceplate_fetchAnimationCBValue(dpes, values, "ProcessInput.PosSt:_alert_hdl." + g_alertPositions["H"] + "._l_limit");
            alarmState = cpcFaceplate_fetchAnimationCBValue(dpes, values, "ProcessInput.PosSt:_alert_hdl." + g_alertPositions["H"] + "._act_state");
            unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, alarmRange, "ValueH", "unDisplayValue_Parameter", false);
            cpcGenericObject_ColorBoxAnimateWarning("Alarm_H", alarmState, g_bBoolAlert, alarmActive, false);
        } else {
            setMultiValue("ValueH.display", "text", "", "ValueH.display", "foreCol", "unAlarmDoesNotExist");
            cpcGenericObject_ColorBoxAnimateWarning("Alarm_H", 0, false, false, false);
        }
        if (g_alertPositions["L"] > 0) {
            alarmRange = cpcFaceplate_fetchAnimationCBValue(dpes, values, "ProcessInput.PosSt:_alert_hdl." + g_alertPositions["L"] + "._u_limit");
            alarmState = cpcFaceplate_fetchAnimationCBValue(dpes, values, "ProcessInput.PosSt:_alert_hdl." + g_alertPositions["L"] + "._act_state");
            unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, alarmRange, "ValueL", "unDisplayValue_Parameter", false);
            cpcGenericObject_ColorBoxAnimateWarning("Alarm_L", alarmState, g_bBoolAlert, alarmActive, false);
        } else {
            setMultiValue("ValueL.display", "text", "", "ValueL.display", "foreCol", "unAlarmDoesNotExist");
            cpcGenericObject_ColorBoxAnimateWarning("Alarm_L", 0, false, false, false);
        }
    }
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogInput_FaceplateStatusDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogInput_FaceplateStatusDisconnection()
{
    unGenericObject_ColorBoxDisconnect(makeDynString("Alarm_HH", "Alarm_H", "Alarm_UnAck", "Alarm_LL", "Alarm_L",
                                                     "AUMOST", "FOMOST", "AUIHFOMOST",
                                                     "IOERRORW", "IOSIMUW", "FODIPRO", "MIOERBRST",
                                                     "eventState"));
    unGenericObject_DisplayValueDisconnect(makeDynString("PosStRangeMax", "PosStRangeMin", "PosSt", "HFStOrAuPosRSt",
                                                     "ValueHH", "ValueH", "ValueL", "ValueLL"));
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogInput_ButtonRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogInput_ButtonRegisterCB(string sDp, bool bSystemConnected)
{
    string deviceName;
    int iAction, iRes;
    dyn_string exceptionInfo;

    deviceName = unGenericDpFunctions_getDpName($sDpName);
    unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);

    switch (iAction) {
        case UN_ACTION_DISCONNECT:
            CPC_AnalogInput_ButtonDisconnection();
            break;
        case UN_ACTION_DPCONNECT:
            g_bDisconnectCB = false;
            iRes = dpConnect("CPC_AnalogInput_ButtonLockCallBack", deviceName + ".ProcessInput.PosSt:_lock._alert_hdl._locked");
            g_bCallbackConnected = (iRes >= 0);
            break;
        case UN_ACTION_DPDISCONNECT_DISCONNECT:
            iRes = dpDisconnect("CPC_AnalogInput_ButtonLockCallBack", deviceName + ".ProcessInput.PosSt:_lock._alert_hdl._locked");

            if(g_bDisconnectCB)
                CPC_AnalogInput_ButtonDisconnect(deviceName, iRes);

            g_bCallbackConnected = !(iRes >= 0);
            g_bDisconnectCB = false;
            CPC_AnalogInput_ButtonDisconnection();
            break;
        case UN_ACTION_NOTHING:
        default:
            break;
    }
}

CPC_AnalogInput_ButtonLockCallBack(string sdpe, bool bLockState)
{
    int iRes;
    string deviceName;

    deviceName = unGenericDpFunctions_getDpName(sdpe);
    if(bLockState) { // lock set so disconnect if not already disconnected
        if(g_bDisconnectCB) {
            CPC_AnalogInput_ButtonDisconnect(deviceName, iRes);
            g_bDisconnectCB = false;
        }
        CPC_AnalogInput_ButtonDisconnection();
    } else { // lock not set so connect if not already connected
        g_bDisconnectCB = true;
        CPC_AnalogInput_ButtonConnect(deviceName);
    }
}

CPC_AnalogInput_ButtonConnect(string deviceName)
{
    int iRes, iManReg01;
    string sDpType, sFunction;
    dyn_string dsFunctions, dsUserAccess, exceptionInfo;

    // Send 0 to ManReg01 if necessary
    iRes = dpGet(deviceName + ".ProcessOutput.ManReg01", iManReg01);
    if ((iRes < 0) || (iManReg01 != 0)) {
        dpSet(deviceName + ".ProcessOutput.ManReg01", 0);
    }

    sDpType=UN_CONFIG_CPC_ANALOGINPUT_DPT_NAME;
    unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
    sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];

    if (sFunction != "") {
        evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" +
                     "dyn_string dsUserAccessReturn;" +
                     "if (isFunctionDefined(\"" + sFunction  + "\"))" +
                     "    {" +
                        sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
                     "    }" +
                     "return dsUserAccessReturn; }", makeDynString(), deviceName, sDpType);
        g_dsUserAccess=dsUserAccess;
    } else {
        fwException_raise(exceptionInfo,"ERROR", "CPC_AnalogInput_ButtonRegisterCB:" + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");
    }

    iRes = dpConnect("CPC_AnalogInput_ButtonAnimationCB",
                     deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                     deviceName + ".statusInformation.selectedManager",
                     deviceName + ".ProcessInput.StsReg01",
                     deviceName + ".ProcessInput.StsReg01:_online.._invalid");
    g_bCallbackConnected = (iRes >= 0);
}

CPC_AnalogInput_ButtonDisconnect(string deviceName, int &iRes) {
    iRes = dpDisconnect("CPC_AnalogInput_ButtonAnimationCB",
                     deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                     deviceName + ".statusInformation.selectedManager",
                     deviceName + ".ProcessInput.StsReg01",
                     deviceName + ".ProcessInput.StsReg01:_online.._invalid");
    g_bCallbackConnected = !(iRes >= 0);
    CPC_AnalogInput_ButtonDisconnection();
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogInput_ButtonAnimationCB
/**
Purpose: animate buttons

Parameters: 
	- dbPermissions, dyn_bool, input, list of permissions, length = 4, order : 1=monitor, 2=operator, 3=expert, 4=admin
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogInput_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager, 
						   		string sDpStsReg01, bit32 stsReg01Value, string sDpStsReg01Bad, bool bStsReg01Bad)
{
	CPC_AnalogInput_ButtonSetState(unGenericDpFunctions_getDpName(sDpStsReg01), UN_CONFIG_CPC_ANALOGINPUT_DPT_NAME, g_dsUserAccess, 
									makeDynString(bLocked, sSelectedManager, stsReg01Value, bStsReg01Bad, 0, 0));
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogInput_ButtonUserAccess
/**
Purpose: User access for AnalogInput buttons

Parameters: 
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogInput_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	bool operator, expert, admin;
	int i, length;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
	string sOperAction, sExpAction, sAdminAction;
	bool bActionDefined = false;

	unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
	if ((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME)) {
		bActionDefined=true;
	}
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	if (operator) {
		if (bActionDefined) {
			dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
		} else {
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_SET_VALUE, UN_FACEPLATE_BUTTON_MASK_ALARM, UN_FACEPLATE_BUTTON_SELECT));
			if (cpcExportGenericFunctions_get5AlertAcknowledgeAlarmValue(sDpName, exceptionInfo) == "TRUE") {
				dynAppend(dsAccess, UN_FACEPLATE_BUTTON_ACK_ALARM);
			}
		}
	}
	if (expert) {
		if(bActionDefined) {
			dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
		} else {
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_BUTTON_SET_ALARM_LIMITS, UN_FACEPLATE_MASKEVENT));
		}
	}
	if (admin) {
		if(bActionDefined) {
			dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
		}
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogInput_UserLoginGetButtonState
/**
Purpose: function called before the button set state when a user logs in. 

Parameters:
	deviceName: string, input, the device name
	dsReturnData: dyn_string, output, the data are returned here

Usage: Internal function

PVSS manager usage:  Ctrl, NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
		-> $sDpName, string, the device DP name
		-> g_sFaceplateButtonType, string, faceplate type
	. PVSS version: 2.12.1 
	. operating system: NT, W2000 and Linux, but tested only under W2000.
	. distributed system: yes but not tested.
*/
CPC_AnalogInput_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
	unGenericObject_UserCBGetButtonState(deviceName, sType, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogInput_ButtonSetState
/**
Purpose: set button state

Parameters: 
	- deviceName, string, input, device
	- sDpType, string, input, dptype
	- dsUserAccess, dyn_string, input, user access
	- dsData, dyn_string, parameter
		
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogInput_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected, buttonEnabled, bit1, bit2;
	dyn_string dsButtons;
	int i, length, alertType;
	string localManager;
	bool bLocked, bStsReg01Bad, bStsReg02Bad;
	string sSelectedManager;
	bit32 stsReg01Value, stsReg02Value; 

	bLocked = dsData[1];
	sSelectedManager = dsData[2];
	stsReg01Value = dsData[3];
	bStsReg01Bad = dsData[4];
	stsReg02Value = dsData[5];
	bStsReg02Bad = dsData[6];

	dsButtons = makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_SET_VALUE, UN_FACEPLATE_BUTTON_ACK_ALARM, 
							  UN_FACEPLATE_BUTTON_SET_ALARM_LIMITS, UN_FACEPLATE_BUTTON_FORCED_MODE); // Each button except "Select"
	length = dynlen(dsButtons);
// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");
// 2. Button state
	for(i=1;i<=length;i++)
		{
		buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);	// User access
		switch (dsButtons[i])
			{
			case UN_FACEPLATE_BUTTON_FORCED_MODE:
				bit1 = !(getBit(stsReg01Value, UN_STSREG01_FOMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				bit2 = !(getBit(stsReg01Value, UN_STSREG01_AUINHFMO) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				buttonEnabled = buttonEnabled && bSelected && !(bit1 || bit2);
				break;
			case UN_FACEPLATE_BUTTON_AUTO_MODE:
			case UN_FACEPLATE_BUTTON_SET_VALUE:
				bit1 = !(getBit(stsReg01Value, UN_STSREG01_AUMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				buttonEnabled = buttonEnabled && bSelected && !(bit1);
				break;
			case UN_FACEPLATE_BUTTON_SET_ALARM_LIMITS:
			case UN_FACEPLATE_BUTTON_ACK_ALARM:
			default:
				break;
			}
		setValue(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", buttonEnabled);
		}
// 3. Button Select state
	unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogInput_ButtonDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogInput_ButtonDisconnection()
{
	unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_SET_VALUE,  
												   UN_FACEPLATE_BUTTON_ACK_ALARM, UN_FACEPLATE_BUTTON_SET_ALARM_LIMITS, 
												   UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_BUTTON_SELECT));
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogInput_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogInput widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogInput_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
    string deviceName, deviceType, sSystemName, sPlcName;
    int iAction, iRes;
    dyn_string exceptionInfo;
    bool bRemote;

    deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
    if (deviceName == "") {// In case of disconnection
        deviceName = g_sDpName;
    }

    sSystemName = unGenericDpFunctions_getSystemName(deviceName);
    sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);

    unDistributedControl_isRemote(bRemote, sSystemName);
    if(bRemote) g_bSystemConnected = bSystemConnected;
    else g_bSystemConnected = true;
    g_bUnSystemAlarmPlc = sPlcName != "" && dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName);

    if (deviceName != "") {    
        deviceType = dpTypeName(deviceName);
        if((deviceType != UN_CONFIG_CPC_ANALOGINPUT_DPT_NAME) && (deviceType != UN_CONFIG_CPC_ANALOGOUTPUT_DPT_NAME))        
            g_bUnSystemAlarmPlc = false;
    }

	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			CPC_AnalogInput_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:

			g_sDpName = deviceName;
			g_bDisconnectCB = false;

		if(g_bUnSystemAlarmPlc) {
				iRes = dpConnect("CPC_AnalogInput_WidgetLockCallBack", deviceName + ".ProcessInput.PosSt:_lock._alert_hdl._locked");
				g_bCallbackConnected = (iRes >= 0);
			}
			else
					CPC_AnalogInput_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			if(g_bUnSystemAlarmPlc) 
				iRes = dpDisconnect("CPC_AnalogInput_WidgetLockCallBack", deviceName + ".ProcessInput.PosSt:_lock._alert_hdl._locked");

			if(g_bDisconnectCB) {
                CPC_AnalogInput_AIAO_WidgetDisconnect(g_sDpName, iRes);
			}
			g_bCallbackConnected = !(iRes >= 0);
			g_bDisconnectCB = false;

			CPC_AnalogInput_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogInput_WidgetLockCallBack
/**
Purpose: set up the lock callback function for the widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogInput widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogInput_WidgetLockCallBack(string sdpe, bool bLockState)
{
	int iRes;

	if(bLockState) { // lock set so disconnect if not already disconnected
// no need to check if it was already set before because it is not possible to do 2 dpSet on the lock the Ev reject the message
		if(g_bDisconnectCB) {
            CPC_AnalogInput_AIAO_WidgetDisconnect(g_sDpName, iRes);
			g_bDisconnectCB = false;
		}
		CPC_AnalogInput_WidgetDisconnection(g_sWidgetType);
        setValue("WidgetArea", "visible", true);
	}
	else { // lock not set so connect if not already connected
// no need to check if it was already set before because it is not possible to do 2 dpSet on the lock the Ev reject the message
		g_bDisconnectCB = true;

        CPC_AnalogInput_AIAO_WidgetConnect(g_sDpName);
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogInput_AIAO_WidgetConnect
/**
Purpose: Connect function of the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnalogInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogInput_AIAO_WidgetConnect(string deviceName)
{
    int iRes;
    dyn_string exceptionInfo;
    string sPlcName, sSystemName;

    sSystemName = unGenericDpFunctions_getSystemName(deviceName);
    sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
    g_bUnSystemAlarmPlc = (sPlcName != "") && dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName);

    cpcGenericObject_GetAnalogAlarmType(deviceName + ".ProcessInput.PosSt", g_iAlertType, exceptionInfo);

    g_sPosStFormat = dpGetFormat(deviceName + ".ProcessInput.PosSt");
    g_sPosStUnit = dpGetUnit(deviceName + ".ProcessInput.PosSt");

    if(g_bUnSystemAlarmPlc){
        if (g_iAlertType == DPCONFIG_ALERT_NONBINARYSIGNAL) {
            iRes = dpConnect("CPC_AnalogInput_WidgetAnimationCB", 
                                deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                                deviceName + ".statusInformation.selectedManager",
                                deviceName + ".ProcessInput.StsReg01",
                                deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                deviceName + ".ProcessInput.PosSt",
                                deviceName + ".ProcessInput.PosSt:_online.._invalid",
                                deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
                                deviceName + ".ProcessInput.PosSt:_alert_hdl.._active",
                                deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_text",
                                deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_range",
                                deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible",
                                sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
                                sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
                                deviceName + ".eventMask");
        } else {
            iRes = dpConnect("CPC_AnalogInput_WidgetAnimationCB_NoAlarm", 
                            deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                            deviceName + ".statusInformation.selectedManager",
                            deviceName + ".ProcessInput.StsReg01",
                            deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                            deviceName + ".ProcessInput.PosSt",
                            deviceName + ".ProcessInput.PosSt:_online.._invalid",
                            deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
                            deviceName + ".eventMask");
        }
    } else {
        CPC_AnalogInput_WidgetDisconnection(g_sWidgetType);
    }
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogInput_AIAO_WidgetDisconnect
/**
Purpose: Disconnect function of the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnalogInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogInput_AIAO_WidgetDisconnect(string deviceName, int &iRes)
{
    string sPlcName, sSystemName;

    if(g_bUnSystemAlarmPlc) {
        sSystemName = unGenericDpFunctions_getSystemName(deviceName);
        sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);

        if (g_iAlertType == DPCONFIG_ALERT_NONBINARYSIGNAL) {
            iRes = dpDisconnect("CPC_AnalogInput_WidgetAnimationCB", 
                        deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                        deviceName + ".statusInformation.selectedManager",
                        deviceName + ".ProcessInput.StsReg01",
                        deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                        deviceName + ".ProcessInput.PosSt",
                        deviceName + ".ProcessInput.PosSt:_online.._invalid",
                        deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
                        deviceName + ".ProcessInput.PosSt:_alert_hdl.._active",
                        deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_text",
                        deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_range",
                        deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible",
                        sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
                        sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
                        deviceName + ".eventMask");
        } else {
            iRes = dpDisconnect("CPC_AnalogInput_WidgetAnimationCB_NoAlarm", 
                        deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                        deviceName + ".statusInformation.selectedManager",
                        deviceName + ".ProcessInput.StsReg01",
                        deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                        deviceName + ".ProcessInput.PosSt",
                        deviceName + ".ProcessInput.PosSt:_online.._invalid",
                        deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
                        sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
                        sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
                        deviceName + ".eventMask");
        }
    }
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogInput_WidgetAnimationCB_NoAlarm
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnalogInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogInput_WidgetAnimationCB_NoAlarm(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
                                        string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
                                        string sDpPosSt, float fPosSt, string sDp6, bool bPosStInvalid,
                                        string sDp7, bool bHFStOrAuPosRStInvalid,
                                        string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled, string sEvtMask, int iMask)
{
    CPC_AnalogInput_WidgetAnimationCB(sDpLock, bLocked, sDpManager, sSelectedManager, sDpStsReg01, bit32StsReg01, sDp4, bStsReg01Invalid,
                                    sDpPosSt, fPosSt, sDp6, bPosStInvalid, sDp7, bHFStOrAuPosRStInvalid,
                                    "", 0, "", 0, "", 0, "", 0, sDpAl, iAlValue, sDpEnabled, bPlcEnabled, sEvtMask, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogInput_WidgetAnimationCB
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnalogInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogInput_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
                                string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
                                string sDpPosSt, float fPosSt, string sDp6, bool bPosStInvalid,
                                string sDp7, bool bHFStOrAuPosRStInvalid,
                                string sDp10, bool isAlarmActive, string sDp13, string actText, string sDp11, int actRange, string sDp12, bool isUnAck,
                                string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled, string sEvtMask, int iMask)
{
    string formattedValue;
    string warningLetter, warningColor, alarmLetter, alarmColor, controlLetter, controlColor, body1Text, body1Color, selectColor;
    bool lockVisible;

    body1Color = "cpcColor_Widget_Status";					// Auto mode (default)
    controlLetter = "";
    if (!bStsReg01Invalid) {
        if (getBit(bit32StsReg01, UN_STSREG01_FOMOST) == 1) {
        body1Color = "unWidget_ControlStateForced";				// Forced mode
        controlLetter = CPC_WIDGET_TEXT_CONTROL_FORCED;
        controlColor = "unWidget_ControlStateForced";
        } else if (getBit(bit32StsReg01, CPC_STSREG01_AUIHFOMOST) == 1) {	// Forced mode inhibited
            controlLetter = CPC_WIDGET_TEXT_CONTROL_AUIHFOMOST;
        }
    }

    if (g_iAlertType != DPCONFIG_ALERT_NONBINARYSIGNAL) {
        alarmLetter = CPC_WIDGET_TEXT_ALARM_DOES_NOT_EXIST;
        alarmColor = "unAlarmDoesNotExist";
    } else if (!isAlarmActive) {
        alarmLetter = CPC_WIDGET_TEXT_ALARM_MASKED;
        alarmColor = "unAlarmMasked";
    } else {
        if (isUnAck) {
            if (actText == "HH" || actText == "LL") {
                alarmLetter = actText;
            }
            alarmColor = "unWidget_AlarmNotAck";
        } else {
            if (actText == "H" || actText == "L") {
                alarmLetter = actText;
                alarmColor = "unFaceplate_WarningActive";
            } else if (actText == "HH" || actText == "LL") {
                alarmLetter = actText;
                alarmColor = "unFaceplate_AlarmActive";
            }
        }
        if (alarmColor != "") {
            body1Color = alarmColor;
        }
    }

    if (!bStsReg01Invalid) {
        unGenericObject_WidgetWarningAnimation(bit32StsReg01, 
                       makeDynInt(UN_STSREG01_FODIPRO, UN_STSREG01_IOSIMUW, UN_STSREG01_IOERRORW, CPC_STSREG01_MIOERBRST),
                       makeDynString(CPC_WIDGET_TEXT_WARNING_DEFAULT, CPC_WIDGET_TEXT_WARNING_SIMU, CPC_WIDGET_TEXT_WARNING_ERROR, CPC_WIDGET_TEXT_WARNING_ALARM_BLOCKED),
                       warningLetter, warningColor);
    }
    unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, selectColor, lockVisible);

    formattedValue = unGenericObject_FormatValue(g_sPosStFormat, fPosSt);
    body1Text = formattedValue + " " + g_sPosStUnit;
    if (bPosStInvalid) {
        body1Color = "unDataNotValid";
    }

    if ((!bPlcEnabled) || (iAlValue > c_unSystemIntegrity_no_alarm_value)) {
        warningLetter = CPC_WIDGET_TEXT_OLD_DATA;
        warningColor = "unDataNotValid";
    }

    if (bStsReg01Invalid || bPosStInvalid || bHFStOrAuPosRStInvalid) {
        warningLetter = CPC_WIDGET_TEXT_INVALID;
        warningColor = "unDataNotValid";
    }

    if (g_bSystemConnected)
        setMultiValue("Body1", "text", body1Text, "Body1", "foreCol", body1Color,
                      "WarningText", "text", warningLetter, "WarningText", "foreCol", warningColor,
                      "AlarmText", "text", alarmLetter, "AlarmText", "foreCol", alarmColor,
                      "ControlStateText", "text", controlLetter, "ControlStateText", "foreCol", controlColor,
                      "SelectArea", "foreCol", selectColor, "LockBmp", "visible", lockVisible,
                      "WidgetArea", "visible", true);

    if (g_bSystemConnected) {
        eventState.visible	= iMask == 0;
    }
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogInput_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType, string, input, widget type. ex: "heater"

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogInput_WidgetDisconnection(string sWidgetType)
{
    cpcGenericObject_WidgetDisconnection(sWidgetType, UN_CONFIG_CPC_ANALOGINPUT_DPT_NAME);
    eventState.visible	= false;
}

