/**@name LIBRARY: unAnalogOutput.ctl

@author: Vincent Forest (AB-CO)

Creation Date: 21 05 2003

Modification History: 
	07/07/2011: Alexey
		[UCPC-265] update alarm acknowledge

	28/04/2004: Herve
		- add function CPC_AnalogOutput_ObjectListGetValueTime

	14/04/2004: Herve
		- add the function CPC_AnalogOutput_HandleMenu
		- add the function CPC_AnalogOutput_UserLoginGetButtonState

	13/04/2004: Herve
		- modification of the signature CPC_AnalogOutput_MenuConfiguration.

	08/04/2004: Herve 
		modification of the signature of the function CPC_AnalogOutput_ButtonSetState

	12/11/2003: Herve
		for the faceplate CB function, call the analogInput function.
		
	06/10/2003: Herve
		add dpConnect on the _lock._alert_hdl._locked in order to detect when the alert_hdl config modified.

version 1.0

External Function : 
	. CPC_AnalogOutput_AcknowledgeAlarm : acknowledge AnalogOutput alarms
	. CPC_AnalogOutput_FaceplateStatusRegisterCB : register callback function
	. CPC_AnalogInput_FaceplateStatusDisconnection : animate faceplate disconnection
	. CPC_AnalogOutput_ButtonRegisterCB : register callback function
	. CPC_AnalogOutput_ButtonSetState : set button state
	. CPC_AnalogOutput_ButtonDisconnection : animate buttons disconnection
	. CPC_AnalogOutput_ButtonUserAccess : User access for AnalogOutput buttons
	. CPC_AnalogOutput_WidgetRegisterCB : register callback function
	. CPC_AnalogOutput_WidgetDisconnection : animate widget disconnection
	. CPC_AnalogOutput_MenuConfiguration : configuration of popup menu

Internal Functions :

Purpose: This library contains AnalogOutput object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. AnalogOutput object (DPType etc.)
	. Global variables and $parameters defined in AnalogOutput faceplate and widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogOutput_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogOutput_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	CPC_AnalogInput_ObjectListGetValueTime(sDeviceName, sDeviceType, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogOutput_AcknowledgeAlarm
/**
Purpose: acknowledge AnalogOutput alarms

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsNeedAck, dyn_string, output, DPEs names that need acknowledgement. 
									 The PLC acknowledgement is specified using UN_ACKNOWLEDGE_PLC string
	
Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogOutput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogOutput_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
	CPC_AnalogInput_AcknowledgeAlarm(sDpName, sDpType, dsNeedAck);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogOutput_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsAccessOk, dyn_string, input, authorized action
	- menuList, dyn_string, output, menu to pop-up
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogOutput_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
	CPC_AnalogInput_MenuConfiguration(sDpName, sDpType, dsAccessOk, menuList);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogOutput_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogOutput_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
	CPC_AnalogInput_HandleMenu(deviceName, sDpType, menuList, menuAnswer);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogOutput_FaceplateStatusRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogOutput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogOutput_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
	CPC_AnalogInput_FaceplateStatusRegisterCB(sDp, bSystemConnected);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogOutput_FaceplateStatusDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogOutput_FaceplateStatusDisconnection()
{
	CPC_AnalogInput_FaceplateStatusDisconnection();
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogOutput_ButtonRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogOutput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogOutput_ButtonRegisterCB(string sDp, bool bSystemConnected)
{
	CPC_AnalogInput_ButtonRegisterCB(sDp, bSystemConnected);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogOutput_ButtonUserAccess
/**
Purpose: User access for AnalogOutput buttons

Parameters: 
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogOutput_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	CPC_AnalogInput_ButtonUserAccess(sDpName, sDpType, dsAccess);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogOutput_UserLoginGetButtonState
/**
Purpose: function called before the button set state when a user logs in. 

Parameters:
	deviceName: string, input, the device name
	dsReturnData: dyn_string, output, the data are returned here

Usage: Internal function

PVSS manager usage:  Ctrl, NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
		-> $sDpName, string, the device DP name
		-> g_sFaceplateButtonType, string, faceplate type
	. PVSS version: 2.12.1 
	. operating system: NT, W2000 and Linux, but tested only under W2000.
	. distributed system: yes but not tested.
*/
CPC_AnalogOutput_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
	CPC_AnalogInput_UserLoginGetButtonState(deviceName, sType, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogOutput_ButtonSetState
/**
Purpose: set button state

Parameters: 
	- deviceName, string, input, device
	- sDpType, string, input, dptype
	- dsUserAccess, dyn_string, input, user access
	- dsData, dyn_string, parameter
		
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogOutput_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	CPC_AnalogInput_ButtonSetState(deviceName, sDpType, dsUserAccess, dsData);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogOutput_ButtonDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogOutput_ButtonDisconnection()
{
	CPC_AnalogInput_ButtonDisconnection();
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogOutput_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogOutput widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogOutput_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	CPC_AnalogInput_WidgetRegisterCB(sDp, bSystemConnected);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_AnalogOutput_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType, string, input, widget type. ex: "heater"

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_AnalogOutput_WidgetDisconnection(string sWidgetType)
{
	cpcGenericObject_WidgetDisconnection(sWidgetType, UN_CONFIG_CPC_ANALOGOUTPUT_DPT_NAME);
}
