/**@name LIBRARY: unController.ctl

@author: Vincent Forest (AB-CO)

Creation Date: 21 05 2003

Modification History:
    14/02/2012: Alexey
        [UCPC-665] avoid ControlStateAnimation when stsreg is invalid

    23/11/2011: Alexey
        [UCPC-603] refactor menu creation and handling

    26/10/2011: Alexey
        fixed ControlWorkingModeText animation

    20/10/2011: Alexey
        [UCPC-585] added ArmedRecipe box to the AA faceplate

    20/09/2011: Alexey
        [UCPC-548][UCPC-547] upd faceplate

    13/09/2011: Alexey
        [UCPC-531] updated cpcGenericObject_WidgetAnimationDoubleStsReg

    26/08/2011: Alexey
        fixed CPC_Controller_FaceplateDisconnect and repaired missed box in the CPC_Controller_FaceplateStatusDisconnection

	15/07/2011: Alexey
		[UCPC-406] Disable "Setpoint" button when AuESPSt = 1

	13/07/2011: Alexey
		[UCPC-280]fixed button access policy

	28/06/2011: Alexey
		use new version of cpcGenericObject_WidgetAnimation
		[UCPC-345] display soft local drive in the widget and faceplate

	25/05/2011: Alexey
		[UCPC-280] updated faceplate's buttons behavior
		[UCPC-280] made Out bar relative to Out's ranges

	18/05/2011: Alexey
		use cpcGenericObject_HandleEventMaskMenu and CPC functions to manage menu
		use cpcGenericObject_WidgetAnimation instead of unGenericObject_WidgetAnimation

  15/11/2010: Quique
  - [UCPC-154] Inhibit manual/forced mode must be shown in faceplate and widget (letter "h")
  
  27/03/2009: Frederic
	  - CPC_Controller_WidgetConnect: move g_bCallbackConnected upper 
	  - CPC_Controller_ButtonRegisterCB(): remove fwException_raise(...) before g_dsUserAccess

  21/11/2008: Quique
    - New bit logic for button AUTO when received AuRegR

  05/11/2008: Quique
    - Assign the right Kd to a Tds (derivative FILTER) and Td to D (derivative TERM)
    
  27/02/2008: Herve
    - remove the faceplate button in unWidget_ControllerActiveX.pnl
    
  21/02/2008: Herve
    - new get/set function to encapsulate PidCon.
    
  28/12/2007: Herve
    - device acccess control
    
	29/06/2007: Herve
		- new bit in StsReg01
		
	02/05/2006: Herve
		- implementation of maskEvent in widget, faceplate and rigthClick
		
	11/04/2006: Herve
		- remove dynAppend(menuList, "SEPARATOR"); in CPC_Controller_MenuConfiguration function

	15/12/2005: Celine
		- CPC_Controller_WidgetAnimationCB, CPC_Controller_WidgetRegisterCB, 
		CPC_Controller_FaceplateStatusRegisterCB,  CPC_Controller_FaceplateStatusAnimationCB : add g_bSystemConnected checking
	
	13/09/2005: Herve
		- CPC_Controller_ButtonSetState: use UN_STSREG01_AUINHMMO instead of UN_STSREG01_AUINHFMO 
		to the enable state of the manual button
		
	09/08/2005: Frederic
		- CPC_Controller_ButtonRegisterCB(), add evalScript to run dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS] function

	15/04/2005: Herve
		- unGenericObject_GetPLCNameFromDpName: return the PLCtype+"_"+plcName, in case of _UnPlc it returns _unPlc_+plcName
		
	01/12/2004: Herve
		add the WarningText letter, set it to PLC state and set back clo to cyan in case of data invalid.
		
	26/11/2004: Herve
		- if remote system off: set facepalte button to disable and set the ControllerActiveX color to DpDoesNotExist
		
	05/11/2004: Herve
		- in the WidgetRegisterCB, faceplateRegiterCB: use the systemName of the $sIdentifier to dpConnect to the PLC

	26/10/2004: Herve
		- in the CPC_Controller_ButtonRegisterCB function: call the CPC_Controller_ButtonUserAccess to get the list of allowed buttons. This list is 
		reseted if the user logs in/out while the remote system is off.

	22/10/2004: Herve
		- modify the unGenericObject_FormatValue call

	18/10/2004: Herve
		- add to CPC_Controller_MenuConfiguration the function call unGenericObject_addTrendActionToMenu

	03/05/2004: Herve
		- during WidgetRegister and FaceplateRegister call unGenericObject_GetPLCNameFromDpName to get the PLC DPName
		- during WidgetRegister check if the dpType is correct

	28/04/2004: Herve
		- add function CPC_Controller_ObjectListGetValueTime

	14/04/2004: Herve
		- add the function CPC_Controller_HandleMenu
		- add the function CPC_Controller_UserLoginGetButtonState

	13/04/2004: Herve
		- in function CPC_Controller_ButtonRegisterCB add the set Manreg01 to 0 in case of dpConnect.
		- modification of the signature CPC_Controller_MenuConfiguration and add the creation of the menu.

	08/04/2004: Herve 
		modification of the signature of the function CPC_Controller_ButtonSetState

	16/01/2004: Herve
		remove the dpConnect, dpDisconnect and callback function in the case of no PLC systemAlarm. Ease the development 
		and test.
		
	01/01/2004: Herve
		- add in all dpConnect and dpDisconnect the _unSystemAlarm_PLC_DS dp if the PLC is configured for systemIntegrity. 
		The g_bUnSystemAlarmPlc is set to true if the PLC is configured for systemIntegrity
		change the dpConnect and dpDisconnect of _aut_inv to _invalid

version 1.0

External Function : 
	. CPC_Controller_AcknowledgeAlarm : acknowledge Controller alarms
	. CPC_Controller_FaceplateStatusDisconnection : animate faceplate disconnection
	. CPC_Controller_ButtonUserAccess : User access for Controller buttons
	. CPC_Controller_ButtonSetState : set button state
	. CPC_Controller_WidgetDisconnection : animate widget disconnection
	. CPC_Controller_MenuConfiguration : configuration of popup menu

Internal Functions :
	. CPC_Controller_FaceplateStatusRegisterCB : register callback function
	. CPC_Controller_FaceplateStatusAnimationCB_NoRange : animate faceplate
	. CPC_Controller_FaceplateStatusAnimationCB : animate faceplate
	. CPC_Controller_ButtonRegisterCB : register callback function
	. CPC_Controller_ButtonAnimationCB : animate buttons
	. CPC_Controller_ButtonDisconnection : animate buttons disconnection
	. CPC_Controller_WidgetRegisterCB : register callback function
	. CPC_Controller_WidgetAnimationCB_NoRange : animate widget
	. CPC_Controller_WidgetAnimationCB : animate widget

Purpose: This library contains Controller object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Controller object (DPType etc.)
	. Global variables and $parameters defined in Controller faceplate and widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

//@{

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Controller_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Controller_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	float value;
	string sTime;
	
	dpGet(sDeviceName+".ProcessInput.OutOVSt", value, sDeviceName+".ProcessInput.OutOVSt:_online.._stime", sTime);
	dsReturnData[1] = sTime;
	dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.OutOVSt"), value) + " " 
							+ dpGetUnit(sDeviceName+".ProcessInput.OutOVSt");
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Controller_AcknowledgeAlarm
/**
Purpose: acknowledge Controller alarms

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsNeedAck, dyn_string, output, DPEs names that need acknowledgement. 
									 The PLC acknowledgement is specified using UN_ACKNOWLEDGE_PLC string
	
Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Controller faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Controller_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
	dsNeedAck = makeDynString();
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Controller_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsAccessOk, dyn_string, input, authorized action
	- menuList, dyn_string, output, menu to pop-up
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Controller_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
	dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_FACEPLATE_TEXT, UN_POPUPMENU_TREND_TEXT);

    cpcGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
    cpcGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
    cpcGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Controller_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_Controller_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
	dyn_string exceptionInfo;

	cpcGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);

	if (dynlen(exceptionInfo) > 0) {
		unSendMessage_toExpertException("Right Click",exceptionInfo);
	}

}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Controller_FaceplateStatusRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Controller faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Controller_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sSystemName;
	int iAction;
	dyn_string exceptionInfo;
	bool bRemote;
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

        if(!g_bBargraphInitialised) {
          g_bBargraphInitialised = true;
          _CPC_Controller_initBargraph("FALSE");
        }
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			CPC_Controller_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
			CPC_Controller_FaceplateConnect(deviceName);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			CPC_Controller_FaceplateDisconnect(deviceName);
			CPC_Controller_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

CPC_Controller_GetAnimationCallback(string deviceName, string sSystemName, string sPlcName, string &callbackFunctionPostfix, dyn_string &statusAnimationDPs) {
	statusAnimationDPs =
			makeDynString(deviceName + ".ProcessInput.StsReg01", 
						 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
						 deviceName + ".ProcessInput.StsReg02", 
						 deviceName + ".ProcessInput.StsReg02:_online.._invalid",								 
						 deviceName + ".ProcessInput.AuSPSt",							// Setpoint
						 deviceName + ".ProcessInput.AuSPSt:_online.._invalid",
						 deviceName + ".ProcessInput.MSPSt",
						 deviceName + ".ProcessInput.MSPSt:_online.._invalid",
						 deviceName + ".ProcessInput.ActSP",
						 deviceName + ".ProcessInput.ActSP:_online.._invalid",
						 deviceName + ".ProcessInput.ActSPH",
						 deviceName + ".ProcessInput.ActSPH:_online.._invalid",
						 deviceName + ".ProcessInput.ActSPL",
						 deviceName + ".ProcessInput.ActSPL:_online.._invalid",
						 deviceName + ".ProcessInput.AuPosRSt",							// Output
						 deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
						 deviceName + ".ProcessInput.MPosRSt",
						 deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
						 deviceName + ".ProcessInput.OutOVSt",
						 deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
						 deviceName + ".ProcessInput.ActOutH",
						 deviceName + ".ProcessInput.ActOutH:_online.._invalid",
						 deviceName + ".ProcessInput.ActOutL",
						 deviceName + ".ProcessInput.ActOutL:_online.._invalid",
						 deviceName + ".ProcessInput.ActKc",							// PID parameters
						 deviceName + ".ProcessInput.ActKc:_online.._invalid",
						 deviceName + ".ProcessInput.ActTi",
						 deviceName + ".ProcessInput.ActTi:_online.._invalid",
						 deviceName + ".ProcessInput.ActTd",
						 deviceName + ".ProcessInput.ActTd:_online.._invalid",
						 deviceName + ".ProcessInput.ActTds",
						 deviceName + ".ProcessInput.ActTds:_online.._invalid",
						 deviceName + ".ProcessInput.MV",
						 deviceName + ".ProcessInput.MV:_online.._invalid");

	if (g_iRangeTypeSp == DPCONFIG_MINMAX_PVSS_RANGECHECK && g_iRangeTypeOut == DPCONFIG_MINMAX_PVSS_RANGECHECK) {
		callbackFunctionPostfix = "";
		dynAppend(statusAnimationDPs, deviceName + ".ProcessInput.ActSP:_pv_range.._max");
		dynAppend(statusAnimationDPs, deviceName + ".ProcessInput.ActSP:_pv_range.._min");
		dynAppend(statusAnimationDPs, deviceName + ".ProcessInput.OutOVSt:_pv_range.._max");
		dynAppend(statusAnimationDPs, deviceName + ".ProcessInput.OutOVSt:_pv_range.._min");
	} else if (g_iRangeTypeSp == DPCONFIG_MINMAX_PVSS_RANGECHECK) {
		callbackFunctionPostfix = "_NoOutRange";
		dynAppend(statusAnimationDPs, deviceName + ".ProcessInput.ActSP:_pv_range.._max");
		dynAppend(statusAnimationDPs, deviceName + ".ProcessInput.ActSP:_pv_range.._min");
	} else if (g_iRangeTypeOut == DPCONFIG_MINMAX_PVSS_RANGECHECK) {
		callbackFunctionPostfix = "_NoSPRange";
		dynAppend(statusAnimationDPs, deviceName + ".ProcessInput.OutOVSt:_pv_range.._max");
		dynAppend(statusAnimationDPs, deviceName + ".ProcessInput.OutOVSt:_pv_range.._min");
	} else {
		callbackFunctionPostfix = "_NoRange";
	}
	dynAppend(statusAnimationDPs, sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm");
	dynAppend(statusAnimationDPs, sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled");
	dynAppend(statusAnimationDPs, deviceName + ".eventMask"); 

}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Controller_FaceplateConnect
/**
Purpose: connect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Controller faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_Controller_FaceplateConnect(string deviceName)
{
	int iRes, iRangeType1, iRangeType2;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;

// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else 
		g_bUnSystemAlarmPlc = false;

	string scalingMethod;
	unGenericDpFunctions_getKeyDeviceConfiguration(deviceName, UN_CONFIG_CPC_CONTROLLER_SCALING_METHOD_KEY, scalingMethod, exceptionInfo);
	if (scalingMethod == "1") {
		ScalingMethod.text() = "I Scaling";
	} else if (scalingMethod == "2") {
		ScalingMethod.text() = "I/O Scaling";
	} else if (scalingMethod == "3") {
		ScalingMethod.text() = "No Scaling";
	} else {
		ScalingMethod.text() = "";
	}

	g_sAuSPStFormat=dpGetFormat(deviceName + ".ProcessInput.AuSPSt");
	g_sAuSPStUnit=dpGetUnit(deviceName + ".ProcessInput.AuSPSt");
	g_sMSPStFormat=dpGetFormat(deviceName + ".ProcessInput.MSPSt");
	g_sMSPStUnit=dpGetUnit(deviceName + ".ProcessInput.MSPSt");
	g_sActSPFormat=dpGetFormat(deviceName + ".ProcessInput.ActSP");
	g_sActSPUnit=dpGetUnit(deviceName + ".ProcessInput.ActSP");
	g_sActSPHFormat=dpGetFormat(deviceName + ".ProcessInput.ActSPH");
	g_sActSPHUnit=dpGetUnit(deviceName + ".ProcessInput.ActSPH");
	g_sActSPLFormat=dpGetFormat(deviceName + ".ProcessInput.ActSPL");
	g_sActSPLUnit=dpGetUnit(deviceName + ".ProcessInput.ActSPL");
	g_sAuPosRStFormat=dpGetFormat(deviceName + ".ProcessInput.AuPosRSt");
	g_sAuPosRStUnit=dpGetUnit(deviceName + ".ProcessInput.AuPosRSt");
	g_sMPosRStFormat=dpGetFormat(deviceName + ".ProcessInput.MPosRSt");
	g_sMPosRStUnit=dpGetUnit(deviceName + ".ProcessInput.MPosRSt");
	g_sOutOVStFormat=dpGetFormat(deviceName + ".ProcessInput.OutOVSt");
	g_sOutOVStUnit=dpGetUnit(deviceName + ".ProcessInput.OutOVSt");
	g_sAtOutHFormat=dpGetFormat(deviceName + ".ProcessInput.ActOutH");
	g_sAtOutHUnit=dpGetUnit(deviceName + ".ProcessInput.ActOutH");
	g_sAtOutLFormat=dpGetFormat(deviceName + ".ProcessInput.ActOutL");
	g_sAtOutLUnit=dpGetUnit(deviceName + ".ProcessInput.ActOutL");
	g_sActKcFormat=dpGetFormat(deviceName + ".ProcessInput.ActKc");
	g_sActKcUnit=dpGetUnit(deviceName + ".ProcessInput.ActKc");
	g_sActTiFormat=dpGetFormat(deviceName + ".ProcessInput.ActTi");
	g_sActTiUnit=dpGetUnit(deviceName + ".ProcessInput.ActTi");
	g_sActTdFormat=dpGetFormat(deviceName + ".ProcessInput.ActTd");
	g_sActTdUnit=dpGetUnit(deviceName + ".ProcessInput.ActTd");
	g_sActTdsFormat=dpGetFormat(deviceName + ".ProcessInput.ActTds");
	g_sActTdsUnit=dpGetUnit(deviceName + ".ProcessInput.ActTds");
	unGenericDpFunctions_getParameters(deviceName, g_dsParameters);
	g_sDescription=unGenericDpFunctions_getDescription(deviceName);
	g_sMVUnit=dpGetUnit(deviceName + ".ProcessInput.MV");

	iRes = dpGet(deviceName + ".ProcessInput.ActSP:_pv_range.._type", iRangeType1,
				 deviceName + ".ProcessInput.OutOVSt:_pv_range.._type", iRangeType2);
	g_iRangeTypeSp = (iRes >= 0) && (iRangeType1 == DPCONFIG_MINMAX_PVSS_RANGECHECK) ? DPCONFIG_MINMAX_PVSS_RANGECHECK : DPCONFIG_NONE;
	g_iRangeTypeOut = (iRes >= 0) && (iRangeType2 == DPCONFIG_MINMAX_PVSS_RANGECHECK) ? DPCONFIG_MINMAX_PVSS_RANGECHECK : DPCONFIG_NONE;
	
	string callbackFunctionPostfix;
	dyn_string statusAnimationDPs;

	CPC_Controller_GetAnimationCallback(deviceName, sSystemName, sPlcName, callbackFunctionPostfix, statusAnimationDPs);

	if(g_bUnSystemAlarmPlc) {
		iRes = dpConnect("CPC_Controller_FaceplateStatusAnimationCB" + callbackFunctionPostfix, statusAnimationDPs);
		g_bCallbackConnected = (iRes >= 0);
	} else {
		CPC_Controller_FaceplateStatusDisconnection();
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Controller_FaceplateDisconnect
/**
Purpose: disconnect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Controller faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_Controller_FaceplateDisconnect(string deviceName)
{
	int iRes;
	string sPlcName, sSystemName;

	if(g_bUnSystemAlarmPlc) {
		sSystemName = unGenericDpFunctions_getSystemName(deviceName);
		sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);

		string callbackFunctionPostfix;
		dyn_string statusAnimationDPs;

		CPC_Controller_GetAnimationCallback(deviceName, sSystemName, sPlcName, callbackFunctionPostfix, statusAnimationDPs);
		iRes = dpDisconnect("CPC_Controller_FaceplateStatusAnimationCB" + callbackFunctionPostfix, statusAnimationDPs);
	}
	g_bCallbackConnected = !(iRes >= 0);
}
//---------------------------------------------------------------------------------------------------------------------------------------

CPC_Controller_FaceplateStatusAnimationCB_NoSPRange(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
										string sStsReg02, bit32 bit32StsReg02, string sDp4, bool bStsReg02Invalid,
										string sDpAuSPSt, float fAuSPSt, string sDp6, bool bAuSPStInvalid, 
					 			        string sDpMSPSt, float fMSPSt, string sDp8, bool bMSPStInvalid,
					 			        string sDpActSP, float fActSP, string sDp10, bool bActSPInvalid,
					 			        string sDpActSPH, float fActSPH, string sDp12, bool bActSPHInvalid,
					 			        string sDpActSPL, float fActSPL, string sDp14, bool bActSPLInvalid,
					 			        string sDpAuPosRSt, float fAuPosRSt, string sDp116, bool bAuPosRStInvalid,
					 			        string sDpMPosRSt, float fMPosRSt, string sDp18, bool bMPosRStInvalid,
					 			        string sDpOutOVSt, float fOutOVSt, string sDp20, bool bOutOVSInvalid,
					 			        string sDpAtOuOH, float fAtOutH, string sDp22, bool bAtOuOHInvalid,
					 			        string sDpAtOuOL, float fAtOutL, string sDp24, bool bAtOuOLInvalid,
					 			        string sDpActKc, float fActKc, string sDp26, bool bActKcInvalid,
					 			        string sDpActTi, float fActTi, string sDp28, bool bActTiInvalid,
					 			        string sDpActTd, float fActTd, string sDp30, bool bActTdInvalid,
					 			        string sDpActTds, float fActTds, string sDp32, bool bActTdsInvalid,
					 			        string sDpMV, float fMV, string sDp34, bool bMVInvalid,
					 			        string sDp37, float maxOut, string sDp38, float minOut,
										string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
										string sDpEvt, int iMask)
{
	CPC_Controller_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid,
									sStsReg02, bit32StsReg02, sDp4, bStsReg02Invalid,
									sDpAuSPSt, fAuSPSt, sDp6, bAuSPStInvalid, 
				 			        sDpMSPSt, fMSPSt, sDp8, bMSPStInvalid,
				 			        sDpActSP, fActSP, sDp10, bActSPInvalid,
				 			        sDpActSPH, fActSPH, sDp12, bActSPHInvalid,
				 			        sDpActSPL, fActSPL, sDp14, bActSPLInvalid,
				 			        sDpAuPosRSt, fAuPosRSt, sDp116, bAuPosRStInvalid,
				 			        sDpMPosRSt, fMPosRSt, sDp18, bMPosRStInvalid,
				 			        sDpOutOVSt, fOutOVSt, sDp20, bOutOVSInvalid,
				 			        sDpAtOuOH, fAtOutH, sDp22, bAtOuOHInvalid,
				 			        sDpAtOuOL, fAtOutL, sDp24, bAtOuOLInvalid,
				 			        sDpActKc, fActKc, sDp26, bActKcInvalid,
				 			        sDpActTi, fActTi, sDp28, bActTiInvalid,
				 			        sDpActTd, fActTd, sDp30, bActTdInvalid,
				 			        sDpActTds, fActTds, sDp32, bActTdsInvalid,
				 			        sDpMV, fMV, sDp32, bMVInvalid,
				 			        "", UN_PIDCON_MAX_RANGE_DEFAULT, "", UN_PIDCON_MIN_RANGE_DEFAULT,
									sDp37, maxOut, sDp38, minOut,
					 			    sDpAl, iValAlarm, sDpEnabled, bSystemIntegrityAlarmEnabled,sDpEvt, iMask);
}

CPC_Controller_FaceplateStatusAnimationCB_NoOutRange(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
										string sStsReg02, bit32 bit32StsReg02, string sDp4, bool bStsReg02Invalid,
										string sDpAuSPSt, float fAuSPSt, string sDp6, bool bAuSPStInvalid, 
					 			        string sDpMSPSt, float fMSPSt, string sDp8, bool bMSPStInvalid,
					 			        string sDpActSP, float fActSP, string sDp10, bool bActSPInvalid,
					 			        string sDpActSPH, float fActSPH, string sDp12, bool bActSPHInvalid,
					 			        string sDpActSPL, float fActSPL, string sDp14, bool bActSPLInvalid,
					 			        string sDpAuPosRSt, float fAuPosRSt, string sDp116, bool bAuPosRStInvalid,
					 			        string sDpMPosRSt, float fMPosRSt, string sDp18, bool bMPosRStInvalid,
					 			        string sDpOutOVSt, float fOutOVSt, string sDp20, bool bOutOVSInvalid,
					 			        string sDpAtOuOH, float fAtOutH, string sDp22, bool bAtOuOHInvalid,
					 			        string sDpAtOuOL, float fAtOutL, string sDp24, bool bAtOuOLInvalid,
					 			        string sDpActKc, float fActKc, string sDp26, bool bActKcInvalid,
					 			        string sDpActTi, float fActTi, string sDp28, bool bActTiInvalid,
					 			        string sDpActTd, float fActTd, string sDp30, bool bActTdInvalid,
					 			        string sDpActTds, float fActTds, string sDp32, bool bActTdsInvalid,
					 			        string sDpMV, float fMV, string sDp34, bool bMVInvalid,
					 			        string sDp35, float maxActSP, string sDp36, float minActSP,
										string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
										string sDpEvt, int iMask)
{
	CPC_Controller_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid,
									sStsReg02, bit32StsReg02, sDp4, bStsReg02Invalid,
									sDpAuSPSt, fAuSPSt, sDp6, bAuSPStInvalid, 
				 			        sDpMSPSt, fMSPSt, sDp8, bMSPStInvalid,
				 			        sDpActSP, fActSP, sDp10, bActSPInvalid,
				 			        sDpActSPH, fActSPH, sDp12, bActSPHInvalid,
				 			        sDpActSPL, fActSPL, sDp14, bActSPLInvalid,
				 			        sDpAuPosRSt, fAuPosRSt, sDp116, bAuPosRStInvalid,
				 			        sDpMPosRSt, fMPosRSt, sDp18, bMPosRStInvalid,
				 			        sDpOutOVSt, fOutOVSt, sDp20, bOutOVSInvalid,
				 			        sDpAtOuOH, fAtOutH, sDp22, bAtOuOHInvalid,
				 			        sDpAtOuOL, fAtOutL, sDp24, bAtOuOLInvalid,
				 			        sDpActKc, fActKc, sDp26, bActKcInvalid,
				 			        sDpActTi, fActTi, sDp28, bActTiInvalid,
				 			        sDpActTd, fActTd, sDp30, bActTdInvalid,
				 			        sDpActTds, fActTds, sDp32, bActTdsInvalid,
				 			        sDpMV, fMV, sDp32, bMVInvalid,
					 			    sDp35, maxActSP, sDp36, minActSP,
				 			        "", UN_PIDCON_MAX_RANGE_DEFAULT, "", UN_PIDCON_MIN_RANGE_DEFAULT,
					 			    sDpAl, iValAlarm, sDpEnabled, bSystemIntegrityAlarmEnabled,sDpEvt, iMask);
}

CPC_Controller_FaceplateStatusAnimationCB_NoRange(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
									string sStsReg02, bit32 bit32StsReg02, string sDp4, bool bStsReg02Invalid,
									string sDpAuSPSt, float fAuSPSt, string sDp6, bool bAuSPStInvalid, 
				 			        string sDpMSPSt, float fMSPSt, string sDp8, bool bMSPStInvalid,
				 			        string sDpActSP, float fActSP, string sDp10, bool bActSPInvalid,
				 			        string sDpActSPH, float fActSPH, string sDp12, bool bActSPHInvalid,
				 			        string sDpActSPL, float fActSPL, string sDp14, bool bActSPLInvalid,
				 			        string sDpAuPosRSt, float fAuPosRSt, string sDp116, bool bAuPosRStInvalid,
				 			        string sDpMPosRSt, float fMPosRSt, string sDp18, bool bMPosRStInvalid,
				 			        string sDpOutOVSt, float fOutOVSt, string sDp20, bool bOutOVSInvalid,
				 			        string sDpAtOuOH, float fAtOutH, string sDp22, bool bAtOuOHInvalid,
				 			        string sDpAtOuOL, float fAtOutL, string sDp24, bool bAtOuOLInvalid,
				 			        string sDpActKc, float fActKc, string sDp26, bool bActKcInvalid,
				 			        string sDpActTi, float fActTi, string sDp28, bool bActTiInvalid,
				 			        string sDpActTd, float fActTd, string sDp30, bool bActTdInvalid,
				 			        string sDpActTds, float fActTds, string sDp32, bool bActTdsInvalid,
				 			        string sDpMV, float fMV, string sDp34, bool bMVInvalid,
									string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
									string sDpEvt, int iMask)
{
	CPC_Controller_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid,
									sStsReg02, bit32StsReg02, sDp4, bStsReg02Invalid,
									sDpAuSPSt, fAuSPSt, sDp6, bAuSPStInvalid, 
				 			        sDpMSPSt, fMSPSt, sDp8, bMSPStInvalid,
				 			        sDpActSP, fActSP, sDp10, bActSPInvalid,
				 			        sDpActSPH, fActSPH, sDp12, bActSPHInvalid,
				 			        sDpActSPL, fActSPL, sDp14, bActSPLInvalid,
				 			        sDpAuPosRSt, fAuPosRSt, sDp116, bAuPosRStInvalid,
				 			        sDpMPosRSt, fMPosRSt, sDp18, bMPosRStInvalid,
				 			        sDpOutOVSt, fOutOVSt, sDp20, bOutOVSInvalid,
				 			        sDpAtOuOH, fAtOutH, sDp22, bAtOuOHInvalid,
				 			        sDpAtOuOL, fAtOutL, sDp24, bAtOuOLInvalid,
				 			        sDpActKc, fActKc, sDp26, bActKcInvalid,
				 			        sDpActTi, fActTi, sDp28, bActTiInvalid,
				 			        sDpActTd, fActTd, sDp30, bActTdInvalid,
				 			        sDpActTds, fActTds, sDp32, bActTdsInvalid,
				 			        sDpMV, fMV, sDp32, bMVInvalid,
				 			        "", UN_PIDCON_MAX_RANGE_DEFAULT, "", UN_PIDCON_MIN_RANGE_DEFAULT,
				 			        "", UN_PIDCON_MAX_RANGE_DEFAULT, "", UN_PIDCON_MIN_RANGE_DEFAULT,
					 			    sDpAl, iValAlarm, sDpEnabled, bSystemIntegrityAlarmEnabled,sDpEvt, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Controller_FaceplateStatusAnimationCB
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Controller faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Controller_FaceplateStatusAnimationCB(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
										string sStsReg02, bit32 bit32StsReg02, string sDp4, bool bStsReg02Invalid,
										string sDpAuSPSt, float fAuSPSt, string sDp6, bool bAuSPStInvalid, 
					 			        string sDpMSPSt, float fMSPSt, string sDp8, bool bMSPStInvalid,
					 			        string sDpActSP, float fActSP, string sDp10, bool bActSPInvalid,
					 			        string sDpActSPH, float fActSPH, string sDp12, bool bActSPHInvalid,
					 			        string sDpActSPL, float fActSPL, string sDp14, bool bActSPLInvalid,
					 			        string sDpAuPosRSt, float fAuPosRSt, string sDp116, bool bAuPosRStInvalid,
					 			        string sDpMPosRSt, float fMPosRSt, string sDp18, bool bMPosRStInvalid,
					 			        string sDpOutOVSt, float fOutOVSt, string sDp20, bool bOutOVSInvalid,
					 			        string sDpAtOuOH, float fAtOutH, string sDp22, bool bAtOuOHInvalid,
					 			        string sDpAtOuOL, float fAtOutL, string sDp24, bool bAtOuOLInvalid,
					 			        string sDpActKc, float fActKc, string sDp26, bool bActKcInvalid,
					 			        string sDpActTi, float fActTi, string sDp28, bool bActTiInvalid,
					 			        string sDpActTd, float fActTd, string sDp30, bool bActTdInvalid,
					 			        string sDpActTds, float fActTds, string sDp32, bool bActTdsInvalid,
					 			        string sDpMV, float fMV, string sDp34, bool bMVInvalid,
					 			        string sDp35, float maxActSP, string sDp36, float minActSP,
					 			        string sDp37, float maxOut, string sDp38, float minOut,
					 			        string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
					 			        string sDpEvt, int iMask)
{
	string pidName;
// 1. Animate display values
if(g_bSystemConnected)
	unGenericObject_DisplayValue(g_sAuSPStFormat, g_sAuSPStUnit, fAuSPSt, "SetPointRequestAuto", "cpcColor_Faceplate_Request", 
			bAuSPStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)			
	unGenericObject_DisplayValue(g_sMSPStFormat, g_sMSPStUnit, fMSPSt, "SetPointRequestManual", "cpcColor_Faceplate_Request", 
			bMSPStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)			
	unGenericObject_DisplayValue(g_sActSPFormat, g_sActSPUnit, fActSP, "SetPointRequestActive", "cpcColor_Faceplate_Request", 
			bActSPInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)			
	unGenericObject_DisplayValue(g_sActSPHFormat, g_sActSPHUnit, fActSPH, "SetPointLimitHigh", "unDisplayValue_Parameter", 
			bActSPHInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)			
	unGenericObject_DisplayValue(g_sActSPLFormat, g_sActSPLUnit, fActSPL, "SetPointLimitLow", "unDisplayValue_Parameter", 
			bActSPLInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)			
	unGenericObject_DisplayValue(g_sAuPosRStFormat, g_sAuPosRStUnit, fAuPosRSt, "OutputPositionRequestAuto", "cpcColor_Faceplate_Request", 
			bAuPosRStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)			
	unGenericObject_DisplayValue(g_sMPosRStFormat, g_sMPosRStUnit, fMPosRSt, "OutputPositionRequestManual", "cpcColor_Faceplate_Request", 
			bMPosRStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)			
	unGenericObject_DisplayValue(g_sOutOVStFormat, g_sOutOVStUnit, fOutOVSt, "OutputPositionRequestActive", "cpcColor_Faceplate_Request", 
			bOutOVSInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)			
	unGenericObject_DisplayValue(g_sAtOutHFormat, g_sAtOutHUnit, fAtOutH, "OutputLimitHigh", "unDisplayValue_Parameter", 
			bAtOuOHInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)			
	unGenericObject_DisplayValue(g_sAtOutLFormat, g_sAtOutLUnit, fAtOutL, "OutputLimitLow", "unDisplayValue_Parameter", 
			bAtOuOLInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)			
	unGenericObject_DisplayValue(g_sActKcFormat, g_sActKcUnit, fActKc, "PIDParameterP", "unDisplayValue_Parameter", 
			bActKcInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)			
	unGenericObject_DisplayValue(g_sActTiFormat, g_sActTiUnit, fActTi, "PIDParameterI", "unDisplayValue_Parameter", 
			bActTiInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)                   
	unGenericObject_DisplayValue(g_sActTdsFormat, g_sActTdUnit, fActTd, "PIDParameterD", "unDisplayValue_Parameter", 
			bActTdsInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_DisplayValue(g_sActTdFormat, g_sActTdsUnit, fActTds, "PIDParameterTds", "unDisplayValue_Parameter", 
			bActTdInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));

	
// 2. Animate colorboxes
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Auto", getBit(bit32StsReg01, UN_STSREG01_AUMOST), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Active", getBit(bit32StsReg01, UN_STSREG01_ACTIVE), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Manual", getBit(bit32StsReg01, UN_STSREG01_MMOST), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Forced", getBit(bit32StsReg01, UN_STSREG01_FOMOST), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Local", getBit(bit32StsReg01, CPC_STSREG01_SOFTLDST), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Tracking", getBit(bit32StsReg01, CPC_STSREG01_TRST), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Positioning", getBit(bit32StsReg01, CPC_STSREG01_OUTPST), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Regulation", getBit(bit32StsReg01, CPC_STSREG01_REGST), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_IOError", getBit(bit32StsReg01, UN_STSREG01_IOERRORW), 1, "unFaceplate_WarningActive", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_IOSimulated", getBit(bit32StsReg01, UN_STSREG01_IOSIMUW), 1, "unFaceplate_WarningActive", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("PIDParameter_P", getBit(bit32StsReg02, UN_STSREG02_BOPKP), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("PIDParameter_I", getBit(bit32StsReg02, UN_STSREG02_BOPTI), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("PIDParameter_D", getBit(bit32StsReg02, UN_STSREG02_BOPTD), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)                                                            
	unGenericObject_ColorBoxAnimate("PIDParameter_Tds", getBit(bit32StsReg02, UN_STSREG02_BOPKD), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("SetPointLimit_High", getBit(bit32StsReg02, UN_STSREG02_BOPSPOH), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("SetPointLimit_Low", getBit(bit32StsReg02, UN_STSREG02_BOPSPOL), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OutputLimit_High", getBit(bit32StsReg02, UN_STSREG02_BOPOUTH), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OutputLimit_Low", getBit(bit32StsReg02, UN_STSREG02_BOPOUTL), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Status_AuInhFor", getBit(bit32StsReg01, UN_STSREG01_AUINHFMO), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Status_AuRegR", getBit(bit32StsReg01, UN_STSREG01_AUREGR), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Status_AuInhMan", getBit(bit32StsReg01, UN_STSREG01_AUINHMMO), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));

if(g_bSystemConnected)
    unGenericObject_ColorBoxAnimate("ArmedRecipe", getBit(bit32StsReg01, CPC_STSREG01_CONTROLLER_ARMRCPST), 1, "cpcColor_Faceplate_Status",
        bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));

// 3. Animate ActiveX
		// Color
if(g_bSystemConnected)
		CPC_Controller_setProperty("Color_MV", "cpcColor_Widget_Status");
if(g_bSystemConnected)
		CPC_Controller_setProperty("Color_OutO", "cpcColor_Widget_Request");
if(g_bSystemConnected)
		CPC_Controller_setProperty("Color_SP", "cpcColor_Widget_Controller_SP");
if(g_bSystemConnected)
		CPC_Controller_setProperty("BackColor", "unSynopticBackground");
		
		// Name & Description
		pidName = g_dsParameters[UN_PARAMETER_NAME];
		if (pidName == "")
			{
			pidName = "PID";
			}
if(g_bSystemConnected)
		CPC_Controller_setProperty("PIDName", pidName);
if(g_bSystemConnected)		
		CPC_Controller_setProperty("Description", g_sDescription);
			
		// MV
if(g_bSystemConnected)
		CPC_Controller_setProperty("MV_Unit", g_sMVUnit);
if(g_bSystemConnected)		
		CPC_Controller_setProperty("MV", fMV);
	if(g_bSystemConnected)		
		CPC_Controller_setProperty("MVLabel", unGenericObject_FormatValue(g_sActSPFormat, fMV));

if(g_bSystemConnected)
		CPC_Controller_setProperty("MV_MaxRange", maxActSP);
if(g_bSystemConnected)		
		CPC_Controller_setProperty("MV_MinRange", minActSP);
if(g_bSystemConnected)
		CPC_Controller_setProperty("Out_MaxRange", maxOut);
if(g_bSystemConnected)		
		CPC_Controller_setProperty("Out_MinRange", minOut);
if(g_bSystemConnected)
		CPC_Controller_setProperty("LabelMV_MaxRange", unGenericObject_FormatValue(g_sActSPFormat, maxActSP));
if(g_bSystemConnected)		
		CPC_Controller_setProperty("LabelMV_MinRange", unGenericObject_FormatValue(g_sActSPFormat, minActSP));
		
		// Setpoint active
if(g_bSystemConnected)
		CPC_Controller_setProperty("SP", fActSP);
	if(g_bSystemConnected)		
		CPC_Controller_setProperty("SPLabel", unGenericObject_FormatValue(g_sActSPFormat, fActSP));
			
		// Setpoint high limit
if(g_bSystemConnected)		
		CPC_Controller_setProperty("SP_LimitHigh", fActSPH);
			
		// Setpoint low limit
if(g_bSystemConnected)		
		CPC_Controller_setProperty("SP_LimitLow", fActSPL);
				
		// Output active
if(g_bSystemConnected)		
		CPC_Controller_setProperty("OutO", fOutOVSt);
	if(g_bSystemConnected)		
		CPC_Controller_setProperty("OutOLabel", unGenericObject_FormatValue(g_sOutOVStFormat, fOutOVSt));
if(g_bSystemConnected)		
		CPC_Controller_setProperty("OutO_Unit", g_sOutOVStUnit);
if(g_bSystemConnected)		
		CPC_Controller_setProperty("LabelMaxO", unGenericObject_FormatValue(g_sOutOVStFormat, maxOut) +" "+ g_sOutOVStUnit);
if(g_bSystemConnected)		
		CPC_Controller_setProperty("LabelMinO", unGenericObject_FormatValue(g_sOutOVStFormat, minOut) +" "+ g_sOutOVStUnit);
		// Output high limit
if(g_bSystemConnected)		
		CPC_Controller_setProperty("OutO_LimitHigh", fAtOutH);
				
		// Output low limit
if(g_bSystemConnected)
		CPC_Controller_setProperty("OutO_LimitLow", fAtOutL);
				
		// Operation mode
	if(g_bSystemConnected)	
		CPC_Controller_setProperty("Operation_Mode", bit32StsReg01);
        // Update															
if(g_bSystemConnected)
		CPC_Controller_setProperty("Update", "");

    if(g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("eventState", iMask == 0, 1, "unFaceplate_WarningActive", 
                bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Controller_FaceplateStatusDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Controller_FaceplateStatusDisconnection()
{
	unGenericObject_ColorBoxDisconnect(makeDynString("OperationMode_Regulation", "OperationMode_Auto", "OperationMode_Active",
													"OperationMode_Forced", "OperationMode_Manual", "OperationMode_Tracking", "OperationMode_Positioning", "OperationMode_Local",
													"Warning_IOError", "Warning_IOSimulated",
													"PIDParameter_P","PIDParameter_I","PIDParameter_D","PIDParameter_Tds",
													"SetPointLimit_High","SetPointLimit_Low",
													"OutputLimit_High","OutputLimit_Low",
                                                    "Status_AuInhFor", "Status_AuRegR", "Status_AuInhMan", "ArmedRecipe"));

	unGenericObject_DisplayValueDisconnect(makeDynString("SetPointRequestAuto", "SetPointRequestManual", "SetPointRequestActive",
														"OutputPositionRequestAuto", "OutputPositionRequestManual", "OutputPositionRequestActive",
														"PIDParameterP", "PIDParameterI", "PIDParameterD","PIDParameterTds",
														"PIDParameterP", "PIDParameterI", "PIDParameterD","PIDParameterTds",
														"SetPointLimitHigh","SetPointLimitLow",
														"OutputLimitHigh","OutputLimitLow"));
// set eventState
	unGenericObject_ColorBoxDisconnect(makeDynString("eventState"));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Controller_ButtonRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Controller faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Controller_ButtonRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sDpType, sFunction;
	int iAction, iRes, iManReg01;
	dyn_string exceptionInfo, dsUserAccess, dsFunctions;
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			CPC_Controller_ButtonDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
// Send 0 to ManReg01 if necessary
			iRes = dpGet(deviceName + ".ProcessOutput.ManReg01", iManReg01);
			if ((iRes < 0) || (iManReg01 != 0))
			{
				dpSet(deviceName + ".ProcessOutput.ManReg01", 0);
			}
		
// call the CPC_Controller_ButtonUserAccess to get the list of allowed buttons.
//			CPC_Controller_ButtonUserAccess(deviceName, UN_CONFIG_CPC_CONTROLLER_DPT_NAME, g_dsUserAccess);

			sDpType=UN_CONFIG_CPC_CONTROLLER_DPT_NAME;
			unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
			sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
					 
			if (sFunction != "")
				{
				evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" + 
							 "dyn_string dsUserAccessReturn;" +
							 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
							 "    {" +
					 			sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
							 "    }" +
							 "return dsUserAccessReturn; }", makeDynString(), deviceName, sDpType);
						
				g_dsUserAccess=dsUserAccess;	
				}
			else
				fwException_raise(exceptionInfo,"ERROR", "CPC_Controller_ButtonRegisterCB: " + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");
					
			
			iRes = dpConnect("CPC_Controller_ButtonAnimationCB",
							 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 deviceName + ".statusInformation.selectedManager",
							 deviceName + ".ProcessInput.StsReg01",
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid");
			g_bCallbackConnected = (iRes >= 0);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			iRes = dpDisconnect("CPC_Controller_ButtonAnimationCB",
							 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 deviceName + ".statusInformation.selectedManager",
							 deviceName + ".ProcessInput.StsReg01",
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid");
			g_bCallbackConnected = !(iRes >= 0);
			CPC_Controller_ButtonDisconnection();
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Controller_ButtonAnimationCB
/**
Purpose: animate buttons

Parameters: 
	- dbPermissions, dyn_bool, input, list of permissions, length = 4, order : 1=monitor, 2=operator, 3=expert, 4=admin
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Controller_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager, 
						   	   string sDpStsReg01, bit32 stsReg01Value, string sDpStsReg01Bad, bool bStsReg01Bad)
{
	CPC_Controller_ButtonSetState(unGenericDpFunctions_getDpName(sDpStsReg01), UN_CONFIG_CPC_CONTROLLER_DPT_NAME, g_dsUserAccess, 
									makeDynString(bLocked, sSelectedManager, stsReg01Value, bStsReg01Bad, 0, 0));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Controller_ButtonUserAccess
/**
Purpose: User access for Controller buttons

Parameters: 
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Controller_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	bool operator, expert, admin;
	int i, length;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
	string sOperAction, sExpAction, sAdminAction;
	bool bActionDefined = false;

	unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
	if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME)) {
		bActionDefined=true;
	}
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	if (operator) {
		if(bActionDefined) {
			dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
		} else {
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE, UN_FACEPLATE_BUTTON_SETPOINT, 
											  UN_FACEPLATE_BUTTON_PID_PARAMETERS, UN_FACEPLATE_BUTTON_SET_PID_LIMITS,
											  UN_FACEPLATE_BUTTON_SETOUTPUT, UN_FACEPLATE_BUTTON_SELECT, UN_FACEPLATE_BUTTON_REGULATION_MODE, CPC_FACEPLATE_BUTTON_OUTPUTPOSITIONING_MODE));
		}
	}
	if (expert)	{
		if(bActionDefined) {
			dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
		} else {
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_MASKEVENT));
		}
	}
	if(admin){
		if(bActionDefined) {
			dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
		}
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Controller_UserLoginGetButtonState
/**
Purpose: function called before the button set state when a user logs in. 

Parameters:
	deviceName: string, input, the device name
	dsReturnData: dyn_string, output, the data are returned here

Usage: Internal function

PVSS manager usage:  Ctrl, NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
		-> $sDpName, string, the device DP name
		-> g_sFaceplateButtonType, string, faceplate type
	. PVSS version: 2.12.1 
	. operating system: NT, W2000 and Linux, but tested only under W2000.
	. distributed system: yes but not tested.
*/

CPC_Controller_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
	unGenericObject_UserCBGetButtonState(deviceName, sType, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Controller_ButtonSetState
/**
Purpose: set button state

Parameters: 
	- deviceName, string, input, device
	- sDpType, string, input, dptype
	- dsUserAccess, dyn_string, input, user access
	- dsData, dyn_string, parameter
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Controller_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected, buttonEnabled, bit1, bit2, bit3;
	dyn_string dsButtons;
	int i, length;
	string localManager;
	bool bLocked, bStsReg01Bad, bStsReg02Bad;
	string sSelectedManager;
	bit32 stsReg01Value, stsReg02Value; 

	bLocked = dsData[1];
	sSelectedManager = dsData[2];
	stsReg01Value = dsData[3];
	bStsReg01Bad = dsData[4];
	stsReg02Value = dsData[5];
	bStsReg02Bad = dsData[6];

	dsButtons = makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE, UN_FACEPLATE_BUTTON_SETPOINT, 
							  UN_FACEPLATE_BUTTON_SETOUTPUT, UN_FACEPLATE_BUTTON_PID_PARAMETERS, UN_FACEPLATE_BUTTON_REGULATION_MODE, CPC_FACEPLATE_BUTTON_OUTPUTPOSITIONING_MODE,
							  UN_FACEPLATE_BUTTON_SET_PID_LIMITS, UN_FACEPLATE_BUTTON_FORCED_MODE); // Each button except "Select" button
	length = dynlen(dsButtons);
// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");
// 2. Button state
	for(i=1;i<=length;i++) {
		buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);	// User access
		switch (dsButtons[i]) {
			case UN_FACEPLATE_BUTTON_AUTO_MODE:
				buttonEnabled = buttonEnabled && bSelected && 
					!getBit(stsReg01Value, UN_STSREG01_AUMOST) &&
					!getBit(stsReg01Value, CPC_STSREG01_SOFTLDST) &&
					!bStsReg01Bad && !bStsReg02Bad;
				break;
			case UN_FACEPLATE_BUTTON_MANUAL_MODE:
				buttonEnabled = buttonEnabled && bSelected && 
					!getBit(stsReg01Value, UN_STSREG01_MMOST) &&
					!getBit(stsReg01Value, UN_STSREG01_AUINHMMO) &&
					!getBit(stsReg01Value, CPC_STSREG01_SOFTLDST) &&
					!bStsReg01Bad && !bStsReg02Bad;
				break;
			case UN_FACEPLATE_BUTTON_FORCED_MODE:
				buttonEnabled = buttonEnabled && bSelected && 
					!getBit(stsReg01Value, UN_STSREG01_FOMOST) &&
					!getBit(stsReg01Value, UN_STSREG01_AUINHFMO) &&
					!bStsReg01Bad && !bStsReg02Bad;
				break;
			case UN_FACEPLATE_BUTTON_SETPOINT:
				buttonEnabled = buttonEnabled && bSelected && 
					!getBit(stsReg01Value, UN_STSREG01_AUESPO) &&
					!getBit(stsReg01Value, UN_STSREG01_AUMOST) &&
					!getBit(stsReg01Value, CPC_STSREG01_SOFTLDST) &&
					!bStsReg01Bad && !bStsReg02Bad;
				break;
			case UN_FACEPLATE_BUTTON_PID_PARAMETERS:
			case UN_FACEPLATE_BUTTON_SET_PID_LIMITS:
				buttonEnabled = buttonEnabled && bSelected && 
					!getBit(stsReg01Value, UN_STSREG01_AUMOST) &&
					!getBit(stsReg01Value, CPC_STSREG01_SOFTLDST) &&
					!bStsReg01Bad && !bStsReg02Bad;
				break;
			case CPC_FACEPLATE_BUTTON_OUTPUTPOSITIONING_MODE:
				buttonEnabled = buttonEnabled && bSelected && 
					!getBit(stsReg01Value, UN_STSREG01_AUMOST) &&
					!getBit(stsReg01Value, CPC_STSREG01_OUTPST) &&
					!getBit(stsReg01Value, CPC_STSREG01_TRST) &&
					!getBit(stsReg01Value, CPC_STSREG01_SOFTLDST) &&
					!bStsReg01Bad && !bStsReg02Bad;
				break;
			case UN_FACEPLATE_BUTTON_REGULATION_MODE:
			case UN_FACEPLATE_BUTTON_SETOUTPUT:
				buttonEnabled = buttonEnabled && bSelected && 
					!getBit(stsReg01Value, UN_STSREG01_AUMOST) &&
					!getBit(stsReg01Value, CPC_STSREG01_REGST) &&
					!getBit(stsReg01Value, CPC_STSREG01_TRST) &&
					!getBit(stsReg01Value, CPC_STSREG01_SOFTLDST) &&
					!bStsReg01Bad && !bStsReg02Bad;
				break;
		}
		setValue(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", buttonEnabled);
	}
// 3. Button Select state
	unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Controller_ButtonDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Controller_ButtonDisconnection()
{
	unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE, 
							UN_FACEPLATE_BUTTON_SETPOINT, UN_FACEPLATE_BUTTON_SETOUTPUT, UN_FACEPLATE_BUTTON_PID_PARAMETERS, 
							UN_FACEPLATE_BUTTON_REGULATION_MODE, CPC_FACEPLATE_BUTTON_OUTPUTPOSITIONING_MODE, UN_FACEPLATE_BUTTON_SET_PID_LIMITS, UN_FACEPLATE_BUTTON_FORCED_MODE,
							UN_FACEPLATE_BUTTON_SELECT));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Controller_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Controller widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Controller_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName;
	int iAction;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;
	bool bRemote;

	deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
	if (deviceName == "")	// In case of disconnection
		{
		deviceName = g_sDpName;
		}
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;
	
	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;

	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			CPC_Controller_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = deviceName;
			CPC_Controller_WidgetConnect(deviceName, sPlcName, sSystemName);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			CPC_Controller_WidgetDisconnect(deviceName, sPlcName, sSystemName);
			CPC_Controller_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}
//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Controller_WidgetConnect
/**
Purpose: Connect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Controller faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_Controller_WidgetConnect(string deviceName, string sPlcName, string sSystemName)
{
	int iRes, iRangeType1, iRangeType2;
	string callbackFunctionPostfix;
	dyn_string statusAnimationDPs, exceptionInfo;

//  check if the device name is of type UN_CONFIG_CPC_CONTROLLER_DPT_NAME if not like of no PLC. -> not available.
	if(dpTypeName(deviceName) != UN_CONFIG_CPC_CONTROLLER_DPT_NAME)
		g_bUnSystemAlarmPlc = false;
				
	if(g_bUnSystemAlarmPlc)
	{
		unGenericDpFunctions_getParameters(deviceName, g_dsParameters);
		g_sMVUnit = dpGetUnit(deviceName + ".ProcessInput.MV");
		g_sDescription = unGenericDpFunctions_getDescription(deviceName);
		g_sOutOVStUnit = dpGetUnit(deviceName + ".ProcessInput.OutOVSt");
        if (g_sWidgetType=="ControllerActiveX") {
            g_sActSPFormat=dpGetFormat(deviceName + ".ProcessInput.ActSP");
            g_sOutOVStFormat=dpGetFormat(deviceName + ".ProcessInput.OutOVSt");
            if(!g_bBargraphInitialised) {
                g_bBargraphInitialised = true;
                _CPC_Controller_initBargraph("TRUE");
            }
            string scalingMethod;
            unGenericDpFunctions_getKeyDeviceConfiguration(deviceName, UN_CONFIG_CPC_CONTROLLER_SCALING_METHOD_KEY, scalingMethod, exceptionInfo);
            if (scalingMethod == "1") {
                ScalingMethod.text() = "I Scaling";
            } else if (scalingMethod == "2") {
                ScalingMethod.text() = "I/O Scaling";
            } else if (scalingMethod == "3") {
                ScalingMethod.text() = "No Scaling";
            } else {
                ScalingMethod.text() = "";
            }
        }
                
		iRes = dpGet(deviceName + ".ProcessInput.ActSP:_pv_range.._type", iRangeType1,
					 deviceName + ".ProcessInput.OutOVSt:_pv_range.._type", iRangeType2);
		g_iRangeTypeSp = (iRes >= 0) && (iRangeType1 == DPCONFIG_MINMAX_PVSS_RANGECHECK) ? DPCONFIG_MINMAX_PVSS_RANGECHECK : DPCONFIG_NONE;
		g_iRangeTypeOut = (iRes >= 0) && (iRangeType2 == DPCONFIG_MINMAX_PVSS_RANGECHECK) ? DPCONFIG_MINMAX_PVSS_RANGECHECK : DPCONFIG_NONE;

		CPC_Controller_GetAnimationCallback(deviceName, sSystemName, sPlcName, callbackFunctionPostfix, statusAnimationDPs);
		dynInsertAt(statusAnimationDPs, deviceName + ".statusInformation.selectedManager:_lock._original._locked", 1);
		dynInsertAt(statusAnimationDPs, deviceName + ".statusInformation.selectedManager", 2);
		iRes = dpConnect("CPC_Controller_WidgetAnimationCB" + callbackFunctionPostfix, statusAnimationDPs);
		g_bCallbackConnected = (iRes >= 0);	
  }
	else 
		CPC_Controller_WidgetDisconnection(g_sWidgetType);

}
//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Controller_WidgetDisconnect
/**
Purpose: Disconnect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Controller faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_Controller_WidgetDisconnect(string deviceName, string sPlcName, string sSystemName)
{
	int iRes;
	string callbackFunctionPostfix;
	dyn_string statusAnimationDPs;
	
	if(g_bUnSystemAlarmPlc)
	{
		CPC_Controller_GetAnimationCallback(deviceName, sSystemName, sPlcName, callbackFunctionPostfix, statusAnimationDPs);
		dynInsertAt(statusAnimationDPs, deviceName + ".statusInformation.selectedManager:_lock._original._locked", 1);
		dynInsertAt(statusAnimationDPs, deviceName + ".statusInformation.selectedManager", 2);
		iRes = dpDisconnect("CPC_Controller_WidgetAnimationCB" + callbackFunctionPostfix, statusAnimationDPs);
	}
	g_bCallbackConnected = !(iRes >= 0);
}

CPC_Controller_WidgetAnimationCB_NoOutRange(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
								string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
								string sStsReg02, bit32 bit32StsReg02, string sDp4, bool bStsReg02Invalid,
								string sDpAuSPSt, float fAuSPSt, string sDp6, bool bAuSPStInvalid, 
			 			        string sDpMSPSt, float fMSPSt, string sDp8, bool bMSPStInvalid,
			 			        string sDpActSP, float fActSP, string sDp10, bool bActSPInvalid,
			 			        string sDpActSPH, float fActSPH, string sDp12, bool bActSPHInvalid,
			 			        string sDpActSPL, float fActSPL, string sDp14, bool bActSPLInvalid,
			 			        string sDpAuPosRSt, float fAuPosRSt, string sDp116, bool bAuPosRStInvalid,
			 			        string sDpMPosRSt, float fMPosRSt, string sDp18, bool bMPosRStInvalid,
			 			        string sDpOutOVSt, float fOutOVSt, string sDp20, bool bOutOVStInvalid,
			 			        string sDpAtOuOH, float fAtOutH, string sDp22, bool bAtOuOHInvalid,
			 			        string sDpAtOuOL, float fAtOutL, string sDp24, bool bAtOuOLInvalid,
			 			        string sDpActKc, float fActKc, string sDp26, bool bActKcInvalid,
			 			        string sDpActTi, float fActTi, string sDp28, bool bActTiInvalid,
			 			        string sDpActTd, float fActTd, string sDp30, bool bActTdInvalid,
			 			        string sDpActTds, float fActTds, string sDp32, bool bActTdsInvalid,
			 			        string sDpMV, float fMV, string sDp34, bool bMVInvalid,
								string sDp35, float maxActSP, string sDp36, float minActSP,
								string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled, string sEvtMask, int iMask)
{
	CPC_Controller_WidgetAnimationCB(sDpLock, bLocked, sDpManager, sSelectedManager,
									sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid,
									sStsReg02, bit32StsReg02, sDp4, bStsReg02Invalid,
									sDpAuSPSt, fAuSPSt, sDp6, bAuSPStInvalid, 
									sDpMSPSt, fMSPSt, sDp8, bMSPStInvalid,
									sDpActSP, fActSP, sDp10, bActSPInvalid,
									sDpActSPH, fActSPH, sDp12, bActSPHInvalid,
									sDpActSPL, fActSPL, sDp14, bActSPLInvalid,
									sDpAuPosRSt, fAuPosRSt, sDp116, bAuPosRStInvalid,
									sDpMPosRSt, fMPosRSt, sDp18, bMPosRStInvalid,
									sDpOutOVSt, fOutOVSt, sDp20, bOutOVStInvalid,
									sDpAtOuOH, fAtOutH, sDp22, bAtOuOHInvalid,
									sDpAtOuOL, fAtOutL, sDp24, bAtOuOLInvalid,
									sDpActKc, fActKc, sDp26, bActKcInvalid,
									sDpActTi, fActTi, sDp28, bActTiInvalid,
									sDpActTd, fActTd, sDp30, bActTdInvalid,
									sDpActTds, fActTds, sDp32, bActTdsInvalid,
									sDpMV, fMV, sDp34, bMVInvalid,
									sDp35, maxActSP, sDp36, minActSP,
									"", UN_PIDCON_MAX_RANGE_DEFAULT, "", UN_PIDCON_MIN_RANGE_DEFAULT,
									sDpAl, iAlValue, sDpEnabled, bPlcEnabled, sEvtMask, iMask);
}

CPC_Controller_WidgetAnimationCB_NoSPRange(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
								string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
								string sStsReg02, bit32 bit32StsReg02, string sDp4, bool bStsReg02Invalid,
								string sDpAuSPSt, float fAuSPSt, string sDp6, bool bAuSPStInvalid, 
			 			        string sDpMSPSt, float fMSPSt, string sDp8, bool bMSPStInvalid,
			 			        string sDpActSP, float fActSP, string sDp10, bool bActSPInvalid,
			 			        string sDpActSPH, float fActSPH, string sDp12, bool bActSPHInvalid,
			 			        string sDpActSPL, float fActSPL, string sDp14, bool bActSPLInvalid,
			 			        string sDpAuPosRSt, float fAuPosRSt, string sDp116, bool bAuPosRStInvalid,
			 			        string sDpMPosRSt, float fMPosRSt, string sDp18, bool bMPosRStInvalid,
			 			        string sDpOutOVSt, float fOutOVSt, string sDp20, bool bOutOVStInvalid,
			 			        string sDpAtOuOH, float fAtOutH, string sDp22, bool bAtOuOHInvalid,
			 			        string sDpAtOuOL, float fAtOutL, string sDp24, bool bAtOuOLInvalid,
			 			        string sDpActKc, float fActKc, string sDp26, bool bActKcInvalid,
			 			        string sDpActTi, float fActTi, string sDp28, bool bActTiInvalid,
			 			        string sDpActTd, float fActTd, string sDp30, bool bActTdInvalid,
			 			        string sDpActTds, float fActTds, string sDp32, bool bActTdsInvalid,
			 			        string sDpMV, float fMV, string sDp34, bool bMVInvalid,
								string sDp37, float maxOut, string sDp38, float minOut,
								string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled, string sEvtMask, int iMask)
{
	CPC_Controller_WidgetAnimationCB(sDpLock, bLocked, sDpManager, sSelectedManager,
									sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid,
									sStsReg02, bit32StsReg02, sDp4, bStsReg02Invalid,
									sDpAuSPSt, fAuSPSt, sDp6, bAuSPStInvalid, 
									sDpMSPSt, fMSPSt, sDp8, bMSPStInvalid,
									sDpActSP, fActSP, sDp10, bActSPInvalid,
									sDpActSPH, fActSPH, sDp12, bActSPHInvalid,
									sDpActSPL, fActSPL, sDp14, bActSPLInvalid,
									sDpAuPosRSt, fAuPosRSt, sDp116, bAuPosRStInvalid,
									sDpMPosRSt, fMPosRSt, sDp18, bMPosRStInvalid,
									sDpOutOVSt, fOutOVSt, sDp20, bOutOVStInvalid,
									sDpAtOuOH, fAtOutH, sDp22, bAtOuOHInvalid,
									sDpAtOuOL, fAtOutL, sDp24, bAtOuOLInvalid,
									sDpActKc, fActKc, sDp26, bActKcInvalid,
									sDpActTi, fActTi, sDp28, bActTiInvalid,
									sDpActTd, fActTd, sDp30, bActTdInvalid,
									sDpActTds, fActTds, sDp32, bActTdsInvalid,
									sDpMV, fMV, sDp34, bMVInvalid,
									"", UN_PIDCON_MAX_RANGE_DEFAULT, "", UN_PIDCON_MIN_RANGE_DEFAULT,
									sDp37, maxOut, sDp38, minOut,
									sDpAl, iAlValue, sDpEnabled, bPlcEnabled, sEvtMask, iMask);
}
CPC_Controller_WidgetAnimationCB_NoRange(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
								string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
								string sStsReg02, bit32 bit32StsReg02, string sDp4, bool bStsReg02Invalid,
								string sDpAuSPSt, float fAuSPSt, string sDp6, bool bAuSPStInvalid, 
			 			        string sDpMSPSt, float fMSPSt, string sDp8, bool bMSPStInvalid,
			 			        string sDpActSP, float fActSP, string sDp10, bool bActSPInvalid,
			 			        string sDpActSPH, float fActSPH, string sDp12, bool bActSPHInvalid,
			 			        string sDpActSPL, float fActSPL, string sDp14, bool bActSPLInvalid,
			 			        string sDpAuPosRSt, float fAuPosRSt, string sDp116, bool bAuPosRStInvalid,
			 			        string sDpMPosRSt, float fMPosRSt, string sDp18, bool bMPosRStInvalid,
			 			        string sDpOutOVSt, float fOutOVSt, string sDp20, bool bOutOVStInvalid,
			 			        string sDpAtOuOH, float fAtOutH, string sDp22, bool bAtOuOHInvalid,
			 			        string sDpAtOuOL, float fAtOutL, string sDp24, bool bAtOuOLInvalid,
			 			        string sDpActKc, float fActKc, string sDp26, bool bActKcInvalid,
			 			        string sDpActTi, float fActTi, string sDp28, bool bActTiInvalid,
			 			        string sDpActTd, float fActTd, string sDp30, bool bActTdInvalid,
			 			        string sDpActTds, float fActTds, string sDp32, bool bActTdsInvalid,
			 			        string sDpMV, float fMV, string sDp34, bool bMVInvalid,
								string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled, string sEvtMask, int iMask)
{
	CPC_Controller_WidgetAnimationCB(sDpLock, bLocked, sDpManager, sSelectedManager,
									sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid,
									sStsReg02, bit32StsReg02, sDp4, bStsReg02Invalid,
									sDpAuSPSt, fAuSPSt, sDp6, bAuSPStInvalid, 
									sDpMSPSt, fMSPSt, sDp8, bMSPStInvalid,
									sDpActSP, fActSP, sDp10, bActSPInvalid,
									sDpActSPH, fActSPH, sDp12, bActSPHInvalid,
									sDpActSPL, fActSPL, sDp14, bActSPLInvalid,
									sDpAuPosRSt, fAuPosRSt, sDp116, bAuPosRStInvalid,
									sDpMPosRSt, fMPosRSt, sDp18, bMPosRStInvalid,
									sDpOutOVSt, fOutOVSt, sDp20, bOutOVStInvalid,
									sDpAtOuOH, fAtOutH, sDp22, bAtOuOHInvalid,
									sDpAtOuOL, fAtOutL, sDp24, bAtOuOLInvalid,
									sDpActKc, fActKc, sDp26, bActKcInvalid,
									sDpActTi, fActTi, sDp28, bActTiInvalid,
									sDpActTd, fActTd, sDp30, bActTdInvalid,
									sDpActTds, fActTds, sDp32, bActTdsInvalid,
									sDpMV, fMV, sDp34, bMVInvalid,
									"", UN_PIDCON_MAX_RANGE_DEFAULT, "", UN_PIDCON_MIN_RANGE_DEFAULT,
									"", UN_PIDCON_MAX_RANGE_DEFAULT, "", UN_PIDCON_MIN_RANGE_DEFAULT,
									sDpAl, iAlValue, sDpEnabled, bPlcEnabled, sEvtMask, iMask);
}
//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_Controller_WidgetAnimationCB
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Controller faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_Controller_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
								string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
								string sStsReg02, bit32 bit32StsReg02, string sDp4, bool bStsReg02Invalid,
								string sDpAuSPSt, float fAuSPSt, string sDp6, bool bAuSPStInvalid, 
								string sDpMSPSt, float fMSPSt, string sDp8, bool bMSPStInvalid,
								string sDpActSP, float fActSP, string sDp10, bool bActSPInvalid,
								string sDpActSPH, float fActSPH, string sDp12, bool bActSPHInvalid,
								string sDpActSPL, float fActSPL, string sDp14, bool bActSPLInvalid,
								string sDpAuPosRSt, float fAuPosRSt, string sDp116, bool bAuPosRStInvalid,
								string sDpMPosRSt, float fMPosRSt, string sDp18, bool bMPosRStInvalid,
								string sDpOutOVSt, float fOutOVSt, string sDp20, bool bOutOVStInvalid,
								string sDpAtOuOH, float fAtOutH, string sDp22, bool bAtOuOHInvalid,
								string sDpAtOuOL, float fAtOutL, string sDp24, bool bAtOuOLInvalid,
								string sDpActKc, float fActKc, string sDp26, bool bActKcInvalid,
								string sDpActTi, float fActTi, string sDp28, bool bActTiInvalid,
								string sDpActTd, float fActTd, string sDp30, bool bActTdInvalid,
								string sDpActTds, float fActTds, string sDp32, bool bActTdsInvalid,
								string sDpMV, float fMV, string sDp34, bool bMVInvalid,
								string sDp35, float maxActSP, string sDp36, float minActSP,
								string sDp37, float maxOut, string sDp38, float minOut,
								string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled, string sEvtMask, int iMask)
{
	string pidName;
	bool bInvalid;
	
// 1. Generic animation
	cpcGenericObject_WidgetAnimation(g_sWidgetType, UN_CONFIG_CPC_CONTROLLER_DPT_NAME, bLocked, sSelectedManager, bit32StsReg01,
		makeDynInt(UN_STSREG01_IOSIMUW, UN_STSREG01_IOERRORW),
		makeDynString(CPC_WIDGET_TEXT_WARNING_SIMU, CPC_WIDGET_TEXT_WARNING_ERROR),"","",
		makeDynString("CPC_STSREG01_SOFTLDST", "UN_STSREG01_AUINHFMO", "UN_STSREG01_AUINHMMO", "UN_STSREG01_MMOST", "UN_STSREG01_FOMOST"),
		"", "", -1,
		bStsReg01Invalid || bStsReg02Invalid || bAuSPStInvalid || bMSPStInvalid || bActSPInvalid || bActSPHInvalid || bActSPLInvalid || bAuPosRStInvalid || bMPosRStInvalid || bOutOVStInvalid || bAtOuOHInvalid || bAtOuOLInvalid || bActKcInvalid || bActTiInvalid || bActTdInvalid || bActTdsInvalid, 
		bStsReg01Invalid || bStsReg02Invalid,
		iAlValue, bPlcEnabled);

    string letter, color;
    if (!bStsReg01Invalid && !bStsReg02Invalid)
        unGenericObject_WidgetControlStateAnimation(bit32StsReg01, makeDynInt(CPC_STSREG01_REGST, CPC_STSREG01_TRST, CPC_STSREG01_OUTPST), makeDynString("R", "T", "P"), letter, color);
    if(g_bSystemConnected)
        setMultiValue("ControlWorkingModeText", "text", letter, "ControlWorkingModeText", "foreCol", color);

// 2. ActiveX animation
	if (g_sWidgetType=="ControllerActiveX") {
		bInvalid = bStsReg01Invalid || bStsReg02Invalid || bAuSPStInvalid || bMSPStInvalid || bActSPInvalid || bActSPHInvalid || bActSPLInvalid || bAuPosRStInvalid || bMPosRStInvalid || bOutOVStInvalid || bAtOuOHInvalid || bAtOuOLInvalid || bActKcInvalid || bActTiInvalid || bActTdInvalid || bActTdsInvalid;
		
		// Color
		if(g_bSystemConnected)
			CPC_Controller_setProperty("Color_MV", "cpcColor_Widget_Status");
		if(g_bSystemConnected)
			CPC_Controller_setProperty("Color_OutO", "cpcColor_Widget_Request");
		if(g_bSystemConnected)
			CPC_Controller_setProperty("Color_SP", "cpcColor_Widget_Controller_SP");
		
		if(bInvalid)
		{
			if(g_bSystemConnected)
				CPC_Controller_setProperty("BackColor", "unDataNotValid");
		}
		else
		{
			if(g_bSystemConnected)
				CPC_Controller_setProperty("BackColor", "unSynopticBackground");
		}

		pidName = g_dsParameters[UN_PARAMETER_NAME];
		if (pidName == "") {
			pidName = "PID";
		}
		if(g_bSystemConnected)
			CPC_Controller_setProperty("PIDName", pidName);
		if(g_bSystemConnected)
			CPC_Controller_setProperty("Description", g_sDescription);

			// MV
		if(g_bSystemConnected)
			CPC_Controller_setProperty("MV_Unit", g_sMVUnit);
		if(g_bSystemConnected)
			CPC_Controller_setProperty("MV", fMV);
		if(g_bSystemConnected)
			CPC_Controller_setProperty("MVLabel", unGenericObject_FormatValue(g_sActSPFormat, fMV));
		if(g_bSystemConnected)
			CPC_Controller_setProperty("MV_MaxRange", maxActSP);
		if(g_bSystemConnected)
			CPC_Controller_setProperty("MV_MinRange", minActSP);
		if(g_bSystemConnected)
			CPC_Controller_setProperty("Out_MaxRange", maxOut);
		if(g_bSystemConnected)
			CPC_Controller_setProperty("Out_MinRange", minOut);
		if(g_bSystemConnected)
			CPC_Controller_setProperty("LabelMV_MaxRange", unGenericObject_FormatValue(g_sActSPFormat, maxActSP));
		if(g_bSystemConnected)
			CPC_Controller_setProperty("LabelMV_MinRange", unGenericObject_FormatValue(g_sActSPFormat, minActSP));
			// Setpoint active
		if(g_bSystemConnected)
			CPC_Controller_setProperty("SP", fActSP);
		if(g_bSystemConnected)
			CPC_Controller_setProperty("SPLabel", unGenericObject_FormatValue(g_sActSPFormat, fActSP));

		// Setpoint high limit
		if(g_bSystemConnected)
			CPC_Controller_setProperty("SP_LimitHigh", fActSPH);

		// Setpoint low limit
		if(g_bSystemConnected)
			CPC_Controller_setProperty("SP_LimitLow", fActSPL);

		// Output active
		if(g_bSystemConnected)
			CPC_Controller_setProperty("OutO", fOutOVSt);
		if(g_bSystemConnected)
			CPC_Controller_setProperty("OutOLabel", unGenericObject_FormatValue(g_sOutOVStFormat, fOutOVSt));
		if(g_bSystemConnected)
			CPC_Controller_setProperty("OutO_Unit", g_sOutOVStUnit);
		if(g_bSystemConnected)
			CPC_Controller_setProperty("LabelMaxO", unGenericObject_FormatValue(g_sOutOVStFormat, 100) +" "+ g_sOutOVStUnit);
		if(g_bSystemConnected)
			CPC_Controller_setProperty("LabelMinO", unGenericObject_FormatValue(g_sOutOVStFormat, 0) +" "+ g_sOutOVStUnit);
		
		// Output high limit
		if(g_bSystemConnected)
			CPC_Controller_setProperty("OutO_LimitHigh", fAtOutH);
				
		// Output low limit
		if(g_bSystemConnected)
			CPC_Controller_setProperty("OutO_LimitLow", fAtOutL);
				
		// Operation mode
		if(g_bSystemConnected)
			CPC_Controller_setProperty("Operation_Mode", bit32StsReg01);

                // Update															
		if(g_bSystemConnected)
		  CPC_Controller_setProperty("Update", "");

		// set O/N letter
		if(bInvalid) {
			if(g_bSystemConnected)
			WarningText.text = CPC_WIDGET_TEXT_INVALID;
		}
		else if ((!bPlcEnabled) || (iAlValue > c_unSystemIntegrity_no_alarm_value)){
			if(g_bSystemConnected)
			WarningText.text = CPC_WIDGET_TEXT_OLD_DATA;
		}

		WarningText.visible = (!bPlcEnabled) || (iAlValue > c_unSystemIntegrity_no_alarm_value) || bInvalid;
		
		} 

// display event state
	if(g_bSystemConnected) {
		if(iMask == 0)
			eventState.visible	= true;
		else
			eventState.visible	= false;
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Controller_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Controller_WidgetDisconnection(string sWidgetType)
{
	cpcGenericObject_WidgetDisconnection(sWidgetType, UN_CONFIG_CPC_CONTROLLER_DPT_NAME);
	if (sWidgetType=="ControllerActiveX")	{
		// Color
		CPC_Controller_setProperty("Color_MV", "unDataNoAccess");
		CPC_Controller_setProperty("Color_OutO", "unDataNoAccess");
		CPC_Controller_setProperty("Color_SP", "unDataNoAccess");
		CPC_Controller_setProperty("BackColor", "unDataNoAccess");
		WarningText.visible(false);
	}
	
	// set event state
	if(shapeExists("eventState"))
		eventState.visible	= false;
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_Controller_setProperty
/**
Purpose: Set the property value of PID panel

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_Controller_setProperty(string sProperty, anytype aPropertyValue)
{
  float fVal;
  string sString;
  bit32 b32;

  switch(getType(aPropertyValue))
  {
    case BIT32_VAR:
      b32 = aPropertyValue;
      switch(sProperty) {
        case "Operation_Mode":
          Operation_Mode.text() = b32;
          break;
        default:
DebugN("********* UNKNOWN BIT32", sProperty, b32);
          break;
      }
//      setValue("PidActiveX", sProperty, b32);
//DebugN(sProperty, aPropertyValue, b32, PidActiveX.Operation_Mode());
//DebugN(sProperty, b32);
      break;
    case FLOAT_VAR:
      fVal = aPropertyValue;
      switch(sProperty) {
        case "MV":
          MV.text() = fVal;
          break;
        case "MV_MaxRange":
          MV_MaxRange.text() = fVal;
          break;
        case "MV_MinRange":
          MV_MinRange.text() = fVal;
          break;
        case "SP":
          SP.text() = fVal;
          break;
        case "SP_LimitHigh":
          SP_LimitHigh.text() = fVal;
          break;
        case "SP_LimitLow":
          SP_LimitLow.text() = fVal;
          break;
        case "OutO":
          OutO.text() = fVal;
          break;
        case "OutO_LimitHigh":
          OutO_LimitHigh.text() = fVal;
          break;
        case "OutO_LimitLow":
          OutO_LimitLow.text() = fVal;
          break;
        case "Out_MaxRange":
          Out_MaxRange.text() = fVal;
          break;
        case "Out_MinRange":
          Out_MinRange.text() = fVal;
          break;
        default:
DebugN("********* UNKNOWN FLOAT", sProperty, fVal);
          break;
      }
//      setValue("PidActiveX", sProperty, fVal);
//DebugN(sProperty, fVal);
      break;
    case STRING_VAR:
      sString = aPropertyValue;
      switch(sProperty) {
        case "Color_MV":
          ShapeMV.backCol() = sString;
          ShapeMV.foreCol() = sString;
          TxtMV.foreCol() = sString;
          LabelMV.foreCol() = sString;
          LabelMV_Unit.foreCol() = sString;
          break;
        case "Color_OutO":
          ShapeOutO.backCol() = sString;
          ShapeOutO.foreCol() = sString;
          TxtOutO.foreCol() = sString;
          LabelOutO.foreCol() = sString;
          LabelOutO_Unit.foreCol() = sString;
          LabelOutOLow.foreCol() = sString;
          LabelOutOHigh.foreCol() = sString;
          break;
        case "Color_SP":
          ShapeSP.backCol() = sString;
          ShapeSP.foreCol() = sString;
          TxtSP.foreCol() = sString;
          LabelSP.foreCol() = sString;
          LabelSP_Unit.foreCol() = sString;
          LabelSPHigh.foreCol() = sString;
          LabelSPLow.foreCol() = sString;
          break;
        case "BackColor":
          Pidcon.backCol() = sString;
          Pidcon.foreCol() = sString;
          break;
        case "PIDName":
          LabelName.text() = sString;
          break;
        case "Description":
          LabelDescription.text() = sString;
          break;
        case "MV_Unit":
          LabelMV_Unit.text() = sString;
          LabelSP_Unit.text() = sString;
          break;
        case "OutO_Unit":
          LabelOutO_Unit.text() = sString;
          break;
        case "OutOLabel":
          TxtOutO.text() = sString;
          break;
        case "SPLabel":
          TxtSP.text() = sString;
          break;
        case "MVLabel":
          TxtMV.text() = sString;
          break;
        case "LabelMV_MaxRange":
          LabelMV_MaxRange.text() = sString;
          break;
        case "LabelMV_MinRange":
          LabelMV_MinRange.text() = sString;
          break;
        case "LabelMaxO":
          LabelMaxO.text() = sString;
          break;
        case "LabelMinO":
          LabelMinO.text() = sString;
          break;
        case "Update":
          _CPC_Controller_updateBargraph();
          break;
        default:
DebugN("********* UNKNOWN STRING", sProperty, sString);
          break;
      }
//DebugN(sProperty, sString);
      break;
    default:
      break;
  }    
}

//---------------------------------------------------------------------------------------------------------------------------------------

// _CPC_Controller_updateBargraph
/**
Purpose: Update the bargraph of PID panel

Parameters: None

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
_CPC_Controller_updateBargraph()
{
  bit32 b32OperationMode = Operation_Mode.text();
  float fX, fY, fH, fL, fLH, fLL,f, fScale, fPos, fShY, fSizeY;
  
//DebugN("start _CPC_Controller_updateBargraph");
  if(StandAlone.text() == "TRUE"){
    LabelMV_Unit.visible = true;
    LabelSP_Unit.visible = true;
    LabelOutO_Unit.visible = true;
// Update LabelReMoSt based on Operation_Mode
    LabelReMoSt.visible = getBit(b32OperationMode, UN_STSREG01_REMOST);
// Update LabelAuMoSt based on Operation_Mode
    LabelAuMoSt.visible = getBit(b32OperationMode, UN_STSREG01_AUMOST);
// Update LabelMMoSt based on Operation_Mode
    LabelMMoSt.visible = getBit(b32OperationMode, UN_STSREG01_MMOST);
// Update LabelFoMoSt based on Operation_Mode
    LabelFoMoSt.visible = getBit(b32OperationMode, UN_STSREG01_FOMOST);
// Update LabelTracking based on Operation_Mode
    LabelTracking.visible = getBit(b32OperationMode, UN_STSREG01_TST);
  }
  else {
    LabelMV_Unit.visible = false;
    LabelSP_Unit.visible = false;
    LabelOutO_Unit.visible = false;
    LabelReMoSt.visible = false;
    LabelAuMoSt.visible = false;
    LabelMMoSt.visible = false;
    LabelFoMoSt.visible = false;
    LabelTracking.visible = false;
  }
  LabelSPHigh.toolTipText() = "SP High Limit = "+SP_LimitHigh.text();
  LabelSPLow.toolTipText() = "SP Low Limit = "+SP_LimitLow.text();
  LabelOutOHigh.toolTipText() = "Out High Limit = "+OutO_LimitHigh.text();
  LabelOutOLow.toolTipText() = "Out Low Limit = "+OutO_LimitLow.text();
  
// Update ShapeSPLimitH based on SP_LimitHigh --> SP High limit
  fH = MV_MaxRange.text();
  fL = MV_MinRange.text();
  f = SP_LimitHigh.text();
  if((fH-fL) != 0)
    fScale = (fH-f)/(fH-fL);
  else
    fScale = 0;
  if(fScale < 0)
    fScale = 0;
  if(fScale >1)
    fScale = 1;
  setValue("ShapeSPLimitH", "scale", 1, fScale);
//DebugN("ShapeSPLimitH=" + fScale);
// Update ShapeSPLimitL based on SP_LimitLow --> SP Low limit
  f = SP_LimitLow.text();
  if((fH-fL) != 0)
    fScale = (fL-f)/(fL-fH);
  else
    fScale = 0;
  if(fScale < 0)
    fScale = 0;
  if(fScale >1)
    fScale = 1;
  setValue("ShapeSPLimitL", "scale", 1, fScale);
//DebugN("ShapeSPLimitL=" + fScale);  
// Update ShapeMV based on MV
  f = MV.text();
  if((fH-fL) != 0)
    fScale = (fL-f)/(fL-fH);
  else
    fScale = 0;
  if(fScale < 0)
    fScale = 0;
  if(fScale >1)
    fScale = 1;
  setValue("ShapeMV", "scale", 1, fScale);
//DebugN("ShapeMV=" + fScale);  
// Update ShapeSP based on SP
  f = SP.text();
  if((fH-fL) != 0)
    fScale = (fL-f)/(fL-fH);
  else
    fScale = 0;
  if(fScale < 0)
    fScale = 0;
  if(fScale >1)
    fScale = 1;
  setValue("ShapeSP", "scale", 1, fScale);
//DebugN("ShapeSP=" + fScale);  
  fSizeY = YAxisSize.text();
// Update ShapeSPLimit based on SP & SP Limit High & Low
  fX = ShapeX.text();
  fY = ShapeSPY.text();
  f = SP.text();
  fLH = SP_LimitHigh.text();
  fLL = SP_LimitLow.text();
  if(f<=fLL) {
    LabelSPLow.visible = true;
    LabelSPHigh.visible = false;
    if((fH - fLL) < 0)
      fPos = fY;
    else
      fPos = fY + fSizeY - ((fLL-fL) / (fH - fL)) * fSizeY - UN_CONTROLLER_SHAPE_LIMIT_YSIZE;
    setMultiValue("ShapeSPLimit", "position", fX, fPos, "ShapeSPLimit", "visible", true);
//DebugN("ShapeSPLimit f<=fLL", fX, fY, f, fH, fLL, fLH, fPos);
  }
  else if(f>=fLH) {
    LabelSPLow.visible = false;
    LabelSPHigh.visible = true;
    if((fH - fLH) < 0)
      fPos = fY;
    else
      fPos = fY + fSizeY - ((fLH -fL)/ (fH - fL)) * fSizeY - UN_CONTROLLER_SHAPE_LIMIT_YSIZE;
    setMultiValue("ShapeSPLimit", "position", fX, fPos, "ShapeSPLimit", "visible", true);
//DebugN("ShapeSPLimit f>=fLH", fX, fY, f, fH, fLL, fLH, fPos);
  }
  else {
    LabelSPLow.visible = false;
    LabelSPHigh.visible = false;
    ShapeSPLimit.visible = false;
  }
// Update ShapeOutOLimitH based on OutO_LimitHigh --> OutO High limit
  fH = Out_MaxRange.text();
  fL = Out_MinRange.text();
  f = OutO_LimitHigh.text();
  fScale = (100-f)/(100);
  if(fScale < 0)
    fScale = 0;
  if(fScale >1)
    fScale = 1;
  setValue("ShapeOutOLimitH", "scale", 1, fScale);
//DebugN("ShapeOutOLimitH=" + fScale);  
// Update ShapeOutOLimitL based on OutO_LimitLow --> OutO Low limit
  f = OutO_LimitLow.text();
  fScale = (f)/(100);
  if(fScale < 0)
    fScale = 0;
  if(fScale >1)
    fScale = 1;
  setValue("ShapeOutOLimitL", "scale", 1, fScale);
//DebugN("ShapeOutOLimitL=" + fScale);  
// Update ShapeOutO based on OutO
  f = OutO.text();
  if((fH-fL) != 0)
    fScale = (f-fL)/(fH-fL);
  else
    fScale = 0;
  if(fScale < 0)
    fScale = 0;
  if(fScale >1)
    fScale = 1;
  setValue("ShapeOutO", "scale", 1, fScale);
//DebugN("ShapeOutO=" + fScale);  
// Update ShapeOutOLimit based on OutO & OutO Limit High & Low
  fX = ShapeOX.text();
  fY = ShapeOutY.text();
  f = OutO.text();
  fLH = OutO_LimitHigh.text();
  fLL = OutO_LimitLow.text();
  if(f<=fLL) {
    LabelOutOLow.visible = true;
    LabelOutOHigh.visible = false;
    if((fH - fLL) < 0)
      fPos = fY;
    else
      fPos = fY + fSizeY - ((fLL-fL) / (fH - fL)) * fSizeY - UN_CONTROLLER_SHAPE_LIMIT_YSIZE;
    setMultiValue("ShapeOutOLimit", "position", fX, fPos, "ShapeOutOLimit", "visible", true);
//DebugN("ShapeOutOLimit f<=fLL", fX, fY, f, fH, fLL, fLH, fPos);
  }
  else if(f>=fLH) {
    LabelOutOLow.visible = false;
    LabelOutOHigh.visible = true;
    if((fH - fLH) < 0)
      fPos = fY;
    else
      fPos = fY + fSizeY - ((fLH -fL)/ (fH - fL)) * fSizeY - UN_CONTROLLER_SHAPE_LIMIT_YSIZE;
    setMultiValue("ShapeOutOLimit", "position", fX, fPos, "ShapeOutOLimit", "visible", true);
//DebugN("ShapeOutOLimit f>=fLH", fX, fY, f, fH, fLL, fLH, fPos);
  }
  else {
    LabelOutOLow.visible = false;
    LabelOutOHigh.visible = false;
    ShapeOutOLimit.visible = false;
  }
//DebugN("end _CPC_Controller_updateBargraph");
}

//---------------------------------------------------------------------------------------------------------------------------------------

// _CPC_Controller_initBargraph
/**
Purpose: Init the bargraph of PID panel

Parameters: 
  sStandAlone: string, input, TRUE for standalone mode, anything else for non standalone mode

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
_CPC_Controller_initBargraph(string sStandAlone)
{
  float fX, fY;

  StandAlone.text() = sStandAlone;
  MV_MaxRange.text() = 100;
  MV_MinRange.text() = 0;
  MV.text() = 75;
  SP_LimitHigh.text() = 100;
  SP_LimitLow.text() = 0;
  SP.text() = 50;
  OutO_LimitHigh.text() = 100;
  OutO_LimitLow.text() = 0;
  OutO.text() = 50;
  getValue("ShapeSPLimit", "position", fX, fY);
  ShapeX.text() = fX;
  ShapeSPY.text() = fY;
  getValue("ShapeOutOLimit", "position", fX, fY);
  ShapeOX.text() = fX;
  ShapeOutY.text() = fY;
  getValue("YAxis", "size", fX, fY);
  YAxisSize.text() = fY;
}

//---------------------------------------------------------------------------------------------------------------------------------------

//@}


