/**@name LIBRARY: unDigitalAlarm.ctl

@author: Francisco Mico (IT-CO)

Creation Date: 18 06 2003

Modifications History:
	$Log: unDigitalAlarm.ctl,v $
    11/04/2012: Alexey
        [UCPC-759] animate config warning

    29/02/2012: Alexey
        [UCPC-684] "Mail/SMS" available for "operator"

    23/11/2011: Alexey
        [UCPC-603] refactor menu creation and handling
        [UCPC-611] added faceplate item to the right click menu
        [UCPC-610] removed set limit item from the right click menu

    03/10/2011: Alexey
        moved Mail/SMS handle to the cpcGenericObject_HandleUnicosMenu

    20/09/2011: Alexey
        [UCPC-548][UCPC-547] upd faceplate

    15/09/2011: Alexey
        [UCPC-540] apply format and unit of APar PosSt to the delay time

    14/09/2011: Alexey
        [UCPC-534] type is a list now

    30/08/2011: Alexey
        [UCPC-514] animate widget when no alert handler

    26/08/2011: Alexey
        added unAnalogAlarm_animatePlcDisconnection

    24/08/2011: Alexey
        [UCPC-483] added CPC_DigitalAlarm_UserLoginGetButtonState function
        [UCPC-466] animate delay time parameter

	11/08/2011: Alexey
        [UCPC-476] display PosW in the widget

	10/08/2011: Alexey
		[UCPC-460] fixed popup rights; added Mask/Unmask PVSS alarm
		[UCPC-473] added "Select" popup item

	04/08/2011: Alexey
		[UCPC-454] made smart label for Block PLC Alarm
		[UCPC-457] animate faceplate's ISt according StsReg bit

	02/08/2011: Alexey
		[UCPC-451] 'Alarm Not Ack' is never blinking
		changed faceplate's 'Alarm Status' box color to red (blinking on unack)

	27/07/2011: Alexey
		[UCPC-387] more human alarm's type

	07/07/2011: Alexey
		[UCPC-265] update alarm acknowledge

	29/06/2011: Alexey
		[UCPC-352] removed CPC_STSREG01_PREMASKALARM

	28/06/2011: Alexey
		[UCPC-359] removed set mask popup item
		[UCPC-357] fixed MAlBRSt box in the faceplate

	18/05/2011: Alexey
		use CPC functions to manage menu

	12/05/2011: amerezhi
		[UCPC-195] display alarm type in the faceplate

	Revision 1.17  2009/03/30 12:14:48  gamx
	fixed bug in animation: priority of "N" is higher than "O"
	
	Revision 1.15  2009/01/28 14:06:39  gamx
	Fixes in _WidgetDisconnection function:
	A standard "unGenericObject_WidgetDisconnections" function cannot be used for ADA or DDA objects (due to additional graphical objects on the widget panel).
	A custom disconnection function has been introduced.

	Revision 1.14  2008/11/12 08:25:13  gthomas
	-removed function calls to _ButtonUserAccessGcs and _ButtonUserAccessUn

	Revision 1.13  2008/10/20 13:55:47  gthomas
	-removed unused XX_EventList_Configure_YY functions

	Revision 1.12  2008/07/18 07:16:55  gthomas
	added new Access control functionality for unicos-4-1
	modified  xx_ButtonUserAccess function.

	Revision 1.11  2008/06/18 14:54:51  aabadz
	Changes done following the recomendations from AB-CO (from Herve)

	Revision 1.10  2008/01/22 14:46:36  aabadz
	Fixed:
	- Naming
	- Call to other functions to reuse code
	- Some error check

	Revision 1.9  2008/01/14 10:12:21  aabadz
	Changes after code review and before tests

	Revision 1.8  2007/10/30 16:21:41  gthomas
	Introduced the CVS keyword $Log to track changes made in the library

	25.09.2006 Kulman Nikolay
		- Porting to Unicos 3.6
	25.11.2004 Kulman Nikolay
		- function unAnalogAlarm_HandleMenu modified to deselect widget automaticly
		- function unAnalogAlarm_WidgetAnimationCB modified
	03.11.2004 Kulman Nikolay
		- function CPC_DigitalAlarm_HandleMenu modified to handle Mask/UnMask menu commands
	11/05/2004: Herve AB/CO
		- function CPC_DigitalAlarm_MenuConfiguration modified: signature modified add the creation of the menu
		- function CPC_DigitalAlarm_HandleMenu added: this function handles the rigth click menu
		- function CPC_DigitalAlarm_ObjectListGetValueTime added: called in object list.
		- function CPC_DigitalAlarm_ButtonSetState modified: modification of the signature of the function
		- function CPC_DigitalAlarm_ButtonAnimationCB modified: signature of the function CPC_DigitalAlarm_ButtonSetState was modified
		- function CPC_DigitalAlarm_UserLoginGetButtonState added: called when user logs in, returns the state of the buttons
		- function CPC_DigitalAlarm_ButtonRegister modified : add the set Manreg01 to 0 in case of dpConnect
		- function CPC_DigitalAlarm_FaceplateConnect, CPC_DigitalAlarm_FaceplateDisconnect and CPC_DigitalAlarm_WidgetRegisterCB
		bug in case the PLC is not defined and if the PVSS system name contains '-',
		call unGenericObject_GetPLCNameFromDpName to get the PLC DPName instead (this function corrects the bugs).

version 1.0

External Function :
	. CPC_DigitalAlarm_AcknowledgeAlarm : animate Digital Alarm alarms
	. CPC_DigitalAlarm_FaceplateStatusDisconnection : animate faceplate disconnection
	. CPC_DigitalAlarm_ButtonUserAccess : User access for DigitalAlarm buttons
	. CPC_DigitalAlarm_ButtonSetState : set button state
	. CPC_DigitalAlarm_WidgetDisconnection : animate widget disconnection
	. CPC_DigitalAlarm_MenuConfiguration : configuration of popup menu

Internal Functions :
	. CPC_DigitalAlarm_FaceplateStatusRegisterCB : register callback function
	. CPC_DigitalAlarm_FaceplateConnect : dpConnect to the DataPoint
	. CPC_DigitalAlarm_FaceplateDisconnect : dpDisconnect to the DataPoint
	. CPC_DigitalAlarm_FaceplateStatusAnimationCB : animate faceplate
	. CPC_DigitalAlarm_ButtonRegisterCB : register callback function
	. CPC_DigitalAlarm_ButtonAnimationCB : animate buttons
	. CPC_DigitalAlarm_ButtonDisconnection : animate buttons disconnection
	. CPC_DigitalAlarm_WidgetRegisterCB : register callback function
	. CPC_DigitalAlarm_WidgetConnect : dpConnect to the DataPoint
	. CPC_DigitalAlarm_WidgetDisconnect : dpDisconnect to the DataPoint
	. CPC_DigitalAlarm_WidgetAnimationCB : animate widget

Purpose: This library contains DigitalAlarm object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. DigitalAlarm object (DPType etc.)
	. Global variables and $parameters defined in DigitalAlarm faceplate and widget
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_DigitalAlarm_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_DigitalAlarm_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
  string sTime, value;

  dpGet(sDeviceName+".ProcessInput.ISt", value,
        sDeviceName+".ProcessInput.ISt:_online.._stime", sTime);

  dsReturnData[1] = sTime;
  dsReturnData[2] = value;

}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalAlarm_AcknowledgeAlarm
/**
Purpose: acknowledge Digital Alarm alarms

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsNeedAck, dyn_string, output, DPEs names that need acknowledgement.
									 The PLC acknowledgement is specified using UN_ACKNOWLEDGE_PLC string

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Local faceplate
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_DigitalAlarm_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
	dsNeedAck = makeDynString("ProcessInput.ISt", UN_ACKNOWLEDGE_PLC);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalAlarm_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsMenuConfig, dyn_string, output, configuration of menu. Use defined constants UN_POPUPMENU_***

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_DigitalAlarm_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
    dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_ACK_TEXT, CPC_POPUPMENU_DIAG_INFO_TEXT, CPC_POPUPMENU_MASK_IST_TEXT,
        UN_POPUPMENU_TREND_TEXT, UN_POPUPMENU_BLOCK_PLC_TEXT, UN_POPUPMENU_FACEPLATE_TEXT);

    cpcGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
    cpcGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
    cpcGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalAlarm_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_DigitalAlarm_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
    dyn_string exceptionInfo;
    cpcGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);

    if (dynlen(exceptionInfo) > 0) {
        unSendMessage_toExpertException("Right Click",exceptionInfo);
    }
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalAlarm_ButtonUserAccess
/**
Purpose: User access for DigitalAlarm buttons

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_DigitalAlarm_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{	
	bool operator, expert, admin;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
	string sOperAction, sExpAction, sAdminAction;
	bool bActionDefined = false;
	
	unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
	if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME)) {
        bActionDefined=true;
    }
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	
	if (operator){
		if(bActionDefined) {
			dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
		} else {
			dynAppend(dsAccess, makeDynString(CPC_FACEPLATE_BUTTON_SET_DA_MAIL_SMS, UN_FACEPLATE_BUTTON_MASK_ALARM, UN_FACEPLATE_BUTTON_SELECT));
			if (cpcExportGenericFunctions_getAcknowledgeAlarmValue(sDpName, ".ProcessInput.ISt") != "N") {
				dynAppend(dsAccess, UN_FACEPLATE_BUTTON_ACK_ALARM);
			}
		}
	}
	// to check why Block/unblock action is missing !
	if (expert){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
		else
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_MASKEVENT, UN_FACEPLATE_BUTTON_BLOCK_ALARM_ACTION));
	}
	if(admin){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
		else
        		dynAppend(dsAccess, makeDynString());
	}

}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalAlarm_UserLoginGetButtonState
/**
Function is called before the button set state when a user logs in.

	@param[in]	sDeviceName		The device name
	@param[in]	sType			Device type <b>(Currently not used)</b>
	@param[out]	dsReturnData	The data are returned here
	@note	This library works with an unicos button panel and need global variables declared in this panel
		- g_dsUserAccess, dyn_string, button identifier is added to this variable if user access
		- $sDpName, string, the device DP name
		- g_sFaceplateButtonType, string, faceplate type
*/
CPC_DigitalAlarm_UserLoginGetButtonState(string sDeviceName, string sType, dyn_string &dsReturnData)
{
	unGenericObject_UserCBGetButtonState(sDeviceName, sType, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalAlarm_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in DigitalAlarm widget
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_DigitalAlarm_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sPlcName, sPlcDpName, sSystemName;
	int iAction;
	dyn_string exceptionInfo;
	bool bRemote;

	deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
	if (deviceName == "")	// In case of disconnection
	{
		deviceName = g_sDpName;
	}

	sSystemName = unGenericDpFunctions_getSystemName(deviceName);

	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	cpcConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)
		g_bUnSystemAlarmPlc = true;
	else
		g_bUnSystemAlarmPlc = false;

	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			CPC_DigitalAlarm_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = deviceName;
			if(dpTypeName(deviceName) == UN_CONFIG_CPC_DIGITALALARM_DPT_NAME)
				CPC_DigitalAlarm_WidgetConnect(deviceName, sPlcName, sSystemName);
			else{
				g_bUnSystemAlarmPlc = false;
				CPC_DigitalAlarm_WidgetDisconnection(g_sWidgetType);
			}
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			CPC_DigitalAlarm_WidgetDisconnect (deviceName, sPlcName, sSystemName);
			CPC_DigitalAlarm_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}


//--------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalAlarm_WidgetConnect
/**
Purpose: do dpConnect

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in DigitalAlarm faceplate
	. PVSS version: 2.12.1
	. operating system: XP and W2000, but tested only under XP
	. distributed system: yes.
*/
CPC_DigitalAlarm_WidgetConnect(string deviceName, string sPlcName, string sSystemName)
{
    int iRes, iAlertType;
    if (g_bUnSystemAlarmPlc) {
        iRes = dpGet(deviceName + ".ProcessInput.ISt:_alert_hdl.._type", iAlertType);
        g_bBoolAlert = ((iRes >= 0) && (iAlertType == DPCONFIG_ALERT_BINARYSIGNAL));
        if (g_bBoolAlert) {
            iRes = dpConnect("CPC_DigitalAlarm_WidgetAnimationCB",
                deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                deviceName + ".statusInformation.selectedManager",
                deviceName + ".ProcessInput.StsReg01",
                deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                deviceName + ".ProcessInput.ISt:_alert_hdl.._active",
                sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
                sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
                deviceName + ".eventMask");
        } else {
            iRes = dpConnect("CPC_DigitalAlarm_WidgetAnimationCB_NoAlarm",
                deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                deviceName + ".statusInformation.selectedManager",
                deviceName + ".ProcessInput.StsReg01",
                deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
                sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
                deviceName + ".eventMask");
        }
        g_bCallbackConnected = (iRes >= 0);
    } else {
        CPC_DigitalAlarm_WidgetDisconnection (g_sWidgetType);
    }
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalAlarm_WidgetDisconnect
/**
Purpose: do dpDisconnect

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in DigitalAlarm faceplate
	. PVSS version: 2.12.1
	. operating system: XP and W2000, but tested only under XP.
	. distributed system: yes.
*/
CPC_DigitalAlarm_WidgetDisconnect(string deviceName, string sPlcName, string sSystemName)
{
    int iRes;
    if (g_bUnSystemAlarmPlc) {
        if (g_bBoolAlert) {
            iRes = dpDisconnect("CPC_DigitalAlarm_WidgetAnimationCB",
                deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                deviceName + ".statusInformation.selectedManager",
                deviceName + ".ProcessInput.StsReg01",
                deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                deviceName + ".ProcessInput.ISt:_alert_hdl.._active",
                sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
                sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
                deviceName + ".eventMask");
        } else {
            iRes = dpDisconnect("CPC_DigitalAlarm_WidgetAnimationCB_NoAlarm",
                deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                deviceName + ".statusInformation.selectedManager",
                deviceName + ".ProcessInput.StsReg01",
                deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
                sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
                deviceName + ".eventMask");
        }
        g_bCallbackConnected = !(iRes >= 0);
    }
}

CPC_DigitalAlarm_WidgetAnimationCB_NoAlarm(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
	string sDpStsReg01, bit32 bit32StsReg01, string sDp1, bool bStsReg01Invalid,
	string sDpAl, int iSystemIntegrityAlarmValue, string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
	string sEvtMask, int iMask)
{
    CPC_DigitalAlarm_WidgetAnimationCB(sDpLock, bLocked, sDpManager, sSelectedManager,
        sDpStsReg01, bit32StsReg01, sDp1, bStsReg01Invalid,
        "", false,
        sDpAl, iSystemIntegrityAlarmValue, sDpEnabled, bSystemIntegrityAlarmEnabled,
        sEvtMask, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalAlarm_WidgetAnimationCB
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in DigitalAlarm faceplate
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_DigitalAlarm_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
	string sDpStsReg01, bit32 bit32StsReg01, string sDp1, bool bStsReg01Invalid,
	string sDp2, bool bIsAlarmActive,
	string sDpAl, int iSystemIntegrityAlarmValue, string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
	string sEvtMask, int iMask)
{
	string sBody1Color, sBody1BColor = "", sColorSelect = "", sWarningColor = "",sWarningLetter = "";
	string sAlarmTypeLetter = "", sAlarmTypeColor = "", sAlarmTextLetter = "", sAlarmTextColor = "";
	sBody1Color = "unDataNotValid";
	bool bLockVisible;
	bool bAlarmSt = getBit(bit32StsReg01, CPC_STSREG01_IST) == 1;

	if (!bIsAlarmActive) {
		sAlarmTextLetter = CPC_WIDGET_TEXT_ALARM_MASKED;
		sAlarmTextColor = "unAlarmMasked";
	}

	if (bStsReg01Invalid){
		sWarningLetter = CPC_WIDGET_TEXT_INVALID;
		sWarningColor = "unDataNotValid";
	} else if (!bSystemIntegrityAlarmEnabled || iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value) { //data quality indicators
		sWarningLetter = CPC_WIDGET_TEXT_OLD_DATA;
		sWarningColor = "unDataNotValid";
	} else {
		sBody1Color = bAlarmSt ? "cpcColor_Alarm_Bad" : "unAlarm_Ok";
		sBody1BColor = "";

		// Alarm not acknowledged (higher priority)
		if (getBit(bit32StsReg01, UN_STSREG01_ALUNACK) == 1) {
			sBody1Color = "cpcColor_Alarm_AlarmNotAck";
		}

		if (bAlarmSt) {
			sAlarmTypeLetter = CPC_WIDGET_ALARM;
			sAlarmTypeColor  = "cpcColor_Alarm_Bad";
			sBody1BColor = sBody1Color;
		}
		if (getBit(bit32StsReg01, CPC_STSREG01_MALBRST) == 1) {			// Alarm blocked
			sAlarmTextLetter = CPC_WIDGET_TEXT_ALARM_BLOCKED;
			sAlarmTextColor = "unAlarmMasked";
			sBody1Color = "unAlarmMasked";
			sBody1BColor = "";
		}

		// Warning animation
		unGenericObject_WidgetWarningAnimation(bit32StsReg01,
		   makeDynInt(CPC_STSREG01_CONFIGW, CPC_STSREG01_POSW, UN_STSREG01_IOSIMUW, UN_STSREG01_IOERRORW),
		   makeDynString(CPC_WIDGET_TEXT_CONFIGW, CPC_WIDGET_TEXT_POSW, CPC_WIDGET_TEXT_WARNING_SIMU, CPC_WIDGET_TEXT_WARNING_ERROR),
		   sWarningLetter, sWarningColor);
	}

	unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, sColorSelect, bLockVisible);
	if(g_bSystemConnected)
	{
		setMultiValue("WarningText", "text", sWarningLetter, "WarningText", "foreCol", sWarningColor,
			"AlarmType", "text", sAlarmTypeLetter, "AlarmType", "foreCol", sAlarmTypeColor,	//	"A"
			"AlarmText", "text", sAlarmTextLetter, "AlarmText", "foreCol", sAlarmTextColor,	//	"B"
			"ControlStateText", "text", "", "SelectArea", "foreCol", sColorSelect,
			"LockBmp", "visible", bLockVisible, "Body1","fill","[solid]",
			"Body1", "foreCol", sBody1Color, "Body1", "backCol", sBody1BColor,
			"WidgetArea", "visible", true);
	}
    if (g_bSystemConnected) {
        eventState.visible = iMask == 0;
    }
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalAlarm_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType, string, input, widget type. ex: "heater"

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_DigitalAlarm_WidgetDisconnection(string sWidgetType)
{
	setMultiValue("Body1", "foreCol", "unDataNoAccess", "Body1", "backCol", "unWidget_Background", "Body1", "fill", "[solid]",
	   "WarningText", "text", "", "WidgetArea", "visible", false, "AlarmType", "text", "", "AlarmText", "text", "");
	eventState.visible = false;
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalAlarm_FaceplateStatusRegisterCB
/**
Register faceplate callback function

	@param[in]	sSystemName 	The device dp
	@param[in] 	bSystemConnected 	Flag
*/
CPC_DigitalAlarm_FaceplateStatusRegisterCB(string sSystemStatus, bool bSystemConnected)
{
	string sDeviceName, sSystemName, sPlcName;
	int iAction;
	dyn_string exceptionInfo;
	string sPlcDpName;
	bool bRemote;

	sDeviceName = unGenericDpFunctions_getDpName($sDpName);
	if (sDeviceName)
	{
		sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
		unDistributedControl_isRemote(bRemote, sSystemName);
		if(bRemote)
			g_bSystemConnected = bSystemConnected;
		else
			g_bSystemConnected = true;

		sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
		cpcConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)
			g_bUnSystemAlarmPlc = true;
		else
			g_bUnSystemAlarmPlc = false;

		unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);

		switch (iAction)
		{
			case UN_ACTION_DISCONNECT:
				CPC_DigitalAlarm_animatePlcDisconnection();
				break;
			case UN_ACTION_DPCONNECT:
				CPC_DigitalAlarm_FaceplateConnect(sDeviceName);
				break;
			case UN_ACTION_DPDISCONNECT_DISCONNECT:
				CPC_DigitalAlarm_FaceplateDisconnect(sDeviceName);
				CPC_DigitalAlarm_animatePlcDisconnection();
				break;
			case UN_ACTION_NOTHING: break;
			default:
				break;
		}
	}

}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalAlarm_FaceplateConnect
/**
Do faceplate dpConnect

	@param[in]	sDeviceName	Device DP name
*/
CPC_DigitalAlarm_FaceplateConnect(string sDeviceName)
{
	int iRes=0;
	dyn_string exceptionInfo;
	dyn_string sPlcDpName;
	string sPlcName, sSystemName;

	sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
	sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
	cpcConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	g_bUnSystemAlarmPlc = (dynlen(exceptionInfo) <= 0);
	unGenericDpFunctions_getKeyDeviceConfiguration(sDeviceName, CPC_CONFIG_TYPE_KEY, g_dsAlarmTypes, exceptionInfo);
	g_bAlAck = cpcExportGenericFunctions_getAcknowledgeAlarmValue(sDeviceName, ".ProcessInput.ISt") != "N";
	g_sAlarmDelay = cpcGenericDpFunctions_getDeviceProperty(sDeviceName, CPC_PARAMS_ALDT, exceptionInfo);
    string alarmDelayDPe = dpAliasToName(sSystemName + g_sAlarmDelay) + "ProcessInput.PosSt";
    g_bAlarmDelayInhibited = dpExists(dpAliasToName(sSystemName + g_sAlarmDelay));
    g_sAlarmDelayFormat = g_bAlarmDelayInhibited ? dpGetFormat(alarmDelayDPe) : "%2.1f";
    g_sAlarmDelayUnit = g_bAlarmDelayInhibited ? dpGetUnit(alarmDelayDPe) : "s";
    g_bAlarmDelayIsValue = g_bAlarmDelayInhibited || g_sAlarmDelay == (string)((int) g_sAlarmDelay);

	if (g_bUnSystemAlarmPlc) {
        if (g_bAlarmDelayInhibited) {
            iRes = dpConnect("CPC_DigitalAlarm_FaceplateStatusAnimationCB",
                 sDeviceName + ".ProcessInput.StsReg01",
                 sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
                 sDeviceName + ".ProcessInput.ISt:_alert_hdl.._active",
                 alarmDelayDPe,
                 alarmDelayDPe + ":_online.._invalid",
                 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
                 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
                 sDeviceName + ".eventMask");
        } else {
            iRes = dpConnect("CPC_DigitalAlarm_FaceplateStatusAnimationCB_ConstantDelay",
                 sDeviceName + ".ProcessInput.StsReg01",
                 sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
                 sDeviceName + ".ProcessInput.ISt:_alert_hdl.._active",
                 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
                 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
                 sDeviceName + ".eventMask");
        }
	} else {
		CPC_DigitalAlarm_animatePlcDisconnection();
	}

	g_bCallbackConnected = (iRes >= 0);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalAlarm_FaceplateDisconnect
/**
Do faceplate dpDisconnect

	@param[in]	sDeviceName	Device DP name
*/
CPC_DigitalAlarm_FaceplateDisconnect(string sDeviceName)
{
	int iRes = -1;
	string sPlcName, sSystemName;
	string sPlcDpName;
	dyn_string exceptionInfo;

	if (g_bUnSystemAlarmPlc) {
		sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
		sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
        string alarmDelayDPe = dpAliasToName(sSystemName + g_sAlarmDelay) + "ProcessInput.PosSt";

		cpcConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
		if (dynlen(exceptionInfo) <= 0) g_bUnSystemAlarmPlc = true;
		else g_bUnSystemAlarmPlc = false;

        if (g_bAlarmDelayInhibited) {
            iRes = dpDisconnect("CPC_DigitalAlarm_FaceplateStatusAnimationCB",
                 sDeviceName + ".ProcessInput.StsReg01",
                 sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
                 sDeviceName + ".ProcessInput.ISt:_alert_hdl.._active",
                 alarmDelayDPe,
                 alarmDelayDPe + ":_online.._invalid",
                 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
                 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
                 sDeviceName + ".eventMask");
        } else {
            iRes = dpDisconnect("CPC_DigitalAlarm_FaceplateStatusAnimationCB_ConstantDelay",
                 sDeviceName + ".ProcessInput.StsReg01",
                 sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
                 sDeviceName + ".ProcessInput.ISt:_alert_hdl.._active",
                 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
                 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
                 sDeviceName + ".eventMask");
        }

		g_bCallbackConnected = !(iRes >= 0);
	} else {
		CPC_DigitalAlarm_WidgetDisconnection(g_sWidgetType);
		g_bCallbackConnected = false;
	}
}

CPC_DigitalAlarm_FaceplateStatusAnimationCB_ConstantDelay(string sStsReg01, bit32 bit32StsReg01, string sDp1, bool bStsReg01Invalid, string sDpIStActive, bool bIsAlarmActive,
		string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled, string sDpEvt, int iMask)
{
    CPC_DigitalAlarm_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp1, bStsReg01Invalid, sDpIStActive, bIsAlarmActive,
        "", g_sAlarmDelay, "", false,
        sDpAl, iValAlarm, sDpEnabled, bSystemIntegrityAlarmEnabled, sDpEvt, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalAlarm_FaceplateStatusAnimationCB
/**
Animate faceplate
*/
CPC_DigitalAlarm_FaceplateStatusAnimationCB(string sStsReg01, bit32 bit32StsReg01, string sDp1, bool bStsReg01Invalid, string sDpIStActive, bool bIsAlarmActive,
		string sDp3, anytype alarmDelay, string sDp4, bool alarmDelayInvalid,
		string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled, string sDpEvt, int iMask)
{
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("ISt", getBit(bit32StsReg01, CPC_STSREG01_IST), 1, getBit(bit32StsReg01, UN_STSREG01_ALUNACK) ? "unFaceplate_AlarmNotAck" : "unFaceplate_AlarmActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("AuEAISt", getBit(bit32StsReg01, CPC_STSREG01_AUEAIST), 1, "cpcColor_Faceplate_Status",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("AuIhMBSt", getBit(bit32StsReg01, CPC_STSREG01_AUIHMBST), 1, "cpcColor_Faceplate_Status",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));

	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("ConfigW", getBit(bit32StsReg01, CPC_STSREG01_CONFIGW), 1, "unFaceplate_WarningActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("AlUnAck", getBit(bit32StsReg01, UN_STSREG01_ALUNACK), g_bAlAck, "cpcColor_Alarm_Bad",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
    if(g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("eventState", iMask == 0, 1, "unFaceplate_WarningActive", 
                bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));

	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("MAlBRSt", getBit(bit32StsReg01, CPC_STSREG01_MALBRST), 1, "unFaceplate_WarningActive",
					bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("PosW", getBit(bit32StsReg01, CPC_STSREG01_POSW), 1, "unFaceplate_WarningActive", 
					bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("IOErrorW", getBit(bit32StsReg01, UN_STSREG01_IOERRORW), 1, "unFaceplate_WarningActive", 
					bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("IOSimuW", getBit(bit32StsReg01, UN_STSREG01_IOSIMUW), 1, "unFaceplate_WarningActive", 
					bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
    bool bSMSMail = cpcGenericObject_isAlarmWithMail(substr(sDpIStActive,0,strpos(sDpIStActive,":_alert_hdl.._active")), DPCONFIG_ALERT_BINARYSIGNAL);
    cpcGenericObject_CheckboxAnimate("MailSMS", bSMSMail);
	string color = (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value) ? "unDataNotValid" : "unDisplayValue_Parameter";
	if(g_bSystemConnected)
		setMultiValue("AlarmType" + ".display", "foreCol", color, "AlarmType" + ".display", "text", cpcGenericObject_translateAlarmAcronym(g_dsAlarmTypes));

    if (g_bSystemConnected) {
        if (g_bAlarmDelayIsValue) {
            unGenericObject_DisplayValue(g_sAlarmDelayFormat, g_sAlarmDelayUnit, alarmDelay, "AlarmDelayText", "unDisplayValue_Parameter", 
                        alarmDelayInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
        } else {
            setMultiValue("AlarmDelayText" + ".display", "foreCol", color, "AlarmDelayText" + ".display", "text", alarmDelay);
        }
    }
    setMultiValue("AlarmDelayButton", "enabled", g_bAlarmDelayInhibited, "AlarmDelayButton", "text", g_bAlarmDelayInhibited ? g_sAlarmDelay : "none");
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalAlarm_animatePlcDisconnection
/**
Animate faceplate disconnection
*/
CPC_DigitalAlarm_animatePlcDisconnection()
{
    unGenericObject_DisplayValueDisconnect(makeDynString("AlarmDelayText", "AlarmType"));
    unGenericObject_ColorBoxDisconnect(makeDynString("ISt", "AuEAISt", "AuIhMBSt", "AlUnAck", "eventState", "MAlBRSt", "PosW", "ConfigW", "IOErrorW", "IOSimuW"));
    setMultiValue("AlarmDelayButton", "enabled", false);
}

CPC_DigitalAlarm_ButtonRegisterCB(string sSystemStatus, bool bSystemConnected)
{
	string sDeviceName, sDpType, sFunction, sSystemName;
	int iAction, iRes;
	dyn_string exceptionInfo, dsFunctions, dsUserAccess;
	string sPlcName;

	sDeviceName = unGenericDpFunctions_getDpName($sDpName);
	if (sDeviceName) {
		unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);

		switch (iAction) {
			case UN_ACTION_DISCONNECT:
				CPC_DigitalAlarm_ButtonDisconnection();
				break;
			case UN_ACTION_DPCONNECT:
				sDpType=UN_CONFIG_CPC_DIGITALALARM_DPT_NAME;
				unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
				if (dynlen(exceptionInfo) <= 0)	CPC_DigitalAlarm_ProvideAccessToButtons(sDeviceName, sDpType, dsFunctions, exceptionInfo);
				
				iRes = dpConnect("CPC_DigitalAlarm_ButtonAnimationCB",
						sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
						sDeviceName + ".statusInformation.selectedManager",
						sDeviceName + ".ProcessInput.StsReg01",
						sDeviceName + ".ProcessInput.StsReg01:_online.._aut_inv");
				g_bCallbackConnected = (iRes >= 0);
				break;
			case UN_ACTION_DPDISCONNECT_DISCONNECT:
				CPC_DigitalAlarm_ButtonsDisconnect(sDeviceName);
				CPC_DigitalAlarm_ButtonDisconnection();
				break;
			case UN_ACTION_NOTHING: break;
			default:
				break;
		}
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------
//CPC_DigitalAlarm_ButtonsDisconnect
/**
Disconnects callback function to animate buttons states
	@param[in] 	sDeviceName	The name of the device
*/
CPC_DigitalAlarm_ButtonsDisconnect(string deviceName)
{
	int iRes = -1;

	iRes = dpDisconnect("CPC_DigitalAlarm_ButtonAnimationCB",
				deviceName + ".statusInformation.selectedManager:_lock._original._locked",
				deviceName + ".statusInformation.selectedManager",
				deviceName + ".ProcessInput.StsReg01",
				deviceName + ".ProcessInput.StsReg01:_online.._aut_inv");
	g_bCallbackConnected = !(iRes >= 0);
}

CPC_DigitalAlarm_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager,
    string sDpStsReg01, bit32 fStsReg01Value, string sDpStsReg01Bad, bool bStsReg01Bad)
{
	CPC_DigitalAlarm_ButtonSetState(unGenericDpFunctions_getDpName(sDpStsReg01), UN_CONFIG_CPC_DIGITALALARM_DPT_NAME, g_dsUserAccess,
		makeDynString(bLocked, sSelectedManager, fStsReg01Value, bStsReg01Bad, 0, 0));
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalAlarm_ButtonSetState
/**
Set button state

	@param[in]	 sDeviceName		The device name
	@param[in]	 sDpType		Device type
	@param[in]	 dsUserAccess	User access
	@param[in]	 dsData			List of various parameters
*/
CPC_DigitalAlarm_ButtonSetState(string sDeviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected;
	dyn_string dsButtons;
	string localManager;
	bool bLocked, bStsReg01Bad;
	string sSelectedManager;
	bit32 stsReg01Value;

	bLocked = dsData[1];
	sSelectedManager = dsData[2];
	stsReg01Value = dsData[3];
	bStsReg01Bad = dsData[4];

	dsButtons = makeDynString(UN_FACEPLATE_BUTTON_ACK_ALARM, UN_FACEPLATE_BUTTON_BLOCK_ALARM_ACTION, CPC_FACEPLATE_BUTTON_SET_DA_MAIL_SMS);// Each button except "Select" button
	// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = localManager == "S";
	// 2. Button state
	for(int i = 1; i <= dynlen(dsButtons); i++) {
		bool buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);	// User access
		switch (dsButtons[i]) {
			case CPC_FACEPLATE_BUTTON_SET_DA_MAIL_SMS:
				buttonEnabled = buttonEnabled && bSelected;
				break;
            case UN_FACEPLATE_BUTTON_BLOCK_ALARM_ACTION:
                buttonEnabled = buttonEnabled && !getBit(stsReg01Value, CPC_STSREG01_AUIHMBST);
			default:
				break;
		}
		setValue(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", buttonEnabled);
	}
    setValue(UN_FACEPLATE_BUTTON_PREFIX + UN_FACEPLATE_BUTTON_BLOCK_ALARM_ACTION, "text", getBit(stsReg01Value, CPC_STSREG01_MALBRST) ? "Deblock Alarm" : "Block Alarm");
	// 3. Button Select state
	unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalAlarm_ButtonDisconnection
/**
Animate disconnection
*/
CPC_DigitalAlarm_ButtonDisconnection()
{
	unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_SELECT, UN_FACEPLATE_BUTTON_ACK_ALARM, UN_FACEPLATE_BUTTON_BLOCK_ALARM_ACTION, CPC_FACEPLATE_BUTTON_SET_DA_MAIL_SMS));
}


//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalAlarm_ProvideAccessToButtons
/**
Defines the list of buttons available for a user. The result is passed in global variable.
	@param[in]	sDeviceName	Name of the device
	@param[in]	sDpType		Device Type
	@param[in] 	dsFunctions	A list of functions for a device, specified in device internal configuration.
	@param[out]	exceptionInfo	A list of exceptions.
*/
CPC_DigitalAlarm_ProvideAccessToButtons(string sDeviceName, string sDpType, dyn_string dsFunctions, dyn_string &exceptionInfo)
{
	string sFunction;
	dyn_string dsUserAccess;

	if (dynlen(dsFunctions) >= UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS) {
		sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
	} else {
		dynAppend(exceptionInfo, "DigitalAlarm_ButtonRegisterCB: the list of functions is not correct - number of functions is less than expected");
		return;
	}

	if (sFunction != "") {
		evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" +
					 "dyn_string dsUserAccessReturn;" +
					 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
					 "    {" + sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
					 "    }" +
					 "return dsUserAccessReturn; }", makeDynString(), sDeviceName, sDpType);

			g_dsUserAccess=dsUserAccess;
	} else {
		fwException_raise(exceptionInfo,"ERROR", "DigitalAlarm_ButtonRegisterCB:" + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");
	}
}
