/**@name LIBRARY: unDigitalOutput.ctl

@author: Vincent Forest (AB-CO)

Creation Date: 04 06 2003

Modification History: 
	28/04/2004: Herve
		- add function CPC_DigitalOutput_ObjectListGetValueTime

	14/04/2004: Herve
		- add the function CPC_DigitalOutput_HandleMenu
		- add the function CPC_DigitalOutput_UserLoginGetButtonState

	13/04/2004: Herve
		- modification of the signature CPC_DigitalOutput_MenuConfiguration.

	08/04/2004: Herve 
		modification of the signature of the function CPC_DigitalOutput_ButtonSetState

version 1.0

External Function : 
	. CPC_DigitalOutput_AcknowledgeAlarm : acknowledge DigitalOutput alarms
	. CPC_DigitalOutput_FaceplateStatusDisconnection : animate faceplate disconnection
	. CPC_DigitalOutput_ButtonUserAccess : User access for DigitalOutput buttons
	. CPC_DigitalOutput_ButtonSetState : set button state
	. CPC_DigitalOutput_WidgetDisconnection : animate widget disconnection
	. CPC_DigitalOutput_MenuConfiguration : configuration of popup menu

Internal Functions :
	. CPC_DigitalOutput_FaceplateStatusRegisterCB : register callback function
	. CPC_DigitalOutput_ButtonRegisterCB : register callback function
	. CPC_DigitalOutput_ButtonDisconnection : animate buttons disconnection
	. CPC_DigitalOutput_WidgetRegisterCB : register callback function
	
Purpose: This library contains DigitalOutput object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. DigitalOutput object (DPType etc.)
	. Global variables and $parameters defined in DigitalOutput faceplate and widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalOutput_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_DigitalOutput_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	CPC_DigitalInput_ObjectListGetValueTime(sDeviceName, sDeviceType, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalOutput_AcknowledgeAlarm
/**
Purpose: acknowledge DigitalOutput alarms

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsNeedAck, dyn_string, output, DPEs names that need acknowledgement. 
									 The PLC acknowledgement is specified using UN_ACKNOWLEDGE_PLC string
	
Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in DigitalOutput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_DigitalOutput_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
	CPC_DigitalInput_AcknowledgeAlarm(sDpName, sDpType, dsNeedAck);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalOutput_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsAccessOk, dyn_string, input, authorized action
	- menuList, dyn_string, output, menu to pop-up
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_DigitalOutput_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
	CPC_DigitalInput_MenuConfiguration(sDpName, sDpType, dsAccessOk, menuList);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalOutput_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_DigitalOutput_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
	CPC_DigitalInput_HandleMenu(deviceName, sDpType, menuList, menuAnswer);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalOutput_FaceplateStatusRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in DigitalOutput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_DigitalOutput_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
	CPC_DigitalInput_FaceplateStatusRegisterCB(sDp, bSystemConnected);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalOutput_FaceplateStatusDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_DigitalOutput_FaceplateStatusDisconnection()
{
	CPC_DigitalInput_FaceplateStatusDisconnection();
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalOutput_ButtonRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in DigitalOutput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_DigitalOutput_ButtonRegisterCB(string sDp, bool bSystemConnected)
{
	CPC_DigitalInput_ButtonRegisterCB(sDp, bSystemConnected);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalOutput_ButtonUserAccess
/**
Purpose: User access for DigitalOutput buttons

Parameters: 
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_DigitalOutput_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	CPC_DigitalInput_ButtonUserAccess(sDpName, sDpType, dsAccess);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalOutput_UserLoginGetButtonState
/**
Purpose: function called before the button set state when a user logs in. 

Parameters:
	deviceName: string, input, the device name
	dsReturnData: dyn_string, output, the data are returned here

Usage: Internal function

PVSS manager usage:  Ctrl, NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
		-> $sDpName, string, the device DP name
		-> g_sFaceplateButtonType, string, faceplate type
	. PVSS version: 2.12.1 
	. operating system: NT, W2000 and Linux, but tested only under W2000.
	. distributed system: yes but not tested.
*/
CPC_DigitalOutput_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
	CPC_DigitalInput_UserLoginGetButtonState(deviceName, sType, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalOutput_ButtonSetState
/**
Purpose: set button state

Parameters: 
	- deviceName, string, input, device
	- sDpType, string, input, dptype
	- dsUserAccess, dyn_string, input, user access
	- bLocked, bool, input, for selection
	- sSelectedManager, string, input, for selection
	- stsReg01Value, bit32, input, status register 1
	- stsReg01Bad, bool, input, status register 1 invalid
	- stsReg02Value, bit32, input, status register 2
	- stsReg02Bad, bool, input, status register 2 invalid
		
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_DigitalOutput_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	CPC_DigitalInput_ButtonSetState(deviceName, UN_CONFIG_CPC_DIGITALOUTPUT_DPT_NAME, dsUserAccess, dsData);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalOutput_ButtonDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_DigitalOutput_ButtonDisconnection()
{
	CPC_DigitalInput_ButtonDisconnection();
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalOutput_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in DigitalOutput widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_DigitalOutput_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	CPC_DigitalInput_WidgetRegisterCB(sDp, bSystemConnected);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_DigitalOutput_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType, string, input, widget type. ex: "heater"

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_DigitalOutput_WidgetDisconnection(string sWidgetType)
{
	CPC_DigitalInput_WidgetDisconnection(sWidgetType);
}

