/**@name LIBRARY: unLocal.ctl

@author: Frederic Bernard (AB-CO-IS)

Creation Date: 05 06 2003

Modification History:
    23/11/2011: Alexey
        [UCPC-603] refactor menu creation and handling

    31/10/2011: Alexey
        [UCPC-592] animate PosW on Local widget

    26/09/2011: Alexey
        fixed widget animation in case without alerts

    21/09/2011: Alexey
        [UCPC-546] use cpcGenericObject_ColorBoxAnimateAlarm

    20/09/2011: Alexey
        [UCPC-548][UCPC-547] upd faceplate

	02/08/2011: Alexey
		[UCPC-451] 'Alarm Not Ack' is never blinking

	29/07/2011: Alexey
		[UCPC-438] alarm animation based no PVSS alarm; does not acknowledge PLC

	14/07/2011: Alexey
		fixed getting ack settings

	07/07/2011: Alexey
		[UCPC-265] update alarm acknowledge
		removed UN_STSREG01_POSALE

	01/07/2011: Alexey
		use constant for params

	28/06/2011: Alexey
		use new version of cpcGenericObject_WidgetAnimation

	18/05/2011: Alexey
		use cpcGenericObject_HandleEventMaskMenu and CPC functions to manage menu
		use cpcGenericObject_WidgetAnimation instead of unGenericObject_WidgetAnimation

  07/04/2011: Alexey
	- [UCPC-209] added Warning_PositionWarning to the faceplate
	- [UCPC-209] changed Position Alarm bit for the widget 

  27/03/2009: Frederic
	- CPC_Local_WidgetConnect(): move g_bCallbackConnected upper
	- CPC_Local_ButtonRegisterCB(): remove fwException_raise(...) before g_dsUserAccess
        
  28/12/2007: Herve
    - device acccess control
    
	02/05/2006: Herve
		- implementation of maskEvent in widget, faceplate and rigthClick
		
	11/04/2006: Herve
		- remove dynAppend(menuList, "SEPARATOR"); in CPC_Local_MenuConfiguration function

	15/12/2005: Celine
		- CPC_Local_FaceplateStatusRegisterCB, CPC_Local_FaceplateStatusAnimationCB, CPC_Local_WidgetRegisterCB,
		 : add g_bSystemConnected checking

	09/08/2005: Frederic
		- CPC_Local_ButtonRegisterCB(), add evalScript to run dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS] function

	15/04/2005: Herve
		- unGenericObject_GetPLCNameFromDpName: return the PLCtype+"_"+plcName, in case of _UnPlc it returns _unPlc_+plcName
		
	05/11/2004: Herve
		- in the WidgetRegisterCB, faceplateRegiterCB: use the systemName of the $sIdentifier to dpConnect to the PLC

	26/10/2004: Herve
		- in the CPC_Local_ButtonRegisterCB function: call the CPC_Local_ButtonUserAccess to get the list of allowed buttons. This list is 
		reseted if the user logs in/out while the remote system is off.

	03/05/2004: Herve
		- during WidgetRegister and FaceplateRegister call unGenericObject_GetPLCNameFromDpName to get the PLC DPName
		- during WidgetRegister check if the dpType is correct

	28/04/2004: Herve
		- add function CPC_Local_ObjectListGetValueTime

	14/04/2004: Herve
		- add the function CPC_Local_HandleMenu
		- add the function CPC_Local_UserLoginGetButtonState

	13/04/2004: Herve
		- in function CPC_Local_ButtonRegisterCB add the set Manreg01 to 0 in case of dpConnect.
		- modification of the signature CPC_Local_MenuConfiguration and add the creation of the menu.

	08/04/2004: Herve 
		modification of the signature of the function CPC_Local_ButtonSetState

	16/01/2004: Herve
		remove the dpConnect, dpDisconnect and callback function in the case of no PLC systemAlarm. Ease the development 
		and test.
		
	03/01/2004: Herve
		- add in all dpConnect and dpDisconnect the _unSystemAlarm_PLC_DS dp if the PLC is configured for systemIntegrity. 
		The g_bUnSystemAlarmPlc is set to true if the PLC is configured for systemIntegrity
		change the dpConnect and dpDisconnect of _aut_inv to _invalid

version 1.0

External Function : 
	. CPC_Local_AcknowledgeAlarm : acknowledge Local alarms
	. CPC_Local_FaceplateStatusDisconnection : animate faceplate disconnection
	. CPC_Local_ButtonUserAccess : User access for Local buttons
	. CPC_Local_ButtonSetState : set button state
	. CPC_Local_WidgetDisconnection : animate widget disconnection
	. CPC_Local_MenuConfiguration : configuration of popup menu

Internal Functions :
	. CPC_Local_FaceplateStatusRegisterCB : register callback function
	. CPC_Local_FaceplateStatusAnimationCB : animate faceplate
	. CPC_Local_ButtonRegisterCB : register callback function
	. CPC_Local_ButtonAnimationCB : animate buttons
	. CPC_Local_ButtonDisconnection : animate buttons disconnection
	. CPC_Local_WidgetRegisterCB : register callback function
	. CPC_Local_WidgetAnimationCB : animate widget
		
Purpose: This library contains Local object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Local object (DPType etc.)
	. Global variables and $parameters defined in Local faceplate and widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_Local_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Local_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
    float value;
    string sTime;
    
    dpGet(sDeviceName+".ProcessInput.PosAl", value, sDeviceName+".ProcessInput.PosAl:_online.._stime", sTime);
    dsReturnData[1] = sTime;
    dsReturnData[2] = value;
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_Local_AcknowledgeAlarm
/**
Purpose: acknowledge Local alarms

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsNeedAck, dyn_string, output, DPEs names that need acknowledgement. 
									 The PLC acknowledgement is specified using UN_ACKNOWLEDGE_PLC string
	
Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Local faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Local_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
    dsNeedAck = makeDynString("ProcessInput.PosAl");
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_Local_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsAccessOk, dyn_string, input, authorized action
	- menuList, dyn_string, output, menu to pop-up
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Local_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
    dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_ACK_TEXT, UN_POPUPMENU_FACEPLATE_TEXT);

    cpcGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
    cpcGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_Local_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_Local_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
    dyn_string exceptionInfo;

    cpcGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);

    if (dynlen(exceptionInfo) > 0) {
        unSendMessage_toExpertException("Right Click",exceptionInfo);
    }
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_Local_FaceplateStatusRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Local faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Local_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
    string deviceName, sSystemName;
    int iAction;
    dyn_string exceptionInfo;
    bool bRemote;
    
    deviceName = unGenericDpFunctions_getDpName($sDpName);
    sSystemName = unGenericDpFunctions_getSystemName(deviceName);
    unDistributedControl_isRemote(bRemote, sSystemName);
    g_bSystemConnected = bRemote ? bSystemConnected : true;

    g_bPHFOn = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_HFON, exceptionInfo);
    g_bPHFOff = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_HFOFF, exceptionInfo);

    unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
    switch (iAction)
        {
        case UN_ACTION_DISCONNECT:
            CPC_Local_FaceplateStatusDisconnection();
            break;
        case UN_ACTION_DPCONNECT:
            CPC_Local_FaceplateConnect(deviceName);
            break;
        case UN_ACTION_DPDISCONNECT_DISCONNECT:
            CPC_Local_FaceplateDisconnect(deviceName);
            CPC_Local_FaceplateStatusDisconnection();
            break;
        case UN_ACTION_NOTHING:
        default:
            break;
        }
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_Local_FaceplateConnect
/**
Purpose: connect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Local faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_Local_FaceplateConnect(string deviceName)
{
    int iRes, iAlertType1;
    dyn_string exceptionInfo;
    string sPlcName, sSystemName;

    sSystemName = unGenericDpFunctions_getSystemName(deviceName);
    sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
    g_bUnSystemAlarmPlc = (sPlcName != "") && dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName);

    iRes = dpGet(deviceName + ".ProcessInput.PosAl:_alert_hdl.._type", iAlertType1);
    g_bBoolAlert = ((iRes >= 0) && (iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL));
    g_bAlAck = cpcExportGenericFunctions_getAcknowledgeAlarmValue(deviceName, ".ProcessInput.PosAl") != "N";

    if(g_bUnSystemAlarmPlc) {
        iRes = dpConnect("CPC_Local_FaceplateStatusAnimationCB",
                             deviceName + ".ProcessInput.StsReg01", 
                             deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                             deviceName + ".ProcessInput.PosAl:_alert_hdl.._active",
                             deviceName + ".ProcessInput.PosAl:_alert_hdl.._act_state",
                             deviceName + ".ProcessInput.PosAl:_alert_hdl.._ack_possible",
                             sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
                             sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
                             deviceName + ".eventMask");
    } else {
        CPC_Local_FaceplateStatusDisconnection();
    }
    g_bCallbackConnected = (iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_Local_FaceplateDisconnect
/**
Purpose: disconnect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Local faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_Local_FaceplateDisconnect(string deviceName)
{
    int iRes;
    string sPlcName, sSystemName;

    if(g_bUnSystemAlarmPlc) {
        sSystemName = unGenericDpFunctions_getSystemName(deviceName);
        sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);

        iRes = dpDisconnect("CPC_Local_FaceplateStatusAnimationCB",
                             deviceName + ".ProcessInput.StsReg01", 
                             deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                             deviceName + ".ProcessInput.PosAl:_alert_hdl.._active",
                             deviceName + ".ProcessInput.PosAl:_alert_hdl.._act_state",
                             deviceName + ".ProcessInput.PosAl:_alert_hdl.._ack_possible",
                             sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
                             sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
                             deviceName + ".eventMask");
    }
    g_bCallbackConnected = !(iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_Local_FaceplateStatusAnimationCB
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Local faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Local_FaceplateStatusAnimationCB(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
                                   string sDp5, bool bIsAlarmActive, string sDp6, int iAlarmState, string sDp7, bool bAckPossible,
                                   string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled, string sDpEvt, int iMask)
{
    if(g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("Warning_IOError", getBit(bit32StsReg01, UN_STSREG01_IOERRORW), 1, "unFaceplate_WarningActive",
                bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
    if(g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("Warning_IOSimulated", getBit(bit32StsReg01, UN_STSREG01_IOSIMUW), 1, "unFaceplate_WarningActive", 
                bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
    if(g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("Warning_PositionWarning", getBit(bit32StsReg01, CPC_STSREG01_POSW), 1, "unFaceplate_WarningActive", 
                bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
    if(g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("Status_OnOpened", getBit(bit32StsReg01, UN_STSREG01_ONST), g_bPHFOn, 
                "cpcColor_Faceplate_Status", bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
    if(g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("Status_OffClosed", getBit(bit32StsReg01, UN_STSREG01_OFFST), g_bPHFOff, 
                "cpcColor_Faceplate_Status", bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
    if(g_bSystemConnected)
        cpcGenericObject_ColorBoxAnimateAlarm("Alarm_Position", iAlarmState, true, bIsAlarmActive,
            bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
    if(g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("Alarm_UnAck", bAckPossible, g_bAlAck && bIsAlarmActive, "cpcColor_Alarm_Bad", 
            bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
    if(g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("eventState", iMask == 0, 1, "unFaceplate_WarningActive", 
                bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_Local_FaceplateStatusDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Local_FaceplateStatusDisconnection()
{
    unGenericObject_ColorBoxDisconnect(makeDynString("Alarm_Position", "Alarm_UnAck",
                                                     "Warning_IOError", "Warning_IOSimulated", 
                                                     "Warning_PositionWarning",
                                                     "Status_OnOpened", "Status_OffClosed"));
    unGenericObject_ColorBoxDisconnect(makeDynString("eventState"));
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_Local_ButtonRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Local faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Local_ButtonRegisterCB(string sDp, bool bSystemConnected)
{
    string deviceName, sFunction, sDpType;
    int iAction, iRes, iManReg01;
    dyn_string exceptionInfo, dsUserAccess, dsFunctions;
    
    deviceName = unGenericDpFunctions_getDpName($sDpName);
    unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
    switch (iAction) {
        case UN_ACTION_DISCONNECT:
            CPC_Local_ButtonDisconnection();
            break;
        case UN_ACTION_DPCONNECT:
            // Send 0 to ManReg01 if necessary
            iRes = dpGet(deviceName + ".ProcessOutput.ManReg01", iManReg01);
            if ((iRes < 0) || (iManReg01 != 0)) {
                dpSet(deviceName + ".ProcessOutput.ManReg01", 0);
            }

            sDpType=UN_CONFIG_CPC_LOCAL_DPT_NAME;
            unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
            sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
                         
            if (sFunction != "") {
                evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" + 
                             "dyn_string dsUserAccessReturn;" +
                             "if (isFunctionDefined(\"" + sFunction  + "\"))" +
                             "    {" +
                                sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
                             "    }" +
                             "return dsUserAccessReturn; }", makeDynString(), deviceName, sDpType);
                        
                g_dsUserAccess=dsUserAccess;
            } else {
                fwException_raise(exceptionInfo,"ERROR", "CPC_Local_ButtonRegisterCB:" + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");
            }
                    
            
            iRes = dpConnect("CPC_Local_ButtonAnimationCB",
                             deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                             deviceName + ".statusInformation.selectedManager",
                             deviceName + ".ProcessInput.StsReg01",
                             deviceName + ".ProcessInput.StsReg01:_online.._invalid");
            g_bCallbackConnected = (iRes >= 0);
            break;
        case UN_ACTION_DPDISCONNECT_DISCONNECT:
            iRes = dpDisconnect("CPC_Local_ButtonAnimationCB",
                             deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                             deviceName + ".statusInformation.selectedManager",
                             deviceName + ".ProcessInput.StsReg01",
                             deviceName + ".ProcessInput.StsReg01:_online.._invalid");
            g_bCallbackConnected = !(iRes >= 0);
            CPC_Local_ButtonDisconnection();
            break;
        case UN_ACTION_NOTHING:
        default:
            break;
        }
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_Local_ButtonAnimationCB
/**
Purpose: animate buttons

Parameters: 
	- dbPermissions, dyn_bool, input, list of permissions, length = 4, order : 1=monitor, 2=operator, 3=expert, 4=admin
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Local_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager, 
                          string sDpStsReg01, bit32 stsReg01Value, string sDpStsReg01Bad, bool bStsReg01Bad)
{
    CPC_Local_ButtonSetState(unGenericDpFunctions_getDpName(sDpStsReg01), UN_CONFIG_CPC_LOCAL_DPT_NAME, g_dsUserAccess, 
                           makeDynString(bLocked, sSelectedManager, stsReg01Value, bStsReg01Bad, 0, 0));
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_Local_ButtonUserAccess
/**
Purpose: User access for Local buttons

Parameters: 
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Local_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
    bool operator, expert, admin;
    int i, length;
    dyn_bool dbPermissions;
    dyn_string exceptionInfo;
    string sOperAction, sExpAction, sAdminAction;
    bool bActionDefined = false;

    unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
    if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME)) {
        bActionDefined=true;
    }
    operator = dbPermissions[2];
    expert = dbPermissions[3];
    admin = dbPermissions[4];
    dsAccess = makeDynString();
    if (operator) {
        if(bActionDefined) {
            dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
        } else {
            if (cpcExportGenericFunctions_getAcknowledgeAlarmValue(sDpName, ".ProcessInput.PosAl") == "Y") {
                dynAppend(dsAccess, UN_FACEPLATE_BUTTON_ACK_ALARM);
            }
        }
    }
    if (expert) {
        if(bActionDefined)
            dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
        else
            dynAppend(dsAccess, UN_FACEPLATE_MASKEVENT);
    }
    if (admin) {
        if(bActionDefined)
            dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
    }
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_Local_UserLoginGetButtonState
/**
Purpose: function called before the button set state when a user logs in. 

Parameters:
	deviceName: string, input, the device name
	dsReturnData: dyn_string, output, the data are returned here

Usage: Internal function

PVSS manager usage:  Ctrl, NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
		-> $sDpName, string, the device DP name
		-> g_sFaceplateButtonType, string, faceplate type
	. PVSS version: 2.12.1 
	. operating system: NT, W2000 and Linux, but tested only under W2000.
	. distributed system: yes but not tested.
*/

CPC_Local_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
    unGenericObject_UserCBGetButtonState(deviceName, sType, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_Local_ButtonSetState
/**
Purpose: set button state

Parameters: 
	- deviceName, string, input, device
	- sDpType, string, input, dptype
	- dsUserAccess, dyn_string, input, user access
	- dsData, dyn_string, parameter
		
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Local_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
    bool buttonEnabled;

    buttonEnabled = (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_ACK_ALARM) > 0);	// User access
    setValue(UN_FACEPLATE_BUTTON_PREFIX + UN_FACEPLATE_BUTTON_ACK_ALARM, "enabled", buttonEnabled);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_Local_ButtonDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Local_ButtonDisconnection()
{
    unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_ACK_ALARM));
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_Local_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Local widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Local_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
    string deviceName;
    int iAction;
    dyn_string exceptionInfo;
    string sPlcName, sSystemName;
    bool bRemote;

    deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
    if (deviceName == "") {// In case of disconnection
        deviceName = g_sDpName;
    }

    sSystemName = unGenericDpFunctions_getSystemName(deviceName);
    sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
    
    unDistributedControl_isRemote(bRemote, sSystemName);
    g_bSystemConnected = bRemote ? bSystemConnected : true;
    g_bUnSystemAlarmPlc = (sPlcName != "") && dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName);
        
    unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
    switch (iAction) {
        case UN_ACTION_DISCONNECT:
            CPC_Local_WidgetDisconnection(g_sWidgetType);
            break;
        case UN_ACTION_DPCONNECT:
            g_sDpName = deviceName;
            CPC_Local_WidgetConnect(deviceName, sPlcName, sSystemName);
            break;
        case UN_ACTION_DPDISCONNECT_DISCONNECT:
            CPC_Local_WidgetDisconnect(deviceName, sPlcName, sSystemName);
            CPC_Local_WidgetDisconnection(g_sWidgetType);
            break;
        case UN_ACTION_NOTHING:
        default:
            break;
    }
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_Local_WidgetConnect
/**
Purpose: Connect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Local faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_Local_WidgetConnect(string deviceName, string sPlcName, string sSystemName)
{
    int iRes, iAlertType1;
    //  check if the device name is of type UN_CONFIG_CPC_LOCAL_DPT_NAME if not like of no PLC. -> not available.
    if(dpTypeName(deviceName) != UN_CONFIG_CPC_LOCAL_DPT_NAME) {
        g_bUnSystemAlarmPlc = false;
    }

    iRes = dpGet(deviceName + ".ProcessInput.PosAl:_alert_hdl.._type", iAlertType1);
    g_bBoolAlert = ((iRes >= 0) && (iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL));

    if(g_bUnSystemAlarmPlc) {
        if (g_bBoolAlert) {
            iRes = dpConnect("CPC_Local_WidgetAnimationCB",
                                deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                                deviceName + ".statusInformation.selectedManager",
                                deviceName + ".ProcessInput.StsReg01",
                                deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                deviceName + ".ProcessInput.PosAl:_alert_hdl.._active",
                                deviceName + ".ProcessInput.PosAl:_alert_hdl.._act_state",
                                deviceName + ".ProcessInput.PosAl:_alert_hdl.._ack_possible",
                                sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
                                sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
                                deviceName + ".eventMask");
        } else {
            iRes = dpConnect("CPC_Local_WidgetAnimationCB_NoAlarm",
                                deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                                deviceName + ".statusInformation.selectedManager",
                                deviceName + ".ProcessInput.StsReg01",
                                deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
                                sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
                                deviceName + ".eventMask");
        }
    g_bCallbackConnected = (iRes >= 0);
    } else {
        CPC_Local_WidgetDisconnection(g_sWidgetType);
    }
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_Local_WidgetDisconnect
/**
Purpose: Connect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Local faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_Local_WidgetDisconnect(string deviceName, string sPlcName, string sSystemName)
{
    int iRes;

    if(g_bUnSystemAlarmPlc) {
        if (g_bBoolAlert) {
            iRes = dpDisconnect("CPC_Local_WidgetAnimationCB",
                                deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                                deviceName + ".statusInformation.selectedManager",
                                deviceName + ".ProcessInput.StsReg01",
                                deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                deviceName + ".ProcessInput.PosAl:_alert_hdl.._active",
                                deviceName + ".ProcessInput.PosAl:_alert_hdl.._act_state",
                                deviceName + ".ProcessInput.PosAl:_alert_hdl.._ack_possible",
                                sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
                                sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
                                deviceName + ".eventMask");
        } else {
            iRes = dpDisconnect("CPC_Local_WidgetAnimationCB_NoAlarm",
                                deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                                deviceName + ".statusInformation.selectedManager",
                                deviceName + ".ProcessInput.StsReg01",
                                deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
                                sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
                                deviceName + ".eventMask");
        }
    }
    g_bCallbackConnected = !(iRes >= 0);
}

CPC_Local_WidgetAnimationCB_NoAlarm(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
                        string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
                        string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled, string sEvtMask, int iMask)
{
    CPC_Local_WidgetAnimationCB(sDpLock, bLocked, sDpManager, sSelectedManager,
                            sDpStsReg01, bit32StsReg01, sDp4, bStsReg01Invalid,
                            "", 0, "", 0, "", 0,
                            sDpAl, iAlValue, sDpEnabled, bPlcEnabled, sEvtMask, iMask);
}
// CPC_Local_WidgetAnimationCB
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Local faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Local_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
                        string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
                        string sDp5, bool bIsAlarmActive, string sDp6, int iActState, string sDp7, bool bAckPossible,
                        string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled, string sEvtMask, int iMask)
{
    string selectColor, warningLetter, warningColor, alarmLetter, alarmColor, sFunction;
    bool bLockVisible;
    dyn_string dsFunctions, exceptionInfo;
    string sBodyColor = "cpcColor_Widget_Status";

    unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, selectColor, bLockVisible);
    unGenericObject_WidgetWarningAnimation(bit32StsReg01, makeDynInt(UN_STSREG01_POSALW, UN_STSREG01_IOSIMUW, UN_STSREG01_IOERRORW), makeDynString(CPC_WIDGET_TEXT_WARNING_DEFAULT, CPC_WIDGET_TEXT_WARNING_SIMU, CPC_WIDGET_TEXT_WARNING_ERROR), warningLetter, warningColor);

    if ((!bPlcEnabled) || (iAlValue > c_unSystemIntegrity_no_alarm_value)) {
        warningLetter = CPC_WIDGET_TEXT_OLD_DATA;
        warningColor = "unDataNotValid";
    }

    if (bStsReg01Invalid) {
        warningLetter = CPC_WIDGET_TEXT_INVALID;
        warningColor = "unDataNotValid";
    }

    if (bStsReg01Invalid) {
        sBodyColor = "unDataNotValid";
    } else {
        if (bIsAlarmActive) {
            if ((iActState == DPATTR_ALERTSTATE_APP_NOT_ACK) || (iActState == DPATTR_ALERTSTATE_APP_ACK)) {
                sBodyColor = "unFaceplate_AlarmActive";
            }
            if (bAckPossible) {
                sBodyColor = "unWidget_AlarmNotAck";
            }
        } else {
            alarmLetter = CPC_WIDGET_TEXT_ALARM_MASKED;
            alarmColor = "unAlarmMasked";
        }
    }

    if(g_bSystemConnected)
        setMultiValue("WarningText", "text", warningLetter, "WarningText", "foreCol", warningColor,
                      "AlarmText", "text", alarmLetter, "AlarmText", "foreCol", alarmColor,
                      "SelectArea", "foreCol", selectColor, "LockBmp", "visible", bLockVisible,
                      "WidgetArea", "visible", true);

    sFunction = cpcGenericObject_GetAnimationFunction(g_sWidgetType);
    if (isFunctionDefined(sFunction) && sFunction != "") {
        if(g_bSystemConnected) {
            execScript("main(int iOn, int iOff, string sBodyColor) {" +
                   sFunction + "(iOn, iOff, sBodyColor);}",
                   makeDynString(), getBit(bit32StsReg01, UN_STSREG01_ONST), getBit(bit32StsReg01, UN_STSREG01_OFFST), sBodyColor);
        }
    }

    if(g_bSystemConnected) {
        eventState.visible = iMask == 0;
    }
}

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_Local_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType, string, input, widget type. ex: "heater"

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_Local_WidgetDisconnection(string sWidgetType)
{
	cpcGenericObject_WidgetDisconnection(sWidgetType, UN_CONFIG_CPC_LOCAL_DPT_NAME);
	eventState.visible = false;
}

