/**@name LIBRARY: unOnOff.ctl

@author: Vincent Forest (AB-CO)

Creation Date: 04 06 2003

Modification History:
    13/09/2012: Alexey
        [UCPC-931] add alarms item to the right click menu

    16/05/2012: Alexey
        [UCPC-787] introduce AlSt bit and letter constants

    08/03/2012: Alexey
        [UCPC-632] show custom labels for on and off states

    29/02/2012: Alexey
        [UCPC-684] "allow restart" available for "operator"

    23/11/2011: Alexey
        [UCPC-603] refactor menu creation and handling

    04/10/2011: Alexey
        fixed failsafe field animation in the faceplate

    30/09/2011: Alexey
        [UCPC-567] added interlock status to the widget animation cb

    20/09/2011: Alexey
        [UCPC-548][UCPC-547] upd faceplate
        [UCPC-542] removed Request -> Local from the faceplate
        [UCPC-552] animate HWLocal parameter

    13/09/2011: Alexey
        [UCPC-531] took off cpcGenericObject_WidgetAnimateManualRestart, updated cpcGenericObject_WidgetAnimationDoubleStsReg
        [UCPC-532] animated POutOff parameter and OutOffOVSt bit

    26/08/2011: Alexey
        fixed CPC_OnOff_FaceplateStatusDisconnection

	27/07/2011: Alexey
		[UCPC-412] disabled "Manual restart needed" faceplate box when it's necessary

	05/07/2011: Alexey
		[UCPC-323] fixed ManRestart animation

	01/07/2011: Alexey
		use constant for params

	28/06/2011: Alexey
		[UCPC-311] animate Soft/Hard local mode in the faceplate and widget
		removed UN_STSREG02_PHLIST bit

	27/06/2011: Alexey
		[UCPC-328] animate interlock status in the new way
		[UCPC-323] animate NeedRestart box

	23/06/2011: Alexey
		renamed cpcFaceplateButtonState_updateAccess -> cpcFaceplateButtonState_ButtonSetState

	22/06/2011: Alexey
		[UCPC-323] added AllowRestart button

	18/05/2011: Alexey
		use cpcGenericObject_HandleEventMaskMenu and CPC functions to manage menu
		use cpcGenericObject_WidgetAnimationDoubleStsReg instead of unGenericObject_WidgetAnimationDoubleStsReg

	15/04/2011: amerezhi
		- [UCPC-208] added TStopISt/FuStopISt

	29/03/2011: amerezhi
		- [UCPC-208] display forced mode inhibited as 'h' on widget

  15/11/2010: Quique
  - [UCPC-154] Inhibit manual/forced mode must be shown in faceplate and widget (letter "h")

  27/03/2009: Frederic
	- CPC_OnOff_WidgetConnect(): move g_bCallbackConnected upper
	- CPC_OnOff_ButtonRegisterCB(): remove fwException_raise(...) before g_dsUserAccess
        
	24/11/08 Quique
		Include new bit UN_STSREG02_ALBW and animation of the widget (alarm blocked)
		
   04/05/08: Quique
  	- Include the function: CPC_OnOff_ObjectListGetValueTime() to get the snapshot Value
	
  28/12/2007: Herve
    - device acccess control
    
	02/05/2006: Herve
		- implementation of maskEvent in widget, faceplate and rigthClick
		
	11/04/2006: Herve
		- remove dynAppend(menuList, "SEPARATOR"); in CPC_OnOff_MenuConfiguration function

	15/12/2005:Celine
		- CPC_OnOff_FaceplateStatusRegisterCB, CPC_OnOff_FaceplateStatusAnimationCB, CPC_OnOff_WidgetRegisterCB : add g_bSystemConnected checking

	09/08/2005: Frederic
		- CPC_OnOff_ButtonRegisterCB(), add evalScript to run dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS] function

	06/06/2005: Herve
		- dupplication of code (two call to unGenericDpFunctions_getWidgetDpName) in WidgetRegisterCB
		
	15/04/2005: Herve
		- unGenericObject_GetPLCNameFromDpName: return the PLCtype+"_"+plcName, in case of _UnPlc it returns _unPlc_+plcName
		
	05/11/2004: Herve
		- in the WidgetRegisterCB, faceplateRegiterCB: use the systemName of the $sIdentifier to dpConnect to the PLC

	26/10/2004: Herve
		- in the CPC_OnOff_ButtonRegisterCB function: call the CPC_OnOff_ButtonUserAccess to get the list of allowed buttons. This list is 
		reseted if the user logs in/out while the remote system is off.

	03/05/2004: Herve
		- during WidgetRegister and FaceplateRegister call unGenericObject_GetPLCNameFromDpName to get the PLC DPName
		- during WidgetRegister check if the dpType is correct

	14/04/2004: Herve
		- add the function CPC_OnOff_HandleMenu
		- add the function CPC_OnOff_UserLoginGetButtonState

	13/04/2004: Herve
		- in function CPC_OnOff_ButtonRegisterCB add the set Manreg01 to 0 in case of dpConnect.
		- modification of the signature CPC_OnOff_MenuConfiguration and add the creation of the menu.

	08/04/2004: Herve 
		modification of the signature of the function CPC_OnOff_ButtonSetState

	16/01/2004: Herve
		remove the dpConnect, dpDisconnect and callback function in the case of no PLC systemAlarm. Ease the development 
		and test.
		
	01/05/2004: Herve
		- add in all dpConnect and dpDisconnect the _unSystemAlarm_PLC_DS dp if the PLC is configured for systemIntegrity. 
		The g_bUnSystemAlarmPlc is set to true if the PLC is configured for systemIntegrity
		change the dpConnect and dpDisconnect of _aut_inv to _invalid

version 1.0

External Function : 
	. CPC_OnOff_AcknowledgeAlarm : acknowledge OnOff alarms
	. CPC_OnOff_FaceplateStatusDisconnection : animate faceplate disconnection
	. CPC_OnOff_ButtonUserAccess : User access for OnOff buttons
	. CPC_OnOff_ButtonSetState : set button state
	. CPC_OnOff_WidgetDisconnection : animate widget disconnection
	. CPC_OnOff_MenuConfiguration : configuration of popup menu

Internal Functions :
	. CPC_OnOff_FaceplateStatusRegisterCB : register callback function
	. CPC_OnOff_FaceplateStatusAnimationCB_NoAlarm : animate faceplate
	. CPC_OnOff_FaceplateStatusAnimationCB : animate faceplate
	. CPC_OnOff_ButtonRegisterCB : register callback function
	. CPC_OnOff_ButtonAnimationCB : animate buttons
	. CPC_OnOff_ButtonDisconnection : animate buttons disconnection
	. CPC_OnOff_WidgetRegisterCB : register callback function
	. CPC_OnOff_WidgetAnimationCB : animate widget
		
Purpose: This library contains OnOff object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. OnOff object (DPType etc.)
	. Global variables and $parameters defined in OnOff faceplate and widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_OnOff_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 3.1 
	. operating system: XP.
	. distributed system: yes.
*/

CPC_OnOff_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	int iValue;
	bit32 b32StsReg01;
	string sTime,sRes;
	
	
	dpGet(sDeviceName+".ProcessInput.StsReg01", iValue, sDeviceName+".ProcessInput.StsReg01:_online.._stime", sTime);
	dsReturnData[1] = sTime;
	// Treat the first two bits
	b32StsReg01=(bit32)iValue;
  if (getBit(b32StsReg01, UN_STSREG01_ONST)) {
    if (!getBit(b32StsReg01, UN_STSREG01_OFFST)) {
      sRes = "On";
    }
  } else if (getBit(b32StsReg01, UN_STSREG01_OFFST)) {
    sRes = "0ff";
  } else {
    sRes = "";
  }

  dsReturnData[2] = sRes;
}


// CPC_OnOff_AcknowledgeAlarm
/**
Purpose: acknowledge OnOff alarms

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsNeedAck, dyn_string, output, DPEs names that need acknowledgement. 
									 The PLC acknowledgement is specified using UN_ACKNOWLEDGE_PLC string
	
Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in OnOff faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_OnOff_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
	dsNeedAck = makeDynString("ProcessInput.StartISt", "ProcessInput.TStopISt", "ProcessInput.FuStopISt", "ProcessInput.AlSt", UN_ACKNOWLEDGE_PLC);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_OnOff_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsAccessOk, dyn_string, input, authorized action
	- menuList, dyn_string, output, menu to pop-up
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_OnOff_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
	dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_ACK_TEXT, UN_POPUPMENU_FACEPLATE_TEXT, CPC_POPUPMENU_ALARMS_TEXT);

	cpcGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	cpcGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_OnOff_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_OnOff_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
	dyn_string exceptionInfo;

	cpcGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);

	if (dynlen(exceptionInfo) > 0) {
		unSendMessage_toExpertException("Right Click",exceptionInfo);
	}

}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_OnOff_FaceplateStatusRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in OnOff faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_OnOff_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName,sSystemName;
	int iAction;
	dyn_string exceptionInfo;
	bool bRemote;

	deviceName = unGenericDpFunctions_getDpName($sDpName);
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			CPC_OnOff_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
			CPC_OnOff_FaceplateConnect(deviceName);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			CPC_OnOff_FaceplateDisconnect(deviceName);
			CPC_OnOff_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}
//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_OnOff_FaceplateConnect
/**
Purpose: connect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in OnOff faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_OnOff_FaceplateConnect(string deviceName)
{
	int iRes, iAlertType1, iAlertType2, iAlertType3, iAlertType4;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;

// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;
		
	iRes = dpGet(deviceName + ".ProcessInput.StartISt:_alert_hdl.._type", iAlertType1,
				 deviceName + ".ProcessInput.TStopISt:_alert_hdl.._type", iAlertType2,
				 deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._type", iAlertType3,
				 deviceName + ".ProcessInput.AlSt:_alert_hdl.._type", iAlertType4);
    g_bBoolAlert = ((iRes >= 0) && (iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType2 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType3 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType4 == DPCONFIG_ALERT_BINARYSIGNAL));
    g_bPFSPosOn = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_FSPOSON, exceptionInfo);
    g_bPHFOn = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_HFONST, exceptionInfo);
    g_bPHFOff = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_HFOFFST, exceptionInfo);
    g_bPPulseSt = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_PULSEST, exceptionInfo);
    g_bPOutOff = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_OUTOFF, exceptionInfo);
    g_bMRestart = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_MRESTART, exceptionInfo);
    g_bHLDrive = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_HLDRIVE, exceptionInfo);
    setMultiValue("Request_Output_Off_Text", "visible", g_bPOutOff,
                  "Request_Output_Off.colorbox", "visible", g_bPOutOff,
                  "Request_Output_UnOff.colorbox", "visible", g_bPOutOff);
    string onLabel, offLabel;
    CPC_OnOff_getLabels(deviceName, onLabel, offLabel, "On / Opened", "Off / Closed");
    setMultiValue("Status_OnOpenedText", "text", onLabel,
                  "Status_OffClosedText", "text", offLabel);

	if(g_bUnSystemAlarmPlc) {
		if (g_bBoolAlert)
		{
			iRes = dpConnect("CPC_OnOff_FaceplateStatusAnimationCB",
								 deviceName + ".ProcessInput.StsReg01", 
								 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
								 deviceName + ".ProcessInput.StsReg02", 
								 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
								 deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
								 deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state",
								 deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state",
								 deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								 deviceName + ".eventMask");
		}
		else
		{
			iRes = dpConnect("CPC_OnOff_FaceplateStatusAnimationCB_NoAlarm",
								 deviceName + ".ProcessInput.StsReg01", 
								 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
								 deviceName + ".ProcessInput.StsReg02", 
								 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								 deviceName + ".eventMask");
		}
	}
	else 
		CPC_OnOff_FaceplateStatusDisconnection();
	g_bCallbackConnected = (iRes >= 0);
}
//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_OnOff_FaceplateDisconnect
/**
Purpose: disconnect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in OnOff faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_OnOff_FaceplateDisconnect(string deviceName)
{
	int iRes;
	string sPlcName, sSystemName;

	if(g_bUnSystemAlarmPlc) {
// get the systemName
		sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
		sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);

		if (g_bBoolAlert)
		{
			iRes = dpDisconnect("CPC_OnOff_FaceplateStatusAnimationCB",
								 deviceName + ".ProcessInput.StsReg01", 
								 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
								 deviceName + ".ProcessInput.StsReg02", 
								 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
								 deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
								 deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state",
								 deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state",
								 deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								 deviceName + ".eventMask");
		}
		else
		{
			iRes = dpDisconnect("CPC_OnOff_FaceplateStatusAnimationCB_NoAlarm",
								 deviceName + ".ProcessInput.StsReg01", 
								 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
								 deviceName + ".ProcessInput.StsReg02", 
								 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								 deviceName + ".eventMask");
		}
	}
	g_bCallbackConnected = !(iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_OnOff_FaceplateStatusAnimationCB_NoAlarm
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_OnOff_FaceplateStatusAnimationCB_NoAlarm(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
										   string sStsReg02, bit32 bit32StsReg02, string sDp4, bool bStsReg02Invalid,
						 			        string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled, string sDpEvt, int iMask)
{
	CPC_OnOff_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, 
									   sStsReg02, bit32StsReg02, sDp4, bStsReg02Invalid,
									   "", 0, "", 0, "", 0, sDpAl, iValAlarm, sDpEnabled, bPlcEnabled, sDpEvt, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_OnOff_FaceplateStatusAnimationCB
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in OnOff faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_OnOff_FaceplateStatusAnimationCB(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
								   string sStsReg02, bit32 bit32StsReg02, string sDp4, bool bStsReg02Invalid,
								   string sDp5, int iStartIStState, string sDp6, int iTStopIStState, string sDp7, int iFuStopIStState, string sDp8, int iAlSt,
			 			        string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled, string sDpEvt, int iMask)
{
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Auto", getBit(bit32StsReg01, UN_STSREG01_AUMOST), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Manual", getBit(bit32StsReg01, UN_STSREG01_MMOST), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Forced", getBit(bit32StsReg01, UN_STSREG01_FOMOST), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Local", getBit(bit32StsReg01, UN_STSREG01_LDST), g_bHLDrive, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_SoftLocal", getBit(bit32StsReg02, CPC_STSREG02_SOFTLDST), 1, "cpcColor_Faceplate_Status", 
			bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));

if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_IOError", getBit(bit32StsReg01, UN_STSREG01_IOERRORW), 1, "unFaceplate_WarningActive", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_IOSimulated", getBit(bit32StsReg01, UN_STSREG01_IOSIMUW), 1, "unFaceplate_WarningActive", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));

if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_AlBW", getBit(bit32StsReg02, UN_STSREG02_ALBW), 1, "unFaceplate_WarningActive", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));

if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_ManualAuto", getBit(bit32StsReg01, UN_STSREG01_AUMRW), 1, "unFaceplate_WarningActive", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_PositionWarning", getBit(bit32StsReg01, UN_STSREG01_POSALW), 1, "unFaceplate_WarningActive", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Status_OnOpened", getBit(bit32StsReg01, UN_STSREG01_ONST), g_bPHFOn,
		"cpcColor_Faceplate_Status", bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Status_OffClosed", getBit(bit32StsReg01, UN_STSREG01_OFFST), g_bPHFOff, 
		"cpcColor_Faceplate_Status", bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
    cpcGenericObject_CheckboxAnimate("Status_Failsafe", g_bPFSPosOn);
    cpcGenericObject_CheckboxAnimate("Status_Pulse", g_bPPulseSt);
    
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Status_AuInhFor", getBit(bit32StsReg01, UN_STSREG01_AUINHFMO), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Status_AuInhMan", getBit(bit32StsReg01, UN_STSREG01_AUINHMMO), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));

if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Request_Auto_On", getBit(bit32StsReg02, UN_STSREG02_AUONRST), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Request_Auto_Off", getBit(bit32StsReg02, UN_STSREG02_AUOFFRST), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Request_Manual_On", getBit(bit32StsReg02, UN_STSREG02_MONRST), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Request_Manual_Off", getBit(bit32StsReg02, UN_STSREG02_MOFFRST), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Request_Output_On", getBit(bit32StsReg02, UN_STSREG02_OUTOVST), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Request_Output_UnOn", !getBit(bit32StsReg02, UN_STSREG02_OUTOVST), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if (g_bPOutOff) {
    if(g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("Request_Output_Off", getBit(bit32StsReg02, CPC_STSREG02_OUTOFFOVST), 1, "cpcColor_Faceplate_Status", 
                bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
    if(g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("Request_Output_UnOff", !getBit(bit32StsReg02, CPC_STSREG02_OUTOFFOVST), 1, "cpcColor_Faceplate_Status", 
                bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
}

	cpcGenericObject_ColorBoxAnimateInterlock("Alarm_StartISt", iStartIStState, getBit(bit32StsReg01, UN_STSREG01_STARTIST), getBit(bit32StsReg01, UN_STSREG01_ALUNACK),
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	cpcGenericObject_ColorBoxAnimateInterlock("Alarm_TStopISt", iTStopIStState, getBit(bit32StsReg01, CPC_STSREG01_TSTOPIST), getBit(bit32StsReg01, UN_STSREG01_ALUNACK),
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	cpcGenericObject_ColorBoxAnimateInterlock("Alarm_FuStopISt", iFuStopIStState, getBit(bit32StsReg02, CPC_STSREG02_FUSTOPIST), getBit(bit32StsReg01, UN_STSREG01_ALUNACK),
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	cpcGenericObject_ColorBoxAnimateInterlock("Alarm_AlSt", iAlSt, getBit(bit32StsReg01, CPC_STSREG01_ALST), getBit(bit32StsReg01, UN_STSREG01_ALUNACK),
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));

if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Alarm_UnAck", getBit(bit32StsReg01, UN_STSREG01_ALUNACK), 1, "cpcColor_Alarm_Bad", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));

	if (g_bMRestart) {
		if(g_bSystemConnected)
			unGenericObject_ColorBoxAnimate("NeedRestart", getBit(bit32StsReg02, CPC_STSREG02_NEEDRESTART) == 0, 1, "unFaceplate_WarningActive",
				bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	} else {
        cpcGenericObject_ColorBoxDisable("NeedRestart");
	}

    if(g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("eventState", iMask == 0, 1, "unFaceplate_WarningActive", 
                bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_OnOff_FaceplateStatusDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_OnOff_FaceplateStatusDisconnection()
{
    unGenericObject_ColoredSquareDisconnect(makeDynString("Alarm_StartISt", "Alarm_TStopISt", "Alarm_FuStopISt", "Alarm_AlSt"));
    unGenericObject_ColorBoxDisconnect(makeDynString("Alarm_UnAck", "OperationMode_Auto",
                                                     "OperationMode_Forced", "OperationMode_Local", "OperationMode_SoftLocal", "OperationMode_Manual",
                                                     "Warning_IOError", "Warning_IOSimulated", "Warning_ManualAuto", 
                                                     "Warning_PositionWarning", "Request_Auto_On","Request_Auto_Off", "Request_Manual_On",
                                                     "Request_Manual_Off", "Request_Output_On", "Request_Output_UnOn", "Request_Output_Off", "Request_Output_UnOff",
                                                     "Status_OnOpened", "Status_OffClosed", "NeedRestart",
                                                     "Warning_AlBW", "Status_AuInhFor", "Status_AuInhMan",
                                                     "eventState"));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_OnOff_ButtonRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in OnOff faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_OnOff_ButtonRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sFunction, sDpType;
	int iAction, iRes, iManReg01;
	dyn_string exceptionInfo, dsUserAccess, dsFunctions;

	deviceName = unGenericDpFunctions_getDpName($sDpName);
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			CPC_OnOff_ButtonDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
// Send 0 to ManReg01 if necessary
			iRes = dpGet(deviceName + ".ProcessOutput.ManReg01", iManReg01);
			if ((iRes < 0) || (iManReg01 != 0))
			{
				dpSet(deviceName + ".ProcessOutput.ManReg01", 0);
			}
		
// call the CPC_OnOff_ButtonUserAccess to get the list of allowed buttons.
//			CPC_OnOff_ButtonUserAccess(deviceName, UN_CONFIG_CPC_ONOFF_DPT_NAME, g_dsUserAccess);

			sDpType=UN_CONFIG_CPC_ONOFF_DPT_NAME;
			unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
			sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
						 
			if (sFunction != "")
				{
				evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" + 
							 "dyn_string dsUserAccessReturn;" +
							 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
							 "    {" +
					 			sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
							 "    }" +
							 "return dsUserAccessReturn; }", makeDynString(), deviceName, sDpType);
				g_dsUserAccess=dsUserAccess;	
				}
			else
				fwException_raise(exceptionInfo,"ERROR", "CPC_OnOff_ButtonRegisterCB: " + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");
					
      string onLabel, offLabel;
      CPC_OnOff_getLabels(deviceName, onLabel, offLabel, "On", "Off");
      setMultiValue("buttonOnOpen", "text", onLabel,
                    "buttonOffClose", "text", offLabel);
			
			iRes = dpConnect("CPC_OnOff_ButtonAnimationCB",
							 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 deviceName + ".statusInformation.selectedManager",
							 deviceName + ".ProcessInput.StsReg01",
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							 deviceName + ".ProcessInput.StsReg02",
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid");
			g_bCallbackConnected = (iRes >= 0);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			iRes = dpDisconnect("CPC_OnOff_ButtonAnimationCB",
							 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 deviceName + ".statusInformation.selectedManager",
							 deviceName + ".ProcessInput.StsReg01",
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							 deviceName + ".ProcessInput.StsReg02",
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid");
			g_bCallbackConnected = !(iRes >= 0);
			CPC_OnOff_ButtonDisconnection();
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_OnOff_ButtonAnimationCB
/**
Purpose: animate buttons

Parameters: 
	- dbPermissions, dyn_bool, input, list of permissions, length = 4, order : 1=monitor, 2=operator, 3=expert, 4=admin
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_OnOff_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager, 
							string sDpStsReg01, bit32 stsReg01Value, string sDpStsReg01Bad, bool bStsReg01Bad,
							string sDpStsReg02, bit32 stsReg02Value, string sDpStsReg02Bad, bool bStsReg02Bad)
{
	CPC_OnOff_ButtonSetState(unGenericDpFunctions_getDpName(sDpStsReg01), UN_CONFIG_CPC_ONOFF_DPT_NAME, g_dsUserAccess, 
						   makeDynString(bLocked, sSelectedManager, stsReg01Value, bStsReg01Bad, stsReg02Value, bStsReg02Bad));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_OnOff_ButtonUserAccess
/**
Purpose: User access for OnOff buttons

Parameters: 
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_OnOff_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	bool operator, expert, admin;
	int i, length;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
	string sOperAction, sExpAction, sAdminAction;
	bool bActionDefined = false;

	unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
	if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME)) {
		bActionDefined=true;
	}
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	if (operator) {
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
		else
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE, UN_FACEPLATE_BUTTON_SELECT,
							UN_FACEPLATE_BUTTON_ONOPEN_REQUEST, UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST, UN_FACEPLATE_BUTTON_ACK_ALARM, CPC_FACEPLATE_BUTTON_ALLOW_RESTART));
	}
	if (expert) {
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
		else
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_MASKEVENT));
	}
	if(admin){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_OnOff_UserLoginGetButtonState
/**
Purpose: function called before the button set state when a user logs in. 

Parameters:
	deviceName: string, input, the device name
	dsReturnData: dyn_string, output, the data are returned here

Usage: Internal function

PVSS manager usage:  Ctrl, NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
		-> $sDpName, string, the device DP name
		-> g_sFaceplateButtonType, string, faceplate type
	. PVSS version: 2.12.1 
	. operating system: NT, W2000 and Linux, but tested only under W2000.
	. distributed system: yes but not tested.
*/

CPC_OnOff_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
	unGenericObject_UserCBGetButtonState(deviceName, sType, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_OnOff_ButtonSetState
/**
Purpose: set button state

Parameters: 
	- deviceName, string, input, device
	- sDpType, string, input, dptype
	- dsUserAccess, dyn_string, input, user access
	- dsData, dyn_string, parameter
		
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_OnOff_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected, buttonEnabled;
	dyn_string dsButtons;
	int i, length;
	string localManager;
	bool bLocked, bStsReg01Bad, bStsReg02Bad;
	string sSelectedManager;
	bit32 stsReg01Value, stsReg02Value; 

	bLocked = dsData[1];
	sSelectedManager = dsData[2];
	stsReg01Value = dsData[3];
	bStsReg01Bad = dsData[4];
	stsReg02Value = dsData[5];
	bStsReg02Bad = dsData[6];

	dsButtons = makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE, UN_FACEPLATE_BUTTON_FORCED_MODE,
							  UN_FACEPLATE_BUTTON_ONOPEN_REQUEST, UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST, UN_FACEPLATE_BUTTON_ACK_ALARM);
	length = dynlen(dsButtons);
// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");
// 2. Button state
	for(i=1;i<=length;i++)
		{
		buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);	// User access
		switch (dsButtons[i])
			{
			case UN_FACEPLATE_BUTTON_FORCED_MODE:
				buttonEnabled = buttonEnabled && bSelected && 
					!getBit(stsReg01Value, UN_STSREG01_FOMOST) &&
					!getBit(stsReg01Value, UN_STSREG01_AUINHFMO) &&
					!getBit(stsReg01Value, UN_STSREG01_LDST) &&
					!bStsReg01Bad && !bStsReg02Bad;
				break;
			case UN_FACEPLATE_BUTTON_MANUAL_MODE:
				buttonEnabled = buttonEnabled && bSelected &&
					!getBit(stsReg01Value, UN_STSREG01_MMOST) &&
					!getBit(stsReg01Value, UN_STSREG01_AUINHMMO) &&
					!getBit(stsReg01Value, UN_STSREG01_LDST) &&
					!getBit(stsReg02Value, CPC_STSREG02_SOFTLDST) &&
					!bStsReg01Bad && !bStsReg02Bad;
				break;
			case UN_FACEPLATE_BUTTON_ONOPEN_REQUEST:
			case UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST:
			case UN_FACEPLATE_BUTTON_AUTO_MODE:
				buttonEnabled = buttonEnabled && bSelected &&
					!getBit(stsReg01Value, UN_STSREG01_AUMOST) &&
					!getBit(stsReg01Value, UN_STSREG01_LDST) &&
					!getBit(stsReg02Value, CPC_STSREG02_SOFTLDST) &&
					!bStsReg01Bad && !bStsReg02Bad;
				break;
			case UN_FACEPLATE_BUTTON_ACK_ALARM:
				buttonEnabled = buttonEnabled &&
					!getBit(stsReg01Value, UN_STSREG01_LDST) &&
					!getBit(stsReg02Value, CPC_STSREG02_SOFTLDST) &&
					!bStsReg01Bad && !bStsReg02Bad;
			default:
				break;
			}
		setValue(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", buttonEnabled);
	}
	dyn_string exceptionInfo;
	cpcFaceplateButtonState_ButtonSetState(makeDynString(CPC_FACEPLATE_BUTTON_ALLOW_RESTART), dsUserAccess, deviceName, sDpType, dsData, exceptionInfo);
// 3. Button Select state
	unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_OnOff_ButtonDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_OnOff_ButtonDisconnection()
{
	unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE, 
												   UN_FACEPLATE_BUTTON_ONOPEN_REQUEST, UN_FACEPLATE_BUTTON_FORCED_MODE,
												   UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST, UN_FACEPLATE_BUTTON_ACK_ALARM,
												   UN_FACEPLATE_BUTTON_SELECT));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_OnOff_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in OnOff widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_OnOff_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName;
	int iAction;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;
	bool bRemote;
		
	deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
	if (deviceName == "")	// In case of disconnection
		{
		deviceName = g_sDpName;
		}
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;


	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;
		
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			CPC_OnOff_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = deviceName;
			CPC_OnOff_WidgetConnect(deviceName, sPlcName, sSystemName);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			CPC_OnOff_WidgetDisconnect(deviceName, sPlcName, sSystemName);
			CPC_OnOff_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}
//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_OnOff_WidgetConnect
/**
Purpose: Connect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in OnOff faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_OnOff_WidgetConnect(string deviceName, string sPlcName, string sSystemName)
{
    dyn_string exceptionInfo;
    int iRes, iAlertType1, iAlertType2, iAlertType3, iAlertType4;

    if(dpTypeName(deviceName) != UN_CONFIG_CPC_ONOFF_DPT_NAME) g_bUnSystemAlarmPlc = false;

    g_bMRestart = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_MRESTART, exceptionInfo);
    iRes = dpGet(deviceName + ".ProcessInput.StartISt:_alert_hdl.._type", iAlertType1,
                 deviceName + ".ProcessInput.TStopISt:_alert_hdl.._type", iAlertType2,
                 deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._type", iAlertType3,
                 deviceName + ".ProcessInput.AlSt:_alert_hdl.._type", iAlertType4);
    g_bBoolAlert = ((iRes >= 0) && (iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType2 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType3 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType4 == DPCONFIG_ALERT_BINARYSIGNAL));

    if(g_bUnSystemAlarmPlc) {
        if (g_bBoolAlert) {
            iRes = dpConnect("CPC_OnOff_WidgetAnimationCB", 
                            deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                            deviceName + ".statusInformation.selectedManager",
                            deviceName + ".ProcessInput.StsReg01",
                            deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                            deviceName + ".ProcessInput.StsReg02",
                            deviceName + ".ProcessInput.StsReg02:_online.._invalid",
                            deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
                            deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state",
                            deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state",
                            deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
                            deviceName + ".eventMask");
        } else {
            iRes = dpConnect("CPC_OnOff_WidgetAnimationCB_NoAlarm", 
                            deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                            deviceName + ".statusInformation.selectedManager",
                            deviceName + ".ProcessInput.StsReg01",
                            deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                            deviceName + ".ProcessInput.StsReg02",
                            deviceName + ".ProcessInput.StsReg02:_online.._invalid",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
                            deviceName + ".eventMask");
        }
        g_bCallbackConnected = (iRes >= 0);
    } else {
        CPC_OnOff_WidgetDisconnection(g_sWidgetType);
    }
}
//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_OnOff_WidgetDisconnect
/**
Purpose: Disconnect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in OnOff faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_OnOff_WidgetDisconnect(string deviceName, string sPlcName, string sSystemName)
{
    int iRes;
    if(g_bUnSystemAlarmPlc) {
        if (g_bBoolAlert) {
            iRes = dpDisconnect("CPC_OnOff_WidgetAnimationCB", 
                            deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                            deviceName + ".statusInformation.selectedManager",
                            deviceName + ".ProcessInput.StsReg01",
                            deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                            deviceName + ".ProcessInput.StsReg02",
                            deviceName + ".ProcessInput.StsReg02:_online.._invalid",
                            deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
                            deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state",
                            deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state",
                            deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
                            deviceName + ".eventMask");
        } else {
            iRes = dpDisconnect("CPC_OnOff_WidgetAnimationCB_NoAlarm", 
                            deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                            deviceName + ".statusInformation.selectedManager",
                            deviceName + ".ProcessInput.StsReg01",
                            deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                            deviceName + ".ProcessInput.StsReg02",
                            deviceName + ".ProcessInput.StsReg02:_online.._invalid",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
                            deviceName + ".eventMask");
        }
    }
    g_bCallbackConnected = !(iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------
CPC_OnOff_WidgetAnimationCB_NoAlarm(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
                                  string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
                                  string sDpStsReg02, bit32 bit32StsReg02, string sDp5, bool bStsReg02Invalid,
                                  string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled,
                                  string sEvtMask, int iMask)
{
    CPC_OnOff_WidgetAnimationCB(sDpLock, bLocked, sDpManager, sSelectedManager,
                            sDpStsReg01, bit32StsReg01, sDp4, bStsReg01Invalid,
                            sDpStsReg02, bit32StsReg02, sDp5, bStsReg02Invalid,
                            "", 0, "", 0, "", 0,
                            sDpAl, iAlValue, sDpEnabled, bPlcEnabled,
                            sEvtMask, iMask);
}

// CPC_OnOff_WidgetAnimationCB
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in OnOff faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_OnOff_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
                          string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
                          string sDpStsReg02, bit32 bit32StsReg02, string sDp5, bool bStsReg02Invalid,
                          string sDp9, int iStartIStState, string sDp10, int iTStopIStState, string sDp11, int iFuStopIStState, string sDp12, int iAlSt,
                          string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled,
                          string sEvtMask, int iMask)
{
	cpcGenericObject_WidgetAnimationDoubleStsReg(g_sWidgetType, UN_CONFIG_CPC_ONOFF_DPT_NAME, 
		bLocked, sSelectedManager, bit32StsReg01, bit32StsReg02,
		makeDynInt(UN_STSREG01_AUMRW, UN_STSREG01_POSALW, UN_STSREG01_IOSIMUW, UN_STSREG01_IOERRORW),
		makeDynInt(UN_STSREG02_ALBW),
		makeDynString(CPC_WIDGET_TEXT_WARNING_DEFAULT, CPC_WIDGET_TEXT_WARNING_DEFAULT, CPC_WIDGET_TEXT_WARNING_SIMU, CPC_WIDGET_TEXT_WARNING_ERROR),
		makeDynString(CPC_WIDGET_TEXT_WARNING_BLOCKED),
		makeDynInt(CPC_STSREG01_ALST, UN_STSREG01_STARTIST, CPC_STSREG01_TSTOPIST), 
		makeDynString(CPC_WIDGET_TEXT_ALARM_ALST, CPC_WIDGET_TEXT_ALARM_STARTIST, CPC_WIDGET_TEXT_ALARM_TSTOPIST),
		makeDynInt(CPC_STSREG02_FUSTOPIST), 
		makeDynString(CPC_WIDGET_TEXT_ALARM_FUSTOPIST),
        iStartIStState, iTStopIStState, iFuStopIStState, iAlSt,
		makeDynString("UN_STSREG01_LDST", "CPC_STSREG02_SOFTLDST", "CPC_STSREG01_AUIHFOMOST", "UN_STSREG01_AUINHMMO", "UN_STSREG01_MMOST", "UN_STSREG01_FOMOST"),
		"UN_STSREG01_ONST", "UN_STSREG01_OFFST", UN_STSREG01_ALUNACK,
		bStsReg01Invalid || bStsReg02Invalid,
		bStsReg01Invalid || bStsReg02Invalid,
		iAlValue, bPlcEnabled);

	if(g_bSystemConnected) {
		eventState.visible = (iMask == 0);
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_OnOff_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType, string, input, widget type. ex: "heater"

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_OnOff_WidgetDisconnection(string sWidgetType)
{
	cpcGenericObject_WidgetDisconnection(sWidgetType, UN_CONFIG_CPC_ONOFF_DPT_NAME);

	// set event state
	eventState.visible	= false;
}

CPC_OnOff_getLabels(string deviceName, string &onLabel, string &offLabel, string defaultOn, string defaultOff)
{
  onLabel = unGenericDpFunctions_getDescription(deviceName + ".ProcessInput.OnSt");
  if (onLabel == "") {
    onLabel = defaultOn;
  }
  offLabel = unGenericDpFunctions_getDescription(deviceName + ".ProcessInput.OffSt");
  if (offLabel == "") {
    offLabel = defaultOff;
  }
}
