/**@name LIBRARY: cpcOnOffConfig.ctl

@author: Vincent Forest (LHC-IAS)

Creation Date: 05 12 2002
    16/05/2012: Alexey
        [UCPC-787] introduce AlSt bit and letter constants

    16/04/2012: Alexey
        [UCPC-758] archive OnSt/OffSt

    08/03/2012: Alexey
        [UCPC-632] added on and off labels

    20/09/2011: Alexey
        [UCPC-552] added HWLocal parameter

	13/09/2011: Alexey
		[UCPC-532] added POutOff parameter

	08/08/2011: Alexey
		[UCPC-11] added OnSt and OffSt

	08/07/2011: Alexey
		[UCPC-389] moved "parameters" field

	01/07/2011: Alexey
		[UCPC-364] added getParamNames method which returns all possible params
		[UCPC-374] added PRstartFS

	2011/05/30 Alexey
		[UCPC-292] moved meta-info to the end of config line; removed UN_CPC_CONFIG length constants

	2011/05/16 amerezhi
		moved export functions from cpcExportGenericFunctions

	2011/05/09 amerezhi
		[UCPC-235] added metainfo (types relationships configuraiton)
		set archive for evStsReg

	2011/04/15 amerezhi
		[UCPC-208] Changed according new code structure

	18/04/2006: Herve
		- implementation of mask event

	14/06/2005: Enrique
		- S7 adaptation, new functions:
			. S7_PLC_CPC_OnOff_checkConfig
			. S7_PLC_CPC_OnOff_setConfig
		
	22/04/2005: Herve
		- allow archive in import file per device

	15/04/2005: Herve
		- Modbus set and check front-end functions: _UnPlc_CPC_OnOff_checkConfig, _UnPlc_CPC_OnOff_setConfig

Modification History: 

version 1.0

External Function : 
	. CPC_OnOffConfig_setConfig :  set configuration of given OnOff datapoint
	. CPC_OnOffConfig_checkConfig : check data before configuration
	
Internal Functions :
	
Purpose: 
This library contains the function used to set Unicos OnOff configuration.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

// Constants
const string UN_CONFIG_CPC_ONOFF_DPT_NAME = "CPC_OnOff";

const unsigned UN_CONFIG_CPC_ONOFF_LENGTH = 6;

const unsigned UN_CONFIG_CPC_ONOFF_NORMAL_POSITION    = 1;
const unsigned UN_CONFIG_CPC_ONOFF_ADDRESS_STSREG01   = 2;
const unsigned UN_CONFIG_CPC_ONOFF_ADDRESS_STSREG02   = 3;
const unsigned UN_CONFIG_CPC_ONOFF_ADDRESS_EVSTSREG01 = 4;
const unsigned UN_CONFIG_CPC_ONOFF_ADDRESS_EVSTSREG02 = 5;
const unsigned UN_CONFIG_CPC_ONOFF_ADDRESS_MANREG01   = 6;

const unsigned UN_CONFIG_CPC_ONOFF_ADDITIONAL_LENGTH          = 9;
const unsigned UN_CONFIG_CPC_ONOFF_ADDITIONAL_MASKEVENT       = 1;
const unsigned UN_CONFIG_CPC_ONOFF_ADDITIONAL_ON_LABEL        = 2;
const unsigned UN_CONFIG_CPC_ONOFF_ADDITIONAL_OFF_LABEL       = 3;
const unsigned UN_CONFIG_CPC_ONOFF_ADDITIONAL_PARAMETERS      = 4;
const unsigned UN_CONFIG_CPC_ONOFF_ADDITIONAL_MASTER_NAME     = 5;
const unsigned UN_CONFIG_CPC_ONOFF_ADDITIONAL_PARENTS         = 6;
const unsigned UN_CONFIG_CPC_ONOFF_ADDITIONAL_CHILDREN        = 7;
const unsigned UN_CONFIG_CPC_ONOFF_ADDITIONAL_TYPE            = 8;
const unsigned UN_CONFIG_CPC_ONOFF_ADDITIONAL_SECOND_ALIAS    = 9;


mapping CPC_OnOffConfig_getConfig() {
  mapping config;
  mapping props;

  mappingClear(props);
  props["address"]      = ".ProcessInput.StsReg01";
  props["dataType"]     = CPC_UINT16;
  config["StsReg01"]    = props;

  mappingClear(props);
  props["address"]      = ".ProcessInput.StsReg02";
  props["dataType"]     = CPC_UINT16;
  config["StsReg02"]    = props;

  mappingClear(props);
  props["hasArchive"]   = true;
  props["address"]      = ".ProcessInput.evStsReg01";
  props["dataType"]     = CPC_INT32;
  config["evStsReg01"]  = props;

  mappingClear(props);
  props["hasArchive"]   = true;
  props["address"]      = ".ProcessInput.evStsReg02";
  props["dataType"]     = CPC_INT32;
  config["evStsReg02"]  = props;

  mappingClear(props);
  props["address"]      = ".ProcessOutput.ManReg01";
  props["dataType"]     = CPC_UINT16;
  config["ManReg01"]    = props;

  mappingClear(props);
  props["address"]      = ".ProcessInput.StartISt";
  props["dpe"]          = "StsReg01";
  props["bitPosition"]  = UN_STSREG01_STARTIST;
  props["isAlarm"]      = true;
  config["StartISt"]    = props;

  mappingClear(props);
  props["address"]      = ".ProcessInput.TStopISt";
  props["dpe"]          = "StsReg01";
  props["bitPosition"]  = CPC_STSREG01_TSTOPIST;
  props["isAlarm"]      = true;
  config["TStopISt"]    = props;

  mappingClear(props);
  props["address"]      = ".ProcessInput.FuStopISt";
  props["dpe"]          = "StsReg02";
  props["bitPosition"]  = CPC_STSREG02_FUSTOPIST;
  props["isAlarm"]      = true;
  config["FuStopISt"]   = props;

  mappingClear(props);
  props["address"]      = ".ProcessInput.AlSt";
  props["dpe"]          = "StsReg01";
  props["bitPosition"]  = CPC_STSREG01_ALST;
  props["isAlarm"]      = true;
  config["AlSt"]        = props;

  mappingClear(props);
  props["address"]      = ".ProcessInput.OnSt";
  props["dpe"]          = "StsReg01";
  props["bitPosition"]  = UN_STSREG01_ONST;
  config["OnSt"]        = props;

  mappingClear(props);
  props["address"]      = ".ProcessInput.OffSt";
  props["dpe"]          = "StsReg01";
  props["bitPosition"]  = UN_STSREG01_OFFST;
  config["OffSt"]       = props;

  return config;
}

dyn_string CPC_OnOffConfig_getParamNames() {
	return makeDynString(CPC_PARAMS_FSPOSON, CPC_PARAMS_HFONST, CPC_PARAMS_HFOFFST, CPC_PARAMS_PULSEST, CPC_PARAMS_MRESTART, CPC_PARAMS_RSTARTFS, CPC_PARAMS_OUTOFF, CPC_PARAMS_HLDRIVE);
}

//------------------------------------------------------------------------------------------------------------------------
// CPC_OnOffConfig_checkConfig
/**
Check data before configuration

  @param[in]  configLine   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_OnOffConfig_checkConfig(dyn_string configLine, dyn_string &exceptionInfo) {
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
		// 1. Common
		unConfigGenericFunctions_checkParameters(configLine, UN_CONFIG_CPC_ONOFF_DPT_NAME, exceptionInfo);
		// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, UN_CONFIG_CPC_ONOFF_DPT_NAME, exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","CPC_OnOffConfig_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

_UnPlc_CPC_OnOff_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_ONOFF_DPT_NAME, configLine, exceptionInfo);
}

S7_PLC_CPC_OnOff_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_ONOFF_DPT_NAME, configLine, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// CPC_OnOffConfig_setConfig
/**
Creates dp (or checks if it is already presented)and sets configuration for OnOff object.

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_OnOffConfig_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo) {
	string sDpName;
	
	// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, UN_CONFIG_CPC_ONOFF_DPT_NAME, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, UN_CONFIG_CPC_ONOFF_DPT_NAME, exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
			// 2. Generate device
			// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
			// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, UN_CONFIG_CPC_ONOFF_DPT_NAME, exceptionInfo);
		}
	}
}

_UnPlc_CPC_OnOff_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

S7_PLC_CPC_OnOff_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

CPC_OnOffConfig_setCustomConfig(dyn_string dsConfigs, bool hasArchive, dyn_string &exceptionInfo) {
  dyn_string tmpExceptionInfo;
  int shift = UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_ONOFF_LENGTH;
  if (hasArchive) {
    shift += UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH;
  }

  unConfigGenericFunctions_setDescription(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessInput.OnSt", dsConfigs[shift + UN_CONFIG_CPC_ONOFF_ADDITIONAL_ON_LABEL], tmpExceptionInfo);
  if (dynlen(tmpExceptionInfo) > 0) {
    dynAppend(exceptionInfo, tmpExceptionInfo);
    tmpExceptionInfo = makeDynString();
  }
  unConfigGenericFunctions_setDescription(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessInput.OffSt", dsConfigs[shift + UN_CONFIG_CPC_ONOFF_ADDITIONAL_OFF_LABEL], tmpExceptionInfo);
  if (dynlen(tmpExceptionInfo) > 0) {
    dynAppend(exceptionInfo, tmpExceptionInfo);
  }

  fwArchive_set(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessInput.OnSt",
              dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL],
              DPATTR_ARCH_PROC_SIMPLESM,
              DPATTR_COMPARE_OLD_NEW,
              -1,
              0,
              exceptionInfo);
  if (dynlen(tmpExceptionInfo) > 0) {
    dynAppend(exceptionInfo, tmpExceptionInfo);
  }

  fwArchive_set(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessInput.OffSt",
              dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL],
              DPATTR_ARCH_PROC_SIMPLESM,
              DPATTR_COMPARE_OLD_NEW,
              -1,
              0,
              exceptionInfo);
  if (dynlen(tmpExceptionInfo) > 0) {
    dynAppend(exceptionInfo, tmpExceptionInfo);
  }
}

//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_CPC_OnOff_ExportConfig
/**
Purpose: Export OnOff Devices for _UnPlc_CPC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
_UnPlc_CPC_OnOff_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
  string sObject = UN_CONFIG_CPC_ONOFF_DPT_NAME;
  int amount = dynlen(dsDpList);
  for(int i = 1; i <= amount; i++) {
    dyn_string dsDpParameters;
    string currentDP = dsDpList[i];
    string sLimitOn, sLimitOff;

    if(!dpExists(currentDP)) continue;

    unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
    cpcExportGenericFunctions_getDigitalNormalPosition(currentDP, sObject, makeDynString(".ProcessInput.FuStopISt", ".ProcessInput.TStopISt", ".ProcessInput.StartISt"), dsDpParameters);
    cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg01", 	true, dsDpParameters);
    cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg02", 	true, dsDpParameters);
    cpcExportGenericFunctions_processAddress(currentDP, sObject, "evStsReg01", 	true, dsDpParameters);
    cpcExportGenericFunctions_processAddress(currentDP, sObject, "evStsReg02", 	true, dsDpParameters);
    cpcExportGenericFunctions_processAddress(currentDP, sObject, "ManReg01", 	false, dsDpParameters);
    cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString("ProcessInput.OnSt", "ProcessInput.OffSt"), dsDpParameters);
    cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_2, makeDynString(), dsDpParameters);
    cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString("ProcessInput.evStsReg01", "ProcessInput.evStsReg02"), dsDpParameters);
    cpcExportGenericFunctions_getMaskEvent(currentDP, dsDpParameters);
    dynAppend(dsDpParameters, unGenericDpFunctions_getDescription(currentDP + ".ProcessInput.OnSt"));
    dynAppend(dsDpParameters, unGenericDpFunctions_getDescription(currentDP + ".ProcessInput.OffSt"));
    cpcExportGenericFunctions_getParameters(currentDP, dsDpParameters, exceptionInfo);
    cpcExportGenericFunctions_getMetainfo(currentDP, dsDpParameters, exceptionInfo);

    unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
  }
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_CPC_OnOff_ExportConfig
/**
Purpose: Export OnOff Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
S7_PLC_CPC_OnOff_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
  _UnPlc_CPC_OnOff_ExportConfig(dsDpList, exceptionInfo);
}
