/**@name LIBRARY: CPC_ProcessControlObject.ctl

@author: Vincent Forest (AB-CO)

Creation Date: 21 05 2003

Modification History: 
    13/09/2012: Alexey
        [UCPC-931] add alarms item to the right click menu

    16/05/2012: Alexey
        [UCPC-787] introduce AlSt bit and letter constants

    27/03/2012: Alexey
        [UCPC-733] change intelock names to common

    29/02/2012: Alexey
        [UCPC-684] "allow restart" available for "operator"

    14/02/2012: Alexey
        [UCPC-665] don't animate letters in widget when stsreg is invalid

    30/11/2011: Alexey
        [UCPC-414] upd animation when not ack but been fired

    23/11/2011: Alexey
        [UCPC-603] refactor menu creation and handling

    20/10/2011: Alexey
        [UCPC-572] blink when unack without specific alert

    26/09/2011: Alexey
        fixed widget animation in case without alerts

    20/09/2011: Alexey
        [UCPC-548][UCPC-547] upd faceplate

    26/08/2011: Alexey
        fixed CPC_ProcessControlObject_FaceplateStatusDisconnection

	04/08/2011: Alexey
        [UCPC-455] animate inhibited statuses in the faceplate and the widget

	25/07/2011: Alexey
		[UCPC-414] animate not ack alarm in the widget

	15/07/2011: Alexey
		[UCPC-409] Widget Animation of Start Interlock in PCO
		[UCPC-410] button animation problem with Set Full Stop and Block Alarm
		[UCPC-411] removed Request_LocalOn from the faceplate

	05/07/2011: Alexey
		[UCPC-323] fixed ManRestart animation

  01/07/2011: Alexey
		[UCPC-370] removed UN_STSREG01_LDST

  28/06/2011: Alexey
		[UCPC-311] animate Soft/Hard local mode in the faceplate and widget

  27/06/2011: Alexey
		[UCPC-323] added AllowRestart button

  23/06/2011: Alexey
		[UCPC-323] added NeedRestart box to the faceplate and animate it in the widget
		[UCPC-328] change animation logic of interlocks in the PCO faceplate

  18/05/2011: Alexey
		use cpcGenericObject_HandleEventMaskMenu and CPC functions to manage menu

  07/04/2011: Alexey
	[UCPC-197] added Request_LocalOn to the faceplate

  27/03/09: Frederic
 	- CPC_ProcessControlObject_WidgetConnect(): move g_bCallbackConnected upper
	- CPC_ProcessControlObject_ButtonRegisterCB(): remove fwException_raise(...) before g_dsUserAccess
        
  06/11/08: Quique
    - New bit for block alarm information. (faceplate/widget animation)
    
  28/12/2007: Herve
    - device acccess control
    
	02/05/2006: Herve
		- implementation of maskEvent in widget, faceplate and rigthClick
		
	11/04/2006: Herve
		- remove dynAppend(menuList, "SEPARATOR"); in CPC_ProcessControlObject_MenuConfiguration function

	15/12/2005: Celine
		- CPC_ProcessControlObject_FaceplateStatusRegisterCB, CPC_ProcessControlObject_FaceplateStatusAnimationCB, 
		CPC_ProcessControlObject_WidgetRegisterCB, CPC_ProcessControlObject_WidgetAnimationCB : add g_bSystemConnected checking
		
	09/08/2005: Frederic
		- CPC_ProcessControlObject_ButtonRegisterCB(), add evalScript to run dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS] function

	15/04/2005: Herve
		- unGenericObject_GetPLCNameFromDpName: return the PLCtype+"_"+plcName, in case of _UnPlc it returns _unPlc_+plcName
		
	05/11/2004: Herve
		- in the WidgetRegisterCB, faceplateRegiterCB: use the systemName of the $sIdentifier to dpConnect to the PLC

	26/10/2004: Herve
		- in the CPC_ProcessControlObject_ButtonRegisterCB function: call the CPC_ProcessControlObject_ButtonUserAccess to get the list of allowed buttons. This list is 
		reseted if the user logs in/out while the remote system is off.

	03/05/2004: Herve
		- during WidgetRegister and FaceplateRegister call unGenericObject_GetPLCNameFromDpName to get the PLC DPName
		- during WidgetRegister check if the dpType is correct

	14/04/2004: Herve
		- add the function CPC_ProcessControlObject_HandleMenu
		- add the function CPC_ProcessControlObject_UserLoginGetButtonState

	13/04/2004: Herve
		- in function CPC_ProcessControlObject_ButtonRegisterCB add the set Manreg01 to 0 in case of dpConnect.
		- modification of the signature CPC_ProcessControlObject_MenuConfiguration and add the creation of the menu.

	08/04/2004: Herve 
		modification of the signature of the function CPC_ProcessControlObject_ButtonSetState

	16/01/2004: Herve
		remove the dpConnect, dpDisconnect and callback function in the case of no PLC systemAlarm. Ease the development 
		and test.
		
	01/05/2004: Herve
		- add in all dpConnect and dpDisconnect the _unSystemAlarm_PLC_DS dp if the PLC is configured for systemIntegrity. 
		The g_bUnSystemAlarmPlc is set to true if the PLC is configured for systemIntegrity
		change the dpConnect and dpDisconnect of _aut_inv to _invalid

version 1.0

External Function : 
	. CPC_ProcessControlObject_AcknowledgeAlarm : acknowledge ProcessControlObject alarms
	. CPC_ProcessControlObject_FaceplateStatusDisconnection : animate faceplate disconnection
	. CPC_ProcessControlObject_ButtonUserAccess : User access for ProcessControlObject buttons
	. CPC_ProcessControlObject_ButtonSetState : set button state
	. CPC_ProcessControlObject_WidgetDisconnection : animate widget disconnection
	. CPC_ProcessControlObject_MenuConfiguration : configuration of popup menu

	. CPC_ProcessControlObject_getName, to get PCO name
	. CPC_ProcessControlObject_getOptionModesLabel, to get an option mode label
	. CPC_ProcessControlObject_getOptionModesAllowance, to get an option mode allowance
	. CPC_ProcessControlObject_setName, to set PCO name
	. CPC_ProcessControlObject_setOptionModes, to set an option mode label & allowance
	. CPC_ProcessControlObject_getNextAllowedModes, to get the modes you can access from current mode (using allowance tables)

Internal Functions :
	. CPC_ProcessControlObject_FaceplateStatusRegisterCB : register callback function
	. CPC_ProcessControlObject_FaceplateStatusAnimationCB_NoAlarm : animate faceplate
	. CPC_ProcessControlObject_FaceplateStatusAnimationCB : animate faceplate
	. CPC_ProcessControlObject_ButtonRegisterCB : register callback function
	. CPC_ProcessControlObject_ButtonAnimationCB : animate buttons
	. CPC_ProcessControlObject_ButtonDisconnection : animate buttons disconnection
	. CPC_ProcessControlObject_WidgetRegisterCB : register callback function
	. CPC_ProcessControlObject_WidgetAnimationCB : animate widget

Purpose: This library contains ProcessControlObject object functions.

Usage: Public

PVSS manager usage: NG, NV

Option modes are saved in the DPE "<PCO name> + UN_PCO_OPTION_MODE_PATH" in a UN_PCO_OPTION_MODE_DELIMITER delimited string.
First UN_PCO_OPTION_MODE_NUMBER parameters are the labels. 
Next UN_PCO_OPTION_MODE_NUMBER parameters are the allowance parameter for each mode.
Example :
Physics;Standby;Capacity;Warm Up;;;;;11000000;01100000;01100000;00010000;0;0;0;0
While running, you can only switch from physics to standby, from standby to capacity, from capacity to standby.
Warm up can't be changed while running.

Constraints:
	. ProcessControlObject object (DPType etc.)
	. Global variables and $parameters defined in ProcessControlObject faceplate and widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

//---------------------------------------------------------------------------------------------------------------------------------------
// CPC_ProcessControlObject_ObjectListGetValueTime
/**
Function called from the objectList to return the time and value

	@param[in]	sDeviceName		Device name
	@param[in]	sDeviceType		Unicos object
	@param[out]	dsReturnData	Return value
*/

CPC_ProcessControlObject_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
    int value;
    string sTime, result;

    dpGet(sDeviceName+".ProcessInput.OpMoSt", value,
          sDeviceName+".ProcessInput.OpMoSt:_online.._stime", sTime);

    CPC_ProcessControlObject_getOptionModesLabel(sDeviceName, value, result);
    dsReturnData[1] = sTime;
    dsReturnData[2] = result;
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_ProcessControlObject_AcknowledgeAlarm
/**
Purpose: acknowledge ProcessControlObject alarms

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsNeedAck, dyn_string, output, DPEs names that need acknowledgement. 
									 The PLC acknowledgement is specified using UN_ACKNOWLEDGE_PLC string
	
Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in ProcessControlObject faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_ProcessControlObject_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
	dsNeedAck = makeDynString("ProcessInput.FuStopISt", "ProcessInput.StartISt", "ProcessInput.TStopISt", "ProcessInput.AlSt", UN_ACKNOWLEDGE_PLC);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_ProcessControlObject_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsAccessOk, dyn_string, input, authorized action
	- menuList, dyn_string, output, menu to pop-up
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_ProcessControlObject_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
	dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_ACK_TEXT, UN_POPUPMENU_FACEPLATE_TEXT, CPC_POPUPMENU_ALARMS_TEXT);

	cpcGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	cpcGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_ProcessControlObject_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_ProcessControlObject_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
	dyn_string exceptionInfo;

	cpcGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);

	if (dynlen(exceptionInfo) > 0) {
		unSendMessage_toExpertException("Right Click",exceptionInfo);
	}

}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_ProcessControlObject_FaceplateStatusRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in ProcessControlObject faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_ProcessControlObject_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sSystemName;
	int iAction;
	dyn_string exceptionInfo;
	bool bRemote;
		
	deviceName = unGenericDpFunctions_getDpName($sDpName);
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			CPC_ProcessControlObject_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
			CPC_ProcessControlObject_FaceplateConnect(deviceName);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			CPC_ProcessControlObject_FaceplateDisconnect(deviceName);
			CPC_ProcessControlObject_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_ProcessControlObject_FaceplateConnect
/**
Purpose: connect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in ProcessControlObject faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_ProcessControlObject_FaceplateConnect(string deviceName)
{
	int iRes, iAlertType1, iAlertType2, iAlertType3, iAlertType4;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;

// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;
		
	iRes = dpGet(deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._type", iAlertType1,
				 deviceName + ".ProcessInput.StartISt:_alert_hdl.._type", iAlertType2,
				 deviceName + ".ProcessInput.TStopISt:_alert_hdl.._type", iAlertType3,
				 deviceName + ".ProcessInput.AlSt:_alert_hdl.._type", iAlertType4);
	g_bBoolAlert = ((iRes >= 0) && (iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL) && 
					(iAlertType2 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType3 == DPCONFIG_ALERT_BINARYSIGNAL));

	dyn_string exceptionInfo;
	g_bMRestart = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_MRESTART, exceptionInfo);

	if(g_bUnSystemAlarmPlc) {
		if (g_bBoolAlert)
			{
			iRes = dpConnect("CPC_ProcessControlObject_FaceplateStatusAnimationCB",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							 deviceName + ".ProcessInput.StsReg02", 
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							 deviceName + ".ProcessInput.OpMoSt",
							 deviceName + ".ProcessInput.OpMoSt:_online.._invalid",
							 deviceName + ".ProcessInput.AuOpMo",
							 deviceName + ".ProcessInput.AuOpMo:_online.._invalid",
							 deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");
			}
		else
			{
			iRes = dpConnect("CPC_ProcessControlObject_FaceplateStatusAnimationCB_NoAlarm",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							 deviceName + ".ProcessInput.StsReg02", 
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							 deviceName + ".ProcessInput.OpMoSt",
							 deviceName + ".ProcessInput.OpMoSt:_online.._invalid",
							 deviceName + ".ProcessInput.AuOpMo",
							 deviceName + ".ProcessInput.AuOpMo:_online.._invalid",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");
			}
	}
	else 
		CPC_ProcessControlObject_FaceplateStatusDisconnection();
	g_bCallbackConnected = (iRes >= 0);
}
//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_ProcessControlObject_FaceplateDisconnect
/**
Purpose: disconnect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in ProcessControlObject faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_ProcessControlObject_FaceplateDisconnect(string deviceName)
{
	int iRes;
	string sPlcName, sSystemName;

	if(g_bUnSystemAlarmPlc) {
// get the systemName
		sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
		sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);

		if (g_bBoolAlert)
			{
			iRes = dpDisconnect("CPC_ProcessControlObject_FaceplateStatusAnimationCB",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							 deviceName + ".ProcessInput.StsReg02", 
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							 deviceName + ".ProcessInput.OpMoSt",
							 deviceName + ".ProcessInput.OpMoSt:_online.._invalid",
							 deviceName + ".ProcessInput.AuOpMo",
							 deviceName + ".ProcessInput.AuOpMo:_online.._invalid",
							 deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");
			}
		else
			{
			iRes = dpDisconnect("CPC_ProcessControlObject_FaceplateStatusAnimationCB_NoAlarm",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							 deviceName + ".ProcessInput.StsReg02", 
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							 deviceName + ".ProcessInput.OpMoSt",
							 deviceName + ".ProcessInput.OpMoSt:_online.._invalid",
							 deviceName + ".ProcessInput.AuOpMo",
							 deviceName + ".ProcessInput.AuOpMo:_online.._invalid",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");
			}
	}
	g_bCallbackConnected = !(iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_ProcessControlObject_FaceplateStatusAnimationCB_NoAlarm
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_ProcessControlObject_FaceplateStatusAnimationCB_NoAlarm(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
									string sStsReg02, bit32 bit32StsReg02, string sDp4, bool bStsReg02Invalid,
									string sDpOpMoSt, int iOpMoSt, string sDp6, bool bOpMoStInvalid, 
				 			        string sDpAuOpMo, int iAuOpMo, string sDp8, bool bAuOpMoInvalid,
				 			        string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled, string sDpEvt, int iMask)
{
	CPC_ProcessControlObject_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, 
									    sStsReg02, bit32StsReg02, sDp4, bStsReg02Invalid,
										sDpOpMoSt, iOpMoSt, sDp6, bOpMoStInvalid,
					 			        sDpAuOpMo, iAuOpMo, sDp8, bAuOpMoInvalid, "", 0, "", 0, "", 0, sDpAl, iValAlarm, sDpEnabled, bPlcEnabled,
					 			        sDpEvt, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_ProcessControlObject_FaceplateStatusAnimationCB
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in ProcessControlObject faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_ProcessControlObject_FaceplateStatusAnimationCB(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
									string sStsReg02, bit32 bit32StsReg02, string sDp4, bool bStsReg02Invalid,
									string sDpOpMoSt, int iOpMoSt, string sDp6, bool bOpMoStInvalid, 
				 			        string sDpAuOpMo, int iAuOpMo, string sDp8, bool bAuOpMoInvalid,
									string sDp19, int iStartIStState, string sDp20, int iTStopIStState, string sDp21, int iFuStopIStState, string sDp22, int iAlSt,
				 			        string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled, string sDpEvt, int iMask)
{
	string result, deviceName;

// 1. Animate colorboxes
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Auto", getBit(bit32StsReg01, UN_STSREG01_AUMOST), 1, "cpcColor_Faceplate_Status", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Manual", getBit(bit32StsReg01, UN_STSREG01_MMOST), 1, "cpcColor_Faceplate_Status", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Status_AuInhMan", getBit(bit32StsReg01, UN_STSREG01_AUINHMMO), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Forced", getBit(bit32StsReg01, UN_STSREG01_FOMOST), 1, "cpcColor_Faceplate_Status", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Status_AuInhFor", getBit(bit32StsReg01, UN_STSREG01_AUINHFMO), 1, "cpcColor_Faceplate_Status", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_SoftLocal", getBit(bit32StsReg02, CPC_STSREG02_SOFTLDST), 1, "cpcColor_Faceplate_Status", 
			bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_IOError", getBit(bit32StsReg01, UN_STSREG01_IOERRORW), 1, "unFaceplate_WarningActive", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_AlBW", getBit(bit32StsReg02, UN_STSREG02_ALBW), 1, "unFaceplate_WarningActive", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_IOSimulated", getBit(bit32StsReg01, UN_STSREG01_IOSIMUW), 1, "unFaceplate_WarningActive", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_ManualAuto", getBit(bit32StsReg01, UN_STSREG01_AUMRW), 1, "unFaceplate_WarningActive", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Status_OnOpened", getBit(bit32StsReg01, UN_STSREG01_ONST), 1, "cpcColor_Faceplate_Status", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Status_OffClosed", getBit(bit32StsReg01, UN_STSREG01_OFFST), 1, "cpcColor_Faceplate_Status", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Status_AlarmAction", getBit(bit32StsReg02, UN_STSREG02_ALACTB), 1, "cpcColor_Faceplate_Status", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Status_Temporary", getBit(bit32StsReg02, UN_STSREG02_TFUSTOP), 1, "cpcColor_Faceplate_Status", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Request_AutoOn", getBit(bit32StsReg02, UN_STSREG02_AUONRST), 1, "cpcColor_Faceplate_Status", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Request_ManualOn", getBit(bit32StsReg02, UN_STSREG02_MONRST), 1, "cpcColor_Faceplate_Status", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Request_AutoControlledStop", getBit(bit32StsReg02, UN_STSREG02_AUCSTOP), 1, "cpcColor_Faceplate_Status", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Order_Run", getBit(bit32StsReg02, UN_STSREG02_RUNOST), 1, "cpcColor_Faceplate_Status", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Order_ControlStop", getBit(bit32StsReg02, UN_STSREG02_CSTOPOST), 1, "cpcColor_Faceplate_Status", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
// 2. Animate alarm colorboxes
	cpcGenericObject_ColorBoxAnimateInterlock("Alarm_StartISt", iStartIStState, getBit(bit32StsReg01, UN_STSREG01_STARTIST), getBit(bit32StsReg01, UN_STSREG01_ALUNACK),
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	cpcGenericObject_ColorBoxAnimateInterlock("Alarm_TStopISt", iTStopIStState, getBit(bit32StsReg01, CPC_STSREG01_TSTOPIST), getBit(bit32StsReg01, UN_STSREG01_ALUNACK),
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	cpcGenericObject_ColorBoxAnimateInterlock("Alarm_FuStopISt", iFuStopIStState, getBit(bit32StsReg01, CPC_STSREG01_FUSTOPIST), getBit(bit32StsReg01, UN_STSREG01_ALUNACK),
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	cpcGenericObject_ColorBoxAnimateInterlock("Alarm_AlSt", iAlSt, getBit(bit32StsReg01, CPC_STSREG01_ALST), getBit(bit32StsReg01, UN_STSREG01_ALUNACK),
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Alarm_UnAck", getBit(bit32StsReg01, UN_STSREG01_ALUNACK), 1, "cpcColor_Alarm_Bad", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if (g_bMRestart) {
		if(g_bSystemConnected)
			unGenericObject_ColorBoxAnimate("NeedRestart", g_bMRestart && getBit(bit32StsReg02, CPC_STSREG02_NEEDRESTART) == 0, 1, "unFaceplate_WarningActive",
				bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	} else {
        cpcGenericObject_ColorBoxDisable("NeedRestart");
	}

// 3. Display option modes
	deviceName = unGenericDpFunctions_getDpName(sStsReg01);
	CPC_ProcessControlObject_getOptionModesLabel(deviceName, iOpMoSt, result);
if(g_bSystemConnected)
	setValue("OrderActiveOption", "text", result);
if(g_bSystemConnected)
	setValue("OrderActiveOption", "foreCol", "cpcColor_Faceplate_Request");
	if (bOpMoStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value))
		{
		if(g_bSystemConnected)
		setValue("OrderActiveOption", "foreCol", "unDataNotValid");
		}
	CPC_ProcessControlObject_getOptionModesLabel(deviceName, iAuOpMo, result);
	if(g_bSystemConnected)
	setValue("RequestAutoOption", "text", result);
	if(g_bSystemConnected)
	setValue("RequestAutoOption", "foreCol", "cpcColor_Faceplate_Request");
	if (bAuOpMoInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value))
		{
		if(g_bSystemConnected)
		setValue("RequestAutoOption", "foreCol", "unDataNotValid");
		}

    if(g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("eventState", iMask == 0, 1, "unFaceplate_WarningActive",
                bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_ProcessControlObject_FaceplateStatusDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_ProcessControlObject_FaceplateStatusDisconnection()
{
    unGenericObject_ColoredSquareDisconnect(makeDynString("Alarm_StartISt", "Alarm_TStopISt", "Alarm_FuStopISt", "Alarm_AlSt"));
	unGenericObject_ColorBoxDisconnect(makeDynString("Alarm_UnAck", "Warning_AlBW",
													 "OperationMode_Auto", "OperationMode_Forced", "OperationMode_Manual", "OperationMode_SoftLocal", 
													 "Warning_IOError", "Warning_IOSimulated", "Warning_ManualAuto", 
													 "Status_OnOpened", "Status_OffClosed", "Status_AlarmAction", "Status_Temporary", 
                                                     "Status_AuInhFor", "Status_AuInhMan", "NeedRestart",
													 "Request_AutoOn", "Request_ManualOn", "Request_AutoControlledStop",
													 "Order_Run", "Order_ControlStop", "eventState"));
	setValue("OrderActiveOption", "foreCol", "unDataNoAccess");
	setValue("RequestAutoOption", "foreCol", "unDataNoAccess");
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_ProcessControlObject_ButtonRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in ProcessControlObject faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_ProcessControlObject_ButtonRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sDpType, sFunction;
	int iAction, iRes, iManReg01;
	dyn_string exceptionInfo, dsUserAccess, dsFunctions;
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			CPC_ProcessControlObject_ButtonDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
// Send 0 to ManReg01 if necessary
			iRes = dpGet(deviceName + ".ProcessOutput.ManReg01", iManReg01);
			if ((iRes < 0) || (iManReg01 != 0))
			{
				dpSet(deviceName + ".ProcessOutput.ManReg01", 0);
			}
		
// call the CPC_ProcessControlObject_ButtonUserAccess to get the list of allowed buttons.
//	CPC_ProcessControlObject_ButtonUserAccess(deviceName, UN_CONFIG_CPC_PROCESSCONTROLOBJECT_DPT_NAME, g_dsUserAccess);
			sDpType=UN_CONFIG_CPC_PROCESSCONTROLOBJECT_DPT_NAME;
			unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
			sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
						 
			if (sFunction != "")
				{
				evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" + 
							 "dyn_string dsUserAccessReturn;" +
							 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
							 "    {" +
					 			sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
							 "    }" +
							 "return dsUserAccessReturn; }", makeDynString(), deviceName, sDpType);
						
				g_dsUserAccess=dsUserAccess;	
				}
			else
				fwException_raise(exceptionInfo,"ERROR", "CPC_ProcessControlObject_ButtonRegisterCB:" + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");
					

			iRes = dpConnect("CPC_ProcessControlObject_ButtonAnimationCB",
							 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 deviceName + ".statusInformation.selectedManager",
							 deviceName + ".ProcessInput.StsReg01",
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							 deviceName + ".ProcessInput.StsReg02",
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid");
			g_bCallbackConnected = (iRes >= 0);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			iRes = dpDisconnect("CPC_ProcessControlObject_ButtonAnimationCB",
							 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 deviceName + ".statusInformation.selectedManager",
							 deviceName + ".ProcessInput.StsReg01",
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							 deviceName + ".ProcessInput.StsReg02",
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid");
			g_bCallbackConnected = !(iRes >= 0);
			CPC_ProcessControlObject_ButtonDisconnection();
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_ProcessControlObject_ButtonAnimationCB
/**
Purpose: animate buttons

Parameters: 
	- dbPermissions, dyn_bool, input, list of permissions, length = 4, order : 1=monitor, 2=operator, 3=expert, 4=admin
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_ProcessControlObject_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager, 
						   string sDpStsReg01, bit32 stsReg01Value, string sDpStsReg01Bad, bool bStsReg01Bad,
						   string sDpStsReg02, bit32 stsReg02Value, string sDpStsReg02Bad, bool bStsReg02Bad)
{
	CPC_ProcessControlObject_ButtonSetState(unGenericDpFunctions_getDpName(sDpStsReg01), UN_CONFIG_CPC_PROCESSCONTROLOBJECT_DPT_NAME, g_dsUserAccess, 
						 		  makeDynString(bLocked, sSelectedManager, stsReg01Value, bStsReg01Bad, stsReg02Value, bStsReg02Bad));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_ProcessControlObject_ButtonUserAccess
/**
Purpose: User access for ProcessControlObject buttons

Parameters: 
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_ProcessControlObject_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	bool operator, expert, admin;
	int i, length;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
        string sOperAction, sExpAction, sAdminAction;
        bool bActionDefined = false;

        unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
        if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
           (sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
          bActionDefined=true;
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString(UN_FACEPLATE_BUTTON_NEXT, UN_FACEPLATE_BUTTON_BACK);
	if (operator)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
	    dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE,
							UN_FACEPLATE_BUTTON_ONOPEN_REQUEST, UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST, UN_FACEPLATE_BUTTON_ACK_ALARM,
							UN_FACEPLATE_BUTTON_SELECT, UN_FACEPLATE_BUTTON_CONTROLLED_STOP_REQUEST,
							UN_FACEPLATE_BUTTON_OPTION_MODES, UN_FACEPLATE_BUTTON_MANUAL_AUTO_TO_DEPENDENT_OBJECT, CPC_FACEPLATE_BUTTON_ALLOW_RESTART));
	}
	if (expert)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
	    dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_BUTTON_BLOCK_ALARM_ACTION,
							UN_FACEPLATE_BUTTON_DEBLOCK_ALARM_ACTION, UN_FACEPLATE_BUTTON_SET_TEMPORARY_AS_FULL_STOP,
							UN_FACEPLATE_BUTTON_RESET_TEMPORARY_AS_FULL_STOP, UN_FACEPLATE_MASKEVENT));
	}
        if(admin){
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
        }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_ProcessControlObject_UserLoginGetButtonState
/**
Purpose: function called before the button set state when a user logs in. 

Parameters:
	deviceName: string, input, the device name
	dsReturnData: dyn_string, output, the data are returned here

Usage: Internal function

PVSS manager usage:  Ctrl, NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
		-> $sDpName, string, the device DP name
		-> g_sFaceplateButtonType, string, faceplate type
	. PVSS version: 2.12.1 
	. operating system: NT, W2000 and Linux, but tested only under W2000.
	. distributed system: yes but not tested.
*/

CPC_ProcessControlObject_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
	unGenericObject_UserCBGetButtonState(deviceName, sType, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_ProcessControlObject_ButtonSetState
/**
Purpose: set button state

Parameters: 
	- deviceName, string, input, device
	- sDpType, string, input, dptype
	- dsUserAccess, dyn_string, input, user access
	- dsData, dyn_string, parameter
		
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_ProcessControlObject_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected, buttonEnabled, bit1, bit2;
	dyn_string dsButtons;
	int i, length;
	string localManager;
	bool bLocked, bStsReg01Bad, bStsReg02Bad;
	string sSelectedManager;
	bit32 stsReg01Value, stsReg02Value; 

	bLocked = dsData[1];
	sSelectedManager = dsData[2];
	stsReg01Value = dsData[3];
	bStsReg01Bad = dsData[4];
	stsReg02Value = dsData[5];
	bStsReg02Bad = dsData[6];

	dsButtons = makeDynString(UN_FACEPLATE_BUTTON_NEXT, UN_FACEPLATE_BUTTON_BACK, UN_FACEPLATE_BUTTON_AUTO_MODE,
							  UN_FACEPLATE_BUTTON_ONOPEN_REQUEST, UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST, UN_FACEPLATE_BUTTON_ACK_ALARM,
							  UN_FACEPLATE_BUTTON_CONTROLLED_STOP_REQUEST, UN_FACEPLATE_BUTTON_RESET_TEMPORARY_AS_FULL_STOP,
							  UN_FACEPLATE_BUTTON_OPTION_MODES, UN_FACEPLATE_BUTTON_MANUAL_AUTO_TO_DEPENDENT_OBJECT, 
							  UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_BUTTON_BLOCK_ALARM_ACTION, UN_FACEPLATE_BUTTON_MANUAL_MODE,
							  UN_FACEPLATE_BUTTON_DEBLOCK_ALARM_ACTION, UN_FACEPLATE_BUTTON_SET_TEMPORARY_AS_FULL_STOP);
	length = dynlen(dsButtons);
// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");
// 2. Button state
	for(i=1;i<=length;i++) {
		buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0); // User access
		switch (dsButtons[i])
			{
			case UN_FACEPLATE_BUTTON_SET_TEMPORARY_AS_FULL_STOP:
				buttonEnabled = buttonEnabled && bSelected && 
					!getBit(stsReg01Value, UN_STSREG01_AUMOST) &&
					!getBit(stsReg02Value, UN_STSREG02_TFUSTOP) &&
					!getBit(stsReg02Value, CPC_STSREG02_SOFTLDST) &&
					!bStsReg01Bad && !bStsReg02Bad;
				break;
			case UN_FACEPLATE_BUTTON_RESET_TEMPORARY_AS_FULL_STOP:
				buttonEnabled = buttonEnabled && bSelected && 
					!getBit(stsReg01Value, UN_STSREG01_AUMOST) &&
					getBit(stsReg02Value, UN_STSREG02_TFUSTOP) &&
					!getBit(stsReg02Value, CPC_STSREG02_SOFTLDST) &&
					!bStsReg01Bad && !bStsReg02Bad;
				break;
			case UN_FACEPLATE_BUTTON_BLOCK_ALARM_ACTION:
				buttonEnabled = buttonEnabled && bSelected && 
					!getBit(stsReg01Value, UN_STSREG01_AUMOST) &&
					!getBit(stsReg02Value, UN_STSREG02_ALACTB) &&
					!getBit(stsReg02Value, CPC_STSREG02_SOFTLDST) &&
					!bStsReg01Bad && !bStsReg02Bad;
				break;
			case UN_FACEPLATE_BUTTON_DEBLOCK_ALARM_ACTION:
				buttonEnabled = buttonEnabled && bSelected && 
					!getBit(stsReg01Value, UN_STSREG01_AUMOST) &&
					getBit(stsReg02Value, UN_STSREG02_ALACTB) &&
					!getBit(stsReg02Value, CPC_STSREG02_SOFTLDST) &&
					!bStsReg01Bad && !bStsReg02Bad;
				break;
			case UN_FACEPLATE_BUTTON_FORCED_MODE:
				buttonEnabled = buttonEnabled && bSelected && 
					!getBit(stsReg01Value, UN_STSREG01_FOMOST) &&
					!getBit(stsReg01Value, UN_STSREG01_AUINHFMO) &&
					!bStsReg01Bad && !bStsReg02Bad;
				break;
			case UN_FACEPLATE_BUTTON_MANUAL_MODE:
				buttonEnabled = buttonEnabled && bSelected && 
					!getBit(stsReg01Value, UN_STSREG01_MMOST) &&
					!getBit(stsReg01Value, UN_STSREG01_AUINHMMO) &&
					!getBit(stsReg02Value, CPC_STSREG02_SOFTLDST) &&
					!bStsReg01Bad && !bStsReg02Bad;
				break;
			case UN_FACEPLATE_BUTTON_AUTO_MODE:
			case UN_FACEPLATE_BUTTON_ONOPEN_REQUEST:
			case UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST:
			case UN_FACEPLATE_BUTTON_CONTROLLED_STOP_REQUEST:
			case UN_FACEPLATE_BUTTON_OPTION_MODES:
			case UN_FACEPLATE_BUTTON_MANUAL_AUTO_TO_DEPENDENT_OBJECT:
				buttonEnabled = buttonEnabled && bSelected && 
					!getBit(stsReg01Value, UN_STSREG01_AUMOST) &&
					!getBit(stsReg02Value, CPC_STSREG02_SOFTLDST) &&
					!bStsReg01Bad && !bStsReg02Bad;
				break;
			case UN_FACEPLATE_BUTTON_ACK_ALARM:
				buttonEnabled = buttonEnabled &&
					!getBit(stsReg02Value, CPC_STSREG02_SOFTLDST) &&
					!bStsReg01Bad && !bStsReg02Bad;
			case UN_FACEPLATE_BUTTON_NEXT:
			case UN_FACEPLATE_BUTTON_BACK:
			default:
				break;
			}
		setValue(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", buttonEnabled);
		}
	dyn_string exceptionInfo;
	cpcFaceplateButtonState_ButtonSetState(makeDynString(CPC_FACEPLATE_BUTTON_ALLOW_RESTART), dsUserAccess, deviceName, sDpType, dsData, exceptionInfo);
// 3. Button Select state
	unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_ProcessControlObject_ButtonDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_ProcessControlObject_ButtonDisconnection()
{
	unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_NEXT, UN_FACEPLATE_BUTTON_BACK, UN_FACEPLATE_BUTTON_AUTO_MODE,
							  UN_FACEPLATE_BUTTON_ONOPEN_REQUEST, UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST, UN_FACEPLATE_BUTTON_ACK_ALARM,
							  UN_FACEPLATE_BUTTON_CONTROLLED_STOP_REQUEST, UN_FACEPLATE_BUTTON_RESET_TEMPORARY_AS_FULL_STOP,
							  UN_FACEPLATE_BUTTON_OPTION_MODES, UN_FACEPLATE_BUTTON_MANUAL_AUTO_TO_DEPENDENT_OBJECT, 
							  UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_BUTTON_BLOCK_ALARM_ACTION, UN_FACEPLATE_BUTTON_MANUAL_MODE,
							  UN_FACEPLATE_BUTTON_DEBLOCK_ALARM_ACTION, UN_FACEPLATE_BUTTON_SET_TEMPORARY_AS_FULL_STOP,
							  UN_FACEPLATE_BUTTON_SELECT));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_ProcessControlObject_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in ProcessControlObject widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_ProcessControlObject_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName;
	int iAction;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;
	bool bRemote;
		
	deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
	if (deviceName == "")	// In case of disconnection
		{
		deviceName = g_sDpName;
		}
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;
	
	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;
		
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			CPC_ProcessControlObject_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = deviceName;
			CPC_ProcessControlObject_WidgetConnect(deviceName, sPlcName, sSystemName);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			CPC_ProcessControlObject_WidgetDisconnect(deviceName, sPlcName, sSystemName);
			CPC_ProcessControlObject_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}
//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_ProcessControlObject_WidgetConnect
/**
Purpose: Connect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in ProcessControlObject faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_ProcessControlObject_WidgetConnect(string deviceName, string sPlcName, string sSystemName)
{
	int iRes, iAlertType1, iAlertType2, iAlertType3, iAlertType4;
	dyn_string exceptionInfo;

//  check if the device name is of type UN_CONFIG_CPC_PROCESSCONTROLOBJECT_DPT_NAME if not like of no PLC. -> not available.
	if(dpTypeName(deviceName) != UN_CONFIG_CPC_PROCESSCONTROLOBJECT_DPT_NAME)
		g_bUnSystemAlarmPlc = false;

	iRes = dpGet(deviceName + ".ProcessInput.StartISt:_alert_hdl.._type", iAlertType1,
						 deviceName + ".ProcessInput.TStopISt:_alert_hdl.._type", iAlertType2,
						 deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._type", iAlertType3,
						 deviceName + ".ProcessInput.AlSt:_alert_hdl.._type", iAlertType4);
	g_bBoolAlert = ((iRes >= 0) && (iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType2 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType3 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType4 == DPCONFIG_ALERT_BINARYSIGNAL));
	g_bMRestart = cpcGenericDpFunctions_getDeviceProperty(deviceName, CPC_PARAMS_MRESTART, exceptionInfo);
	if(g_bUnSystemAlarmPlc) {
        if (g_bBoolAlert) {
            iRes = dpConnect("CPC_ProcessControlObject_WidgetAnimationCB",
                            deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                            deviceName + ".statusInformation.selectedManager",
                            deviceName + ".ProcessInput.StsReg01",
                            deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                            deviceName + ".ProcessInput.StsReg02",
                            deviceName + ".ProcessInput.StsReg02:_online.._invalid",
                            deviceName + ".ProcessInput.OpMoSt",
                            deviceName + ".ProcessInput.OpMoSt:_online.._invalid",
                            deviceName + ".ProcessInput.AuOpMo:_online.._invalid",
                            deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
                            deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state",
                            deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state",
                            deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
                            deviceName + ".eventMask");
        } else {
            iRes = dpConnect("CPC_ProcessControlObject_WidgetAnimationCB_NoAlarm",
                            deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                            deviceName + ".statusInformation.selectedManager",
                            deviceName + ".ProcessInput.StsReg01",
                            deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                            deviceName + ".ProcessInput.StsReg02",
                            deviceName + ".ProcessInput.StsReg02:_online.._invalid",
                            deviceName + ".ProcessInput.OpMoSt",
                            deviceName + ".ProcessInput.OpMoSt:_online.._invalid",
                            deviceName + ".ProcessInput.AuOpMo:_online.._invalid",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
                            deviceName + ".eventMask");
        }
        g_bCallbackConnected = (iRes >= 0);
    } else {
        CPC_ProcessControlObject_WidgetDisconnection(g_sWidgetType);
    }
}
//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_ProcessControlObject_WidgetDisconnect
/**
Purpose: Disconnect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in ProcessControlObject faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_ProcessControlObject_WidgetDisconnect(string deviceName, string sPlcName, string sSystemName)
{
    int iRes;

    if(g_bUnSystemAlarmPlc) {
        if (g_bBoolAlert) {
            iRes = dpDisconnect("CPC_ProcessControlObject_WidgetAnimationCB",
                            deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                            deviceName + ".statusInformation.selectedManager",
                            deviceName + ".ProcessInput.StsReg01",
                            deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                            deviceName + ".ProcessInput.StsReg02",
                            deviceName + ".ProcessInput.StsReg02:_online.._invalid",
                            deviceName + ".ProcessInput.OpMoSt",
                            deviceName + ".ProcessInput.OpMoSt:_online.._invalid",
                            deviceName + ".ProcessInput.AuOpMo:_online.._invalid",
                            deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
                            deviceName + ".ProcessInput.TStopISt:_alert_hdl.._act_state",
                            deviceName + ".ProcessInput.FuStopISt:_alert_hdl.._act_state",
                            deviceName + ".ProcessInput.AlSt:_alert_hdl.._act_state",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
                            deviceName + ".eventMask");
        } else {
            iRes = dpDisconnect("CPC_ProcessControlObject_WidgetAnimationCB_NoAlarm",
                            deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                            deviceName + ".statusInformation.selectedManager",
                            deviceName + ".ProcessInput.StsReg01",
                            deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                            deviceName + ".ProcessInput.StsReg02",
                            deviceName + ".ProcessInput.StsReg02:_online.._invalid",
                            deviceName + ".ProcessInput.OpMoSt",
                            deviceName + ".ProcessInput.OpMoSt:_online.._invalid",
                            deviceName + ".ProcessInput.AuOpMo:_online.._invalid",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
                            sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
                            deviceName + ".eventMask");
        }
        g_bCallbackConnected = !(iRes >= 0);
    }
}

CPC_ProcessControlObject_WidgetAnimationCB_NoAlarm(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
                                         string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
                                         string sDpStsReg02, bit32 bit32StsReg02, string sDp5, bool bStsReg02Invalid,
                                         string sDpOpMoSt, int iOpMoSt, string sDp7, bool bOpMoStInvalid,
                                         string sDp8, bool bAuOpMoInvalid,
                                         string sDpAl, int iSystemIntegrityAlarmValue, string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
                                         string sEvtMask, int iMask)
{
    CPC_ProcessControlObject_WidgetAnimationCB(sDpLock, bLocked, sDpManager, sSelectedManager,
                                             sDpStsReg01, bit32StsReg01, sDp4, bStsReg01Invalid,
                                             sDpStsReg02, bit32StsReg02, sDp5, bStsReg02Invalid,
                                             sDpOpMoSt, iOpMoSt, sDp7, bOpMoStInvalid,
                                             sDp8, bAuOpMoInvalid,
                                             "", 0, "", 0, "", 0,
                                             sDpAl, iSystemIntegrityAlarmValue, sDpEnabled, bSystemIntegrityAlarmEnabled,
                                             sEvtMask, iMask);
}

// CPC_ProcessControlObject_WidgetAnimationCB
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in ProcessControlObject faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_ProcessControlObject_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
                                         string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
                                         string sDpStsReg02, bit32 bit32StsReg02, string sDp5, bool bStsReg02Invalid,
                                         string sDpOpMoSt, int iOpMoSt, string sDp7, bool bOpMoStInvalid,
                                         string sDp8, bool bAuOpMoInvalid,
                                         string sDp9, int iStartIStState, string sDp10, int iTStopIStState, string sDp11, int iFuStopIStState, string sDp12, int iAlSt,
                                         string sDpAl, int iSystemIntegrityAlarmValue, string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
                                         string sEvtMask, int iMask)
{
	string deviceName, sFunction, blockColor, blockLetter, pcoColor, pcoName, sOpMo, displayColor;
	string sColorSelect, sWarningLetter, sWarningColor, sAlarmLetter, sAlarmColor, sControlLetter, sControlColor, sBodyColor;
	bool bLockVisible, isRestartNeeded;
	dyn_string dsFunctions, exceptionInfo;

	deviceName = unGenericDpFunctions_getDpName(sDpStsReg01);
  isRestartNeeded = g_bMRestart && getBit(bit32StsReg02, CPC_STSREG02_NEEDRESTART) == 0;
// 1. Select
	unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, sColorSelect, bLockVisible);
// 2. Warning (Include the second stsReg: stsReg02 to check the warning letter)
	unGenericObject_WidgetWarningAnimationDoubleStsReg(bit32StsReg01, bit32StsReg02,
						makeDynInt(UN_STSREG01_AUMRW, UN_STSREG01_IOSIMUW, UN_STSREG01_IOERRORW),
						makeDynInt(UN_STSREG02_ALBW),
						makeDynString(CPC_WIDGET_TEXT_WARNING_DEFAULT, CPC_WIDGET_TEXT_WARNING_SIMU, CPC_WIDGET_TEXT_WARNING_ERROR),
						makeDynString(CPC_WIDGET_TEXT_WARNING_BLOCKED),
						sWarningLetter, sWarningColor);
// 2.1 if the device is not correclty running (the iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value) 
// or the checking is not enabled then set the letter to O and the color to not valid data

	if ((!bSystemIntegrityAlarmEnabled) || (iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value))
		{
		sWarningLetter = CPC_WIDGET_TEXT_OLD_DATA;
		sWarningColor = "unDataNotValid";
		}

	if (bStsReg01Invalid || bStsReg02Invalid || bOpMoStInvalid || bAuOpMoInvalid)
		{
		sWarningLetter = CPC_WIDGET_TEXT_INVALID;
		sWarningColor = "unDataNotValid";
		}
    if (!bStsReg01Invalid && !bStsReg02Invalid) {
        // 3. Alarm
        unGenericObject_WidgetAlarmTextAnimation(bit32StsReg01, 
                            makeDynInt(CPC_STSREG01_ALST, UN_STSREG01_STARTIST, CPC_STSREG01_TSTOPIST, CPC_STSREG01_FUSTOPIST), 
                            makeDynString(CPC_WIDGET_TEXT_ALARM_ALST, CPC_WIDGET_TEXT_ALARM_STARTIST, CPC_WIDGET_TEXT_ALARM_TSTOPIST, CPC_WIDGET_TEXT_ALARM_FUSTOPIST),
                            sAlarmLetter, sAlarmColor);
        if (sAlarmLetter == "" && isRestartNeeded) {
            sAlarmLetter = CPC_WIDGET_TEXT_MANUAL_RESTART;
            sAlarmColor = "unFaceplate_WarningActive";
        }
        // 4. Control state
        cpcGenericObject_WidgetControlStateAnimation(bit32StsReg01, bit32StsReg02, makeDynString("CPC_STSREG02_SOFTLDST", "CPC_STSREG01_AUIHFOMOST", "UN_STSREG01_AUINHMMO", "UN_STSREG01_MMOST", "UN_STSREG01_FOMOST"), sControlLetter, sControlColor);
        // 5. Block text
        if (getBit(bit32StsReg02, UN_STSREG02_TFUSTOP) == 1) {
            blockLetter = CPC_WIDGET_TEXT_BLOCK_TFUSTOP;
            blockColor = "unFaceplate_StatusAlarmActive";
        }
        if (getBit(bit32StsReg02, UN_STSREG02_ALACTB) == 1) {
            blockLetter = CPC_WIDGET_TEXT_ALARM_BLOCKED;
            blockColor = "unFaceplate_StatusAlarmActive";
        }
    }
// 6. Animate letters and select
	if(g_bSystemConnected) {
		setMultiValue("WarningText", "text", sWarningLetter, "WarningText", "foreCol", sWarningColor,
					  "AlarmText", "text", sAlarmLetter, "AlarmText", "foreCol", sAlarmColor,
					  "ControlStateText", "text", sControlLetter, "ControlStateText", "foreCol", sControlColor,
					  "SelectArea", "foreCol", sColorSelect, "LockBmp", "visible", bLockVisible,
					  "BlockText", "text", blockLetter, "BlockText", "foreCol", blockColor,
					  "WidgetArea", "visible", true);
	}
// 7. Animate body
	sBodyColor = "unWidget_ControlStateAuto";
	if (bStsReg01Invalid || bStsReg02Invalid) {
		sBodyColor = "unDataNotValid";
	} else {
		if (sControlLetter == CPC_WIDGET_TEXT_CONTROL_FORCED) {
			sBodyColor = "unWidget_ControlStateForced";
		}
    if (isRestartNeeded) {
      sBodyColor = "unFaceplate_WarningActive";
    }
    if (sAlarmLetter == CPC_WIDGET_TEXT_ALARM_TSTOPIST || sAlarmLetter == CPC_WIDGET_TEXT_ALARM_FUSTOPIST) {
      sBodyColor = "unFaceplate_AlarmActive";
    }
    if (getBit(bit32StsReg01, UN_STSREG01_ALUNACK) == 1 && (iTStopIStState == 1 || iTStopIStState == 3 || iFuStopIStState == 1 || iFuStopIStState == 3)) {
      sBodyColor = "unWidget_AlarmNotAck";
    }

		if (blockLetter == CPC_WIDGET_TEXT_ALARM_BLOCKED) {
			sBodyColor = "unFaceplate_StatusAlarmActive";
		}
	}
    sFunction = cpcGenericObject_GetAnimationFunction(g_sWidgetType);
	if (isFunctionDefined(sFunction) && (sFunction != "")) {
		if(g_bSystemConnected)
			execScript("main(int iOn, int iOff, string sBodyColor) {" +
						sFunction + "(iOn, iOff, sBodyColor);}", 
						makeDynString(), getBit(bit32StsReg01, UN_STSREG01_ONST), getBit(bit32StsReg01, UN_STSREG01_OFFST), sBodyColor);
	}
// 8. PCO name
	pcoColor = sBodyColor;
	if (getBit(bit32StsReg01, UN_STSREG01_OFFST) == 0 && g_sWidgetType != "Air") {
		pcoColor = "unWidget_AlternativeColor";
	}
	if (bStsReg01Invalid || bStsReg02Invalid) {
		pcoColor = "unDataNotValid";
	}
	CPC_ProcessControlObject_getName(deviceName, pcoName);
	if(g_bSystemConnected)
		setMultiValue("PCOName", "text", pcoName, "PCOName", "foreCol", pcoColor);
// 9. Option mode
	CPC_ProcessControlObject_getOptionModesLabel(deviceName, iOpMoSt, sOpMo);
	displayColor = "cpcColor_Widget_Request";
	if (bOpMoStInvalid) {
		displayColor = "unDataNotValid";
	}
	if(g_bSystemConnected) {
		setMultiValue("OptionMode", "text", sOpMo, "OptionMode", "foreCol", displayColor);
	}

// display event state
	if(g_bSystemConnected) {
		eventState.visible = iMask == 0;
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_ProcessControlObject_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType, string, input, widget type. ex: "heater"

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CPC_ProcessControlObject_WidgetDisconnection(string sWidgetType)
{
	cpcGenericObject_WidgetDisconnection(sWidgetType, UN_CONFIG_CPC_PROCESSCONTROLOBJECT_DPT_NAME);
	unGenericObject_WidgetDisplayValueDisconnect(makeDynString("OptionMode", "PCOName"));
	setValue("BlockText", "text", "");

	// set event state
	eventState.visible	= false;
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_ProcessControlObject_getOptionModesLabel
/**
Purpose: this function is called by a PCO object to get an option mode label

Parameter:
	-> sDpName, string input, device name
	-> uLabel, unsigned input, label number	
	-> result, string output, result of the function								  

Usage: Public

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_ProcessControlObject_getOptionModesLabel(string sDpName, unsigned uLabel, string &result)
{
	string sOptionModes;
	dyn_string dsOptionModes;
	
	result = "";
	sOptionModes = unGenericDpFunctions_getDescription(sDpName + UN_PCO_OPTION_MODE_PATH);
	dsOptionModes = strsplit(sOptionModes, UN_PCO_OPTION_MODE_DELIMITER);
	if ((dynlen(dsOptionModes) >= uLabel) && (uLabel > 0) && (UN_PCO_OPTION_MODE_NUMBER >= uLabel))
		{
		result = dsOptionModes[uLabel];
		}
	if (result == "")
		{
		//result = "Op Mode " + uLabel;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_ProcessControlObject_getOptionModesAllowance
/**
Purpose: this function is called by a PCO object to get an option mode allowance

Parameter:
	-> sDpName, string input, device name
	-> uLabel, unsigned input, label number	
	-> result, bit32 output, result of the function								  

Usage: Public

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_ProcessControlObject_getOptionModesAllowance(string sDpName, unsigned uLabel, bit32 &result)
{
	string sOptionModes;
	dyn_string dsOptionModes;
	
	result = 0;
	sOptionModes = unGenericDpFunctions_getDescription(sDpName + UN_PCO_OPTION_MODE_PATH);
	dsOptionModes = strsplit(sOptionModes, UN_PCO_OPTION_MODE_DELIMITER);
	if ((dynlen(dsOptionModes) >= (UN_PCO_OPTION_MODE_NUMBER + uLabel)) && (uLabel > 0))
		{
		result = dsOptionModes[UN_PCO_OPTION_MODE_NUMBER + uLabel];
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_ProcessControlObject_setOptionModes
/**
Purpose: this function is called by a PCO object to set the option mode labels & allowance

Parameter:
	-> sDpName, string input, device name
	-> dsLabels, dyn_string input, labels
	-> dsAllowances, dyn_string input, allowances (must be bit32 variables converted to string)
	-> iRes, int output, result of the function, < 0 if an error occured						  

Usage: Public

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_ProcessControlObject_setOptionModes(string sDpName, dyn_string dsLabels, dyn_string dsAllowances, int &iRes)
{
	int i,length;
	string description;
	
	iRes = -1;
	length = dynlen(dsLabels);
	if ((length != dynlen(dsAllowances)) || (length == 0))
		{
		//error
		}
	else
		{
		for(i=1;i<=UN_PCO_OPTION_MODE_NUMBER;i++)			// Labels
			{
			if (i <= length)
				{
				description = description + UN_PCO_OPTION_MODE_DELIMITER + dsLabels[i];
				}
			else
				{
				description = description + UN_PCO_OPTION_MODE_DELIMITER;
				}
			}
		for(i=1;i<=UN_PCO_OPTION_MODE_NUMBER;i++)			// Allowances
			{
			if (i <= length)
				{
				description = description + UN_PCO_OPTION_MODE_DELIMITER + dsAllowances[i];
				}
			else
				{
				description = description + UN_PCO_OPTION_MODE_DELIMITER;
				}
			}
		description = substr(description, 1);
		iRes = dpSetDescription(sDpName + UN_PCO_OPTION_MODE_PATH, description);
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_ProcessControlObject_getName
/**
Purpose: this function is called by a PCO object to get PCO name

Parameter:
	-> sDpName, string input, device name
	-> result, string output, result of the function								  

Usage: Public

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_ProcessControlObject_getName(string sDpName, string &result)
{
	dyn_string dsParameters;
	
	unGenericDpFunctions_getParameters(sDpName, dsParameters);
	result = dsParameters[UN_PARAMETER_NAME];
	if (result == "")
		{
		result = "PCO";
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_ProcessControlObject_setName
/**
Purpose: this function is called by a PCO object to set the PCO name

Parameter:
	-> sDpName, string input, device name
	-> name, string input, name to be set						  
	-> exceptionInfo, dyn_string output, for errors

Usage: Public

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_ProcessControlObject_setName(string sDpName, string name, dyn_string &exceptionInfo)
{
	unGenericDpFunctions_changeDpParameter(sDpName, UN_PARAMETER_NAME, name, exceptionInfo);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CPC_ProcessControlObject_getNextAllowedModes
/**
Purpose: to get the modes you can access from current mode (using allowance tables)

Parameter:
	-> sDpName, string input, device name
	-> iCurrentMode, int input, current mode
	-> bRunning, bool input, true while running
	-> dsAllowedModes, dyn_string output, result of the function
	-> exceptionInfo, dyn_string output, for errors
	
Usage: Public

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CPC_ProcessControlObject_getNextAllowedModes(string sDpName, int iCurrentMode, bool bRunning, dyn_string &dsAllowedModes, dyn_string &dsUnAllowedModes, dyn_string &exceptionInfo)
{
	string parameters;
	dyn_string dsParameters;
	bit32 allowance;

	dynClear(dsAllowedModes);
	dynClear(dsUnAllowedModes);

	// 1. Get parameters
	parameters = unGenericDpFunctions_getDescription(sDpName + UN_PCO_OPTION_MODE_PATH);
	dsParameters = strsplit(parameters, UN_PCO_OPTION_MODE_DELIMITER);
	while (dynlen(dsParameters) < (2*UN_PCO_OPTION_MODE_NUMBER)) {
		dynAppend(dsParameters, "");
	}
	if (bRunning) {
		// 3. While running -> allowed modes are coded in the allowance table
		allowance = dsParameters[iCurrentMode+UN_PCO_OPTION_MODE_NUMBER];
		for(int i = 1; i <= UN_PCO_OPTION_MODE_NUMBER; i++) {
			if (strltrim(dsParameters[i]) != "") {
				if ((getBit(allowance,UN_PCO_OPTION_MODE_NUMBER-i) == 1) && (i != iCurrentMode)) {
					dynAppend(dsAllowedModes,dsParameters[i]);
				} else {
					dynAppend(dsUnAllowedModes,dsParameters[i]);
				}
			}
		}
	} else {
		// 2. Not running -> each defined mode is allowed
		for(int i = 1; i <= UN_PCO_OPTION_MODE_NUMBER; i++) {
			if (strltrim(dsParameters[i]) != "") {
				if (i != iCurrentMode) {
					dynAppend(dsAllowedModes, dsParameters[i]);
				} else {
					dynAppend(dsUnAllowedModes, dsParameters[i]);
				}
			}
		}
	}
}

