/**@name LIBRARY: cpcProcessControlObjectConfig.ctl

@author: Vincent Forest (LHC-IAS)

Creation Date: 05 12 2002

Modification History: 
    16/05/2012: Alexey
        [UCPC-787] introduce AlSt bit and letter constants

    27/03/2012: Alexey
        [UCPC-733] change intelock names to common

	08/07/2011: Alexey
		[UCPC-389] moved "parameters" field

	01/07/2011: Alexey
		[UCPC-364] added getParamNames method which returns all possible params
		[UCPC-374] added PRstartFS

	2011/05/30 Alexey
		[UCPC-292] moved meta-info to the end of config line; removed UN_CPC_CONFIG length constants

	2011/05/16 amerezhi
		moved export functions from cpcExportGenericFunctions

	2011/05/09 amerezhi
		[UCPC-235] added metainfo (types relationships configuraiton)

	28/03/2011: amerezhi
		- updated import of PCO with new code structure

	18/04/2006: Herve
		- implementation of mask event

	15/06/2005: Enrique
		- S7 adaptation, new functions:
			
	22/04/2005: Herve
		- allow archive in import file per device

	15/04/2005: Herve
		- Modbus set and check front-end functions: _UnPlc_CPC_ProcessControlObject_checkConfig, _UnPlc_CPC_ProcessControlObject_setConfig

	20/10/2003: Herve 
		- config of "OpMoSt", "AuOpMo" and "MOpMoR" set to PVSS_MODBUS_FLOAT instead of PVSS_MODBUS_INT16

version 2.0

External Function : 
	. CPC_ProcessControlObjectConfig_setConfig :  set configuration of given ProcessControlObject datapoint
	. CPC_ProcessControlObjectConfig_checkConfig : check data before configuration
	
Internal Functions :
	
Purpose: 
This library contains the function used to set Unicos ProcessControlObject configuration.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

// Constants
const string UN_CONFIG_CPC_PROCESSCONTROLOBJECT_DPT_NAME = "CPC_ProcessControlObject";

const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_LENGTH = 26;

const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_PCONAME = 1;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_NORMAL_POSITION = 2;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_ADDRESS_STSREG01 = 3;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_ADDRESS_STSREG02 = 4;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_ADDRESS_EVSTSREG01 = 5;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_ADDRESS_EVSTSREG02 = 6;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_ADDRESS_AUOPMO = 7;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_ADDRESS_OPMOST = 8;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_ADDRESS_MANREG01 = 9;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_ADDRESS_MOPMOR = 10;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_NAME_1 = 11;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_NAME_2 = 12;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_NAME_3 = 13;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_NAME_4 = 14;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_NAME_5 = 15;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_NAME_6 = 16;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_NAME_7 = 17;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_NAME_8 = 18;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_1 = 19;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_2 = 20;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_3 = 21;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_4 = 22;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_5 = 23;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_6 = 24;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_7 = 25;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_8 = 26;

const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_ADDITIONAL_LENGTH          = 7;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_ADDITIONAL_MASKEVENT       = 1;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_ADDITIONAL_PARAMETERS      = 2;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_ADDITIONAL_MASTER_NAME     = 3;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_ADDITIONAL_PARENTS         = 4;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_ADDITIONAL_CHILDREN        = 5;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_ADDITIONAL_TYPE            = 6;
const unsigned UN_CONFIG_CPC_PROCESSCONTROLOBJECT_ADDITIONAL_SECOND_ALIAS    = 7;

mapping CPC_ProcessControlObjectConfig_getConfig() {
	mapping config;
	mapping props;

	mappingClear(props);
	props["address"] 		= ".ProcessInput.StsReg01";
	props["dataType"] 		= CPC_UINT16;
	config["StsReg01"] 		= props;

	mappingClear(props);
	props["address"] 		= ".ProcessInput.StsReg02";
	props["dataType"] 		= CPC_UINT16;
	config["StsReg02"] 		= props;

	mappingClear(props);
	props["address"] 		= ".ProcessInput.OpMoSt";
	props["dataType"] 		= CPC_FLOAT;
	config["OpMoSt"] 		= props;

	mappingClear(props);
	props["address"] 		= ".ProcessInput.AuOpMo";
	props["dataType"] 		= CPC_FLOAT;
	config["AuOpMo"] 		= props;

	mappingClear(props);
	props["address"] 		= ".ProcessInput.StartISt";
	props["dpe"]			= "StsReg01";
	props["bitPosition"]	= UN_STSREG01_STARTIST;
	props["isAlarm"]		= true;
	config["StartISt"] 		= props;

	mappingClear(props);
	props["address"] 		= ".ProcessInput.TStopISt";
	props["dpe"]			= "StsReg01";
	props["bitPosition"]	= CPC_STSREG01_TSTOPIST;
	props["isAlarm"]		= true;
	config["TStopISt"] 		= props;

	mappingClear(props);
	props["address"] 		= ".ProcessInput.FuStopISt";
	props["dpe"]			= "StsReg01";
	props["bitPosition"]	= CPC_STSREG01_FUSTOPIST;
	props["isAlarm"]		= true;
	config["FuStopISt"] 	= props;

  mappingClear(props);
  props["address"]      = ".ProcessInput.AlSt";
  props["dpe"]          = "StsReg01";
  props["bitPosition"]  = CPC_STSREG01_ALST;
  props["isAlarm"]      = true;
  config["AlSt"]        = props;

	mappingClear(props);
	props["hasArchive"] 	= true;
	props["address"] 		= ".ProcessInput.evStsReg01";
	props["dataType"] 		= CPC_INT32;
	config["evStsReg01"] 	= props;

	mappingClear(props);
	props["hasArchive"] 	= true;
	props["address"] 		= ".ProcessInput.evStsReg02";
	props["dataType"] 		= CPC_INT32;
	config["evStsReg02"] 	= props;

	mappingClear(props);
	props["address"] 		= ".ProcessOutput.ManReg01";
	props["dataType"] 		= CPC_UINT16;
	config["ManReg01"] 		= props;

	mappingClear(props);
	props["address"] 		= ".ProcessOutput.MOpMoR";
	props["dataType"] 		= CPC_FLOAT;
	config["MOpMoR"] 		= props;

	return config;
}

dyn_string CPC_ProcessControlObjectConfig_getParamNames() {
	return makeDynString(CPC_PARAMS_MRESTART, CPC_PARAMS_RSTARTFS);
}

//------------------------------------------------------------------------------------------------------------------------
// CPC_ProcessControlObjectConfig_checkConfig
/**
Check data before configuration

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_ProcessControlObjectConfig_checkConfig(dyn_string configLine, dyn_string &exceptionInfo) {
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
		// 1. Common
		unConfigGenericFunctions_checkParameters(configLine, UN_CONFIG_CPC_PROCESSCONTROLOBJECT_DPT_NAME, exceptionInfo);
		// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, UN_CONFIG_CPC_PROCESSCONTROLOBJECT_DPT_NAME, exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","CPC_ProcessControlObjectConfig_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

_UnPlc_CPC_ProcessControlObject_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_PROCESSCONTROLOBJECT_DPT_NAME, configLine, exceptionInfo);
}

S7_PLC_CPC_ProcessControlObject_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_PROCESSCONTROLOBJECT_DPT_NAME, configLine, exceptionInfo);
}


CPC_ProcessControlObjectConfig_checkCustomConfig(dyn_string configLine, bool hasArchive, dyn_string &exceptionInfo) {
	for (int i=UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_1;i<=UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_8;i++) {
		string allowance = configLine[UN_CONFIG_COMMON_LENGTH + i];
		if ((strlen(allowance) != 0) && (strlen(allowance) != 8)) {
			fwException_raise(exceptionInfo,"ERROR","_UnPlc_CPC_ProcessControlObject_checkConfig: " + getCatStr("unGeneration","BADALLOWANCE"),"");
		} else {
			strreplace(allowance, "1", "");
			strreplace(allowance, "0", "");
			if (allowance != "") {
				fwException_raise(exceptionInfo,"ERROR","_UnPlc_CPC_ProcessControlObject_checkConfig: " + getCatStr("unGeneration","BADALLOWANCE"),"");
			}
		}
	}
}

//------------------------------------------------------------------------------------------------------------------------
// CPC_ProcessControlObjectConfig_setConfig
/**
Creates dp (or checks if it is already presented)and sets configuration for ProcessControlObject object.

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_ProcessControlObjectConfig_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo) {
	string sDpName;
	
	// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, UN_CONFIG_CPC_PROCESSCONTROLOBJECT_DPT_NAME, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, UN_CONFIG_CPC_PROCESSCONTROLOBJECT_DPT_NAME, exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
			// 2. Generate device
			// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
			// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, UN_CONFIG_CPC_PROCESSCONTROLOBJECT_DPT_NAME, exceptionInfo);
		}
	}
}

_UnPlc_CPC_ProcessControlObject_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

S7_PLC_CPC_ProcessControlObject_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

CPC_ProcessControlObjectConfig_setCustomConfig(dyn_string dsConfigs, bool hasArchive, dyn_string &exceptionInfo) {
	int iRes;
	CPC_ProcessControlObject_setName(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_PROCESSCONTROLOBJECT_PCONAME], exceptionInfo);
	CPC_ProcessControlObject_setOptionModes(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME],
								 makeDynString(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_NAME_1],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_NAME_2],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_NAME_3],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_NAME_4],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_NAME_5],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_NAME_6],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_NAME_7],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_NAME_8]),
								 makeDynString(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_1],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_2],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_3],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_4],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_5],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_6],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_7],
								 			   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_PROCESSCONTROLOBJECT_MODE_ALLOWANCE_8]),
								 iRes);
	if (iRes < 0)
	{
		fwException_raise(exceptionInfo,"ERROR","_UnPlc_CPC_ProcessControlObject_setConfig: set option modes failed","");
	}
}

//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_CPC_ProcessControlObject_ExportConfig
/**
Purpose: Export PCO Devices for _UnPlc_CPC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
_UnPlc_CPC_ProcessControlObject_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	string sObject = UN_CONFIG_CPC_PROCESSCONTROLOBJECT_DPT_NAME;
	int amount = dynlen(dsDpList);
	for(int i = 1; i <= amount; i++) {
		dyn_string dsDpParameters;
		string currentDP = dsDpList[i];

		if(!dpExists(currentDP)) continue;

		unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
		cpcExportGenericFunctions_getPCOName(currentDP, dsDpParameters);
		cpcExportGenericFunctions_getDigitalNormalPosition(currentDP, sObject, makeDynString(".ProcessInput.FuStopISt", ".ProcessInput.TStopISt", ".ProcessInput.StartISt"), dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg01", 	true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "StsReg02", 	true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "evStsReg01", 	true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "evStsReg02", 	true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "AuOpMo", 		true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "OpMoSt", 		true, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "ManReg01", 	false, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "MOpMoR", 		false, dsDpParameters);
		cpcExportGenericFunctions_getPCOParameters(currentDP, dsDpParameters);
		cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString(), dsDpParameters);
		cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_2, makeDynString(), dsDpParameters);
		cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString("ProcessInput.evStsReg01", "ProcessInput.evStsReg02"), dsDpParameters);
		cpcExportGenericFunctions_getMaskEvent(currentDP, dsDpParameters);
		cpcExportGenericFunctions_getParameters(currentDP, dsDpParameters, exceptionInfo);
		cpcExportGenericFunctions_getMetainfo(currentDP, dsDpParameters, exceptionInfo);

		unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
	}
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_CPC_ProcessControlObject_ExportConfig
/**
Purpose: Export PCO Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
S7_PLC_CPC_ProcessControlObject_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_CPC_ProcessControlObject_ExportConfig(dsDpList, exceptionInfo);
}
