/**@name LIBRARY: cpcConfigWord2Word2AnalogStatus.ctl

@author: Geraldine Thomas (EN-ICE/SIC)
Modifications History:
  $Log: CPC_Word2AnalogStatusConfig.ctl,v $
	08/07/2011: Alexey
		[UCPC-384] removed "parameters" field

	01/07/2011: Alexey
		[UCPC-364] added getParamNames method which returns all possible params
		fixed archives export

	2011/05/31 Alexey
		[UCPC-297] removed MaskEvent

	2011/05/30 Alexey
		[UCPC-292] moved meta-info to the end of config line; removed UN_CPC_CONFIG length constants

	2011/05/16 amerezhi
		moved export functions from cpcExportGenericFunctions

	2011/05/09 amerezhi
		[UCPC-235] added metainfo (types relationships configuraiton)

  2011/04/19 amerezhi
  [UCPC-236] updated for unCPC6

  2011/02/09 amerezhi
  Check and set device using UNCCPCConfig' general functions

  Revision 1.2  2010/09/27 11:25:05  amerezhi
  The checking for SCALE is changed from RANGE to 2 checkFloat

  Revision 1.1  2009/01/15 15:13:49  gthomas
  Added new functions to configure new type: Word2AnalogStatus
*/
/*
Internal Functions :
mapping CPC_Word2AnalogStatusConfig_getConfig() {
CPC_Word2AnalogStatusConfig_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo) {
CPC_Word2AnalogStatusConfig_checkCustomConfig(dyn_string dsConfigs, dyn_string &exceptionInfo) {
CPC_Word2AnalogStatusConfig_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo) {
CPC_Word2AnalogStatusConfig_setCustomConfig(dyn_string dsConfigs, dyn_string &exceptionInfo) {

*/

// Constants
const string UN_CONFIG_CPC_WORD2ANALOGSTATUS_DPT_NAME = "CPC_Word2AnalogStatus";

const unsigned UN_CONFIG_CPC_WORD2ANALOGSTATUS_LENGTH				= 11;

const unsigned UN_CONFIG_CPC_WORD2ANALOGSTATUS_UNIT					= 1;
const unsigned UN_CONFIG_CPC_WORD2ANALOGSTATUS_FORMAT				= 2;
const unsigned UN_CONFIG_CPC_WORD2ANALOGSTATUS_DEADBAND				= 3;
const unsigned UN_CONFIG_CPC_WORD2ANALOGSTATUS_DEADBAND_TYPE		= 4;
const unsigned UN_CONFIG_CPC_WORD2ANALOGSTATUS_RANGEMAX				= 5;
const unsigned UN_CONFIG_CPC_WORD2ANALOGSTATUS_RANGEMIN				= 6;
const unsigned UN_CONFIG_CPC_WORD2ANALOGSTATUS_OUTRANGEMAX			= 7;
const unsigned UN_CONFIG_CPC_WORD2ANALOGSTATUS_OUTRANGEMIN			= 8;
const unsigned UN_CONFIG_CPC_WORD2ANALOGSTATUS_ARCHIVE_ACTIVE		= 9;
const unsigned UN_CONFIG_CPC_WORD2ANALOGSTATUS_ARCHIVE_TIME_FILTER	= 10;
const unsigned UN_CONFIG_CPC_WORD2ANALOGSTATUS_ADDRESS_POSST		= 11;

const unsigned UN_CONFIG_CPC_WORD2ANALOGSTATUS_ADDITIONAL_LENGTH          = 5;
const unsigned UN_CONFIG_CPC_WORD2ANALOGSTATUS_ADDITIONAL_MASTER_NAME     = 1;
const unsigned UN_CONFIG_CPC_WORD2ANALOGSTATUS_ADDITIONAL_PARENTS         = 2;
const unsigned UN_CONFIG_CPC_WORD2ANALOGSTATUS_ADDITIONAL_CHILDREN        = 3;
const unsigned UN_CONFIG_CPC_WORD2ANALOGSTATUS_ADDITIONAL_TYPE            = 4;
const unsigned UN_CONFIG_CPC_WORD2ANALOGSTATUS_ADDITIONAL_SECOND_ALIAS    = 5;

mapping CPC_Word2AnalogStatusConfig_getConfig() {
	mapping config;
	mapping props;

	mappingClear(props);
	props["hasUnit"] 		= true;
	props["hasFormat"]		= true;
	props["hasSmooth"] 		= true;
	props["hasArchive"] 	= true;
	props["hasPvRange"] 	= true;
	props["address"] 		= ".ProcessInput.PosSt";
	props["dataType"] 		= CPC_INT16;
	config["PosSt"] 		= props;

	return config;
}

//------------------------------------------------------------------------------------------------------------------------
// CPC_Word2AnalogStatusConfig_checkConfig
/**
Check data before configuration

  @param[in]  configLine   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_Word2AnalogStatusConfig_checkConfig(dyn_string configLine, dyn_string &exceptionInfo) {
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
		// 1. Common
		unConfigGenericFunctions_checkParameters(configLine, UN_CONFIG_CPC_WORD2ANALOGSTATUS_DPT_NAME, exceptionInfo);
		// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, UN_CONFIG_CPC_WORD2ANALOGSTATUS_DPT_NAME, exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","CPC_Word2AnalogStatusConfig_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

_UnPlc_CPC_Word2AnalogStatus_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_WORD2ANALOGSTATUS_DPT_NAME, configLine, exceptionInfo);
}

S7_PLC_CPC_Word2AnalogStatus_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_checkDeviceConfig(UN_CONFIG_CPC_WORD2ANALOGSTATUS_DPT_NAME, configLine, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// CPC_Word2AnalogStatusConfig_checkCustomConfig
/**
Check custom data before configuration

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_Word2AnalogStatusConfig_checkCustomConfig(dyn_string dsConfigs, bool hasArchive, dyn_string &exceptionInfo) {
    float sMin, sMax;
	unConfigGenericFunctions_checkFloat(dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_WORD2ANALOGSTATUS_OUTRANGEMIN], sMin, exceptionInfo);
    unConfigGenericFunctions_checkFloat(dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_WORD2ANALOGSTATUS_OUTRANGEMAX], sMax, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// CPC_Word2AnalogStatusConfig_setConfig
/**
Creates dp (or checks if it is already presented)and sets configuration for Word2AnalogStatus object.

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_Word2AnalogStatusConfig_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo) {
	string sDpName;
	
	// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, UN_CONFIG_CPC_WORD2ANALOGSTATUS_DPT_NAME, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, UN_CONFIG_CPC_WORD2ANALOGSTATUS_DPT_NAME, exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
			// 2. Generate device
			// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
			// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, UN_CONFIG_CPC_WORD2ANALOGSTATUS_DPT_NAME, exceptionInfo);
		}
	}
}

_UnPlc_CPC_Word2AnalogStatus_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

S7_PLC_CPC_Word2AnalogStatus_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	cpcConfigGenericFunctions_setDeviceConfig(dsConfigs, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// CPC_Word2AnalogStatusConfig_setCustomConfig
/**
Set custom configuration of given Word2AnalogStatus datapoint

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
CPC_Word2AnalogStatusConfig_setCustomConfig(dyn_string dsConfigs, bool hasArchive, dyn_string &exceptionInfo) {
	float outRangeMax, outRangeMin, rangeMax, rangeMin;
	int iRes;
	outRangeMax = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_WORD2ANALOGSTATUS_OUTRANGEMAX];
	outRangeMin = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_WORD2ANALOGSTATUS_OUTRANGEMIN];
	rangeMax = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_WORD2ANALOGSTATUS_RANGEMAX];
	rangeMin = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_WORD2ANALOGSTATUS_RANGEMIN];

	// (b*x^2 + a) conversion
	float b = (outRangeMax - outRangeMin) / (rangeMax - rangeMin);
	float a = b*rangeMax - outRangeMax;

	string dpe = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessInput.PosSt";
	iRes = dpSetWait(dpe + ":_msg_conv.._type", DPCONFIG_CONVERSION_RAW_TO_ENG_MAIN,
					dpe + ":_msg_conv.1._type", DPDETAIL_CONV_POLY,
					dpe + ":_msg_conv.1._poly_grade", 1,
					dpe + ":_msg_conv.1._poly_a", a,
					dpe + ":_msg_conv.1._poly_b", b);
	if(iRes != 0) {
		fwException_raise(exceptionInfo,"ERROR","CPC_Word2AnalogStatusConfig_setConfig: error in convertion" + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_CPC_Word2AnalogStatus_ExportConfig
/**
Purpose: Export Word2AnalogStatus Devices for _UnPlc_CPC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void _UnPlc_CPC_Word2AnalogStatus_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	string sObject = UN_CONFIG_CPC_WORD2ANALOGSTATUS_DPT_NAME;
	int amount = dynlen(dsDpList);
	for(int i = 1; i <= amount; i++) {
		dyn_string dsDpParameters;
		string currentDP = dsDpList[i];
		string sLimitOn, sLimitOff;

		if(!dpExists(currentDP)) continue;

		unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
		cpcExportGenericFunctions_getUnit(currentDP, dsDpParameters);
		cpcExportGenericFunctions_getFormat(currentDP, dsDpParameters);
		cpcExportGenericFunctions_getDeadband(currentDP, dsDpParameters, exceptionInfo);
		cpcExportGenericFunctions_getRange(currentDP, dsDpParameters, exceptionInfo);
		cpcExportGenericFunctions_getOutRange(currentDP, dsDpParameters, exceptionInfo);
		cpcExportGenericFunctions_getArchive(currentDP, dsDpParameters);
		cpcExportGenericFunctions_processAddress(currentDP, sObject, "PosSt", 	true, dsDpParameters);
		cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString(), dsDpParameters);
		cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_2, makeDynString("ProcessInput.PosSt"), dsDpParameters);
		cpcExportGenericFunctions_getArchiveNameForDpes(currentDP, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString(), dsDpParameters);
		cpcExportGenericFunctions_getMetainfo(currentDP, dsDpParameters, exceptionInfo);

		unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
	}
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_CPC_Word2AnalogStatus_ExportConfig
/**
Purpose: Export Word2AnalogStatus Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void S7_PLC_CPC_Word2AnalogStatus_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_CPC_Word2AnalogStatus_ExportConfig(dsDpList, exceptionInfo);
}
